@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -28,6 +28,7 @@
 ****************************************************************************/
 
 #include "qtsingleapplication.h"
+#include "../tools/qtcreatorcrashhandler/crashhandlersetup.h"
 
 #include <app/app_version.h>
 #include <extensionsystem/iplugin.h>
@@ -70,11 +71,9 @@ static const char fixedOptionsC[] =
 "    -version                      Display program version\n"
 "    -client                       Attempt to connect to already running first instance\n"
 "    -settingspath <path>          Override the default path where user settings are stored\n"
-#if !defined(Q_OS_WIN)
-"    -pid <pid>                    Attempt to connect to instance given by pid\n";
-#else
-;
-#endif
+"    -pid <pid>                    Attempt to connect to instance given by pid\n"
+"    -block                        Block until editor is closed\n";
+
 
 static const char HELP_OPTION1[] = "-h";
 static const char HELP_OPTION2[] = "-help";
@@ -83,9 +82,8 @@ static const char HELP_OPTION4[] = "--help";
 static const char VERSION_OPTION[] = "-version";
 static const char CLIENT_OPTION[] = "-client";
 static const char SETTINGS_OPTION[] = "-settingspath";
-#if !defined(Q_OS_WIN)
 static const char PID_OPTION[] = "-pid";
-#endif
+static const char BLOCK_OPTION[] = "-block";
 
 typedef QList<PluginSpec *> PluginSpecSet;
 
@@ -295,7 +293,7 @@ int main(int argc, char **argv)
 
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
     // QML is unusable with the xlib backend
-    QApplication::setGraphicsSystem("raster");
+    QApplication::setGraphicsSystem(QLatin1String("raster"));
 #endif
 
     SharedTools::QtSingleApplication app((QLatin1String(appNameC)), argc, argv);
@@ -303,10 +301,16 @@ int main(int argc, char **argv)
     const int threadCount = QThreadPool::globalInstance()->maxThreadCount();
     QThreadPool::globalInstance()->setMaxThreadCount(qMax(4, 2 * threadCount));
 
+    setupCrashHandler(); // Display a backtrace once a serious signal is delivered.
+
 #ifdef ENABLE_QT_BREAKPAD
     QtSystemExceptionHandler systemExceptionHandler;
 #endif
 
+#if QT_VERSION >= 0x050100
+    app.setAttribute(Qt::AA_UseHighDpiPixmaps);
+#endif
+
     // Manually determine -settingspath command line option
     // We can't use the regular way of the plugin manager, because that needs to parse pluginspecs
     // but the settings path can influence which plugins are enabled
@@ -350,7 +354,7 @@ int main(int argc, char **argv)
 #else
     uiLanguages << QLocale::system().name();
 #endif
-    QString overrideLanguage = settings->value("General/OverrideLanguage").toString();
+    QString overrideLanguage = settings->value(QLatin1String("General/OverrideLanguage")).toString();
     if (!overrideLanguage.isEmpty())
         uiLanguages.prepend(overrideLanguage);
     const QString &creatorTrPath = QCoreApplication::applicationDirPath()
@@ -410,9 +414,8 @@ int main(int argc, char **argv)
         appOptions.insert(QLatin1String(HELP_OPTION4), false);
         appOptions.insert(QLatin1String(VERSION_OPTION), false);
         appOptions.insert(QLatin1String(CLIENT_OPTION), false);
-#if !defined(Q_OS_WIN)
         appOptions.insert(QLatin1String(PID_OPTION), true);
-#endif
+        appOptions.insert(QLatin1String(BLOCK_OPTION), false);
         QString errorMessage;
         if (!PluginManager::parseOptions(arguments, appOptions, &foundAppOptions, &errorMessage)) {
             displayError(errorMessage);
@@ -452,7 +455,6 @@ int main(int argc, char **argv)
     }
 
     qint64 pid = -1;
-#if !defined(Q_OS_WIN)
     if (foundAppOptions.contains(QLatin1String(PID_OPTION))) {
         QString pidString = foundAppOptions.value(QLatin1String(PID_OPTION));
         bool pidOk;
@@ -460,9 +462,11 @@ int main(int argc, char **argv)
         if (pidOk)
             pid = tmpPid;
     }
-#endif
 
-    if (app.isRunning() && (pid != -1 || foundAppOptions.contains(QLatin1String(CLIENT_OPTION)))) {
+    bool isBlock = foundAppOptions.contains(QLatin1String(BLOCK_OPTION));
+    if (app.isRunning() && (pid != -1 || isBlock
+                            || foundAppOptions.contains(QLatin1String(CLIENT_OPTION)))) {
+        app.setBlock(isBlock);
         if (app.sendMessage(PluginManager::serializedArguments(), 5000 /*timeout*/, pid))
             return 0;
 
@@ -470,7 +474,7 @@ int main(int argc, char **argv)
         if (app.isRunning(pid)) {
             // Nah app is still running, ask the user
             int button = askMsgSendFailed();
-            while(button == QMessageBox::Retry) {
+            while (button == QMessageBox::Retry) {
                 if (app.sendMessage(PluginManager::serializedArguments(), 5000 /*timeout*/, pid))
                     return 0;
                 if (!app.isRunning(pid)) // App quit while we were trying so start a new creator
@@ -493,22 +497,21 @@ int main(int argc, char **argv)
         errorOverview.exec();
     }
 
-    // Set up lock and remote arguments.
-    app.initialize();
-    QObject::connect(&app, SIGNAL(messageReceived(QString)),
-                     &pluginManager, SLOT(remoteArguments(QString)));
+    // Set up remote arguments.
+    QObject::connect(&app, SIGNAL(messageReceived(QString,QObject*)),
+                     &pluginManager, SLOT(remoteArguments(QString,QObject*)));
 
     QObject::connect(&app, SIGNAL(fileOpenRequest(QString)), coreplugin->plugin(),
                      SLOT(fileOpenRequest(QString)));
 
+    // quit when last window (relevant window, see WA_QuitOnClose) is closed
+    // this should actually be the default, but doesn't work in Qt 5
+    // QTBUG-31569
+    QObject::connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
     // shutdown plugin manager on the exit
     QObject::connect(&app, SIGNAL(aboutToQuit()), &pluginManager, SLOT(shutdown()));
 
-#ifdef WITH_TESTS
-    // Do this after the event loop has started
-    if (PluginManager::runningTests())
-        QTimer::singleShot(100, &pluginManager, SLOT(startTests()));
-#endif
-
-    return app.exec();
+    const int r = app.exec();
+    cleanupCrashHandler();
+    return r;
 }
