@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -30,9 +30,9 @@
 #ifndef CPLUSPLUS_DEPRECATEDGENTEMPLATEINSTANCE_H
 #define CPLUSPLUS_DEPRECATEDGENTEMPLATEINSTANCE_H
 
-#include <TypeVisitor.h>
-#include <NameVisitor.h>
-#include <FullySpecifiedType.h>
+#include <cplusplus/TypeVisitor.h>
+#include <cplusplus/NameVisitor.h>
+#include <cplusplus/FullySpecifiedType.h>
 
 #include <QList>
 #include <QPair>
