@@ -28,50 +28,22 @@
 ****************************************************************************/
 
 #include "FindUsages.h"
+
 #include "Overview.h"
-#include <AST.h>
-#include <TranslationUnit.h>
-#include <Control.h>
-#include <Names.h>
-#include <Symbols.h>
-#include <CoreTypes.h>
-#include <Literals.h>
-#include <Scope.h>
+
+#include <cplusplus/AST.h>
+#include <cplusplus/TranslationUnit.h>
+#include <cplusplus/Control.h>
+#include <cplusplus/Names.h>
+#include <cplusplus/Symbols.h>
+#include <cplusplus/CoreTypes.h>
+#include <cplusplus/Literals.h>
+#include <cplusplus/Scope.h>
+
 #include <QDebug>
 
 using namespace CPlusPlus;
 
-namespace {
-
-QString fetchLine(const QByteArray &bytes, const int line)
-{
-    int current = 0;
-    const char *s = bytes.constData();
-
-    if (line) {
-        while (*s) {
-            if (*s == '\n') {
-                ++current;
-                if (line == current) {
-                    ++s;
-                    break;
-                }
-            }
-            ++s;
-        }
-    }
-
-    if (current == line) {
-        const char *e = s;
-        while (*e && *e != '\n')
-            ++e;
-        return QString::fromUtf8(s, e - s);
-    }
-    return QString();
-}
-
-} // Anonymous
-
 FindUsages::FindUsages(const QByteArray &originalSource, Document::Ptr doc, const Snapshot &snapshot)
     : ASTVisitor(doc->translationUnit()),
       _id(0),
@@ -85,6 +57,8 @@ FindUsages::FindUsages(const QByteArray &originalSource, Document::Ptr doc, cons
 {
     _snapshot.insert(_doc);
     typeofExpression.init(_doc, _snapshot, _context.bindings());
+
+    prepareLines(_originalSource);
 }
 
 FindUsages::FindUsages(const LookupContext &context)
@@ -99,6 +73,8 @@ FindUsages::FindUsages(const LookupContext &context)
       _currentScope(0)
 {
     typeofExpression.init(_doc, _snapshot, _context.bindings());
+
+    prepareLines(_originalSource);
 }
 
 QList<Usage> FindUsages::usages() const
@@ -178,8 +154,7 @@ QString FindUsages::matchingLine(const Token &tk) const
             break;
     }
 
-    const QString matchingLine = QString::fromUtf8(cp, lineEnd - cp);
-    return matchingLine;
+    return QString::fromUtf8(cp, lineEnd - cp);
 }
 
 void FindUsages::reportResult(unsigned tokenIndex)
@@ -195,9 +170,8 @@ void FindUsages::reportResult(unsigned tokenIndex)
     unsigned line, col;
     getTokenStartPosition(tokenIndex, &line, &col);
     QString lineText;
-    const int lines = _originalSource.count('\n') + 1;
-    if (((int) line - 1) < lines)
-        lineText = fetchLine(_originalSource, line - 1);
+    if (line < _sourceLineEnds.size())
+        lineText = fetchLine(line);
     else
         lineText = matchingLine(tk);
 
@@ -234,9 +208,17 @@ bool FindUsages::checkCandidates(const QList<LookupItem> &candidates) const
 
             if (isLocalScope(_declSymbol->enclosingScope()) || isLocalScope(s->enclosingScope())) {
                 if (s->enclosingScope()->isTemplate()) {
-                    if (s->enclosingScope()->enclosingScope() != _declSymbol->enclosingScope())
+                    if (_declSymbol->enclosingScope()->isTemplate()) {
+                        if (s->enclosingScope()->enclosingScope() != _declSymbol->enclosingScope()->enclosingScope())
+                            return false;
+                    } else {
+                        if (s->enclosingScope()->enclosingScope() != _declSymbol->enclosingScope())
+                            return false;
+                    }
+                } else if (_declSymbol->enclosingScope()->isTemplate() && s->isTemplate()) {
+                    if (_declSymbol->enclosingScope()->enclosingScope() != s->enclosingScope())
                         return false;
-                } else if (s->enclosingScope() != _declSymbol->enclosingScope()) {
+                } else if (! s->isUsingDeclaration() && s->enclosingScope() != _declSymbol->enclosingScope()) {
                     return false;
                 }
             }
@@ -260,6 +242,8 @@ void FindUsages::checkExpression(unsigned startToken, unsigned endToken, Scope *
     if (! scope)
         scope = _currentScope;
 
+    // make possible to instantiate templates
+    typeofExpression.setExpandTemplates(true);
     const QList<LookupItem> results = typeofExpression(expression, scope, TypeOfExpression::Preprocess);
     reportResult(endToken, results);
 }
@@ -535,11 +519,7 @@ void FindUsages::memInitializer(MemInitializerAST *ast)
             (void) switchScope(previousScope);
         }
     }
-    // unsigned lparen_token = ast->lparen_token;
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        this->expression(it->value);
-    }
-    // unsigned rparen_token = ast->rparen_token;
+    this->expression(ast->expression);
 }
 
 bool FindUsages::visit(NestedNameSpecifierAST *ast)
@@ -558,14 +538,17 @@ void FindUsages::nestedNameSpecifier(NestedNameSpecifierAST *ast)
     // unsigned scope_token = ast->scope_token;
 }
 
-bool FindUsages::visit(NewPlacementAST *ast)
+bool FindUsages::visit(ExpressionListParenAST *ast)
 {
-    (void) ast;
-    Q_ASSERT(!"unreachable");
+    // unsigned lparen_token = ast->lparen_token;
+    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
+        this->expression(it->value);
+    }
+    // unsigned rparen_token = ast->rparen_token;
     return false;
 }
 
-void FindUsages::newPlacement(NewPlacementAST *ast)
+void FindUsages::newPlacement(ExpressionListParenAST *ast)
 {
     if (! ast)
         return;
@@ -594,23 +577,6 @@ void FindUsages::newArrayDeclarator(NewArrayDeclaratorAST *ast)
     // unsigned rbracket_token = ast->rbracket_token;
 }
 
-bool FindUsages::visit(NewInitializerAST *ast)
-{
-    (void) ast;
-    Q_ASSERT(!"unreachable");
-    return false;
-}
-
-void FindUsages::newInitializer(NewInitializerAST *ast)
-{
-    if (! ast)
-        return;
-
-    // unsigned lparen_token = ast->lparen_token;
-    this->expression(ast->expression);
-    // unsigned rparen_token = ast->rparen_token;
-}
-
 bool FindUsages::visit(NewTypeIdAST *ast)
 {
     (void) ast;
@@ -887,6 +853,7 @@ void FindUsages::capture(CaptureAST *ast)
     if (! ast)
         return;
 
+    this->name(ast->identifier);
 }
 
 bool FindUsages::visit(LambdaDeclaratorAST *ast)
@@ -1025,7 +992,6 @@ bool FindUsages::visit(RangeBasedForStatementAST *ast)
         this->specifier(it->value);
     }
     this->declarator(ast->declarator);
-    this->expression(ast->initializer);
     // unsigned comma_token = ast->comma_token;
     this->expression(ast->expression);
     // unsigned rparen_token = ast->rparen_token;
@@ -1296,7 +1262,7 @@ bool FindUsages::visit(NewExpressionAST *ast)
     this->expression(ast->type_id);
     // unsigned rparen_token = ast->rparen_token;
     this->newTypeId(ast->new_type_id);
-    this->newInitializer(ast->new_initializer);
+    this->expression(ast->new_initializer);
     return false;
 }
 
@@ -1313,11 +1279,7 @@ bool FindUsages::visit(TypenameCallExpressionAST *ast)
 {
     // unsigned typename_token = ast->typename_token;
     /*const Name *name =*/ this->name(ast->name);
-    // unsigned lparen_token = ast->lparen_token;
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        this->expression(it->value);
-    }
-    // unsigned rparen_token = ast->rparen_token;
+    this->expression(ast->expression);
     return false;
 }
 
@@ -1326,11 +1288,7 @@ bool FindUsages::visit(TypeConstructorCallAST *ast)
     for (SpecifierListAST *it = ast->type_specifier_list; it; it = it->next) {
         this->specifier(it->value);
     }
-    // unsigned lparen_token = ast->lparen_token;
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        this->expression(it->value);
-    }
-    // unsigned rparen_token = ast->rparen_token;
+    this->expression(ast->expression);
     return false;
 }
 
@@ -2191,3 +2149,28 @@ bool FindUsages::visit(ArrayDeclaratorAST *ast)
     // unsigned rbracket_token = ast->rbracket_token;
     return false;
 }
+
+void FindUsages::prepareLines(const QByteArray &bytes)
+{
+    _sourceLineEnds.reserve(1000);
+    const char *s = bytes.constData();
+    _sourceLineEnds.push_back(s - 1); // we start counting at line 1, so line 0 is always empty.
+
+    for (; *s; ++s)
+        if (*s == '\n')
+            _sourceLineEnds.push_back(s);
+    if (s != _sourceLineEnds.back() + 1) // no newline at the end of the file
+        _sourceLineEnds.push_back(s);
+}
+
+QString FindUsages::fetchLine(unsigned lineNr) const
+{
+    Q_ASSERT(lineNr < _sourceLineEnds.size());
+    if (lineNr == 0)
+        return QString();
+
+    const char *start = _sourceLineEnds.at(lineNr - 1) + 1;
+    const char *end = _sourceLineEnds.at(lineNr);
+    return QString::fromUtf8(start, end - start);
+}
+
