@@ -33,15 +33,18 @@
 #include "CppDocument.h"
 #include "LookupItem.h"
 #include "AlreadyConsideredClassContainer.h"
-#include <FullySpecifiedType.h>
-#include <Type.h>
-#include <SymbolVisitor.h>
-#include <Control.h>
-#include <Name.h>
+
+#include <cplusplus/FullySpecifiedType.h>
+#include <cplusplus/Type.h>
+#include <cplusplus/SymbolVisitor.h>
+#include <cplusplus/Control.h>
+#include <cplusplus/Name.h>
+
 #include <QSet>
+#include <QMap>
+
 #include <map>
 #include <functional>
-#include <QMap>
 
 namespace CPlusPlus {
 
@@ -128,6 +131,7 @@ private:
     QSharedPointer<Control> _control;
     TemplateNameIdTable _specializations;
     QMap<const TemplateNameId *, ClassOrNamespace *> _instantiations;
+    QHash<const AnonymousNameId *, ClassOrNamespace *> _anonymouses;
 
     QHash<Internal::FullyQualifiedName, Symbol *> *_scopeLookupCache;
 
@@ -249,6 +253,9 @@ protected:
     virtual bool visit(ObjCMethod *);
 
 private:
+    Symbol *instantiateTemplateFunction(const TemplateNameId *instantiation,
+                                        Template *specialization) const;
+
     Snapshot _snapshot;
     QSharedPointer<Control> _control;
     QSet<Namespace *> _processed;
@@ -282,7 +289,9 @@ public:
 
     QList<LookupItem> lookup(const Name *name, Scope *scope) const;
     ClassOrNamespace *lookupType(const Name *name, Scope *scope,
-                                 ClassOrNamespace* enclosingTemplateInstantiation = 0) const;
+                                 ClassOrNamespace* enclosingTemplateInstantiation = 0,
+                                 QSet<const Declaration *> typedefsBeingResolved
+                                    = QSet<const Declaration *>()) const;
     ClassOrNamespace *lookupType(Symbol *symbol,
                                  ClassOrNamespace* enclosingTemplateInstantiation = 0) const;
     ClassOrNamespace *lookupParent(Symbol *symbol) const;
@@ -306,6 +315,8 @@ public:
     }
 
 private:
+    QList<LookupItem> lookupByUsing(const Name *name, Scope *scope) const;
+
     // The current expression.
     Document::Ptr _expressionDocument;
 
