@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 /*
   Copyright 2005 Roberto Raggi <roberto@kdevelop.org>
 
@@ -56,13 +53,14 @@ using namespace CPlusPlus;
 Macro::Macro()
     : _next(0),
       _hashcode(0),
+      _fileRevision(0),
       _line(0),
       _offset(0),
       _length(0),
       _state(0)
 { }
 
-QString Macro::toString() const
+QString Macro::decoratedName() const
 {
     QString text;
     if (f._hidden)
@@ -78,13 +76,25 @@ QString Macro::toString() const
                 text += QLatin1String(", ");
             else
                 first = false;
-            text += QString::fromUtf8(formal.constData(), formal.size());
+            if (formal != "__VA_ARGS__")
+                text += QString::fromUtf8(formal.constData(), formal.size());
         }
         if (f._variadic)
             text += QLatin1String("...");
         text += QLatin1Char(')');
     }
     text += QLatin1Char(' ');
-    text += QString::fromUtf8(_definition.constData(), _definition.size());
     return text;
 }
+
+QString Macro::toString() const
+{
+    QString text = decoratedName();
+    text.append(QString::fromUtf8(_definitionText.constData(), _definitionText.size()));
+    return text;
+}
+
+QString Macro::toStringWithLineBreaks() const
+{
+    return toString();
+}
