@@ -1,20 +1,19 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+** This file is part of Qt Creator.
 **
-** Contact: Nokia Corporation (qt-info@nokia.com)
-**
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
-**
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
@@ -22,14 +21,11 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "contextpanetextwidget.h"
 #include "contextpanewidget.h"
@@ -37,7 +33,7 @@
 #include "ui_contextpanetext.h"
 #include <qmljs/qmljspropertyreader.h>
 #include <QTimerEvent>
-
+#include <QVariant>
 namespace QmlEditorWidgets {
 
 ContextPaneTextWidget::ContextPaneTextWidget(QWidget *parent) :
@@ -46,14 +42,21 @@ ContextPaneTextWidget::ContextPaneTextWidget(QWidget *parent) :
     m_fontSizeTimer(-1)
 {
     ui->setupUi(this);
-    ui->boldButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/bold-h-icon.png")));
-    ui->italicButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/italic-h-icon.png")));
-    ui->underlineButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/underline-h-icon.png")));
-    ui->strikeoutButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/strikeout-h-icon.png")));
+    ui->boldButton->setIcon(QIcon::fromTheme(QLatin1String("format-text-bold"),
+            QIcon(QLatin1String(":/qmldesigner/images/bold-h-icon.png"))));
+    ui->italicButton->setIcon(QIcon::fromTheme(QLatin1String("format-text-italic"),
+            QIcon(QLatin1String(":/qmldesigner/images/italic-h-icon.png"))));
+    ui->underlineButton->setIcon(QIcon::fromTheme(QLatin1String("format-text-underline"),
+            QIcon(QLatin1String(":/qmldesigner/images/underline-h-icon.png"))));
+    ui->strikeoutButton->setIcon(QIcon::fromTheme(QLatin1String("format-text-strikethrough"),
+            QIcon(QLatin1String(":/qmldesigner/images/strikeout-h-icon.png"))));
 
-    ui->leftAlignmentButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/alignmentleft-h-icon.png")));
-    ui->centerHAlignmentButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/alignmentcenterh-h-icon.png")));
-    ui->rightAlignmentButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/alignmentright-h-icon.png")));
+    ui->leftAlignmentButton->setIcon(QIcon::fromTheme(QLatin1String("format-justify-left"),
+            QIcon(QLatin1String(":/qmldesigner/images/alignmentleft-h-icon.png"))));
+    ui->centerHAlignmentButton->setIcon(QIcon::fromTheme(QLatin1String("format-justify-center"),
+            QIcon(QLatin1String(":/qmldesigner/images/alignmentcenterh-h-icon.png"))));
+    ui->rightAlignmentButton->setIcon(QIcon::fromTheme(QLatin1String("format-justify-right"),
+            QIcon(QLatin1String(":/qmldesigner/images/alignmentright-h-icon.png"))));
 
     ui->centerVAlignmentButton->setIcon(QIcon(QLatin1String(":/qmldesigner/images/alignmentmiddle-h-icon.png")));
 
@@ -92,7 +95,7 @@ ContextPaneTextWidget::ContextPaneTextWidget(QWidget *parent) :
 
 static inline bool checkIfBoolean(QVariant v)
 {
-    return (v.toString() == "true" || v.toString() == "false");
+    return (v.toString() == QLatin1String("true") || v.toString() == QLatin1String("false"));
 }
 
 void ContextPaneTextWidget::setProperties(QmlJS::PropertyReader *propertyReader)
@@ -100,7 +103,7 @@ void ContextPaneTextWidget::setProperties(QmlJS::PropertyReader *propertyReader)
     if (propertyReader->hasProperty(QLatin1String("font.pointSize"))) {
         QVariant variant = propertyReader->readProperty(QLatin1String("font.pointSize"));
         bool b;
-        ui->fontSizeSpinBox->setValue(variant.toInt(&b));        
+        ui->fontSizeSpinBox->setValue(variant.toInt(&b));
         ui->fontSizeSpinBox->setEnabled(b);
         ui->fontSizeSpinBox->setIsPointSize(true);
     } else if (!propertyReader->hasProperty(QLatin1String("font.pixelSize"))) {
@@ -166,17 +169,15 @@ void ContextPaneTextWidget::setProperties(QmlJS::PropertyReader *propertyReader)
         ui->strikeoutButton->setChecked(false);
     }
 
-    if (propertyReader->hasProperty(QLatin1String("color"))) {
-        ui->colorButton->setColor(propertyReader->readProperty("color").toString());
-    } else {
+    if (propertyReader->hasProperty(QLatin1String("color")))
+        ui->colorButton->setColor(propertyReader->readProperty(QLatin1String("color")).toString());
+    else
         ui->colorButton->setColor(QLatin1String("black"));
-    }
 
-    if (propertyReader->hasProperty(QLatin1String("styleColor"))) {
-        ui->textColorButton->setColor(propertyReader->readProperty("styleColor").toString());
-    } else {
+    if (propertyReader->hasProperty(QLatin1String("styleColor")))
+        ui->textColorButton->setColor(propertyReader->readProperty(QLatin1String("styleColor")).toString());
+    else
         ui->textColorButton->setColor(QLatin1String("black"));
-    }
 
     if (propertyReader->hasProperty(QLatin1String("font.family"))) {
         QString familyName = propertyReader->readProperty(QLatin1String("font.family")).toString();
@@ -194,7 +195,7 @@ void ContextPaneTextWidget::setProperties(QmlJS::PropertyReader *propertyReader)
         QString alignment = propertyReader->readProperty(QLatin1String("horizontalAlignment")).toString();
         ui->leftAlignmentButton->setChecked(true);
         ui->leftAlignmentButton->setEnabled(true);
-        if (alignment == QLatin1String("Text.AlignHCenter") || alignment == "AlignHCenter")
+        if (alignment == QLatin1String("Text.AlignHCenter") || alignment == QLatin1String("AlignHCenter"))
             ui->centerHAlignmentButton->setChecked(true);
         else if (alignment == QLatin1String("Text.AlignRight") || alignment == QLatin1String("AlignRight"))
             ui->rightAlignmentButton->setChecked(true);
@@ -261,7 +262,7 @@ void ContextPaneTextWidget::onTextColorButtonToggled(bool flag)
     if (flag)
         ui->colorButton->setChecked(false);
     QPoint p = mapToGlobal(ui->textColorButton->pos());
-    parentContextWidget->colorDialog()->setupColor(ui->textColorButton->color());
+    parentContextWidget->colorDialog()->setupColor(ui->textColorButton->color().toString());
     p = parentContextWidget->colorDialog()->parentWidget()->mapFromGlobal(p);
     parentContextWidget->onShowColorDialog(flag, p);
 }
@@ -272,8 +273,8 @@ void ContextPaneTextWidget::onColorButtonToggled(bool flag)
         ui->textColorButton->setChecked(false);
     ContextPaneWidget *parentContextWidget = qobject_cast<ContextPaneWidget*>(parentWidget());
     QPoint p = mapToGlobal(ui->colorButton->pos());
-    parentContextWidget->colorDialog()->setupColor(ui->colorButton->color());
-    p = parentContextWidget->colorDialog()->parentWidget()->mapFromGlobal(p);    
+    parentContextWidget->colorDialog()->setupColor(ui->colorButton->color().toString());
+    p = parentContextWidget->colorDialog()->parentWidget()->mapFromGlobal(p);
     parentContextWidget->onShowColorDialog(flag, p);
 }
 
@@ -298,7 +299,7 @@ void ContextPaneTextWidget::onColorDialogCancled()
 }
 
 void ContextPaneTextWidget::onFontSizeChanged(int)
-{  
+{
     if (m_fontSizeTimer > 0)
         killTimer(m_fontSizeTimer);
     m_fontSizeTimer = startTimer(200);
@@ -307,11 +308,10 @@ void ContextPaneTextWidget::onFontSizeChanged(int)
 void ContextPaneTextWidget::onFontFormatChanged()
 {
     int size = ui->fontSizeSpinBox->value();
-    if (ui->fontSizeSpinBox->isPointSize()) {   
+    if (ui->fontSizeSpinBox->isPointSize())
         emit removeAndChangeProperty(QLatin1String("font.pixelSize"), QLatin1String("font.pointSize"), size, true);
-    } else {        
+    else
         emit removeAndChangeProperty(QLatin1String("font.pointSize"), QLatin1String("font.pixelSize"), size, true);
-    }
 
 }
 
@@ -351,7 +351,7 @@ void ContextPaneTextWidget::onStrikeoutCheckedChanged(bool value)
 void ContextPaneTextWidget::onCurrentFontChanged(const QFont &font)
 {
     font.family();
-    emit propertyChanged(QLatin1String("font.family"), QVariant(QString('\"') + font.family() + QString('\"')));
+    emit propertyChanged(QLatin1String("font.family"), QVariant(QLatin1Char('"') + font.family() + QLatin1Char('"')));
 }
 
 void ContextPaneTextWidget::onHorizontalAlignmentChanged()
@@ -398,24 +398,11 @@ void ContextPaneTextWidget::onVerticalAlignmentChanged()
     }
 }
 
-
 ContextPaneTextWidget::~ContextPaneTextWidget()
 {
     delete ui;
 }
 
-void ContextPaneTextWidget::changeEvent(QEvent *e)
-{
-    QWidget::changeEvent(e);
-    switch (e->type()) {
-    case QEvent::LanguageChange:
-        ui->retranslateUi(this);
-        break;
-    default:
-        break;
-    }
-}
-
 void ContextPaneTextWidget::timerEvent(QTimerEvent *event)
 {
     if (event->timerId() == m_fontSizeTimer) {
