@@ -1,42 +1,38 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qmljscodeformatter.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QMetaEnum>
-#include <QtGui/QTextDocument>
-#include <QtGui/QTextCursor>
-#include <QtGui/QTextBlock>
+#include <QDebug>
+#include <QMetaEnum>
+#include <QTextDocument>
+#include <QTextBlock>
 
 using namespace QmlJS;
 
@@ -90,6 +86,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case Identifier:    enter(objectdefinition_or_js); continue;
             case Import:        enter(top_qml); continue;
+            case LeftBrace:     enter(top_js); enter(expression); continue; // if a file starts with {, it's likely json
             default:            enter(top_js); continue;
             } break;
 
@@ -155,6 +152,9 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case Semicolon:     leave(true); break;
             case If:            enter(if_statement); break;
+            case With:          enter(statement_with_condition); break;
+            case Try:           enter(try_statement); break;
+            case Switch:        enter(switch_statement); break;
             case LeftBrace:     enter(jsblock_open); break;
             case On:
             case As:
@@ -163,6 +163,11 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             case Signal:
             case Property:
             case Identifier:    enter(expression_or_objectdefinition); break;
+
+            // error recovery
+            case RightBracket:
+            case RightParenthesis:  leave(true); break;
+
             default:            enter(expression); continue;
             } break;
 
@@ -191,18 +196,18 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case Colon:         enter(binding_assignment); break; // oops, was a binding
             case Var:
-            case Identifier:    enter(property_type); break;
+            case Identifier:    enter(property_name); break;
             case List:          enter(property_list_open); break;
             default:            leave(true); continue;
             } break;
 
-        case property_type:
+        case property_name:
             turnInto(property_maybe_initializer);
             break;
 
         case property_list_open:
             if (m_currentLine.midRef(m_currentToken.begin(), m_currentToken.length) == QLatin1String(">"))
-                turnInto(property_maybe_initializer);
+                turnInto(property_name);
             break;
 
         case property_maybe_initializer:
@@ -247,15 +252,25 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
         case expression_or_objectdefinition:
             switch (kind) {
             case Dot:
-            case Identifier:    break; // need to become an objectdefinition_open in cases like "width: Qt.Foo {"
-            case LeftBrace:     turnInto(objectdefinition_open); break;
-            default:            enter(expression); continue; // really? identifier and more tokens might already be gone
+            case Identifier:        break; // need to become an objectdefinition_open in cases like "width: Qt.Foo {"
+            case LeftBrace:         turnInto(objectdefinition_open); break;
+
+            // propagate 'leave' from expression state
+            case RightBracket:
+            case RightParenthesis:  leave(); continue;
+
+            default:                enter(expression); continue; // really? identifier and more tokens might already be gone
             } break;
 
         case expression_or_label:
             switch (kind) {
-            case Colon:         turnInto(labelled_statement); break;
-            default:            enter(expression); continue;
+            case Colon:             turnInto(labelled_statement); break;
+
+            // propagate 'leave' from expression state
+            case RightBracket:
+            case RightParenthesis:  leave(); continue;
+
+            default:                enter(expression); continue;
             } break;
 
         case ternary_op:
@@ -371,14 +386,17 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             } break;
 
         case maybe_else:
-            if (kind == Else) {
-                turnInto(else_clause);
-                enter(substatement);
-                break;
-            } else {
-                leave(true);
-                continue;
-            }
+            switch (kind) {
+            case Else:              turnInto(else_clause); enter(substatement); break;
+            default:                leave(true); continue;
+            } break;
+
+        case maybe_catch_or_finally:
+            switch (kind) {
+            case Catch:             turnInto(catch_statement); break;
+            case Finally:           turnInto(finally_statement); break;
+            default:                leave(true); continue;
+            } break;
 
         case else_clause:
             // ### shouldn't happen
@@ -388,19 +406,14 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             break;
 
         case condition_open:
+            if (tryInsideExpression())
+                break;
             switch (kind) {
             case RightParenthesis:  turnInto(substatement); break;
-            case LeftParenthesis:   enter(condition_paren_open); break;
-            } break;
-
-        // paren nesting
-        case condition_paren_open:
-            switch (kind) {
-            case RightParenthesis:  leave(); break;
-            case LeftParenthesis:   enter(condition_paren_open); break;
             } break;
 
         case switch_statement:
+        case catch_statement:
         case statement_with_condition:
             switch (kind) {
             case LeftParenthesis:   enter(statement_with_condition_paren_open); break;
@@ -414,7 +427,8 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             case RightParenthesis:  turnInto(substatement); break;
             } break;
 
-        case statement_with_block:
+        case try_statement:
+        case finally_statement:
             switch (kind) {
             case LeftBrace:         enter(jsblock_open); break;
             default:                leave(true); break;
@@ -424,7 +438,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             switch (kind) {
             case While:             break;
             case LeftParenthesis:   enter(do_statement_while_paren_open); break;
-            default:                leave(true); break;
+            default:                leave(true); continue; // error recovery
             } break;
 
         case do_statement_while_paren_open:
@@ -627,6 +641,8 @@ void CodeFormatter::enter(int newState)
     m_currentState.push(s);
     m_newStates.push(s);
 
+    //qDebug() << "enter state" << stateToString(newState);
+
     if (newState == bracket_open)
         enter(bracket_element_start);
 }
@@ -646,10 +662,10 @@ void CodeFormatter::leave(bool statementDone)
 
     int topState = m_currentState.top().type;
 
+    //qDebug() << "left state" << stateToString(poppedState.type) << ", now in state" << stateToString(topState);
+
     // if statement is done, may need to leave recursively
     if (statementDone) {
-        if (!isExpressionEndState(topState))
-            leave(true);
         if (topState == if_statement) {
             if (poppedState.type != maybe_else)
                 enter(maybe_else);
@@ -659,6 +675,15 @@ void CodeFormatter::leave(bool statementDone)
             // leave the else *and* the surrounding if, to prevent another else
             leave();
             leave(true);
+        } else if (topState == try_statement) {
+            if (poppedState.type != maybe_catch_or_finally
+                    && poppedState.type != finally_statement) {
+                enter(maybe_catch_or_finally);
+            } else {
+                leave(true);
+            }
+        } else if (!isExpressionEndState(topState)) {
+            leave(true);
         }
     }
 }
@@ -734,8 +759,7 @@ bool CodeFormatter::tryStatement()
         enter(case_start);
         return true;
     case Try:
-    case Finally:
-        enter(statement_with_block);
+        enter(try_statement);
         return true;
     case LeftBrace:
         enter(jsblock_open);
@@ -756,6 +780,7 @@ bool CodeFormatter::tryStatement()
     case Function:
     case Number:
     case String:
+    case LeftParenthesis:
         enter(expression);
         // look at the token again
         m_tokenIndex -= 1;
@@ -780,8 +805,7 @@ bool CodeFormatter::isExpressionEndState(int type) const
             type == topmost_intro ||
             type == top_js ||
             type == objectdefinition_open ||
-            type == if_statement ||
-            type == else_clause ||
+            type == do_statement ||
             type == jsblock_open ||
             type == substatement_open ||
             type == bracket_open ||
@@ -808,11 +832,10 @@ int CodeFormatter::column(int index) const
     const QChar tab = QLatin1Char('\t');
 
     for (int i = 0; i < index; i++) {
-        if (m_currentLine[i] == tab) {
+        if (m_currentLine[i] == tab)
             col = ((col / m_tabSize) + 1) * m_tabSize;
-        } else {
+        else
             col++;
-        }
     }
     return col;
 }
@@ -896,23 +919,23 @@ CodeFormatter::TokenKind CodeFormatter::extendedTokenKind(const QmlJS::Token &to
     QStringRef text = m_currentLine.midRef(token.begin(), token.length);
 
     if (kind == Identifier) {
-        if (text == "as")
+        if (text == QLatin1String("as"))
             return As;
-        if (text == "import")
+        if (text == QLatin1String("import"))
             return Import;
-        if (text == "signal")
+        if (text == QLatin1String("signal"))
             return Signal;
-        if (text == "property")
+        if (text == QLatin1String("property"))
             return Property;
-        if (text == "on")
-            return On;
-        if (text == "list")
+        if (text == QLatin1String("on"))
             return On;
+        if (text == QLatin1String("list"))
+            return List;
     } else if (kind == Keyword) {
-        const QChar char1 = text.at(0);
-        const QChar char2 = text.at(1);
-        const QChar char3 = (text.size() > 2 ? text.at(2) : QChar());
-        switch (char1.toLatin1()) {
+        const char char1 = text.at(0).toLatin1();
+        const char char2 = text.at(1).toLatin1();
+        const char char3 = (text.size() > 2 ? text.at(2).toLatin1() : 0);
+        switch (char1) {
         case 'v':
             return Var;
         case 'i':
@@ -967,11 +990,11 @@ CodeFormatter::TokenKind CodeFormatter::extendedTokenKind(const QmlJS::Token &to
             return Break;
         }
     } else if (kind == Delimiter) {
-        if (text == "?")
+        if (text == QLatin1String("?"))
             return Question;
-        else if (text == "++")
+        else if (text == QLatin1String("++"))
             return PlusPlus;
-        else if (text == "--")
+        else if (text == QLatin1String("--"))
             return MinusMinus;
     }
 
@@ -980,12 +1003,339 @@ CodeFormatter::TokenKind CodeFormatter::extendedTokenKind(const QmlJS::Token &to
 
 void CodeFormatter::dump() const
 {
-    QMetaEnum metaEnum = staticMetaObject.enumerator(staticMetaObject.indexOfEnumerator("StateType"));
-
     qDebug() << "Current token index" << m_tokenIndex;
     qDebug() << "Current state:";
     foreach (const State &s, m_currentState) {
-        qDebug() << metaEnum.valueToKey(s.type) << s.savedIndentDepth;
+        qDebug() << stateToString(s.type) << s.savedIndentDepth;
     }
     qDebug() << "Current indent depth:" << m_indentDepth;
 }
+
+QString CodeFormatter::stateToString(int type) const
+{
+    const QMetaEnum &metaEnum = staticMetaObject.enumerator(staticMetaObject.indexOfEnumerator("StateType"));
+    return QString::fromUtf8(metaEnum.valueToKey(type));
+}
+
+QtStyleCodeFormatter::QtStyleCodeFormatter()
+    : m_indentSize(4)
+{}
+
+void QtStyleCodeFormatter::setIndentSize(int size)
+{
+    m_indentSize = size;
+}
+
+void QtStyleCodeFormatter::onEnter(int newState, int *indentDepth, int *savedIndentDepth) const
+{
+    const State &parentState = state();
+    const Token &tk = currentToken();
+    const int tokenPosition = column(tk.begin());
+    const bool firstToken = (tokenIndex() == 0);
+    const bool lastToken = (tokenIndex() == tokenCount() - 1);
+
+    switch (newState) {
+    case objectdefinition_open: {
+        // special case for things like "gradient: Gradient {"
+        if (parentState.type == binding_assignment)
+            *savedIndentDepth = state(1).savedIndentDepth;
+
+        if (firstToken)
+            *savedIndentDepth = tokenPosition;
+
+        *indentDepth = *savedIndentDepth + m_indentSize;
+        break;
+    }
+
+    case binding_or_objectdefinition:
+        if (firstToken)
+            *indentDepth = *savedIndentDepth = tokenPosition;
+        break;
+
+    case binding_assignment:
+    case objectliteral_assignment:
+        if (lastToken)
+            *indentDepth = *savedIndentDepth + 4;
+        else
+            *indentDepth = column(tokenAt(tokenIndex() + 1).begin());
+        break;
+
+    case expression_or_objectdefinition:
+        *indentDepth = tokenPosition;
+        break;
+
+    case expression_or_label:
+        if (*indentDepth == tokenPosition)
+            *indentDepth += 2*m_indentSize;
+        else
+            *indentDepth = tokenPosition;
+        break;
+
+    case expression:
+        if (*indentDepth == tokenPosition) {
+            // expression_or_objectdefinition doesn't want the indent
+            // expression_or_label already has it
+            if (parentState.type != expression_or_objectdefinition
+                    && parentState.type != expression_or_label
+                    && parentState.type != binding_assignment) {
+                *indentDepth += 2*m_indentSize;
+            }
+        }
+        // expression_or_objectdefinition and expression_or_label have already consumed the first token
+        else if (parentState.type != expression_or_objectdefinition
+                 && parentState.type != expression_or_label) {
+            *indentDepth = tokenPosition;
+        }
+        break;
+
+    case expression_maybe_continuation:
+        // set indent depth to indent we'd get if the expression ended here
+        for (int i = 1; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (isExpressionEndState(type) && !isBracelessState(type)) {
+                *indentDepth = state(i - 1).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+
+    case bracket_open:
+        if (parentState.type == expression && state(1).type == binding_assignment) {
+            *savedIndentDepth = state(2).savedIndentDepth;
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        } else if (parentState.type == objectliteral_assignment) {
+            *savedIndentDepth = parentState.savedIndentDepth;
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        } else if (!lastToken) {
+            *indentDepth = tokenPosition + 1;
+        } else {
+            *indentDepth = *savedIndentDepth + m_indentSize;
+        }
+        break;
+
+    case function_start:
+        // align to the beginning of the line
+        *savedIndentDepth = *indentDepth = column(tokenAt(0).begin());
+        break;
+
+    case do_statement_while_paren_open:
+    case statement_with_condition_paren_open:
+    case signal_arglist_open:
+    case function_arglist_open:
+    case paren_open:
+        if (!lastToken)
+            *indentDepth = tokenPosition + 1;
+        else
+            *indentDepth += m_indentSize;
+        break;
+
+    case ternary_op:
+        if (!lastToken)
+            *indentDepth = tokenPosition + tk.length + 1;
+        else
+            *indentDepth += m_indentSize;
+        break;
+
+    case jsblock_open:
+        // closing brace should be aligned to case
+        if (parentState.type == case_cont) {
+            *savedIndentDepth = parentState.savedIndentDepth;
+            break;
+        }
+        // fallthrough
+    case substatement_open:
+        // special case for "foo: {" and "property int foo: {"
+        if (parentState.type == binding_assignment)
+            *savedIndentDepth = state(1).savedIndentDepth;
+        *indentDepth = *savedIndentDepth + m_indentSize;
+        break;
+
+    case substatement:
+        *indentDepth += m_indentSize;
+        break;
+
+    case objectliteral_open:
+        if (parentState.type == expression
+                || parentState.type == objectliteral_assignment) {
+            // undo the continuation indent of the expression
+            if (state(1).type == expression_or_label)
+                *indentDepth = state(1).savedIndentDepth;
+            else
+                *indentDepth = parentState.savedIndentDepth;
+            *savedIndentDepth = *indentDepth;
+        }
+        *indentDepth += m_indentSize;
+        break;
+
+
+    case statement_with_condition:
+    case try_statement:
+    case catch_statement:
+    case finally_statement:
+    case if_statement:
+    case do_statement:
+    case switch_statement:
+        if (firstToken || parentState.type == binding_assignment)
+            *savedIndentDepth = tokenPosition;
+        // ### continuation
+        *indentDepth = *savedIndentDepth; // + 2*m_indentSize;
+        // special case for 'else if'
+        if (!firstToken
+                && newState == if_statement
+                && parentState.type == substatement
+                && state(1).type == else_clause) {
+            *indentDepth = state(1).savedIndentDepth;
+            *savedIndentDepth = *indentDepth;
+        }
+        break;
+
+    case maybe_else:
+    case maybe_catch_or_finally: {
+        // set indent to where leave(true) would put it
+        int lastNonEndState = 0;
+        while (!isExpressionEndState(state(lastNonEndState + 1).type))
+            ++lastNonEndState;
+        *indentDepth = state(lastNonEndState).savedIndentDepth;
+        break;
+    }
+
+    case condition_open:
+        // fixed extra indent when continuing 'if (', but not for 'else if ('
+        if (tokenPosition <= *indentDepth + m_indentSize)
+            *indentDepth += 2*m_indentSize;
+        else
+            *indentDepth = tokenPosition + 1;
+        break;
+
+    case case_start:
+        *savedIndentDepth = tokenPosition;
+        break;
+
+    case case_cont:
+        *indentDepth += m_indentSize;
+        break;
+
+    case multiline_comment_start:
+        *indentDepth = tokenPosition + 2;
+        break;
+
+    case multiline_comment_cont:
+        *indentDepth = tokenPosition;
+        break;
+    }
+}
+
+void QtStyleCodeFormatter::adjustIndent(const QList<Token> &tokens, int startLexerState, int *indentDepth) const
+{
+    State topState = state();
+    State previousState = state(1);
+
+    // keep user-adjusted indent in multiline comments
+    if (topState.type == multiline_comment_start
+            || topState.type == multiline_comment_cont) {
+        if (!tokens.isEmpty()) {
+            *indentDepth = column(tokens.at(0).begin());
+            return;
+        }
+    }
+    // don't touch multi-line strings at all
+    if ((startLexerState & Scanner::MultiLineMask) == Scanner::MultiLineStringDQuote
+            || (startLexerState & Scanner::MultiLineMask) == Scanner::MultiLineStringSQuote) {
+        *indentDepth = -1;
+        return;
+    }
+
+    const int kind = extendedTokenKind(tokenAt(0));
+    switch (kind) {
+    case LeftBrace:
+        if (topState.type == substatement
+                || topState.type == binding_assignment
+                || topState.type == case_cont) {
+            *indentDepth = topState.savedIndentDepth;
+        }
+        break;
+    case RightBrace: {
+        if (topState.type == jsblock_open && previousState.type == case_cont) {
+            *indentDepth = previousState.savedIndentDepth;
+            break;
+        }
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == objectdefinition_open
+                    || type == jsblock_open
+                    || type == substatement_open
+                    || type == objectliteral_open) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+    }
+    case RightBracket:
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == bracket_open) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            }
+        }
+        break;
+
+    case LeftBracket:
+    case LeftParenthesis:
+    case Delimiter:
+        if (topState.type == expression_maybe_continuation)
+            *indentDepth = topState.savedIndentDepth;
+        break;
+
+    case Else:
+        if (topState.type == maybe_else) {
+            *indentDepth = state(1).savedIndentDepth;
+        } else if (topState.type == expression_maybe_continuation) {
+            bool hasElse = false;
+            for (int i = 1; state(i).type != topmost_intro; ++i) {
+                const int type = state(i).type;
+                if (type == else_clause)
+                    hasElse = true;
+                if (type == if_statement) {
+                    if (hasElse) {
+                        hasElse = false;
+                    } else {
+                        *indentDepth = state(i).savedIndentDepth;
+                        break;
+                    }
+                }
+            }
+        }
+        break;
+
+    case Catch:
+    case Finally:
+        if (topState.type == maybe_catch_or_finally)
+            *indentDepth = state(1).savedIndentDepth;
+        break;
+
+    case Colon:
+        if (topState.type == ternary_op)
+            *indentDepth -= 2;
+        break;
+
+    case Question:
+        if (topState.type == expression_maybe_continuation)
+            *indentDepth = topState.savedIndentDepth;
+        break;
+
+    case Default:
+    case Case:
+        for (int i = 0; state(i).type != topmost_intro; ++i) {
+            const int type = state(i).type;
+            if (type == switch_statement || type == case_cont) {
+                *indentDepth = state(i).savedIndentDepth;
+                break;
+            } else if (type == topmost_intro) {
+                break;
+            }
+        }
+        break;
+    }
+}
