@@ -30,11 +30,8 @@
 #ifndef QMLJS_INTERPRETER_H
 #define QMLJS_INTERPRETER_H
 
-#include <languageutils/componentversion.h>
-#include <languageutils/fakemetaobject.h>
 #include <qmljs/qmljsdocument.h>
 #include <qmljs/qmljs_global.h>
-#include <qmljs/parser/qmljsastfwd_p.h>
 
 #include <QFileInfoList>
 #include <QList>
@@ -611,10 +608,9 @@ public:
     static BuiltinObjects defaultLibraryObjects;
 
     // parses the contents of a qmltypes file and fills the newObjects map
-    static void parseQmlTypeDescriptions(
-        const QByteArray &qmlTypes,
+    static void parseQmlTypeDescriptions(const QByteArray &contents,
         BuiltinObjects *newObjects,
-        QList<ModuleApiInfo> *newModuleApis, QString *errorMessage, QString *warningMessage);
+        QList<ModuleApiInfo> *newModuleApis, QString *errorMessage, QString *warningMessage, const QString &fileName);
 };
 
 class QMLJS_EXPORT CppQmlTypes
@@ -876,6 +872,8 @@ public:
         LibraryImport,
         FileImport,
         DirectoryImport,
+        QrcFileImport,
+        QrcDirectoryImport,
         UnknownFileImport // refers a file/directory that wasn't found
     };
 
