@@ -32,6 +32,7 @@
 
 #include "qmljs_global.h"
 #include "qmljsdocument.h"
+#include "qmljsbundle.h"
 
 #include <utils/environment.h>
 
@@ -52,16 +53,21 @@ class QMLJS_EXPORT ModelManagerInterface: public QObject
     Q_OBJECT
 
 public:
+    enum QrcResourceSelector {
+        ActiveQrcResources,
+        AllQrcResources
+    };
+
     class ProjectInfo
     {
     public:
         ProjectInfo()
-            : tryQmlDump(false)
+            : tryQmlDump(false), qmlDumpHasRelocatableFlag(true)
         { }
 
         ProjectInfo(QPointer<ProjectExplorer::Project> project)
             : project(project)
-            , tryQmlDump(false)
+            , tryQmlDump(false), qmlDumpHasRelocatableFlag(true)
         { }
 
         operator bool() const
@@ -73,19 +79,26 @@ public:
         bool isNull() const
         { return project.isNull(); }
 
+        QStringList completeImportPaths();
+
     public: // attributes
         QPointer<ProjectExplorer::Project> project;
         QStringList sourceFiles;
         QStringList importPaths;
+        QStringList activeResourceFiles;
+        QStringList allResourceFiles;
 
         // whether trying to run qmldump makes sense
         bool tryQmlDump;
+        bool qmlDumpHasRelocatableFlag;
         QString qmlDumpPath;
         ::Utils::Environment qmlDumpEnvironment;
 
         QString qtImportsPath;
         QString qtQmlPath;
         QString qtVersionString;
+        QmlJS::QmlLanguageBundles activeBundle;
+        QmlJS::QmlLanguageBundles extendedBundle;
     };
 
     class WorkingCopy
@@ -136,13 +149,24 @@ public:
                                    bool emitDocumentOnDiskChanged) = 0;
     virtual void fileChangedOnDisk(const QString &path) = 0;
     virtual void removeFiles(const QStringList &files) = 0;
+    virtual QStringList filesAtQrcPath(const QString &path, const QLocale *locale = 0,
+                                       ProjectExplorer::Project *project = 0,
+                                       QrcResourceSelector resources = AllQrcResources) = 0;
+    virtual QMap<QString,QStringList> filesInQrcPath(const QString &path,
+                                                     const QLocale *locale = 0,
+                                                     ProjectExplorer::Project *project = 0,
+                                                     bool addDirs = false,
+                                                     QrcResourceSelector resources = AllQrcResources) = 0;
 
     virtual QList<ProjectInfo> projectInfos() const = 0;
     virtual ProjectInfo projectInfo(ProjectExplorer::Project *project) const = 0;
     virtual void updateProjectInfo(const ProjectInfo &pinfo) = 0;
     Q_SLOT virtual void removeProjectInfo(ProjectExplorer::Project *project) = 0;
+    virtual ProjectInfo projectInfoForPath(QString path) = 0;
 
     virtual QStringList importPaths() const = 0;
+    virtual QmlJS::QmlLanguageBundles activeBundles() const = 0;
+    virtual QmlJS::QmlLanguageBundles extendedBundles() const = 0;
 
     virtual void loadPluginTypes(const QString &libraryPath, const QString &importPath,
                                  const QString &importUri, const QString &importVersion) = 0;
