@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 ** Author: Andreas Hartmetz, KDAB (andreas.hartmetz@kdab.com)
 **
@@ -30,12 +30,7 @@
 
 #include "analyzermanager.h"
 
-#include "analyzerconstants.h"
 #include "analyzerplugin.h"
-#include "analyzerruncontrol.h"
-#include "analyzeroptionspage.h"
-#include "analyzerstartparameters.h"
-#include "analyzerutils.h"
 #include "ianalyzertool.h"
 #include "analyzersettings.h"
 
@@ -43,34 +38,24 @@
 #include <coreplugin/findplaceholder.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/imode.h>
-#include <coreplugin/minisplitter.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/navigationwidget.h>
 #include <coreplugin/outputpane.h>
 #include <coreplugin/rightpane.h>
-#include <coreplugin/id.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/editormanager/editormanager.h>
 
-#include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/project.h>
-#include <projectexplorer/runconfiguration.h>
 #include <projectexplorer/buildconfiguration.h>
-#include <projectexplorer/iprojectmanager.h>
 #include <projectexplorer/target.h>
-#include <projectexplorer/session.h>
-
-#include <extensionsystem/iplugin.h>
 
 #include <utils/fancymainwindow.h>
 #include <utils/styledbar.h>
 #include <utils/qtcassert.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/statuslabel.h>
-#include <ssh/sshconnection.h>
 
 #include <QVariant>
 #include <QDebug>
@@ -78,16 +63,12 @@
 #include <QVBoxLayout>
 #include <QDockWidget>
 #include <QAction>
-#include <QMenu>
-#include <QMenuBar>
 #include <QToolButton>
 #include <QComboBox>
 #include <QStackedWidget>
 #include <QDialog>
-#include <QApplication>
-#include <QLabel>
-#include <QCheckBox>
 #include <QDialogButtonBox>
+#include <QPointer>
 #include <QPushButton>
 
 using namespace Core;
@@ -121,7 +102,6 @@ public:
         setIcon(QIcon(QLatin1String(":/images/analyzer_mode.png")));
         setPriority(P_MODE_ANALYZE);
         setId(MODE_ANALYZE);
-        setType(MODE_EDIT_TYPE);
     }
 
     ~AnalyzerMode()
@@ -211,7 +191,7 @@ public:
     MainWindowSettingsMap m_defaultSettings;
 
     // list of dock widgets to prevent memory leak
-    typedef QWeakPointer<QDockWidget> DockPtr;
+    typedef QPointer<QDockWidget> DockPtr;
     QList<DockPtr> m_dockWidgets;
 };
 
@@ -235,8 +215,6 @@ AnalyzerManagerPrivate::AnalyzerManagerPrivate(AnalyzerManager *qq):
 
     setupActions();
 
-    connect(ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode*)),
-            this, SLOT(modeChanged(Core::IMode*)));
     ProjectExplorerPlugin *pe = ProjectExplorerPlugin::instance();
     connect(pe, SIGNAL(updateRunActions()), SLOT(updateRunActions()));
 }
@@ -294,6 +272,9 @@ void AnalyzerManagerPrivate::delayedInit()
     m_mode = new AnalyzerMode(q);
     createModeMainWindow();
 
+    connect(ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode*)),
+            this, SLOT(modeChanged(Core::IMode*)));
+
     // Right-side window with editor, output etc.
     MiniSplitter *mainWindowSplitter = new MiniSplitter;
     mainWindowSplitter->addWidget(m_mainWindow);
@@ -409,8 +390,8 @@ void AnalyzerManagerPrivate::activateDock(Qt::DockWidgetArea area, QDockWidget *
 
     QAction *toggleViewAction = dockWidget->toggleViewAction();
     toggleViewAction->setText(dockWidget->windowTitle());
-    Command *cmd = Core::ActionManager::registerAction(toggleViewAction,
-        Id(QLatin1String("Analyzer.") + dockWidget->objectName()), globalContext);
+    Command *cmd = ActionManager::registerAction(toggleViewAction,
+        Id("Analyzer.").withSuffix(dockWidget->objectName()), globalContext);
     cmd->setAttribute(Command::CA_Hide);
 
     ActionContainer *viewsMenu = Core::ActionManager::actionContainer(Id(M_WINDOW_VIEWS));
@@ -420,7 +401,8 @@ void AnalyzerManagerPrivate::activateDock(Qt::DockWidgetArea area, QDockWidget *
 void AnalyzerManagerPrivate::deactivateDock(QDockWidget *dockWidget)
 {
     QAction *toggleViewAction = dockWidget->toggleViewAction();
-    Core::ActionManager::unregisterAction(toggleViewAction, Id(QLatin1String("Analyzer.") + dockWidget->objectName()));
+    ActionManager::unregisterAction(toggleViewAction,
+        Id("Analyzer.").withSuffix(dockWidget->objectName()));
     m_mainWindow->removeDockWidget(dockWidget);
     dockWidget->hide();
     // Prevent saveState storing the data of the wrong children.
@@ -580,13 +562,16 @@ QAction *AnalyzerManagerPrivate::actionFromToolAndMode(IAnalyzerTool *tool, Star
 void AnalyzerManagerPrivate::selectSavedTool()
 {
     const QSettings *settings = ICore::settings();
-    const Id lastActiveAction(settings->value(QLatin1String(LAST_ACTIVE_TOOL)).toString());
-    foreach (QAction *action, m_actions) {
-        IAnalyzerTool *tool = m_toolFromAction.value(action);
-        StartMode mode = m_modeFromAction.value(action);
-        if (tool->actionId(mode) == lastActiveAction) {
-            selectTool(tool, mode);
-            return;
+
+    if (settings->contains(QLatin1String(LAST_ACTIVE_TOOL))) {
+        const Id lastActiveAction = Id::fromSetting(settings->value(QLatin1String(LAST_ACTIVE_TOOL)));
+        foreach (QAction *action, m_actions) {
+            IAnalyzerTool *tool = m_toolFromAction.value(action);
+            StartMode mode = m_modeFromAction.value(action);
+            if (tool->actionId(mode) == lastActiveAction) {
+                selectTool(tool, mode);
+                return;
+            }
         }
     }
     // fallback to first available tool
