@@ -45,6 +45,7 @@
 #include <qtsupport/baseqtversion.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
+#include <utils/environment.h>
 
 #include <QDateTime>
 #include <QSettings>
@@ -80,7 +81,9 @@ namespace {
     const QLatin1String OpenJDKLocationKey("OpenJDKLocation");
     const QLatin1String KeystoreLocationKey("KeystoreLocation");
     const QLatin1String AutomaticKitCreationKey("AutomatiKitCreation");
+    const QLatin1String MakeExtraSearchDirectory("MakeExtraSearchDirectory");
     const QLatin1String PartitionSizeKey("PartitionSize");
+    const QLatin1String ToolchainHostKey("ToolchainHost");
     const QLatin1String ArmToolchainPrefix("arm-linux-androideabi");
     const QLatin1String X86ToolchainPrefix("x86");
     const QLatin1String MipsToolchainPrefix("mipsel-linux-android");
@@ -152,7 +155,13 @@ AndroidConfig::AndroidConfig(const QSettings &settings)
     antLocation = FileName::fromString(settings.value(AntLocationKey).toString());
     openJDKLocation = FileName::fromString(settings.value(OpenJDKLocationKey).toString());
     keystoreLocation = FileName::fromString(settings.value(KeystoreLocationKey).toString());
+    toolchainHost = settings.value(ToolchainHostKey).toString();
     automaticKitCreation = settings.value(AutomaticKitCreationKey, true).toBool();
+    QString extraDirectory = settings.value(MakeExtraSearchDirectory).toString();
+    if (extraDirectory.isEmpty())
+        makeExtraSearchDirectories = QStringList();
+    else
+        makeExtraSearchDirectories << extraDirectory;
 
     PersistentSettingsReader reader;
     if (reader.load(FileName::fromString(sdkSettingsFileName()))
@@ -163,9 +172,15 @@ AndroidConfig::AndroidConfig(const QSettings &settings)
         antLocation = FileName::fromString(reader.restoreValue(AntLocationKey).toString());
         openJDKLocation = FileName::fromString(reader.restoreValue(OpenJDKLocationKey).toString());
         keystoreLocation = FileName::fromString(reader.restoreValue(KeystoreLocationKey).toString());
+        toolchainHost = reader.restoreValue(ToolchainHostKey).toString();
         QVariant v = reader.restoreValue(AutomaticKitCreationKey);
         if (v.isValid())
             automaticKitCreation = v.toBool();
+        QString extraDirectory = reader.restoreValue(MakeExtraSearchDirectory).toString();
+        if (extraDirectory.isEmpty())
+            makeExtraSearchDirectories = QStringList();
+        else
+            makeExtraSearchDirectories << extraDirectory;
         // persistent settings
     }
 
@@ -190,14 +205,23 @@ void AndroidConfig::save(QSettings &settings) const
     settings.setValue(KeystoreLocationKey, keystoreLocation.toString());
     settings.setValue(PartitionSizeKey, partitionSize);
     settings.setValue(AutomaticKitCreationKey, automaticKitCreation);
+    settings.setValue(ToolchainHostKey, toolchainHost);
+    settings.setValue(MakeExtraSearchDirectory,
+                      makeExtraSearchDirectories.isEmpty() ? QString()
+                                                           : makeExtraSearchDirectories.at(0));
 }
 
 void AndroidConfigurations::setConfig(const AndroidConfig &devConfigs)
 {
     m_config = devConfigs;
+
+    if (m_config.toolchainHost.isEmpty())
+        detectToolchainHost();
+
     save();
     updateAvailablePlatforms();
     updateAutomaticKitList();
+    updateAndroidDevice();
     emit updated();
 }
 
@@ -277,7 +301,7 @@ FileName AndroidConfigurations::toolPath(Abi::Architecture architecture, const Q
     return path.appendPath(QString::fromLatin1("toolchains/%1-%2/prebuilt/%3/bin/%4")
             .arg(toolchainPrefix(architecture))
             .arg(ndkToolChainVersion)
-            .arg(ToolchainHost)
+            .arg(m_config.toolchainHost)
             .arg(toolsPrefix(architecture)));
 }
 
@@ -291,6 +315,11 @@ FileName AndroidConfigurations::readelfPath(Abi::Architecture architecture, cons
     return toolPath(architecture, ndkToolChainVersion).append(QLatin1String("-readelf" QTC_HOST_EXE_SUFFIX));
 }
 
+FileName AndroidConfigurations::gccPath(Abi::Architecture architecture, const QString &ndkToolChainVersion) const
+{
+    return toolPath(architecture, ndkToolChainVersion).append(QLatin1String("-gcc" QTC_HOST_EXE_SUFFIX));
+}
+
 FileName AndroidConfigurations::gdbPath(Abi::Architecture architecture, const QString &ndkToolChainVersion) const
 {
     return toolPath(architecture, ndkToolChainVersion).append(QLatin1String("-gdb" QTC_HOST_EXE_SUFFIX));
@@ -301,6 +330,30 @@ FileName AndroidConfigurations::openJDKPath() const
     return m_config.openJDKLocation;
 }
 
+void AndroidConfigurations::detectToolchainHost()
+{
+    QStringList hostPatterns;
+    switch (HostOsInfo::hostOs()) {
+    case HostOsInfo::HostOsLinux:
+        hostPatterns << QLatin1String("linux*");
+        break;
+    case HostOsInfo::HostOsWindows:
+        hostPatterns << QLatin1String("windows*");
+        break;
+    case HostOsInfo::HostOsMac:
+        hostPatterns << QLatin1String("darwin*");
+        break;
+    default: /* unknown host */ return;
+    }
+
+    FileName path = m_config.ndkLocation;
+    QDirIterator it(path.appendPath(QLatin1String("prebuilt")).toString(), hostPatterns, QDir::Dirs);
+    if (it.hasNext()) {
+        it.next();
+        m_config.toolchainHost = it.fileName();
+    }
+}
+
 FileName AndroidConfigurations::openJDKBinPath() const
 {
     FileName path = m_config.openJDKLocation;
@@ -325,12 +378,12 @@ FileName AndroidConfigurations::zipalignPath() const
     return path.appendPath(QLatin1String("tools/zipalign" QTC_HOST_EXE_SUFFIX));
 }
 
-QString AndroidConfigurations::getDeployDeviceSerialNumber(int *apiLevel) const
+QString AndroidConfigurations::getDeployDeviceSerialNumber(int *apiLevel, const QString &abi) const
 {
     QVector<AndroidDeviceInfo> devices = connectedDevices();
 
     foreach (AndroidDeviceInfo device, devices) {
-        if (device.sdk >= *apiLevel) {
+        if (device.sdk >= *apiLevel && device.cpuABI.contains(abi)) {
             *apiLevel = device.sdk;
             return device.serialNumber;
         }
@@ -368,6 +421,7 @@ QVector<AndroidDeviceInfo> AndroidConfigurations::connectedDevices(int apiLevel)
 
         dev.serialNumber = serialNo;
         dev.sdk = getSDKVersion(dev.serialNumber);
+        dev.cpuABI = getAbis(dev.serialNumber);
         if (apiLevel != -1 && dev.sdk != apiLevel)
             continue;
         devices.push_back(dev);
@@ -457,7 +511,7 @@ QVector<AndroidDeviceInfo> AndroidConfigurations::androidVirtualDevices() const
             if (line.contains(QLatin1String("Target:")))
                 dev.sdk = line.mid(line.lastIndexOf(QLatin1Char(' '))).remove(QLatin1Char(')')).toInt();
             if (line.contains(QLatin1String("ABI:")))
-                dev.cpuABI = line.mid(line.lastIndexOf(QLatin1Char(' '))).trimmed();
+                dev.cpuABI = QStringList() << line.mid(line.lastIndexOf(QLatin1Char(' '))).trimmed();
         }
         devices.push_back(dev);
     }
@@ -551,6 +605,33 @@ int AndroidConfigurations::getSDKVersion(const QString &device) const
     return adbProc.readAll().trimmed().toInt();
 }
 
+QStringList AndroidConfigurations::getAbis(const QString &device) const
+{
+    QStringList result;
+    int i = 1;
+    while (true) {
+        QStringList arguments = AndroidDeviceInfo::adbSelector(device);
+        arguments << QLatin1String("shell") << QLatin1String("getprop");
+        if (i == 1)
+            arguments << QLatin1String("ro.product.cpu.abi");
+        else
+            arguments << QString::fromLatin1("ro.product.cpu.abi%1").arg(i);
+
+        QProcess adbProc;
+        adbProc.start(adbToolPath().toString(), arguments);
+        if (!adbProc.waitForFinished(-1)) {
+            adbProc.kill();
+            return result;
+        }
+        QString abi = QString::fromLocal8Bit(adbProc.readAll().trimmed());
+        if (abi.isEmpty())
+            break;
+        result << abi;
+        ++i;
+    }
+    return result;
+}
+
 QString AndroidConfigurations::bestMatch(const QString &targetAPI) const
 {
     int target = targetAPI.mid(targetAPI.lastIndexOf(QLatin1Char('-')) + 1).toInt();
@@ -561,6 +642,11 @@ QString AndroidConfigurations::bestMatch(const QString &targetAPI) const
     return QLatin1String("android-8");
 }
 
+QStringList AndroidConfigurations::makeExtraSearchDirectories() const
+{
+    return m_config.makeExtraSearchDirectories;
+}
+
 bool equalKits(Kit *a, Kit *b)
 {
     return ToolChainKitInformation::toolChain(a) == ToolChainKitInformation::toolChain(b)
@@ -693,10 +779,79 @@ AndroidConfigurations::AndroidConfigurations(QObject *parent)
 
 void AndroidConfigurations::load()
 {
+    bool saveSettings = false;
     QSettings *settings = Core::ICore::instance()->settings();
     settings->beginGroup(SettingsGroup);
     m_config = AndroidConfig(*settings);
+
+    if (m_config.antLocation.isEmpty()) {
+        Utils::Environment env = Utils::Environment::systemEnvironment();
+        QString location = env.searchInPath(QLatin1String("ant"));
+        QFileInfo fi(location);
+        if (fi.exists() && fi.isExecutable() && !fi.isDir()) {
+            m_config.antLocation = Utils::FileName::fromString(location);
+            saveSettings = true;
+        }
+    }
+
+    if (m_config.openJDKLocation.isEmpty()) {
+        Utils::Environment env = Utils::Environment::systemEnvironment();
+        QString location = env.searchInPath(QLatin1String("javac"));
+        QFileInfo fi(location);
+        if (fi.exists() && fi.isExecutable() && !fi.isDir()) {
+            QDir parentDirectory = fi.canonicalPath();
+            parentDirectory.cdUp(); // one up from bin
+            m_config.openJDKLocation = Utils::FileName::fromString(parentDirectory.absolutePath());
+            saveSettings = true;
+        } else if (Utils::HostOsInfo::isWindowsHost()) {
+            QSettings settings(QLatin1String("HKEY_LOCAL_MACHINE\\SOFTWARE\\Javasoft\\Java Development Kit"), QSettings::NativeFormat);
+            QStringList allVersions = settings.childGroups();
+            QString javaHome;
+            int major = -1;
+            int minor = -1;
+            foreach (const QString &version, allVersions) {
+                QStringList parts = version.split(QLatin1String("."));
+                if (parts.size() != 2) // not interested in 1.7.0_u21
+                    continue;
+                bool okMajor, okMinor;
+                int tmpMajor = parts.at(0).toInt(&okMajor);
+                int tmpMinor = parts.at(1).toInt(&okMinor);
+                if (!okMajor || !okMinor)
+                    continue;
+                if (tmpMajor > major
+                        || (tmpMajor == major
+                            && tmpMinor > minor)) {
+                    settings.beginGroup(version);
+                    QString tmpJavaHome = settings.value(QLatin1String("JavaHome")).toString();
+                    settings.endGroup();
+                    if (!QFileInfo(tmpJavaHome).exists())
+                        continue;
+
+                    major = tmpMajor;
+                    minor = tmpMinor;
+                    javaHome = tmpJavaHome;
+                }
+            }
+            if (!javaHome.isEmpty()) {
+                m_config.openJDKLocation = Utils::FileName::fromString(javaHome);
+                saveSettings = true;
+            }
+        }
+    }
+
     settings->endGroup();
+
+    if (saveSettings)
+        save();
+}
+
+void AndroidConfigurations::updateAndroidDevice()
+{
+    ProjectExplorer::DeviceManager * const devMgr = ProjectExplorer::DeviceManager::instance();
+    if (adbToolPath().toFileInfo().exists())
+        devMgr->addDevice(ProjectExplorer::IDevice::Ptr(new Internal::AndroidDevice));
+    else if (devMgr->find(Constants::ANDROID_DEVICE_ID))
+        devMgr->removeDevice(Core::Id(Constants::ANDROID_DEVICE_ID));
 }
 
 AndroidConfigurations *AndroidConfigurations::m_instance = 0;
