@@ -169,6 +169,7 @@ bool AndroidPackageCreationStep::init()
     m_certificatePasswdForRun = m_certificatePasswd;
     m_jarSigner = AndroidConfigurations::instance().jarsignerPath();
     m_zipAligner = AndroidConfigurations::instance().zipalignPath();
+    m_environment = bc->environment();
 
     ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
     if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
@@ -207,7 +208,7 @@ static void parseSharedLibs(const QByteArray &buffer, QStringList *libs)
 #else
     QList<QByteArray> lines = buffer.trimmed().split('\n');
 #endif
-    foreach (QByteArray line, lines) {
+    foreach (const QByteArray &line, lines) {
         if (line.contains("(NEEDED)") && line.contains("Shared library:") ) {
             const int pos = line.lastIndexOf('[') + 1;
             (*libs) << QString::fromLatin1(line.mid(pos, line.length() - pos - 1));
@@ -290,14 +291,6 @@ void AndroidPackageCreationStep::checkRequiredLibraries()
     parseSharedLibs(readelfProc.readAll(), &libs);
     AndroidManager::setQtLibs(target(), requiredLibraries(AndroidManager::availableQtLibsWithDependencies(target()),
                                                           AndroidManager::qtLibs(target()), libs));
-
-    QStringList checkedLibs = AndroidManager::prebundledLibs(target());
-    QStringList prebundledLibraries;
-    foreach (const QString &qtLib, AndroidManager::availableQtLibs(target())) {
-        if (libs.contains(qtLib) || checkedLibs.contains(qtLib))
-            prebundledLibraries << qtLib;
-    }
-    AndroidManager::setPrebundledLibs(target(), prebundledLibraries);
     emit updateRequiredLibrariesModels();
 }
 
@@ -346,16 +339,8 @@ void AndroidPackageCreationStep::checkRequiredLibrariesForRun()
     QMetaObject::invokeMethod(this, "setQtLibs",Qt::BlockingQueuedConnection,
                               Q_ARG(QStringList, m_qtLibsWithDependencies));
 
-    QMetaObject::invokeMethod(this, "getBundleInformation");
+    QMetaObject::invokeMethod(this, "getBundleInformation", Qt::BlockingQueuedConnection);
 
-    QStringList prebundledLibraries;
-    foreach (const AndroidManager::Library &qtLib, m_availableQtLibs) {
-        if (libs.contains(qtLib.name) || m_prebundledLibs.contains(qtLib.name))
-            prebundledLibraries << qtLib.name;
-    }
-
-    QMetaObject::invokeMethod(this, "setPrebundledLibs", Qt::BlockingQueuedConnection,
-                              Q_ARG(QStringList, prebundledLibraries));
     emit updateRequiredLibrariesModels();
 }
 
@@ -413,6 +398,9 @@ QAbstractItemModel *AndroidPackageCreationStep::keystoreCertificates()
         if (!m_keystorePasswd.length())
             return 0;
         params << m_keystorePasswd;
+        Utils::Environment env = Utils::Environment::systemEnvironment();
+        env.set(QLatin1String("LANG"), QLatin1String("C"));
+        keytoolProc.setProcessEnvironment(env.toProcessEnvironment());
         keytoolProc.start(AndroidConfigurations::instance().keytoolPath().toString(), params);
         if (!keytoolProc.waitForStarted() || !keytoolProc.waitForFinished()) {
             QMessageBox::critical(0, tr("Error"),
@@ -487,7 +475,23 @@ void AndroidPackageCreationStep::collectFiles(QList<DeployItem> *deployList,
                 + qtLib
                 + QLatin1String(".so");
 
-        DeployItem deployItem(fullPath, 0, destinationPath, true);
+        // If the Qt lib/ folder contains libgnustl_shared.so, don't deploy it from there, since
+        // it will be deployed directly from the NDK instead.
+        if (qtLib != QLatin1String("gnustl_shared")) {
+            DeployItem deployItem(fullPath, 0, destinationPath, true);
+            deployList->append(deployItem);
+        }
+    }
+
+    if (!androidTargetArch.isEmpty()) {
+        ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+        if (tc->type() != QLatin1String(Constants::ANDROID_TOOLCHAIN_TYPE))
+            return;
+
+        AndroidToolChain *atc = static_cast<AndroidToolChain *>(tc);
+
+        QString libgnustl = AndroidManager::libGnuStl(androidTargetArch, atc->ndkToolChainVersion());
+        DeployItem deployItem(libgnustl, 0, androidLibPath + QLatin1String("/libgnustl_shared.so"), false);
         deployList->append(deployItem);
     }
 
@@ -504,12 +508,20 @@ void AndroidPackageCreationStep::collectFiles(QList<DeployItem> *deployList,
 
     QSet<QString> alreadyListed;
     foreach (QString bundledFile, m_otherBundledFiles) {
-        if (!bundledFile.endsWith(QLatin1Char('/')))
-            bundledFile.append(QLatin1Char('/'));
+        QStringList allFiles;
+        if (QFileInfo(qtVersionSourcePath + QLatin1Char('/') + bundledFile).isDir()) {
+            if (!bundledFile.endsWith(QLatin1Char('/')))
+                bundledFile.append(QLatin1Char('/'));
+
+            allFiles = collectRelativeFilePaths(qtVersionSourcePath + QLatin1Char('/') + bundledFile);
+        } else {
+            // If we need to bundle a specific file, we just add an empty string and the file
+            // names and data will be prepared correctly in the loop below.
+            allFiles = QStringList(QString());
+        }
 
-        QStringList allFiles = collectRelativeFilePaths(qtVersionSourcePath + QLatin1Char('/') + bundledFile);
         foreach (QString file, allFiles) {
-            QString fullPath = qtVersionSourcePath + QLatin1Char('/') + bundledFile + QLatin1Char('/') + file;
+            QString fullPath = qtVersionSourcePath + QLatin1Char('/') + bundledFile + file;
             if (alreadyListed.contains(fullPath))
                 continue;
 
@@ -518,22 +530,31 @@ void AndroidPackageCreationStep::collectFiles(QList<DeployItem> *deployList,
             QString garbledFileName;
             QString destinationPath;
             bool shouldStrip = false;
-            if (file.endsWith(QLatin1String(".so"))) {
-                garbledFileName = QLatin1String("lib")
-                    + AndroidManager::libraryPrefix()
-                    + QString(bundledFile).replace(QLatin1Char('/'), QLatin1Char('_'))
-                    + QString(file).replace(QLatin1Char('/'), QLatin1Char('_'));
+
+            QString fullFileName = bundledFile + file;
+            if (fullFileName.endsWith(QLatin1String(".so"))) {
+                if (fullFileName.startsWith(QLatin1String("lib/"))) {
+                    // Special case when the destination folder is lib/
+                    // Since this is also the source folder, there is no need to garble the file
+                    // name and copy it. We also won't have write access to this folder, so we
+                    // couldn't if we wanted to.
+                    garbledFileName = fullFileName.mid(sizeof("lib/") - 1);
+                } else {
+                    garbledFileName = QLatin1String("lib")
+                        + AndroidManager::libraryPrefix()
+                        + QString(fullFileName).replace(QLatin1Char('/'), QLatin1Char('_'));
+                }
                 destinationPath = androidLibPath + QLatin1Char('/') + garbledFileName;
                 shouldStrip = true;
             } else {
-                garbledFileName = AndroidManager::libraryPrefix() + bundledFile + file;
+                garbledFileName = AndroidManager::libraryPrefix() + QLatin1Char('/') + fullFileName;
                 destinationPath = androidAssetsPath + garbledFileName;
             }
 
             deployList->append(DeployItem(fullPath, 0, destinationPath, shouldStrip));
             pluginsAndImportsList->append(DeployItem(garbledFileName,
                                                      0,
-                                                     bundledFile + file,
+                                                     fullFileName,
                                                      shouldStrip));
         }
     }
@@ -558,6 +579,8 @@ void AndroidPackageCreationStep::removeManagedFilesFromPackage()
             }
         }
     }
+
+    removeDirectory(m_androidDir.toString() + QLatin1String("/assets/") + AndroidManager::libraryPrefix());
 }
 
 void AndroidPackageCreationStep::copyFilesIntoPackage(const QList<DeployItem> &deployList)
@@ -667,11 +690,12 @@ bool AndroidPackageCreationStep::createPackage()
     emit addOutput(tr("Creating package file ..."), MessageOutput);
 
     QProcess *const buildProc = new QProcess;
+    buildProc->setProcessEnvironment(m_environment.toProcessEnvironment());
 
     connect(buildProc, SIGNAL(readyReadStandardOutput()), this,
-        SLOT(handleBuildStdOutOutput()));
+        SLOT(handleBuildStdOutOutput()), Qt::DirectConnection);
     connect(buildProc, SIGNAL(readyReadStandardError()), this,
-        SLOT(handleBuildStdErrOutput()));
+        SLOT(handleBuildStdErrOutput()), Qt::DirectConnection);
 
     buildProc->setWorkingDirectory(m_androidDir.toString());
 
@@ -783,6 +807,10 @@ bool AndroidPackageCreationStep::runCommand(QProcess *buildProc
         return false;
     }
     buildProc->waitForFinished(-1);
+
+    handleProcessOutput(buildProc, false);
+    handleProcessOutput(buildProc, true);
+
     if (buildProc->error() != QProcess::UnknownError
         || buildProc->exitCode() != 0) {
         QString mainMessage = tr("Packaging Error: Command '%1 %2' failed.")
@@ -802,13 +830,7 @@ void AndroidPackageCreationStep::handleBuildStdOutOutput()
     QProcess *const process = qobject_cast<QProcess *>(sender());
     if (!process)
         return;
-
-    process->setReadChannel(QProcess::StandardOutput);
-    while (process->canReadLine()) {
-        QString line = QString::fromLocal8Bit(process->readLine());
-        m_outputParser.stdOutput(line);
-        emit addOutput(line, BuildStep::NormalOutput, BuildStep::DontAppendNewline);
-    }
+    handleProcessOutput(process, false);
 }
 
 void AndroidPackageCreationStep::handleBuildStdErrOutput()
@@ -817,11 +839,21 @@ void AndroidPackageCreationStep::handleBuildStdErrOutput()
     if (!process)
         return;
 
-    process->setReadChannel(QProcess::StandardError);
+    handleProcessOutput(process, true);
+}
+
+void AndroidPackageCreationStep::handleProcessOutput(QProcess *process, bool stdErr)
+{
+    process->setReadChannel(stdErr ? QProcess::StandardError : QProcess::StandardOutput);
     while (process->canReadLine()) {
         QString line = QString::fromLocal8Bit(process->readLine());
-        m_outputParser.stdError(line);
-        emit addOutput(line, BuildStep::ErrorOutput, BuildStep::DontAppendNewline);
+        if (stdErr)
+            m_outputParser.stdError(line);
+        else
+            m_outputParser.stdOutput(line);
+        emit addOutput(line, stdErr ? BuildStep::ErrorOutput
+                                    : BuildStep::NormalOutput,
+                       BuildStep::DontAppendNewline);
     }
 }
 
