@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -35,16 +35,20 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
 #include <coreplugin/icontext.h>
+#include <texteditor/codeassist/keywordscompletionassist.h>
 
 #include <utils/environment.h>
 #include <utils/pathchooser.h>
 
 #include <QFuture>
 #include <QStringList>
+#include <QCheckBox>
 #include <QDir>
+#include <QVector>
 #include <QAction>
 
-QT_FORWARD_DECLARE_CLASS(QProcess)
+#include "cmakevalidator.h"
+
 QT_FORWARD_DECLARE_CLASS(QLabel)
 
 namespace Utils {
@@ -78,6 +82,7 @@ public:
                        const QString &generator);
     bool hasCodeBlocksMsvcGenerator() const;
     bool hasCodeBlocksNinjaGenerator() const;
+    bool preferNinja() const;
     static QString findCbpFile(const QDir &);
 
     static QString findDumperLibrary(const Utils::Environment &env);
@@ -95,17 +100,6 @@ private:
     ProjectExplorer::Project *m_contextProject;
 };
 
-struct CMakeValidator
-{
-    enum STATE { VALID, INVALID, RUNNING };
-    STATE state;
-    QProcess *process;
-    bool hasCodeBlocksMsvcGenerator;
-    bool hasCodeBlocksNinjaGenerator;
-    QString version;
-    QString executable;
-};
-
 class CMakeSettingsPage : public Core::IOptionsPage
 {
     Q_OBJECT
@@ -123,21 +117,18 @@ public:
     bool isCMakeExecutableValid() const;
     bool hasCodeBlocksMsvcGenerator() const;
     bool hasCodeBlocksNinjaGenerator() const;
+    bool preferNinja() const;
 
-private slots:
-    void userCmakeFinished();
-    void pathCmakeFinished();
+    TextEditor::Keywords keywords();
 
 private:
-    void cmakeFinished(CMakeValidator *cmakeValidator) const;
     void saveSettings() const;
     QString findCmakeExecutable() const;
-    void startProcess(CMakeValidator *cmakeValidator);
-    void updateInfo(CMakeValidator *cmakeValidator);
 
     Utils::PathChooser *m_pathchooser;
-    mutable CMakeValidator m_userCmake;
-    mutable CMakeValidator m_pathCmake;
+    QCheckBox *m_preferNinja;
+    CMakeValidator m_cmakeValidatorForUser;
+    CMakeValidator m_cmakeValidatorForSystem;
 };
 
 } // namespace Internal
