@@ -30,7 +30,7 @@
 #ifndef CMAKERUNCONFIGURATION_H
 #define CMAKERUNCONFIGURATION_H
 
-#include <projectexplorer/applicationrunconfiguration.h>
+#include <projectexplorer/localapplicationrunconfiguration.h>
 #include <utils/environment.h>
 
 QT_BEGIN_NAMESPACE
@@ -42,14 +42,9 @@ class PathChooser;
 class DetailsWidget;
 }
 
-namespace ProjectExplorer {
-class EnvironmentWidget;
-}
-
 namespace CMakeProjectManager {
 namespace Internal {
 
-class CMakeBuildConfiguration;
 class CMakeTarget;
 
 class CMakeRunConfiguration : public ProjectExplorer::LocalApplicationRunConfiguration
@@ -68,7 +63,6 @@ public:
     void setRunMode(RunMode runMode);
     QString workingDirectory() const;
     QString commandLineArguments() const;
-    Utils::Environment environment() const;
     QWidget *createConfigurationWidget();
 
     void setExecutable(const QString &executable);
@@ -87,8 +81,6 @@ public:
     QString disabledReason() const;
 
 signals:
-    void baseEnvironmentChanged();
-    void userEnvironmentChangesChanged(const QList<Utils::EnvironmentItem> &diff);
     void baseWorkingDirectoryChanged(const QString&);
 
 private slots:
@@ -104,25 +96,12 @@ private:
     QString baseWorkingDirectory() const;
     void ctor();
 
-    enum BaseEnvironmentBase { CleanEnvironmentBase = 0,
-                               SystemEnvironmentBase = 1,
-                               BuildEnvironmentBase = 2};
-    void setBaseEnvironmentBase(BaseEnvironmentBase env);
-    BaseEnvironmentBase baseEnvironmentBase() const;
-    Utils::Environment baseEnvironment() const;
-    QString baseEnvironmentText() const;
-
-    void setUserEnvironmentChanges(const QList<Utils::EnvironmentItem> &diff);
-    QList<Utils::EnvironmentItem> userEnvironmentChanges() const;
-
     RunMode m_runMode;
     QString m_buildTarget;
     QString m_workingDirectory;
     QString m_userWorkingDirectory;
     QString m_title;
     QString m_arguments;
-    QList<Utils::EnvironmentItem> m_userEnvironmentChanges;
-    BaseEnvironmentBase m_baseEnvironmentBase;
     bool m_enabled;
 };
 
@@ -134,14 +113,11 @@ public:
 
 private slots:
     void setArguments(const QString &args);
-    void baseEnvironmentChanged();
-    void userEnvironmentChangesChanged();
-    void userChangesChanged();
     void setWorkingDirectory();
     void resetWorkingDirectory();
     void runInTerminalToggled(bool toggled);
+    void environmentWasChanged();
 
-    void baseEnvironmentComboBoxChanged(int index);
     void workingDirectoryChanged(const QString &workingDirectory);
 
 private:
@@ -149,8 +125,6 @@ private:
     bool m_ignoreChange;
     CMakeRunConfiguration *m_cmakeRunConfiguration;
     Utils::PathChooser *m_workingDirectoryEdit;
-    QComboBox *m_baseEnvironmentComboBox;
-    ProjectExplorer::EnvironmentWidget *m_environmentWidget;
     Utils::DetailsWidget *m_detailsContainer;
 };
 
@@ -163,9 +137,7 @@ public:
     ~CMakeRunConfigurationFactory();
 
     bool canCreate(ProjectExplorer::Target *parent, const Core::Id id) const;
-    ProjectExplorer::RunConfiguration *create(ProjectExplorer::Target *parent, const Core::Id id);
     bool canRestore(ProjectExplorer::Target *parent, const QVariantMap &map) const;
-    ProjectExplorer::RunConfiguration *restore(ProjectExplorer::Target *parent, const QVariantMap &map);
     bool canClone(ProjectExplorer::Target *parent, ProjectExplorer::RunConfiguration *product) const;
     ProjectExplorer::RunConfiguration *clone(ProjectExplorer::Target *parent, ProjectExplorer::RunConfiguration *product);
 
@@ -177,6 +149,10 @@ public:
 
 private:
     bool canHandle(ProjectExplorer::Target *parent) const;
+
+    ProjectExplorer::RunConfiguration *doCreate(ProjectExplorer::Target *parent, const Core::Id id);
+    ProjectExplorer::RunConfiguration *doRestore(ProjectExplorer::Target *parent,
+                                                 const QVariantMap &map);
 };
 
 }
