@@ -37,15 +37,9 @@
 #include "icore.h"
 #include "id.h"
 
-#include <utils/treewidgetcolumnstretcher.h>
-
 #include <QKeyEvent>
-#include <QShortcut>
-#include <QHeaderView>
 #include <QFileDialog>
 #include <QLineEdit>
-#include <QAction>
-#include <QKeyEvent>
 #include <QTreeWidgetItem>
 #include <QCoreApplication>
 #include <QDebug>
@@ -60,9 +54,9 @@ ShortcutSettings::ShortcutSettings(QObject *parent)
 {
     connect(ActionManager::instance(), SIGNAL(commandListChanged()), this, SLOT(initialize()));
 
-    setId(QLatin1String(Core::Constants::SETTINGS_ID_SHORTCUTS));
+    setId(Core::Constants::SETTINGS_ID_SHORTCUTS);
     setDisplayName(tr("Keyboard"));
-    setCategory(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE));
+    setCategory(Core::Constants::SETTINGS_CATEGORY_CORE);
     setDisplayCategory(QCoreApplication::translate("Core", Core::Constants::SETTINGS_TR_CATEGORY_CORE));
     setCategoryIcon(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE_ICON));
 }
@@ -82,6 +76,7 @@ QWidget *ShortcutSettings::createPage(QWidget *parent)
     setTargetLabelText(targetLabelText);
     setTargetEditTitle(editTitle);
     setTargetHeader(editTitle);
+    targetEdit()->setPlaceholderText(tr("Type to set shortcut"));
 
     if (m_searchKeywords.isEmpty()) {
         QTextStream(&m_searchKeywords) << ' ' << pageTitle
@@ -141,7 +136,7 @@ void ShortcutSettings::commandChanged(QTreeWidgetItem *current)
     CommandMappings::commandChanged(current);
     if (!current || !current->data(0, Qt::UserRole).isValid())
         return;
-    ShortcutItem *scitem = qVariantValue<ShortcutItem *>(current->data(0, Qt::UserRole));
+    ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
     setKeySequence(scitem->m_key);
 }
 
@@ -149,13 +144,13 @@ void ShortcutSettings::targetIdentifierChanged()
 {
     QTreeWidgetItem *current = commandList()->currentItem();
     if (current && current->data(0, Qt::UserRole).isValid()) {
-        ShortcutItem *scitem = qVariantValue<ShortcutItem *>(current->data(0, Qt::UserRole));
+        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
         scitem->m_key = QKeySequence(m_key[0], m_key[1], m_key[2], m_key[3]);
         if (scitem->m_cmd->defaultKeySequence() != scitem->m_key)
             setModified(current, true);
         else
             setModified(current, false);
-        current->setText(2, scitem->m_key);
+        current->setText(2, scitem->m_key.toString(QKeySequence::NativeText));
         resetCollisionMarker(scitem);
         markPossibleCollisions(scitem);
     }
@@ -168,14 +163,14 @@ void ShortcutSettings::setKeySequence(const QKeySequence &key)
     for (int i = 0; i < m_keyNum; ++i) {
         m_key[i] = key[i];
     }
-    targetEdit()->setText(key);
+    targetEdit()->setText(key.toString(QKeySequence::NativeText));
 }
 
 void ShortcutSettings::resetTargetIdentifier()
 {
     QTreeWidgetItem *current = commandList()->currentItem();
     if (current && current->data(0, Qt::UserRole).isValid()) {
-        ShortcutItem *scitem = qVariantValue<ShortcutItem *>(current->data(0, Qt::UserRole));
+        ShortcutItem *scitem = qvariant_cast<ShortcutItem *>(current->data(0, Qt::UserRole));
         setKeySequence(scitem->m_cmd->defaultKeySequence());
     }
 }
@@ -200,7 +195,7 @@ void ShortcutSettings::importAction()
             QString sid = item->m_cmd->id().toString();
             if (mapping.contains(sid)) {
                 item->m_key = mapping.value(sid);
-                item->m_item->setText(2, item->m_key);
+                item->m_item->setText(2, item->m_key.toString(QKeySequence::NativeText));
                 if (item->m_item == commandList()->currentItem())
                     commandChanged(item->m_item);
 
@@ -222,7 +217,7 @@ void ShortcutSettings::defaultAction()
 {
     foreach (ShortcutItem *item, m_scitems) {
         item->m_key = item->m_cmd->defaultKeySequence();
-        item->m_item->setText(2, item->m_key);
+        item->m_item->setText(2, item->m_key.toString(QKeySequence::NativeText));
         setModified(item->m_item, false);
         if (item->m_item == commandList()->currentItem())
             commandChanged(item->m_item);
@@ -293,7 +288,7 @@ void ShortcutSettings::initialize()
         s->m_key = c->keySequence();
         item->setText(0, subId);
         item->setText(1, c->description());
-        item->setText(2, s->m_key);
+        item->setText(2, s->m_key.toString(QKeySequence::NativeText));
         if (s->m_cmd->defaultKeySequence() != s->m_key)
             setModified(item, true);
 
@@ -333,7 +328,7 @@ void ShortcutSettings::handleKeyEvent(QKeyEvent *e)
     }
     m_keyNum++;
     QKeySequence ks(m_key[0], m_key[1], m_key[2], m_key[3]);
-    targetEdit()->setText(ks);
+    targetEdit()->setText(ks.toString(QKeySequence::NativeText));
     e->accept();
 }
 
@@ -362,7 +357,7 @@ void ShortcutSettings::markPossibleCollisions(ShortcutItem *item)
     if (item->m_key.isEmpty())
         return;
 
-    int globalId = Context(Constants::C_GLOBAL).at(0);
+    Id globalId = Context(Constants::C_GLOBAL).at(0);
 
     foreach (ShortcutItem *currentItem, m_scitems) {
 
@@ -371,11 +366,10 @@ void ShortcutSettings::markPossibleCollisions(ShortcutItem *item)
             continue;
         }
 
-        foreach (int context, currentItem->m_cmd->context()) {
-
+        foreach (Id id, currentItem->m_cmd->context()) {
             // conflict if context is identical, OR if one
             // of the contexts is the global context
-            if (item->m_cmd->context().contains(context) ||
+            if (item->m_cmd->context().contains(id) ||
                (item->m_cmd->context().contains(globalId) &&
                 !currentItem->m_cmd->context().isEmpty()) ||
                 (currentItem->m_cmd->context().contains(globalId) &&
