@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -31,7 +31,8 @@
 
 /*!
   \class Core::IEditor
-  \brief The IEditor is an interface for providing different editors for different file types.
+  \brief The IEditor class is an interface for providing different editors for
+  different file types.
 
   Classes that implement this interface are for example the editors for
   C++ files, ui-files and resource files.
@@ -42,13 +43,13 @@
 
   Guidelines for implementing:
   \list
-  \o displayName() is used as a user visible description of the document (usually filename w/o path).
-  \o kind() must be the same value as the kind() of the corresponding EditorFactory.
-  \o The changed() signal should be emitted when the modified state of the document changes
+  \li displayName() is used as a user visible description of the document (usually filename w/o path).
+  \li kind() must be the same value as the kind() of the corresponding EditorFactory.
+  \li The changed() signal should be emitted when the modified state of the document changes
      (so /bold{not} every time the document changes, but /bold{only once}).
-  \o If duplication is supported, you need to ensure that all duplicates
+  \li If duplication is supported, you need to ensure that all duplicates
         return the same file().
-  \o QString preferredMode() const is the mode the editor manager should activate.
+  \li QString preferredMode() const is the mode the editor manager should activate.
      Some editors use a special mode (such as Design mode).
   \endlist
 
