@@ -1,58 +1,51 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
-#include "mimetypesettings.h"
-#include "ui_mimetypesettingspage.h"
-#include "mimetypemagicdialog.h"
-#include "mimedatabase.h"
 #include "coreconstants.h"
 #include "editormanager.h"
 #include "icore.h"
 #include "ieditorfactory.h"
 #include "iexternaleditor.h"
+#include "mimedatabase.h"
+#include "mimetypemagicdialog.h"
+#include "mimetypesettings.h"
+#include "ui_mimetypesettingspage.h"
 
-#include <extensionsystem/pluginmanager.h>
-
-#include <QtCore/QCoreApplication>
-#include <QtCore/QLatin1String>
-#include <QtCore/QStringList>
-#include <QtCore/QVector>
-#include <QtCore/QSet>
-#include <QtCore/QScopedPointer>
-#include <QtCore/QAbstractTableModel>
-#include <QtCore/QHash>
-#include <QtGui/QIcon>
-#include <QtGui/QTableWidgetItem>
-#include <QtGui/QMessageBox>
+#include <QAbstractTableModel>
+#include <QCoreApplication>
+#include <QHash>
+#include <QMessageBox>
+#include <QScopedPointer>
+#include <QSet>
+#include <QStringList>
+#include <QTableWidgetItem>
+#include <QSortFilterProxyModel>
 #include <QtAlgorithms>
 
 #include <algorithm>
@@ -73,7 +66,7 @@ class MimeTypeSettingsModel : public QAbstractTableModel
 
 public:
     MimeTypeSettingsModel(QObject *parent = 0)
-        : QAbstractTableModel(parent), m_core(ICore::instance()) {}
+        : QAbstractTableModel(parent) {}
     virtual ~MimeTypeSettingsModel() {}
 
     virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
@@ -86,7 +79,6 @@ public:
     void validatePatterns(QStringList *candidates, const MimeType &mimeType) const;
     void updateKnownPatterns(const QStringList &oldPatterns, const QStringList &newPatterns);
 
-    ICore *m_core;
     QList<MimeType> m_mimeTypes;
     QSet<QString> m_knownPatterns;
     QHash<QString, QString> m_handlersByMimeType;
@@ -134,20 +126,20 @@ QVariant MimeTypeSettingsModel::data(const QModelIndex &modelIndex, int role) co
 
 void MimeTypeSettingsModel::load()
 {
-    m_mimeTypes = m_core->mimeDatabase()->mimeTypes();
+    m_mimeTypes = ICore::mimeDatabase()->mimeTypes();
     qSort(m_mimeTypes.begin(), m_mimeTypes.end(), MimeTypeComp());
     m_knownPatterns = QSet<QString>::fromList(
-        MimeDatabase::fromGlobPatterns(m_core->mimeDatabase()->globPatterns()));
+        MimeDatabase::fromGlobPatterns(ICore::mimeDatabase()->globPatterns()));
 
     foreach (const MimeType &mimeType, m_mimeTypes) {
         QString value;
         const QList<IEditorFactory *> factories =
-            m_core->editorManager()->editorFactories(mimeType);
+            EditorManager::editorFactories(mimeType);
         if (!factories.isEmpty()) {
             value = factories.front()->displayName();
         } else {
             const QList<IExternalEditor *> externalEditors =
-                m_core->editorManager()->externalEditors(mimeType);
+                EditorManager::externalEditors(mimeType);
             if (!externalEditors.isEmpty())
                 value = externalEditors.front()->displayName();
             else
@@ -239,31 +231,41 @@ public slots:
     void removeMagicHeader();
     void editMagicHeader();
     void resetMimeTypes();
+    void updateMagicHeaderButtons();
+
+private slots:
+    void setFilterPattern(const QString &pattern);
 
 public:
     static const QChar kSemiColon;
 
     QString m_keywords;
     MimeDatabase *m_mimeDatabase;
-    QScopedPointer<MimeTypeSettingsModel> m_model;
+    MimeTypeSettingsModel *m_model;
+    QSortFilterProxyModel *m_filterModel;
     int m_mimeForPatternSync;
     int m_mimeForMagicSync;
     bool m_reset;
     bool m_persist;
     QList<int> m_modifiedMimeTypes;
+    QString m_filterPattern;
     Ui::MimeTypeSettingsPage m_ui;
 };
 
 const QChar MimeTypeSettingsPrivate::kSemiColon(QLatin1Char(';'));
 
 MimeTypeSettingsPrivate::MimeTypeSettingsPrivate()
-    : m_mimeDatabase(ICore::instance()->mimeDatabase())
-    , m_model(new MimeTypeSettingsModel)
+    : m_mimeDatabase(ICore::mimeDatabase())
+    , m_model(new MimeTypeSettingsModel(this))
+    , m_filterModel(new QSortFilterProxyModel(this))
     , m_mimeForPatternSync(-1)
     , m_mimeForMagicSync(-1)
     , m_reset(false)
     , m_persist(false)
-{}
+{
+    m_filterModel->setSourceModel(m_model);
+    m_filterModel->setFilterKeyColumn(-1);
+}
 
 MimeTypeSettingsPrivate::~MimeTypeSettingsPrivate()
 {}
@@ -271,9 +273,12 @@ MimeTypeSettingsPrivate::~MimeTypeSettingsPrivate()
 void MimeTypeSettingsPrivate::configureUi(QWidget *w)
 {
     m_ui.setupUi(w);
+    m_ui.filterLineEdit->setText(m_filterPattern);
 
     m_model->load();
-    m_ui.mimeTypesTableView->setModel(m_model.data());
+    connect(m_ui.filterLineEdit, SIGNAL(textChanged(QString)),
+            this, SLOT(setFilterPattern(QString)));
+    m_ui.mimeTypesTableView->setModel(m_filterModel);
 
     configureTable(m_ui.mimeTypesTableView);
     configureTable(m_ui.magicHeadersTableWidget);
@@ -288,6 +293,11 @@ void MimeTypeSettingsPrivate::configureUi(QWidget *w)
     connect(m_ui.removeMagicButton, SIGNAL(clicked()), this, SLOT(removeMagicHeader()));
     connect(m_ui.editMagicButton, SIGNAL(clicked()), this, SLOT(editMagicHeader()));
     connect(m_ui.resetButton, SIGNAL(clicked()), this, SLOT(resetMimeTypes()));
+    connect(m_ui.magicHeadersTableWidget->selectionModel(),
+            SIGNAL(currentChanged(QModelIndex,QModelIndex)),
+            this,
+            SLOT(updateMagicHeaderButtons()));
+    updateMagicHeaderButtons();
 }
 
 void MimeTypeSettingsPrivate::configureTable(QTableView *tableView)
@@ -386,7 +396,7 @@ void MimeTypeSettingsPrivate::clearSyncData()
 }
 
 void MimeTypeSettingsPrivate::syncData(const QModelIndex &current,
-                                           const QModelIndex &previous)
+                                       const QModelIndex &previous)
 {
     if (previous.isValid()) {
         if (m_mimeForPatternSync == previous.row())
@@ -401,11 +411,12 @@ void MimeTypeSettingsPrivate::syncData(const QModelIndex &current,
     }
 
     if (current.isValid()) {
-        const MimeType &currentMimeType = m_model->m_mimeTypes.at(current.row());
+        const MimeType &currentMimeType =
+                m_model->m_mimeTypes.at(m_filterModel->mapToSource(current).row());
 
         QStringList formatedPatterns;
         foreach (const MimeGlobPattern &pattern, currentMimeType.globPatterns())
-            formatedPatterns.append(pattern.regExp().pattern());
+            formatedPatterns.append(pattern.pattern());
         m_ui.patternsLineEdit->setText(formatedPatterns.join(kSemiColon));
 
         // Consider only rule-based matchers.
@@ -518,7 +529,7 @@ void MimeTypeSettingsPrivate::updateMimeDatabase()
     m_modifiedMimeTypes.erase(std::unique(m_modifiedMimeTypes.begin(), m_modifiedMimeTypes.end()),
                               m_modifiedMimeTypes.end());
 
-    MimeDatabase *db = ICore::instance()->mimeDatabase();
+    MimeDatabase *db = ICore::mimeDatabase();
     QList<MimeType> allModified;
     foreach (int index, m_modifiedMimeTypes) {
         const MimeType &mimeType = m_model->m_mimeTypes.at(index);
@@ -545,79 +556,78 @@ void MimeTypeSettingsPrivate::resetMimeTypes()
     m_reset = true;
 }
 
+void MimeTypeSettingsPrivate::updateMagicHeaderButtons()
+{
+    const QModelIndex &modelIndex = m_ui.magicHeadersTableWidget->selectionModel()->currentIndex();
+    const bool enabled = modelIndex.isValid();
+
+    m_ui.removeMagicButton->setEnabled(enabled);
+    m_ui.editMagicButton->setEnabled(enabled);
+}
+
+void MimeTypeSettingsPrivate::setFilterPattern(const QString &pattern)
+{
+    m_filterPattern = pattern;
+    m_filterModel->setFilterWildcard(pattern);
+}
+
 // MimeTypeSettingsPage
 MimeTypeSettings::MimeTypeSettings(QObject *parent)
     : IOptionsPage(parent)
-    , m_d(new MimeTypeSettingsPrivate)
-{}
+    , d(new MimeTypeSettingsPrivate)
+{
+    setId(Core::Constants::SETTINGS_ID_MIMETYPES);
+    setDisplayName(tr("MIME Types"));
+    setCategory(Core::Constants::SETTINGS_CATEGORY_CORE);
+    setDisplayCategory(QCoreApplication::translate("Core",
+        Core::Constants::SETTINGS_TR_CATEGORY_CORE));
+    setCategoryIcon(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE_ICON));
+}
 
 MimeTypeSettings::~MimeTypeSettings()
-{}
-
-QString MimeTypeSettings::id() const
 {
-    return QLatin1String(Core::Constants::SETTINGS_ID_MIMETYPES);
-}
-
-QString MimeTypeSettings::displayName() const
-{
-    return tr("MIME Types");
-}
-
-QString MimeTypeSettings::category() const
-{
-    return QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE);
-}
-
-QString MimeTypeSettings::displayCategory() const
-{
-    return QCoreApplication::translate("Core", Core::Constants::SETTINGS_TR_CATEGORY_CORE);
-}
-
-QIcon MimeTypeSettings::categoryIcon() const
-{
-    return QIcon(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE_ICON));
+    delete d;
 }
 
 bool MimeTypeSettings::matches(const QString &s) const
 {
-    return m_d->m_keywords.contains(s, Qt::CaseInsensitive);
+    return d->m_keywords.contains(s, Qt::CaseInsensitive);
 }
 
 QWidget *MimeTypeSettings::createPage(QWidget *parent)
 {
     QWidget *w = new QWidget(parent);
-    m_d->configureUi(w);
+    d->configureUi(w);
     return w;
 }
 
 void MimeTypeSettings::apply()
 {
-    if (!m_d->m_modifiedMimeTypes.isEmpty()) {
+    if (!d->m_modifiedMimeTypes.isEmpty()) {
         const QModelIndex &modelIndex =
-            m_d->m_ui.mimeTypesTableView->selectionModel()->currentIndex();
+            d->m_ui.mimeTypesTableView->selectionModel()->currentIndex();
         if (modelIndex.isValid()) {
-            if (m_d->m_mimeForPatternSync == modelIndex.row())
-                m_d->syncMimePattern();
-            if (m_d->m_mimeForMagicSync == modelIndex.row())
-                m_d->syncMimeMagic();
+            if (d->m_mimeForPatternSync == modelIndex.row())
+                d->syncMimePattern();
+            if (d->m_mimeForMagicSync == modelIndex.row())
+                d->syncMimeMagic();
         }
-        m_d->clearSyncData();
+        d->clearSyncData();
     }
 
-    if (!m_d->m_persist)
-        m_d->m_persist = true;
+    if (!d->m_persist)
+        d->m_persist = true;
 }
 
 void MimeTypeSettings::finish()
 {
-    if (m_d->m_persist) {
-        if (m_d->m_reset)
-            ICore::instance()->mimeDatabase()->clearUserModifiedMimeTypes();
+    if (d->m_persist) {
+        if (d->m_reset)
+            ICore::mimeDatabase()->clearUserModifiedMimeTypes();
         else
-            m_d->updateMimeDatabase();
+            d->updateMimeDatabase();
     }
-    m_d->resetState();
+    d->resetState();
 }
 
 } // Internal
