@@ -27,19 +27,13 @@
 **
 ****************************************************************************/
 
-#include "cpptoolsreuse.h"
-#include "cpprefactoringchanges.h"
 #include "insertionpointlocator.h"
-#include "symbolfinder.h"
 
-#include <AST.h>
-#include <ASTVisitor.h>
-#include <TranslationUnit.h>
-#include <Literals.h>
-#include <Symbols.h>
+#include "cpptoolsreuse.h"
+#include "symbolfinder.h"
+#include "cpptoolsconstants.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/mimedatabase.h>
 
 #include <utils/qtcassert.h>
 
@@ -48,33 +42,6 @@ using namespace CppTools;
 
 namespace {
 
-static QString generate(InsertionPointLocator::AccessSpec xsSpec)
-{
-    switch (xsSpec) {
-    default:
-    case InsertionPointLocator::Public:
-        return QLatin1String("public:\n");
-
-    case InsertionPointLocator::Protected:
-        return QLatin1String("protected:\n");
-
-    case InsertionPointLocator::Private:
-        return QLatin1String("private:\n");
-
-    case InsertionPointLocator::PublicSlot:
-        return QLatin1String("public slots:\n");
-
-    case InsertionPointLocator::ProtectedSlot:
-        return QLatin1String("protected slots:\n");
-
-    case InsertionPointLocator::PrivateSlot:
-        return QLatin1String("private slots:\n");
-
-    case InsertionPointLocator::Signals:
-        return QLatin1String("signals:\n");
-    }
-}
-
 static int ordering(InsertionPointLocator::AccessSpec xsSpec)
 {
     static QList<InsertionPointLocator::AccessSpec> order = QList<InsertionPointLocator::AccessSpec>()
@@ -167,7 +134,7 @@ protected:
         if (needsLeadingEmptyLine)
             prefix += QLatin1String("\n");
         if (needsPrefix)
-            prefix += generate(_xsSpec);
+            prefix += InsertionPointLocator::accessSpecToString(_xsSpec);
 
         QString suffix;
         if (needsSuffix)
@@ -305,6 +272,33 @@ InsertionLocation::InsertionLocation(const QString &fileName,
     , m_column(column)
 {}
 
+QString InsertionPointLocator::accessSpecToString(InsertionPointLocator::AccessSpec xsSpec)
+{
+    switch (xsSpec) {
+    default:
+    case InsertionPointLocator::Public:
+        return QLatin1String("public:\n");
+
+    case InsertionPointLocator::Protected:
+        return QLatin1String("protected:\n");
+
+    case InsertionPointLocator::Private:
+        return QLatin1String("private:\n");
+
+    case InsertionPointLocator::PublicSlot:
+        return QLatin1String("public slots:\n");
+
+    case InsertionPointLocator::ProtectedSlot:
+        return QLatin1String("protected slots:\n");
+
+    case InsertionPointLocator::PrivateSlot:
+        return QLatin1String("private slots:\n");
+
+    case InsertionPointLocator::Signals:
+        return QLatin1String("signals:\n");
+    }
+}
+
 InsertionPointLocator::InsertionPointLocator(const CppRefactoringChanges &refactoringChanges)
     : m_refactoringChanges(refactoringChanges)
 {
@@ -324,19 +318,6 @@ InsertionLocation InsertionPointLocator::methodDeclarationInClass(
     }
 }
 
-static bool isSourceFile(const QString &fileName)
-{
-    const Core::MimeDatabase *mimeDb = Core::ICore::mimeDatabase();
-    Core::MimeType cSourceTy = mimeDb->findByType(QLatin1String("text/x-csrc"));
-    Core::MimeType cppSourceTy = mimeDb->findByType(QLatin1String("text/x-c++src"));
-    Core::MimeType mSourceTy = mimeDb->findByType(QLatin1String("text/x-objcsrc"));
-    QStringList suffixes = cSourceTy.suffixes();
-    suffixes += cppSourceTy.suffixes();
-    suffixes += mSourceTy.suffixes();
-    QFileInfo fileInfo(fileName);
-    return suffixes.contains(fileInfo.suffix());
-}
-
 namespace {
 template <class Key, class Value>
 class HighestValue
@@ -382,7 +363,7 @@ public:
         : ASTVisitor(translationUnit)
     {}
 
-    void operator()(Declaration *decl, unsigned *line, unsigned *column)
+    void operator()(Symbol *decl, unsigned *line, unsigned *column)
     {
         // default to end of file
         _bestToken.maybeSet(-1, translationUnit()->ast()->lastToken());
@@ -489,7 +470,9 @@ static Declaration *isNonVirtualFunctionDeclaration(Symbol *s)
     return declaration;
 }
 
-static InsertionLocation nextToSurroundingDefinitions(Declaration *declaration, const CppRefactoringChanges &changes)
+static InsertionLocation nextToSurroundingDefinitions(Symbol *declaration,
+                                                      const CppRefactoringChanges &changes,
+                                                      const QString& destinationFile)
 {
     InsertionLocation noResult;
     Class *klass = declaration->enclosingClass();
@@ -508,85 +491,90 @@ static InsertionLocation nextToSurroundingDefinitions(Declaration *declaration,
     if (declIndex == -1)
         return noResult;
 
-    // scan preceding declarations for a function declaration
+    // scan preceding declarations for a function declaration (and see if it is defined)
+    CppTools::SymbolFinder symbolFinder;
+    Function *definitionFunction = 0;
     QString prefix, suffix;
     Declaration *surroundingFunctionDecl = 0;
     for (int i = declIndex - 1; i >= 0; --i) {
         Symbol *s = klass->memberAt(i);
-        surroundingFunctionDecl = isNonVirtualFunctionDeclaration(s);
-        if (surroundingFunctionDecl) {
-            prefix = QLatin1String("\n\n");
-            break;
+        if (s->isGenerated() || !(surroundingFunctionDecl = isNonVirtualFunctionDeclaration(s)))
+            continue;
+        if ((definitionFunction = symbolFinder.findMatchingDefinition(surroundingFunctionDecl,
+                                                                      changes.snapshot())))
+        {
+            if (destinationFile.isEmpty() || destinationFile == QString::fromUtf8(
+                        definitionFunction->fileName(), definitionFunction->fileNameLength())) {
+                prefix = QLatin1String("\n\n");
+                break;
+            }
+            definitionFunction = 0;
         }
     }
-    if (!surroundingFunctionDecl) {
+    if (!definitionFunction) {
         // try to find one below
         for (unsigned i = declIndex + 1; i < klass->memberCount(); ++i) {
             Symbol *s = klass->memberAt(i);
             surroundingFunctionDecl = isNonVirtualFunctionDeclaration(s);
-            if (surroundingFunctionDecl) {
-                suffix = QLatin1String("\n\n");
-                break;
+            if (!surroundingFunctionDecl)
+                continue;
+            if ((definitionFunction = symbolFinder.findMatchingDefinition(surroundingFunctionDecl,
+                                                                          changes.snapshot())))
+            {
+                if (destinationFile.isEmpty() || destinationFile == QString::fromUtf8(
+                            definitionFunction->fileName(), definitionFunction->fileNameLength())) {
+                    suffix = QLatin1String("\n\n");
+                    break;
+                }
+                definitionFunction = 0;
             }
         }
-        if (!surroundingFunctionDecl)
-            return noResult;
     }
 
-    // find the declaration's definition
-    CppTools::SymbolFinder symbolFinder;
-    Symbol *definition = symbolFinder.findMatchingDefinition(surroundingFunctionDecl,
-                                                             changes.snapshot());
-    if (!definition)
+    if (!definitionFunction)
         return noResult;
 
     unsigned line, column;
     if (suffix.isEmpty()) {
-        Function *definitionFunction = definition->asFunction();
-        if (!definitionFunction)
-            return noResult;
-
-        Document::Ptr targetDoc = changes.snapshot().document(QString::fromUtf8(definition->fileName()));
+        Document::Ptr targetDoc = changes.snapshot().document(QString::fromUtf8(definitionFunction->fileName()));
         if (!targetDoc)
             return noResult;
 
         targetDoc->translationUnit()->getPosition(definitionFunction->endOffset(), &line, &column);
     } else {
         // we don't have an offset to the start of the function definition, so we need to manually find it...
-        CppRefactoringFilePtr targetFile = changes.file(QString::fromUtf8(definition->fileName()));
+        CppRefactoringFilePtr targetFile = changes.file(QString::fromUtf8(definitionFunction->fileName()));
         if (!targetFile->isValid())
             return noResult;
 
         FindFunctionDefinition finder(targetFile->cppDocument()->translationUnit());
-        FunctionDefinitionAST *functionDefinition = finder(definition->line(), definition->column());
+        FunctionDefinitionAST *functionDefinition = finder(definitionFunction->line(), definitionFunction->column());
         if (!functionDefinition)
             return noResult;
 
         targetFile->cppDocument()->translationUnit()->getTokenStartPosition(functionDefinition->firstToken(), &line, &column);
     }
 
-    return InsertionLocation(QString::fromUtf8(definition->fileName()), prefix, suffix, line, column);
+    return InsertionLocation(QString::fromUtf8(definitionFunction->fileName()), prefix, suffix, line, column);
 }
 
-QList<InsertionLocation> InsertionPointLocator::methodDefinition(
-    Declaration *declaration) const
+QList<InsertionLocation> InsertionPointLocator::methodDefinition(Symbol *declaration,
+                                                                 bool useSymbolFinder,
+                                                                 const QString &destinationFile) const
 {
     QList<InsertionLocation> result;
     if (!declaration)
         return result;
 
-    CppTools::SymbolFinder symbolFinder;
-    if (Symbol *s = symbolFinder.findMatchingDefinition(declaration,
-                                                        m_refactoringChanges.snapshot(),
-                                                        true)) {
-        if (Function *f = s->asFunction()) {
-            if (f->isConst() == declaration->type().isConst()
-                    && f->isVolatile() == declaration->type().isVolatile())
-                return result;
-        }
+    if (useSymbolFinder) {
+        CppTools::SymbolFinder symbolFinder;
+        if (symbolFinder.findMatchingDefinition(declaration, m_refactoringChanges.snapshot(), true))
+            return result;
     }
 
-    const InsertionLocation location = nextToSurroundingDefinitions(declaration, m_refactoringChanges);
+    const InsertionLocation location = nextToSurroundingDefinitions(declaration,
+                                                                    m_refactoringChanges,
+                                                                    destinationFile);
     if (location.isValid()) {
         result += location;
         return result;
@@ -595,7 +583,7 @@ QList<InsertionLocation> InsertionPointLocator::methodDefinition(
     const QString declFileName = QString::fromUtf8(declaration->fileName(),
                                                    declaration->fileNameLength());
     QString target = declFileName;
-    if (!isSourceFile(declFileName)) {
+    if (!ProjectFile::isSource(ProjectFile::classify(declFileName))) {
         QString candidate = CppTools::correspondingHeaderOrSource(declFileName);
         if (!candidate.isEmpty())
             target = candidate;
