@@ -33,9 +33,7 @@
 #include "searchsymbols.h"
 
 #include <find/ifindfilter.h>
-#include <find/searchresultwindow.h>
 
-#include <QFutureInterface>
 #include <QFutureWatcher>
 #include <QPointer>
 #include <QWidget>
@@ -50,12 +48,11 @@ class CppModelManager;
 class SymbolsFindFilter : public Find::IFindFilter
 {
     Q_OBJECT
-public:
-    enum SearchScope {
-        SearchProjectsOnly,
-        SearchGlobal
-    };
 
+public:
+    typedef SymbolSearcher::SearchScope SearchScope;
+
+public:
     explicit SymbolsFindFilter(CppModelManager *manager);
 
     QString id() const;
@@ -102,15 +99,6 @@ private:
     SearchScope m_scope;
 };
 
-class SymbolsFindParameters
-{
-public:
-    QString text;
-    Find::FindFlags flags;
-    SearchSymbols::SymbolTypes types;
-    SymbolsFindFilter::SearchScope scope;
-};
-
 class SymbolsFindFilterConfigWidget : public QWidget
 {
     Q_OBJECT
@@ -137,7 +125,4 @@ private:
 } // Internal
 } // CppTools
 
-Q_DECLARE_METATYPE(CppTools::Internal::SymbolsFindFilter::SearchScope)
-Q_DECLARE_METATYPE(CppTools::Internal::SymbolsFindParameters)
-
 #endif // SYMBOLSFINDFILTER_H
