@@ -28,7 +28,6 @@
 ****************************************************************************/
 
 #include "breakwindow.h"
-#include "debuggerinternalconstants.h"
 #include "breakhandler.h"
 #include "debuggerengine.h"
 #include "debuggeractions.h"
@@ -38,28 +37,19 @@
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
 
-#include <QAction>
-#include <QApplication>
-#include <QButtonGroup>
 #include <QCheckBox>
 #include <QComboBox>
-#include <QDebug>
 #include <QDialog>
 #include <QDialogButtonBox>
 #include <QFormLayout>
 #include <QGroupBox>
-#include <QHBoxLayout>
-#include <QHeaderView>
-#include <QIntValidator>
 #include <QKeyEvent>
 #include <QLabel>
 #include <QLineEdit>
+#include <QMessageBox>
 #include <QMenu>
-#include <QSpacerItem>
 #include <QSpinBox>
 #include <QTextEdit>
-#include <QVariant>
-#include <QVBoxLayout>
 
 namespace Debugger {
 namespace Internal {
@@ -147,7 +137,7 @@ private:
 };
 
 BreakpointDialog::BreakpointDialog(BreakpointModelId id, QWidget *parent)
-    : QDialog(parent), m_enabledParts(~0), m_previousType(UnknownType),
+    : QDialog(parent), m_enabledParts(~0), m_previousType(UnknownBreakpointType),
       m_firstTypeChange(true)
 {
     setWindowTitle(tr("Edit Breakpoint Properties"));
@@ -169,7 +159,8 @@ BreakpointDialog::BreakpointDialog(BreakpointModelId id, QWidget *parent)
           << tr("Break on data access at address given by expression")
           << tr("Break on QML signal emit")
           << tr("Break when JavaScript exception is thrown");
-    QTC_ASSERT(types.size() == BreakpointAtJavaScriptThrow, return);
+    // We don't list UnknownBreakpointType, so 1 less:
+    QTC_CHECK(types.size() + 1 == LastBreakpointType);
     m_comboBoxType = new QComboBox(groupBoxBasic);
     m_comboBoxType->setMaxVisibleItems(20);
     m_comboBoxType->addItems(types);
@@ -494,11 +485,10 @@ void BreakpointDialog::setParts(unsigned mask, const BreakpointParameters &data)
     }
 
     if (mask & ExpressionPart) {
-        if (!data.expression.isEmpty()) {
+        if (!data.expression.isEmpty())
             m_lineEditExpression->setText(data.expression);
-        } else {
+        else
             m_lineEditExpression->clear();
-        }
     }
 
     if (mask & ConditionPart)
@@ -523,8 +513,9 @@ void BreakpointDialog::typeChanged(int)
     const BreakpointType newType = type();
     m_previousType = newType;
     // Save current state.
-    switch(previousType) {
-    case UnknownType:
+    switch (previousType) {
+    case UnknownBreakpointType:
+    case LastBreakpointType:
         break;
     case BreakpointByFileAndLine:
         getParts(FileAndLinePart|ModulePart|AllConditionParts|TracePointPart, &m_savedParameters);
@@ -554,7 +545,8 @@ void BreakpointDialog::typeChanged(int)
 
     // Enable and set up new state from saved values.
     switch (newType) {
-    case UnknownType:
+    case UnknownBreakpointType:
+    case LastBreakpointType:
         break;
     case BreakpointByFileAndLine:
         setParts(FileAndLinePart|AllConditionParts|ModulePart|TracePointPart, m_savedParameters);
@@ -706,6 +698,18 @@ void BreakTreeView::keyPressEvent(QKeyEvent *ev)
         int row = qMin(model()->rowCount() - ids.size() - 1, currentIndex().row());
         deleteBreakpoints(ids);
         setCurrentIndex(si.at(0).sibling(row, 0));
+    } else if (ev->key() == Qt::Key_Space) {
+        QItemSelectionModel *sm = selectionModel();
+        QTC_ASSERT(sm, return);
+        const QModelIndexList selectedIds = sm->selectedIndexes();
+        if (!selectedIds.isEmpty()) {
+            BreakHandler *handler = breakHandler();
+            const BreakpointModelIds validIds = handler->findBreakpointsByIndex(selectedIds);
+            const bool isEnabled = validIds.isEmpty() || handler->isEnabled(validIds.at(0));
+            setBreakpointsEnabled(validIds, !isEnabled);
+            foreach (const QModelIndex &id, selectedIds)
+                update(id);
+        }
     }
     QTreeView::keyPressEvent(ev);
 }
@@ -713,11 +717,15 @@ void BreakTreeView::keyPressEvent(QKeyEvent *ev)
 void BreakTreeView::mouseDoubleClickEvent(QMouseEvent *ev)
 {
     QModelIndex indexUnderMouse = indexAt(ev->pos());
-    if (indexUnderMouse.isValid() && indexUnderMouse.column() >= 4) {
-        BreakpointModelId id = breakHandler()->findBreakpointByIndex(indexUnderMouse);
-        editBreakpoints(BreakpointModelIds() << id);
+    if (indexUnderMouse.isValid()) {
+        if (indexUnderMouse.column() >= 4) {
+            BreakpointModelId id = breakHandler()->findBreakpointByIndex(indexUnderMouse);
+            editBreakpoints(BreakpointModelIds() << id);
+        }
+    } else {
+        addBreakpoint();
     }
-    QTreeView::mouseDoubleClickEvent(ev);
+    BaseTreeView::mouseDoubleClickEvent(ev);
 }
 
 void BreakTreeView::setModel(QAbstractItemModel *model)
@@ -826,7 +834,7 @@ void BreakTreeView::contextMenuEvent(QContextMenuEvent *ev)
     if (act == deleteAction)
         deleteBreakpoints(selectedIds);
     else if (act == deleteAllAction)
-        deleteBreakpoints(handler->allBreakpointIds());
+        deleteAllBreakpoints();
     else if (act == deleteByFileAction)
         deleteBreakpoints(breakpointsInFile);
     else if (act == adjustColumnAction)
@@ -852,6 +860,16 @@ void BreakTreeView::setBreakpointsEnabled(const BreakpointModelIds &ids, bool en
         handler->setEnabled(id, enabled);
 }
 
+void BreakTreeView::deleteAllBreakpoints()
+{
+    if (QMessageBox::warning(debuggerCore()->mainWindow(),
+           tr("Remove All Breakpoints"),
+           tr("Are you sure you want to remove all breakpoints "
+              "from all files in the current session?"),
+           QMessageBox::Yes|QMessageBox::No) == QMessageBox::Yes)
+        deleteBreakpoints(breakHandler()->allBreakpointIds());
+}
+
 void BreakTreeView::deleteBreakpoints(const BreakpointModelIds &ids)
 {
     BreakHandler *handler = breakHandler();
