@@ -1,49 +1,42 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef DEBUGGER_CDBENGINE_H
 #define DEBUGGER_CDBENGINE_H
 
 #include "debuggerengine.h"
-#include "breakpoint.h"
 
-#include <QtCore/QSharedPointer>
-#include <QtCore/QProcess>
-#include <QtCore/QVariantList>
-#include <QtCore/QMap>
-#include <QtCore/QMultiHash>
-#include <QtCore/QTime>
-#include <QtCore/QPair>
-#include <QtCore/QList>
+#include <QSharedPointer>
+#include <QProcess>
+#include <QMap>
+#include <QVariant>
+#include <QTime>
 
 namespace Utils {
 class ConsoleProcess;
@@ -54,7 +47,6 @@ namespace Internal {
 class DisassemblerAgent;
 struct CdbBuiltinCommand;
 struct CdbExtensionCommand;
-struct CdbOptions;
 struct MemoryViewCookie;
 class ByteArrayInputStream;
 class GdbMi;
@@ -64,8 +56,6 @@ class CdbEngine : public Debugger::DebuggerEngine
     Q_OBJECT
 
 public:
-    typedef QSharedPointer<CdbOptions> OptionsPtr;
-
     enum CommandFlags { QuietCommand = 0x1 };
     // Flag bits for a sequence of commands
     enum CommandSequenceFlags {
@@ -81,11 +71,9 @@ public:
     typedef void (CdbEngine::*BuiltinCommandHandler)(const CdbBuiltinCommandPtr &);
     typedef void (CdbEngine::*ExtensionCommandHandler)(const CdbExtensionCommandPtr &);
 
-    explicit CdbEngine(const DebuggerStartParameters &sp,
-        DebuggerEngine *masterEngine,
-        const OptionsPtr &options);
+    CdbEngine(const DebuggerStartParameters &sp);
+    ~CdbEngine();
 
-    virtual ~CdbEngine();
     // Factory function that returns 0 if the debug engine library cannot be found.
 
     virtual bool setToolTipExpression(const QPoint &mousePos, TextEditor::ITextEditor *editor,
@@ -98,8 +86,7 @@ public:
     virtual void detachDebugger();
     virtual void updateWatchData(const WatchData &data,
                                  const WatchUpdateFlags & flags = WatchUpdateFlags());
-    virtual unsigned debuggerCapabilities() const;
-    virtual bool canWatchWidgets() const;
+    virtual bool hasCapability(unsigned cap) const;
     virtual void watchPoint(const QPoint &);
     virtual void setRegisterValue(int regnr, const QString &value);
 
@@ -116,10 +103,10 @@ public:
     virtual void executeRunToFunction(const QString &functionName);
     virtual void executeJumpToLine(const ContextData &data);
     virtual void assignValueInDebugger(const WatchData *w, const QString &expr, const QVariant &value);
-    virtual void executeDebuggerCommand(const QString &command);
+    virtual void executeDebuggerCommand(const QString &command, DebuggerLanguages languages);
 
     virtual void activateFrame(int index);
-    virtual void selectThread(int index);
+    virtual void selectThread(ThreadId threadId);
 
     virtual bool stateAcceptsBreakpointChanges() const;
     virtual bool acceptsBreakpoint(BreakpointModelId id) const;
@@ -161,6 +148,7 @@ private slots:
 
     void postCommandSequence(unsigned mask);
     void operateByInstructionTriggered(bool);
+    void verboseLogTriggered(bool);
 
     void consoleStubError(const QString &);
     void consoleStubProcessStarted();
@@ -186,7 +174,9 @@ private:
     };
     enum ParseStackResultFlags // Flags returned by parseStackTrace
     {
-        ParseStackStepInto = 1 // Need to execute a step, hit on a call frame in "Step into"
+        ParseStackStepInto = 1, // Need to execute a step, hit on a call frame in "Step into"
+        ParseStackStepOut = 2, // Need to step out, hit on a frame without debug information
+        ParseStackWow64 = 3 // Hit on a frame with 32bit emulation, switch debugger to 32 bit mode
     };
 
 
@@ -203,13 +193,14 @@ private:
     void handleSessionAccessible(unsigned long cdbExState);
     void handleSessionInaccessible(unsigned long cdbExState);
     void handleSessionIdle(const QByteArray &message);
-    void doInterruptInferior(SpecialStopMode sm);
+    bool doInterruptInferior(SpecialStopMode sm);
     void doInterruptInferiorCustomSpecialStop(const QVariant &v);
     void doContinueInferior();
     inline void parseOutputLine(QByteArray line);
     inline bool isCdbProcessRunning() const { return m_process.state() != QProcess::NotRunning; }
     bool canInterruptInferior() const;
     void syncOperateByInstruction(bool operateByInstruction);
+    void syncVerboseLog(bool verboseLog);
     void postWidgetAtCommand();
     void handleCustomSpecialStop(const QVariant &v);
     void postFetchMemory(const MemoryViewCookie &c);
@@ -228,6 +219,9 @@ private:
     void handleExpression(const CdbExtensionCommandPtr &);
     void handleResolveSymbol(const CdbBuiltinCommandPtr &command);
     void handleResolveSymbol(const QList<quint64> &addresses, const QVariant &cookie);
+    void handleCheckWow64(const CdbBuiltinCommandPtr &cmd);
+    void ensureUsing32BitStackInWow64(const CdbBuiltinCommandPtr &cmd);
+    void handleSwitchWow64Stack(const CdbBuiltinCommandPtr &cmd);
     void jumpToAddress(quint64 address);
 
     // Extension commands
@@ -248,11 +242,10 @@ private:
     int elapsedLogTime() const;
     void addLocalsOptions(ByteArrayInputStream &s) const;
     unsigned parseStackTrace(const GdbMi &data, bool sourceStepInto);
-    void parseThreads(const GdbMi &, int forceCurrentThreadId = -1);
+    void mergeStartParametersSourcePathMap();
 
     const QByteArray m_creatorExtPrefix;
     const QByteArray m_tokenPrefix;
-    const OptionsPtr m_options;
 
     QProcess m_process;
     QScopedPointer<Utils::ConsoleProcess> m_consoleStub;
@@ -269,8 +262,17 @@ private:
     const QByteArray m_extensionCommandPrefixBA; //!< Library name used as prefix
     bool m_operateByInstructionPending; //!< Creator operate by instruction action changed.
     bool m_operateByInstruction;
+    bool m_verboseLogPending; //!< Creator verbose log action changed.
+    bool m_verboseLog;
     bool m_notifyEngineShutdownOnTermination;
     bool m_hasDebuggee;
+    bool m_cdbIs64Bit;
+    enum Wow64State {
+        wow64Uninitialized,
+        noWow64Stack,
+        wow64Stack32Bit,
+        wow64Stack64Bit
+    } m_wow64State;
     QTime m_logTime;
     mutable int m_elapsedLogTime;
     QByteArray m_extensionMessageBuffer;
@@ -280,9 +282,11 @@ private:
     PendingBreakPointMap m_pendingBreakpointMap;
     QHash<QString, QString> m_fileNameModuleHash;
     QMultiHash<QString, quint64> m_symbolAddressCache;
+    QHash<QByteArray, QString> m_watchInameToName;
     bool m_ignoreCdbOutput;
     QVariantList m_customSpecialStopData;
     QList<SourcePathMapping> m_sourcePathMappings;
+    QScopedPointer<GdbMi> m_coreStopReason;
 };
 
 } // namespace Internal
