@@ -32,34 +32,32 @@
 #include "debuggerinternalconstants.h"
 #include "debuggeractions.h"
 #include "debuggercore.h"
-#include "debuggerplugin.h"
 #include "debuggerrunner.h"
 #include "debuggerstringutils.h"
 #include "debuggerstartparameters.h"
 
-#include "memoryagent.h"
-#include "disassembleragent.h"
 #include "breakhandler.h"
+#include "disassembleragent.h"
+#include "memoryagent.h"
 #include "moduleshandler.h"
+#include "peutils.h"
 #include "registerhandler.h"
-#include "snapshothandler.h"
 #include "sourcefileshandler.h"
 #include "stackhandler.h"
 #include "threadshandler.h"
 #include "watchhandler.h"
 
 #include <coreplugin/icore.h>
-#include <coreplugin/idocument.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 
-#include <texteditor/itexteditor.h>
-#include <texteditor/basetextmark.h>
-
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/taskhub.h>
 
+#include <texteditor/itexteditor.h>
+
 #include <utils/savedaction.h>
 #include <utils/qtcassert.h>
 #include <utils/fileinprojectfinder.h>
@@ -68,9 +66,7 @@
 
 #include <QDebug>
 #include <QTimer>
-#include <QFile>
 #include <QFileInfo>
-#include <QFutureInterface>
 
 #include <QMessageBox>
 
@@ -172,8 +168,6 @@ public:
         m_disassemblerAgent(engine),
         m_memoryAgent(engine),
         m_isStateDebugging(false),
-        m_testsPossible(true),
-        m_testsRunning(false),
         m_taskHub(0)
     {
         connect(&m_locationTimer, SIGNAL(timeout()), SLOT(resetLocation()));
@@ -307,17 +301,7 @@ public:
     bool m_isStateDebugging;
 
     Utils::FileInProjectFinder m_fileFinder;
-    // Testing
-    void handleAutoTests();
-    void handleAutoTestLine(int line);
-    void reportTestError(const QString &msg, int line);
-    bool m_testsPossible;
-    bool m_testsRunning;
-    bool m_breakOnError;
-    bool m_foundError;
-    QStringList m_testContents;
     TaskHub *m_taskHub;
-    QString m_testFileName;
 };
 
 
@@ -603,13 +587,12 @@ void DebuggerEngine::gotoLocation(const Location &loc)
     if (editors.isEmpty()) {
         editor = EditorManager::openEditor(file, Core::Id(),
             EditorManager::IgnoreNavigationHistory);
-        if (editor) {
-            editors.append(editor);
-            editor->setProperty(Constants::OPENED_BY_DEBUGGER, true);
-        }
+        QTC_ASSERT(editor, return); // Unreadable file?
+        editor->setProperty(Constants::OPENED_BY_DEBUGGER, true);
     } else {
         editor = editors.back();
     }
+
     ITextEditor *texteditor = qobject_cast<ITextEditor *>(editor);
     if (texteditor)
         texteditor->gotoLine(line, 0);
@@ -760,6 +743,7 @@ void DebuggerEnginePrivate::doSetupEngine()
 {
     m_engine->showMessage(_("CALL: SETUP ENGINE"));
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << m_engine << state());
+    m_engine->checkForReleaseBuild(m_startParameters);
     m_engine->setupEngine();
 }
 
@@ -879,6 +863,11 @@ void DebuggerEngine::notifyEngineRequestRemoteSetup()
     emit requestRemoteSetup();
 }
 
+void DebuggerEngine::notifyEngineRemoteServerRunning(const QByteArray &, int /*pid*/)
+{
+    showMessage(_("NOTE: REMOTE SERVER RUNNING IN MULTIMODE"));
+}
+
 void DebuggerEngine::notifyEngineRemoteSetupDone(int gdbServerPort, int qmlPort)
 {
     showMessage(_("NOTE: REMOTE SETUP DONE: GDB SERVER PORT: %1  QML PORT %2")
@@ -1789,150 +1778,94 @@ void DebuggerEngine::setStateDebugging(bool on)
     d->m_isStateDebugging = on;
 }
 
-void DebuggerEngine::handleAutoTests()
+void DebuggerEngine::checkForReleaseBuild(const DebuggerStartParameters &sp)
 {
-    d->handleAutoTests();
-}
-
-bool DebuggerEngine::isAutoTestRunning() const
-{
-    return d->m_testsRunning;
-}
-
-void DebuggerEnginePrivate::handleAutoTests()
-{
-    if (!m_testsPossible)
+    if (!debuggerCore()->boolSetting(WarnOnReleaseBuilds) || !(sp.languages & CppLanguage))
+        return;
+    QString binary = sp.executable;
+    if (binary.isEmpty())
         return;
 
-    StackFrame frame = m_engine->stackHandler()->currentFrame();
-    if (!frame.file.endsWith(QLatin1String("debugger/simple/simple_test_app.cpp")))
-        return;
+    QString detailedWarning;
+    switch (sp.toolChainAbi.binaryFormat()) {
+    case ProjectExplorer::Abi::PEFormat: {
+        if (sp.masterEngineType != CdbEngineType)
+            return;
+        if (!binary.endsWith(QLatin1String(".exe"), Qt::CaseInsensitive))
+            binary.append(QLatin1String(".exe"));
+        QString errorMessage;
+        QStringList rc;
+        if (getPDBFiles(binary, &rc, &errorMessage) && !rc.isEmpty())
+            return;
+        if (!errorMessage.isEmpty()) {
+            detailedWarning.append(QLatin1Char('\n'));
+            detailedWarning.append(errorMessage);
+        }
+        break;
+    }
+    case ProjectExplorer::Abi::ElfFormat: {
 
-    if (m_testContents.isEmpty()) {
-        QFile file(frame.file);
-        file.open(QIODevice::ReadOnly);
-        QTextStream ts(&file);
-        m_testFileName = QFileInfo(frame.file).absoluteFilePath();
-        m_testContents = ts.readAll().split(QLatin1Char('\n'));
-        if (m_testContents.isEmpty()) {
-            m_testsPossible = false;
+        Utils::ElfReader reader(binary);
+        Utils::ElfData elfData = reader.readHeaders();
+        QString error = reader.errorString();
+
+        debuggerCore()->showMessage(_("EXAMINING ") + binary, LogDebug);
+        QByteArray msg = "ELF SECTIONS: ";
+
+        static QList<QByteArray> interesting;
+        if (interesting.isEmpty()) {
+            interesting.append(".debug_info");
+            interesting.append(".debug_abbrev");
+            interesting.append(".debug_line");
+            interesting.append(".debug_str");
+            interesting.append(".debug_loc");
+            interesting.append(".debug_range");
+            interesting.append(".gdb_index");
+            interesting.append(".note.gnu.build-id");
+            interesting.append(".gnu.hash");
+            interesting.append(".gnu_debuglink");
+        }
+
+        QSet<QByteArray> seen;
+        foreach (const Utils::ElfSectionHeader &header, elfData.sectionHeaders) {
+            msg.append(header.name);
+            msg.append(' ');
+            if (interesting.contains(header.name))
+                seen.insert(header.name);
+        }
+        debuggerCore()->showMessage(_(msg), LogDebug);
+
+        if (!error.isEmpty()) {
+            debuggerCore()->showMessage(_("ERROR WHILE READING ELF SECTIONS: ") + error,
+                                        LogDebug);
             return;
         }
-        foreach (const QString &s, m_testContents) {
-            if (s.startsWith(QLatin1String("#define USE_AUTORUN"))) {
-                if (s.startsWith(QLatin1String("#define USE_AUTORUN 1"))) {
-                    m_testsPossible = true;
-                    m_breakOnError = false;
-                    m_testsRunning = true;
-                } else if (s.startsWith(QLatin1String("#define USE_AUTORUN 2"))) {
-                    m_testsPossible = true;
-                    m_testsRunning = true;
-                    m_breakOnError = true;
-                } else {
-                    m_testsPossible = false;
-                    m_testsRunning = false;
-                    m_breakOnError = false;
-                }
-                break;
-            }
+
+        if (elfData.sectionHeaders.isEmpty()) {
+            debuggerCore()->showMessage(_("NO SECTION HEADERS FOUND. IS THIS AN EXECUTABLE?"),
+                                        LogDebug);
+            return;
         }
+
+        // Note: .note.gnu.build-id also appears in regular release builds.
+        // bool hasBuildId = elfData.indexOf(".note.gnu.build-id") >= 0;
+        bool hasEmbeddedInfo = elfData.indexOf(".debug_info") >= 0;
+        bool hasLink = elfData.indexOf(".gnu_debuglink") >= 0;
+        if (hasEmbeddedInfo || hasLink)
+            return;
+
+        foreach (const QByteArray &name, interesting) {
+            const QString found = seen.contains(name) ? tr("Found.") : tr("Not found.");
+            detailedWarning.append(tr("\nSection %1: %2").arg(_(name)).arg(found));
+        }
+        break;
     }
-
-    if (!m_testsPossible)
+    default:
         return;
-
-    int line = frame.line;
-    if (line > 1 && line < m_testContents.size())
-        handleAutoTestLine(line);
-}
-
-void DebuggerEnginePrivate::handleAutoTestLine(int line)
-{
-    QString s = m_testContents.at(line).trimmed();
-    if (s.endsWith(QLatin1Char('.')))
-        s.chop(1);
-    int pos = s.indexOf(QLatin1String("//"));
-    if (pos == -1)
-        return;
-    s = s.mid(pos + 2).trimmed();
-    QString cmd = s.section(QLatin1Char(' '), 0, 0);
-    if (cmd == QLatin1String("Skip")) {
-        m_engine->showMessage(_("Skipping test %1").arg(line));
-        handleAutoTestLine(line + 1);
-    } else if (cmd == QLatin1String("Expand")) {
-        m_engine->showMessage(_("'Expand' found in line %1, "
-            "but is not implemented yet.").arg(line));
-        handleAutoTestLine(line + 1);
-    } else if (cmd == QLatin1String("Check")) {
-        QString name = s.section(QLatin1Char(' '), 1, 1);
-        if (name.isEmpty()) {
-            reportTestError(_("'Check' needs arguments."), line);
-        } else if (name.count(QLatin1Char('.')) >= 2) {
-            m_engine->showMessage(_("Variable %1 found in line %2 is nested "
-                "too deeply for the current implementation.").arg(name).arg(line));
-        } else {
-            QByteArray iname = "local." + name.toLatin1();
-            QString found = m_engine->watchHandler()->displayForAutoTest(iname);
-            if (found.isEmpty()) {
-                reportTestError(_("Check referes to unknown variable %1.")
-                    .arg(name), line);
-            } else {
-                QString needle = s.section(QLatin1Char(' '), 2, -1);
-                if (needle == found) {
-                    m_engine->showMessage(_("Check in line %1 for %2 was successful")
-                        .arg(line).arg(needle));
-                } else {
-                    reportTestError(_("Check for %1 failed. Got %2.")
-                        .arg(needle).arg(found), line);
-                }
-            }
-        }
-        handleAutoTestLine(line + 1);
-    } else if (cmd == QLatin1String("CheckType")) {
-        QString name = s.section(QLatin1Char(' '), 1, 1);
-        if (name.isEmpty()) {
-            reportTestError(_("'CheckType' needs arguments."), line);
-        } else if (name.count(QLatin1Char('.')) >= 2) {
-            m_engine->showMessage(_("Variable %1 found in line %2 is nested "
-                "too deeply for the current implementation.").arg(name).arg(line));
-        } else {
-            QByteArray iname = "local." + name.toLatin1();
-            QString found = m_engine->watchHandler()->displayForAutoTest(iname);
-            if (found.isEmpty()) {
-                reportTestError(_("CheckType referes to unknown variable %1.")
-                    .arg(name), line);
-            } else {
-                QString needle = s.section(QLatin1Char(' '), 2, -1);
-                if (found.endsWith(needle)) {
-                    m_engine->showMessage(_("CheckType in line %1 for %2 was successful")
-                        .arg(line).arg(needle));
-                } else {
-                    reportTestError(_("CheckType for %1 failed. Got %2.")
-                        .arg(needle).arg(found), line);
-                }
-            }
-        }
-        handleAutoTestLine(line + 1);
-    } else if (cmd == QLatin1String("Continue")) {
-        if (state() == InferiorStopOk) {
-            m_engine->showMessage(_("Continue in line %1 processed.").arg(line));
-            if (!m_breakOnError || !m_foundError)
-                m_engine->continueInferior();
-            else
-                m_foundError = false;
-        } else {
-            m_engine->showMessage(_("Auto-run aborted in line %1. State is %2.")
-                .arg(line).arg(state()));
-        }
     }
-}
-
-void DebuggerEnginePrivate::reportTestError(const QString &msg, int line)
-{
-    m_engine->showMessage(_("### Line %1: %2").arg(line).arg(msg));
-    m_foundError = true;
-    Task task(Task::Error, msg, Utils::FileName::fromUserInput(m_testFileName), line + 1, Core::Id("DebuggerTest"));
-    taskHub()->addTask(task);
+    showMessageBox(QMessageBox::Information, tr("Warning"),
+                   tr("This does not seem to be a \"Debug\" build.\n"
+                      "Setting breakpoints by file name and line number may fail.\n").append(detailedWarning));
 }
 
 TaskHub *DebuggerEnginePrivate::taskHub()
