@@ -36,7 +36,6 @@
 #include "threaddata.h" // For ThreadId.
 
 #include <QObject>
-#include <QStringList>
 
 QT_BEGIN_NAMESPACE
 class QDebug;
@@ -299,6 +298,7 @@ protected:
 
     virtual void notifyEngineRequestRemoteSetup();
     public:
+    virtual void notifyEngineRemoteServerRunning(const QByteArray &, int pid);
     virtual void notifyEngineRemoteSetupDone(int gdbServerPort, int qmlPort);
     virtual void notifyEngineRemoteSetupFailed(const QString &message);
 
@@ -385,6 +385,8 @@ protected:
     bool isStateDebugging() const;
     void setStateDebugging(bool on);
 
+    static void checkForReleaseBuild(const DebuggerStartParameters& sp);
+
     virtual void setupSlaveInferior();
     virtual void setupSlaveEngine();
     virtual void runSlaveEngine();
@@ -393,9 +395,6 @@ protected:
     virtual void slaveEngineStateChanged(DebuggerEngine *engine,
         DebuggerState state);
 
-    virtual void handleAutoTests();
-    virtual bool isAutoTestRunning() const;
-
 private:
     // Wrapper engine needs access to state of its subengines.
     friend class Internal::QmlCppEngine;
