@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -31,45 +31,35 @@
 #include "debuggerruncontrolfactory.h"
 
 #include "debuggeractions.h"
-#include "debuggerinternalconstants.h"
 #include "debuggercore.h"
 #include "debuggerengine.h"
-#include "debuggermainwindow.h"
-#include "debuggerplugin.h"
-#include "debuggerstringutils.h"
-#include "debuggerstartparameters.h"
 #include "debuggerkitinformation.h"
-#include "lldb/lldbenginehost.h"
+#include "debuggerplugin.h"
+#include "debuggerrunconfigurationaspect.h"
+#include "debuggerstartparameters.h"
+#include "debuggerstringutils.h"
 #include "debuggertooltipmanager.h"
+#include "breakhandler.h"
 
 #ifdef Q_OS_WIN
 #  include "peutils.h"
 #  include <utils/winutils.h>
 #endif
 
-#include <projectexplorer/abi.h>
-#include <projectexplorer/applicationrunconfiguration.h> // For LocalApplication*
+#include <projectexplorer/localapplicationrunconfiguration.h> // For LocalApplication*
+#include <projectexplorer/environmentaspect.h> // For the environment
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/target.h>
-#include <projectexplorer/toolchain.h>
+#include <projectexplorer/taskhub.h>
 
-#include <utils/outputformat.h>
-#include <utils/synchronousprocess.h>
 #include <utils/qtcassert.h>
-#include <utils/fancymainwindow.h>
 #include <utils/qtcprocess.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/helpmanager.h>
 
-#include <QDir>
-#include <QCheckBox>
-#include <QSpinBox>
-#include <QDebug>
 #include <QErrorMessage>
-#include <QFormLayout>
-#include <QLabel>
+#include <QTcpServer>
 
 using namespace Debugger::Internal;
 using namespace ProjectExplorer;
@@ -86,6 +76,7 @@ DebuggerEngine *createScriptEngine(const DebuggerStartParameters &sp);
 DebuggerEngine *createPdbEngine(const DebuggerStartParameters &sp);
 DebuggerEngine *createQmlEngine(const DebuggerStartParameters &sp);
 DebuggerEngine *createQmlCppEngine(const DebuggerStartParameters &sp, QString *error);
+DebuggerEngine *createLldbLibEngine(const DebuggerStartParameters &sp);
 DebuggerEngine *createLldbEngine(const DebuggerStartParameters &sp);
 
 static const char *engineTypeName(DebuggerEngineType et)
@@ -105,147 +96,16 @@ static const char *engineTypeName(DebuggerEngineType et)
         return "QML engine";
     case Debugger::QmlCppEngineType:
         return "QML C++ engine";
+    case Debugger::LldbLibEngineType:
+        return "LLDB binary engine";
     case Debugger::LldbEngineType:
-        return "LLDB engine";
+        return "LLDB command line engine";
     case Debugger::AllEngineTypes:
         break;
     }
     return "No engine";
 }
 
-////////////////////////////////////////////////////////////////////////
-//
-// DebuggerRunConfigWidget
-//
-////////////////////////////////////////////////////////////////////////
-
-class DebuggerRunConfigWidget : public RunConfigWidget
-{
-    Q_OBJECT
-
-public:
-    explicit DebuggerRunConfigWidget(RunConfiguration *runConfiguration);
-    QString displayName() const { return tr("Debugger Settings"); }
-
-private slots:
-    void useCppDebuggerToggled(bool on);
-    void useQmlDebuggerToggled(bool on);
-    void qmlDebugServerPortChanged(int port);
-    void useMultiProcessToggled(bool on);
-
-public:
-    DebuggerRunConfigurationAspect *m_aspect; // not owned
-
-    QCheckBox *m_useCppDebugger;
-    QCheckBox *m_useQmlDebugger;
-    QSpinBox *m_debugServerPort;
-    QLabel *m_debugServerPortLabel;
-    QLabel *m_qmlDebuggerInfoLabel;
-    QCheckBox *m_useMultiProcess;
-};
-
-DebuggerRunConfigWidget::DebuggerRunConfigWidget(RunConfiguration *runConfiguration)
-{
-    m_aspect = runConfiguration->debuggerAspect();
-
-    m_useCppDebugger = new QCheckBox(tr("Enable C++"), this);
-    m_useQmlDebugger = new QCheckBox(tr("Enable QML"), this);
-
-    m_debugServerPort = new QSpinBox(this);
-    m_debugServerPort->setMinimum(1);
-    m_debugServerPort->setMaximum(65535);
-
-    m_debugServerPortLabel = new QLabel(tr("Debug port:"), this);
-    m_debugServerPortLabel->setBuddy(m_debugServerPort);
-
-    m_qmlDebuggerInfoLabel = new QLabel(tr("<a href=\""
-        "qthelp://com.nokia.qtcreator/doc/creator-debugging-qml.html"
-        "\">What are the prerequisites?</a>"));
-
-    m_useCppDebugger->setChecked(m_aspect->useCppDebugger());
-    m_useQmlDebugger->setChecked(m_aspect->useQmlDebugger());
-
-    m_debugServerPort->setValue(m_aspect->qmlDebugServerPort());
-
-    static const QByteArray env = qgetenv("QTC_DEBUGGER_MULTIPROCESS");
-    m_useMultiProcess =
-        new QCheckBox(tr("Enable Debugging of Subprocesses"), this);
-    m_useMultiProcess->setChecked(m_aspect->useMultiProcess());
-    m_useMultiProcess->setVisible(env.toInt());
-
-    connect(m_qmlDebuggerInfoLabel, SIGNAL(linkActivated(QString)),
-            Core::HelpManager::instance(), SLOT(handleHelpRequest(QString)));
-    connect(m_useQmlDebugger, SIGNAL(toggled(bool)),
-            SLOT(useQmlDebuggerToggled(bool)));
-    connect(m_useCppDebugger, SIGNAL(toggled(bool)),
-            SLOT(useCppDebuggerToggled(bool)));
-    connect(m_debugServerPort, SIGNAL(valueChanged(int)),
-            SLOT(qmlDebugServerPortChanged(int)));
-    connect(m_useMultiProcess, SIGNAL(toggled(bool)),
-            SLOT(useMultiProcessToggled(bool)));
-
-    if (m_aspect->isDisplaySuppressed())
-        hide();
-
-    if (m_aspect->areQmlDebuggingOptionsSuppressed()) {
-        m_debugServerPortLabel->hide();
-        m_debugServerPort->hide();
-        m_useQmlDebugger->hide();
-    }
-
-    if (m_aspect->areCppDebuggingOptionsSuppressed())
-        m_useCppDebugger->hide();
-
-    if (m_aspect->isQmlDebuggingSpinboxSuppressed()) {
-        m_debugServerPort->hide();
-        m_debugServerPortLabel->hide();
-    }
-
-    QHBoxLayout *qmlLayout = new QHBoxLayout;
-    qmlLayout->setMargin(0);
-    qmlLayout->addWidget(m_useQmlDebugger);
-    qmlLayout->addWidget(m_debugServerPortLabel);
-    qmlLayout->addWidget(m_debugServerPort);
-    qmlLayout->addWidget(m_qmlDebuggerInfoLabel);
-    qmlLayout->addStretch();
-
-    QVBoxLayout *layout = new QVBoxLayout;
-    layout->setMargin(0);
-    layout->addWidget(m_useCppDebugger);
-    layout->addLayout(qmlLayout);
-    layout->addWidget(m_useMultiProcess);
-    setLayout(layout);
-}
-
-void DebuggerRunConfigWidget::qmlDebugServerPortChanged(int port)
-{
-    m_aspect->m_qmlDebugServerPort = port;
-}
-
-void DebuggerRunConfigWidget::useCppDebuggerToggled(bool on)
-{
-    m_aspect->m_useCppDebugger = on;
-    if (!on && !m_useQmlDebugger->isChecked())
-        m_useQmlDebugger->setChecked(true);
-}
-
-void DebuggerRunConfigWidget::useQmlDebuggerToggled(bool on)
-{
-    m_debugServerPort->setEnabled(on);
-    m_debugServerPortLabel->setEnabled(on);
-
-    m_aspect->m_useQmlDebugger = on
-            ? DebuggerRunConfigurationAspect::EnableQmlDebugger
-            : DebuggerRunConfigurationAspect::DisableQmlDebugger;
-    if (!on && !m_useCppDebugger->isChecked())
-        m_useCppDebugger->setChecked(true);
-}
-
-void DebuggerRunConfigWidget::useMultiProcessToggled(bool on)
-{
-    m_aspect->m_useMultiProcess = on;
-}
-
 ////////////////////////////////////////////////////////////////////////
 //
 // DebuggerRunControlPrivate
@@ -261,7 +121,7 @@ public:
 public:
     DebuggerRunControl *q;
     DebuggerEngine *m_engine;
-    const QWeakPointer<RunConfiguration> m_myRunConfiguration;
+    const QPointer<RunConfiguration> m_myRunConfiguration;
     bool m_running;
 };
 
@@ -465,11 +325,6 @@ bool DebuggerRunControlFactory::canRun(RunConfiguration *runConfiguration, RunMo
             && qobject_cast<LocalApplicationRunConfiguration *>(runConfiguration);
 }
 
-QString DebuggerRunControlFactory::displayName() const
-{
-    return DebuggerPlugin::tr("Debug");
-}
-
 static DebuggerStartParameters localStartParameters(RunConfiguration *runConfiguration, QString *errorMessage)
 {
     DebuggerStartParameters sp;
@@ -477,6 +332,8 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
     LocalApplicationRunConfiguration *rc =
             qobject_cast<LocalApplicationRunConfiguration *>(runConfiguration);
     QTC_ASSERT(rc, return sp);
+    EnvironmentAspect *environment = rc->extraAspect<ProjectExplorer::EnvironmentAspect>();
+    QTC_ASSERT(environment, return sp);
     if (!rc->ensureConfigured(errorMessage))
         return sp;
 
@@ -484,7 +341,7 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
     Kit *kit = target ? target->kit() : KitManager::instance()->defaultKit();
     if (!fillParameters(&sp, kit, errorMessage))
         return sp;
-    sp.environment = rc->environment();
+    sp.environment = environment->environment();
     sp.workingDirectory = rc->workingDirectory();
 
 #if defined(Q_OS_WIN)
@@ -495,7 +352,7 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
     sp.executable = rc->executable();
     if (sp.executable.isEmpty())
         return sp;
-    sp.startMode = StartInternal;
+
     sp.processArgs = rc->commandLineArguments();
     sp.useTerminal = rc->runMode() == LocalApplicationRunConfiguration::Console;
     sp.dumperLibrary = rc->dumperLibrary();
@@ -510,15 +367,27 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
         }
     }
 
-    DebuggerRunConfigurationAspect *aspect = runConfiguration->debuggerAspect();
-    sp.multiProcess = aspect->useMultiProcess();
+    DebuggerRunConfigurationAspect *debugger
+            = runConfiguration->extraAspect<Debugger::DebuggerRunConfigurationAspect>();
+    sp.multiProcess = debugger->useMultiProcess();
 
-    if (aspect->useCppDebugger())
+    if (debugger->useCppDebugger())
         sp.languages |= CppLanguage;
 
-    if (aspect->useQmlDebugger()) {
-        sp.qmlServerAddress = _("127.0.0.1");
-        sp.qmlServerPort = aspect->qmlDebugServerPort();
+    if (debugger->useQmlDebugger()) {
+        const ProjectExplorer::IDevice::ConstPtr device =
+                DeviceKitInformation::device(runConfiguration->target()->kit());
+        QTC_ASSERT(device->type() == ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE, return sp);
+        QTcpServer server;
+        const bool canListen = server.listen(QHostAddress::LocalHost)
+                || server.listen(QHostAddress::LocalHostIPv6);
+        if (!canListen) {
+            if (errorMessage)
+                *errorMessage = DebuggerPlugin::tr("Not enough free ports for QML debugging. ");
+            return sp;
+        }
+        sp.qmlServerAddress = server.serverAddress().toString();
+        sp.qmlServerPort = server.serverPort();
         sp.languages |= QmlLanguage;
 
         // Makes sure that all bindings go through the JavaScript engine, so that
@@ -530,6 +399,8 @@ static DebuggerStartParameters localStartParameters(RunConfiguration *runConfigu
         QtcProcess::addArg(&sp.processArgs, QString::fromLatin1("-qmljsdebugger=port:%1,block").arg(sp.qmlServerPort));
     }
 
+    sp.startMode = StartInternal;
+
     // FIXME: If it's not yet build this will be empty and not filled
     // when rebuild as the runConfiguration is not stored and therefore
     // cannot be used to retrieve the dumper location.
@@ -568,12 +439,13 @@ static bool fixupEngineTypes(DebuggerStartParameters &sp, RunConfiguration *rc,
     }
 
     if (rc) {
-        DebuggerRunConfigurationAspect *aspect = rc->debuggerAspect();
+        DebuggerRunConfigurationAspect *aspect
+                = rc->extraAspect<Debugger::DebuggerRunConfigurationAspect>();
         if (const Target *target = rc->target())
             if (!fillParameters(&sp, target->kit(), errorMessage))
                 return false;
-        const bool useCppDebugger = aspect->useCppDebugger();
-        const bool useQmlDebugger = aspect->useQmlDebugger();
+        const bool useCppDebugger = aspect->useCppDebugger() && (sp.languages & CppLanguage);
+        const bool useQmlDebugger = aspect->useQmlDebugger() && (sp.languages & QmlLanguage);
         if (useQmlDebugger) {
             if (useCppDebugger) {
                 sp.masterEngineType = QmlCppEngineType;
@@ -594,20 +466,21 @@ static bool fixupEngineTypes(DebuggerStartParameters &sp, RunConfiguration *rc,
 DebuggerRunControl *DebuggerRunControlFactory::doCreate
     (const DebuggerStartParameters &sp0, RunConfiguration *rc, QString *errorMessage)
 {
-    Q_UNUSED(errorMessage);
+    TaskHub *th = ProjectExplorerPlugin::instance()->taskHub();
+    th->clearTasks(Debugger::Constants::TASK_CATEGORY_DEBUGGER_DEBUGINFO);
+    th->clearTasks(Debugger::Constants::TASK_CATEGORY_DEBUGGER_TEST);
+    th->clearTasks(Debugger::Constants::TASK_CATEGORY_DEBUGGER_RUNTIME);
+
     DebuggerStartParameters sp = sp0;
     if (!debuggerCore()->boolSetting(AutoEnrichParameters)) {
         const QString sysroot = sp.sysRoot;
-        if (sp.debugInfoLocation.isEmpty()) {
+        if (sp.debugInfoLocation.isEmpty())
             sp.debugInfoLocation = sysroot + QLatin1String("/usr/lib/debug");
-        }
         if (sp.debugSourceLocation.isEmpty()) {
             QString base = sysroot + QLatin1String("/usr/src/debug/");
             sp.debugSourceLocation.append(base + QLatin1String("qt5base/src/corelib"));
             sp.debugSourceLocation.append(base + QLatin1String("qt5base/src/gui"));
             sp.debugSourceLocation.append(base + QLatin1String("qt5base/src/network"));
-            sp.debugSourceLocation.append(base + QLatin1String("qt5base/src/v8"));
-            sp.debugSourceLocation.append(base + QLatin1String("qt5declarative/src/qml"));
         }
     }
 
@@ -617,6 +490,11 @@ DebuggerRunControl *DebuggerRunControlFactory::doCreate
     return new DebuggerRunControl(rc, sp);
 }
 
+IRunConfigurationAspect *DebuggerRunControlFactory::createRunConfigurationAspect(RunConfiguration *rc)
+{
+    return new DebuggerRunConfigurationAspect(rc);
+}
+
 DebuggerRunControl *DebuggerRunControlFactory::createAndScheduleRun
     (const DebuggerStartParameters &sp, RunConfiguration *runConfiguration)
 {
@@ -634,12 +512,6 @@ DebuggerRunControl *DebuggerRunControlFactory::createAndScheduleRun
     return rc;
 }
 
-RunConfigWidget *DebuggerRunControlFactory::createConfigurationWidget
-    (RunConfiguration *runConfiguration)
-{
-    return new DebuggerRunConfigWidget(runConfiguration);
-}
-
 DebuggerEngine *DebuggerRunControlFactory::createEngine(DebuggerEngineType et,
     const DebuggerStartParameters &sp, QString *errorMessage)
 {
@@ -656,6 +528,8 @@ DebuggerEngine *DebuggerRunControlFactory::createEngine(DebuggerEngineType et,
         return createQmlEngine(sp);
     case LldbEngineType:
         return createLldbEngine(sp);
+    case LldbLibEngineType:
+        return createLldbLibEngine(sp);
     case QmlCppEngineType:
         return createQmlCppEngine(sp, errorMessage);
     default:
@@ -667,5 +541,3 @@ DebuggerEngine *DebuggerRunControlFactory::createEngine(DebuggerEngineType et,
 }
 
 } // namespace Debugger
-
-#include "debuggerrunner.moc"
