@@ -38,7 +38,6 @@
 #include <utils/environment.h>
 #include <projectexplorer/abi.h>
 #include <projectexplorer/kit.h>
-#include <projectexplorer/projectexplorerconstants.h>
 
 #include <QMetaType>
 
@@ -67,7 +66,7 @@ public:
         multiProcess(false),
         languages(AnyLanguage),
         qmlServerAddress(QLatin1String("127.0.0.1")),
-        qmlServerPort(ProjectExplorer::Constants::QML_DEFAULT_DEBUG_SERVER_PORT),
+        qmlServerPort(Constants::QML_DEFAULT_DEBUG_SERVER_PORT),
         remoteSetupNeeded(false),
         startMode(NoStartMode),
         closeMode(KillAtClose),
@@ -115,7 +114,6 @@ public:
     QString remoteChannel;
     QString symbolFileName;
     QString serverStartScript;
-    QString searchPath; // Gdb "set solib-search-path"
     QString debugInfoLocation; // Gdb "set-debug-file-directory".
     QStringList debugSourceLocation; // Gdb "directory"
     QByteArray remoteSourcesDir;
@@ -142,7 +140,7 @@ public:
 
 namespace Internal {
 
-bool fillParameters(DebuggerStartParameters *sp, const ProjectExplorer::Kit *kit = 0, QString *errorMessage = 0);
+bool fillParameters(DebuggerStartParameters *sp, const ProjectExplorer::Kit *kit, QString *errorMessage = 0);
 
 } // namespace Internal
 } // namespace Debugger
