@@ -35,8 +35,6 @@
 #include "debuggerstartparameters.h"
 #include "debuggerstringutils.h"
 
-#include <projectexplorer/abi.h>
-#include <utils/fancymainwindow.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
@@ -251,7 +249,7 @@ void GdbRemoteServerEngine::handleFileExecAndSymbols(const GdbResponse &response
     if (response.resultClass == GdbResultDone) {
         callTargetRemote();
     } else {
-        QByteArray reason = response.data.findChild("msg").data();
+        QByteArray reason = response.data["msg"].data();
         QString msg = tr("Reading debug information failed:\n");
         msg += QString::fromLocal8Bit(reason);
         if (reason.endsWith("No such file or directory.")) {
@@ -305,7 +303,7 @@ void GdbRemoteServerEngine::handleTargetRemote(const GdbResponse &response)
     } else {
         // 16^error,msg="hd:5555: Connection timed out."
         QString msg = msgConnectRemoteServerFailed(
-            QString::fromLocal8Bit(response.data.findChild("msg").data()));
+            QString::fromLocal8Bit(response.data["msg"].data()));
         notifyInferiorSetupFailed(msg);
     }
 }
@@ -325,7 +323,7 @@ void GdbRemoteServerEngine::handleTargetExtendedRemote(const GdbResponse &respon
         postCommand("attach " + QByteArray::number(m_targetPid), CB(handleTargetExtendedAttach));
     } else {
         QString msg = msgConnectRemoteServerFailed(
-            QString::fromLocal8Bit(response.data.findChild("msg").data()));
+            QString::fromLocal8Bit(response.data["msg"].data()));
         notifyInferiorSetupFailed(msg);
     }
 }
@@ -338,7 +336,7 @@ void GdbRemoteServerEngine::handleTargetExtendedAttach(const GdbResponse &respon
         handleInferiorPrepared();
     } else {
         QString msg = msgConnectRemoteServerFailed(
-            QString::fromLocal8Bit(response.data.findChild("msg").data()));
+            QString::fromLocal8Bit(response.data["msg"].data()));
         notifyInferiorSetupFailed(msg);
     }
 }
@@ -366,7 +364,7 @@ void GdbRemoteServerEngine::handleTargetQnx(const GdbResponse &response)
     } else {
         // 16^error,msg="hd:5555: Connection timed out."
         QString msg = msgConnectRemoteServerFailed(
-            QString::fromLocal8Bit(response.data.findChild("msg").data()));
+            QString::fromLocal8Bit(response.data["msg"].data()));
         notifyInferiorSetupFailed(msg);
     }
 }
@@ -383,13 +381,13 @@ void GdbRemoteServerEngine::handleAttach(const GdbResponse &response)
         break;
     }
     case GdbResultError:
-        if (response.data.findChild("msg").data() == "ptrace: Operation not permitted.") {
+        if (response.data["msg"].data() == "ptrace: Operation not permitted.") {
             notifyInferiorSetupFailed(DumperHelper::msgPtraceError(startParameters().startMode));
             break;
         }
         // if msg != "ptrace: ..." fall through
     default:
-        QString msg = QString::fromLocal8Bit(response.data.findChild("msg").data());
+        QString msg = QString::fromLocal8Bit(response.data["msg"].data());
         notifyInferiorSetupFailed(msg);
     }
 }
@@ -402,8 +400,13 @@ void GdbRemoteServerEngine::runEngine()
     if (!remoteExecutable.isEmpty()) {
         // Cannot use -exec-run for QNX gdb 7.4 as it does not support path parameter for the MI call
         const bool useRun = m_isQnxGdb && m_gdbVersion > 70300;
-        const QByteArray command = useRun ? "run" : "-exec-run";
-        postCommand(command + " " + remoteExecutable.toLocal8Bit(), GdbEngine::RunRequest, CB(handleExecRun));
+        QByteArray command = useRun ? "run" : "-exec-run";
+        command += " " + remoteExecutable.toLocal8Bit();
+
+        const QByteArray arguments = isMasterEngine() ? startParameters().processArgs.toLocal8Bit() : masterEngine()->startParameters().processArgs.toLocal8Bit();
+        command += " " + arguments;
+
+        postCommand(command, GdbEngine::RunRequest, CB(handleExecRun));
     } else {
         notifyEngineRunAndInferiorStopOk();
         continueInferiorInternal();
@@ -418,7 +421,7 @@ void GdbRemoteServerEngine::handleExecRun(const GdbResponse &response)
         showMessage(_("INFERIOR STARTED"));
         showMessage(msgInferiorSetupOk(), StatusBar);
     } else {
-        QString msg = QString::fromLocal8Bit(response.data.findChild("msg").data());
+        QString msg = QString::fromLocal8Bit(response.data["msg"].data());
         showMessage(msg);
         notifyEngineRunFailed();
     }
