@@ -38,7 +38,6 @@
 #include "qmllivetextpreview.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/icore.h>
 #include <qmldebug/declarativeenginedebugclient.h>
 #include <qmldebug/declarativeenginedebugclientv2.h>
@@ -50,6 +49,8 @@
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
 
+#include <QFileInfo>
+
 using namespace QmlDebug;
 
 namespace Debugger {
@@ -471,17 +472,7 @@ void QmlInspectorAdapter::jumpToObjectDefinitionInEditor(
 {
     const QString fileName = m_engine->toFileInProject(objSource.url());
 
-    Core::EditorManager *editorManager = Core::EditorManager::instance();
-    Core::IEditor *editor = editorManager->openEditor(fileName);
-    TextEditor::ITextEditor *textEditor
-            = qobject_cast<TextEditor::ITextEditor*>(editor);
-
-    if (textEditor) {
-        editorManager->addCurrentPositionToNavigationHistory();
-        textEditor->gotoLine(objSource.lineNumber());
-        textEditor->widget()->setFocus();
-    }
-
+    Core::EditorManager::openEditorAt(fileName, objSource.lineNumber());
     if (debugId != -1 && debugId != m_currentSelectedDebugId) {
         m_currentSelectedDebugId = debugId;
         m_currentSelectedDebugName = agent()->displayName(debugId);
@@ -503,8 +494,8 @@ void QmlInspectorAdapter::selectObject(const ObjectReference &obj,
 
 void QmlInspectorAdapter::deletePreviews()
 {
-    foreach (const QString &key, m_textPreviews.keys())
-        delete m_textPreviews.take(key);
+    qDeleteAll(m_textPreviews);
+    m_textPreviews.clear();
 }
 
 void QmlInspectorAdapter::enableTools(const bool enable)
