@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -30,12 +30,10 @@
 #ifndef QMLINSPECTORAGENT_H
 #define QMLINSPECTORAGENT_H
 
-#include <QObject>
 #include <QStack>
 #include <QTimer>
 
 #include <qmldebug/baseenginedebugclient.h>
-#include <watchdata.h>
 
 namespace Debugger {
 
@@ -145,7 +143,6 @@ private:
     QList<int> m_objectWatches;
     QList<int> m_fetchDataIds;
     QTimer m_delayQueryTimer;
-    bool m_newObjectsCreated;
 };
 
 } // Internal
