@@ -32,24 +32,20 @@
 #include "debuggerstringutils.h"
 
 #include "watchhandler.h"
-#include "breakpoint.h"
 #include "breakhandler.h"
 #include "qmlengine.h"
 #include "stackhandler.h"
-#include "qtmessageloghandler.h"
+#include "debuggercore.h"
+#include "debuggeractions.h"
 
 #include <utils/qtcassert.h>
-#include <coreplugin/editormanager/editormanager.h>
 #include <texteditor/basetexteditor.h>
 
+#include <qmljs/consolemanagerinterface.h>
+
 #include <QTextBlock>
-#include <QVariant>
-#include <QStack>
-#include <QQueue>
 #include <QFileInfo>
-#include <QTextDocument>
 #include <QScriptEngine>
-#include <QScriptValue>
 
 #define DEBUG_QML 0
 #if DEBUG_QML
@@ -116,19 +112,14 @@ public:
     void version();
     //void profile(ProfileCommand command); //NOT SUPPORTED
     void gc();
-
-    QmlV8ObjectData extractData(const QVariant &data, const QVariant &refsVal);
     void clearCache();
 
     void logSendMessage(const QString &msg) const;
     void logReceiveMessage(const QString &msg) const;
 
-    QtMessageLogItem *constructLogItemTree(QtMessageLogItem *parent,
-                                           const QmlV8ObjectData &objectData, const QVariant &refsVal);
 private:
     QByteArray packMessage(const QByteArray &type, const QByteArray &message = QByteArray());
     QScriptValue initObject();
-    QVariant valueFromRef(int handle, const QVariant &refsVal, bool *success);
 
 public:
     QmlV8DebuggerClient *q;
@@ -761,8 +752,23 @@ void QmlV8DebuggerClientPrivate::gc()
     q->sendMessage(packMessage(V8REQUEST, jsonMessage.toString().toUtf8()));
 }
 
-QmlV8ObjectData QmlV8DebuggerClientPrivate::extractData(const QVariant &data,
-                                                        const QVariant &refsVal)
+QVariant valueFromRef(int handle, const QVariant &refsVal, bool *success)
+{
+    *success = false;
+    QVariant variant;
+    const QVariantList refs = refsVal.toList();
+    foreach (const QVariant &ref, refs) {
+        const QVariantMap refData = ref.toMap();
+        if (refData.value(_(HANDLE)).toInt() == handle) {
+            variant = refData;
+            *success = true;
+            break;
+        }
+    }
+    return variant;
+}
+
+QmlV8ObjectData extractData(const QVariant &data, const QVariant &refsVal)
 {
     //    { "handle" : <handle>,
     //      "type"   : <"undefined", "null", "boolean", "number", "string", "object", "function" or "frame">
@@ -900,24 +906,6 @@ QScriptValue QmlV8DebuggerClientPrivate::initObject()
     return jsonVal;
 }
 
-QVariant QmlV8DebuggerClientPrivate::valueFromRef(int handle,
-                                           const QVariant &refsVal,
-                                           bool *success)
-{
-    *success = false;
-    QVariant variant;
-    const QVariantList refs = refsVal.toList();
-    foreach (const QVariant &ref, refs) {
-        const QVariantMap refData = ref.toMap();
-        if (refData.value(_(HANDLE)).toInt() == handle) {
-            variant = refData;
-            *success = true;
-            break;
-        }
-    }
-    return variant;
-}
-
 void QmlV8DebuggerClientPrivate::logSendMessage(const QString &msg) const
 {
     if (engine)
@@ -930,34 +918,6 @@ void QmlV8DebuggerClientPrivate::logReceiveMessage(const QString &msg) const
         engine->logMessage(QLatin1String("V8DebuggerClient"), QmlEngine::LogReceive, msg);
 }
 
-QtMessageLogItem *QmlV8DebuggerClientPrivate::constructLogItemTree(
-        QtMessageLogItem *parent,
-        const QmlV8ObjectData &objectData,
-        const QVariant &refsVal)
-{
-    if (!objectData.value.isValid())
-        return 0;
-
-    QString text;
-    if (objectData.name.isEmpty())
-        text = objectData.value.toString();
-    else
-        text = QString(_("%1: %2")).arg(QString::fromAscii(objectData.name))
-                .arg(objectData.value.toString());
-
-    QtMessageLogItem *item = new QtMessageLogItem(parent,
-                                                  QtMessageLogHandler::UndefinedType, text);
-
-    foreach (const QVariant &property, objectData.properties) {
-        QtMessageLogItem *child = constructLogItemTree(
-                    item, extractData(property, refsVal), refsVal);
-        if (child)
-            item->insertChild(child);
-    }
-
-    return item;
-}
-
 ///////////////////////////////////////////////////////////////////////
 //
 // QmlV8DebuggerClient
@@ -989,6 +949,11 @@ void QmlV8DebuggerClient::endSession()
     d->disconnect();
 }
 
+void QmlV8DebuggerClient::resetSession()
+{
+    clearExceptionSelection();
+}
+
 void QmlV8DebuggerClient::executeStep()
 {
     clearExceptionSelection();
@@ -1125,9 +1090,8 @@ void QmlV8DebuggerClient::assignValueInDebugger(const WatchData * /*data*/,
         d->evaluate(expression, false, false, stackHandler->currentIndex());
         d->updateLocalsAndWatchers.append(d->sequence);
     } else {
-        d->engine->showMessage(QString(_("Cannot evaluate"
-                                         "%1 in current stack frame")).
-                               arg(expression), QtMessageLogOutput);
+        d->engine->showMessage(QString(_("Cannot evaluate %1 in current stack frame")).arg(
+                                   expression), ConsoleOutput);
     }
 }
 
@@ -1144,9 +1108,8 @@ void QmlV8DebuggerClient::executeDebuggerCommand(const QString &command)
         d->debuggerCommands.append(d->sequence);
     } else {
         //Currently cannot evaluate if not in a javascript break
-        d->engine->showMessage(QString(_("Cannot evaluate %1"
-                                         "in current stack frame")).
-                               arg(command), QtMessageLogOutput);
+        d->engine->showMessage(QString(_("Cannot evaluate %1 in current stack frame")).arg(
+                                   command), ConsoleOutput);
     }
 }
 
@@ -1241,14 +1204,12 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
                     //do nothing, wait for next break
 
                 } else if (debugCommand == _(BACKTRACE)) {
-                    if (success) {
+                    if (success)
                         updateStack(resp.value(_(BODY)), resp.value(_(REFS)));
-                    }
 
                 } else if (debugCommand == _(LOOKUP)) {
-                    if (success) {
+                    if (success)
                         expandLocalsAndWatchers(resp.value(_(BODY)), resp.value(_(REFS)));
-                    }
 
                 } else if (debugCommand == _(EVALUATE)) {
                     int seq = resp.value(_("request_seq")).toInt();
@@ -1262,9 +1223,8 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
                     }
 
                 } else if (debugCommand == _(LISTBREAKPOINTS)) {
-                    if (success) {
+                    if (success)
                         updateBreakpoints(resp.value(_(BODY)));
-                    }
 
                 } else if (debugCommand == _(SETBREAKPOINT)) {
                     //                { "seq"         : <number>,
@@ -1330,14 +1290,12 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
 
 
                 } else if (debugCommand == _(FRAME)) {
-                    if (success) {
+                    if (success)
                         setCurrentFrameDetails(resp.value(_(BODY)), resp.value(_(REFS)));
-                    }
 
                 } else if (debugCommand == _(SCOPE)) {
-                    if (success) {
+                    if (success)
                         updateScope(resp.value(_(BODY)), resp.value(_(REFS)));
-                    }
 
                 } else if (debugCommand == _(SCOPES)) {
                 } else if (debugCommand == _(SOURCE)) {
@@ -1429,13 +1387,7 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
                         BreakHandler *handler = d->engine->breakHandler();
 
                         foreach (int v8Id, v8BreakpointIds) {
-                            BreakpointModelId internalId;
-                            foreach (const BreakpointModelId &id, d->breakpoints.keys()) {
-                                if (d->breakpoints.value(id) == v8Id) {
-                                    internalId = id;
-                                    break;
-                                }
-                            }
+                            const BreakpointModelId internalId = d->breakpoints.key(v8Id);
 
                             if (internalId.isValid()) {
                                 const BreakpointParameters &params = handler->breakpointData(internalId);
@@ -1465,19 +1417,18 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
                         //Update breakpoint data
                         BreakHandler *handler = d->engine->breakHandler();
                         foreach (int v8Id, v8BreakpointIds) {
-                            foreach (const BreakpointModelId &id, d->breakpoints.keys()) {
-                                if (d->breakpoints.value(id) == v8Id) {
-                                    BreakpointResponse br = handler->response(id);
-                                    if (br.functionName.isEmpty()) {
-                                        br.functionName = invocationText;
-                                        handler->setResponse(id, br);
-                                    }
-                                    if (handler->state(id) != BreakpointInserted) {
-                                        br.lineNumber = breakData.value(
-                                                    _("sourceLine")).toInt() + 1;
-                                        handler->setResponse(id, br);
-                                        handler->notifyBreakpointInsertOk(id);
-                                    }
+                            const BreakpointModelId id = d->breakpoints.key(v8Id);
+                            if (id.isValid()) {
+                                BreakpointResponse br = handler->response(id);
+                                if (br.functionName.isEmpty()) {
+                                    br.functionName = invocationText;
+                                    handler->setResponse(id, br);
+                                }
+                                if (handler->state(id) != BreakpointInserted) {
+                                    br.lineNumber = breakData.value(
+                                                _("sourceLine")).toInt() + 1;
+                                    handler->setResponse(id, br);
+                                    handler->notifyBreakpointInsertOk(id);
                                 }
                             }
                         }
@@ -1512,9 +1463,8 @@ void QmlV8DebuggerClient::messageReceived(const QByteArray &data)
                         d->backtrace();
                     }
 
-                    if (d->engine->state() == InferiorStopOk) {
+                    if (d->engine->state() == InferiorStopOk)
                         d->backtrace();
-                    }
 
                 } else if (eventType == _("afterCompile")) {
                     //Currently break point relocation is disabled.
@@ -1630,17 +1580,17 @@ StackFrame QmlV8DebuggerClient::extractStackFrame(const QVariant &bodyVal, const
         return stackFrame;
     }
 
-    QmlV8ObjectData objectData = d->extractData(body.value(_("func")), refsVal);
+    QmlV8ObjectData objectData = extractData(body.value(_("func")), refsVal);
     QString functionName = objectData.value.toString();
     if (functionName.isEmpty())
         functionName = tr("Anonymous Function");
     stackFrame.function = functionName;
 
-    objectData = d->extractData(body.value(_("script")), refsVal);
+    objectData = extractData(body.value(_("script")), refsVal);
     stackFrame.file = d->engine->toFileInProject(objectData.value.toString());
     stackFrame.usable = QFileInfo(stackFrame.file).isReadable();
 
-    objectData = d->extractData(body.value(_("receiver")), refsVal);
+    objectData = extractData(body.value(_("receiver")), refsVal);
     stackFrame.to = objectData.value.toString();
 
     stackFrame.line = body.value(_("line")).toInt() + 1;
@@ -1687,6 +1637,15 @@ void QmlV8DebuggerClient::setCurrentFrameDetails(const QVariant &bodyVal, const
     d->clearCache();
 
     const int frameIndex = stackHandler->currentIndex();
+    QSet<QByteArray> expandedInames = watchHandler->expandedINames();
+    QHash<quint64, QByteArray> handlesToLookup;
+    // Store handles of all expanded watch data
+    foreach (const QByteArray &iname, expandedInames) {
+        const WatchData *wd = watchHandler->findData(iname);
+        if (!wd || !wd->isLocal())
+            continue;
+        handlesToLookup.insert(wd->id, iname);
+    }
     watchHandler->removeAllData();
     if (frameIndex < 0)
         return;
@@ -1700,8 +1659,7 @@ void QmlV8DebuggerClient::setCurrentFrameDetails(const QVariant &bodyVal, const
         data.exp = QByteArray("this");
         data.name = QLatin1String(data.exp);
         data.iname = QByteArray("local.") + data.exp;
-        QmlV8ObjectData objectData = d->extractData(
-                    currentFrame.value(_("receiver")), refsVal);
+        QmlV8ObjectData objectData = extractData(currentFrame.value(_("receiver")), refsVal);
         data.id = objectData.handle;
         data.type = objectData.type;
         data.value = objectData.value.toString();
@@ -1726,6 +1684,11 @@ void QmlV8DebuggerClient::setCurrentFrameDetails(const QVariant &bodyVal, const
         d->scope(scopeIndex);
     }
     d->engine->gotoLocation(stackHandler->currentFrame());
+
+    // Expand watch data that were previously expanded
+    QHash<quint64, QByteArray>::const_iterator itEnd = handlesToLookup.end();
+    for (QHash<quint64, QByteArray>::const_iterator it = handlesToLookup.begin(); it != itEnd; ++it)
+        expandObject(it.value(), it.key());
     emit stackFrameCompleted();
 }
 
@@ -1759,12 +1722,12 @@ void QmlV8DebuggerClient::updateScope(const QVariant &bodyVal, const QVariant &r
     if (bodyMap.value(_("frameIndex")).toInt() != stackHandler->currentIndex())
         return;
 
-    QmlV8ObjectData objectData = d->extractData(bodyMap.value(_("object")), refsVal);
+    QmlV8ObjectData objectData = extractData(bodyMap.value(_("object")), refsVal);
 
     QList<int> handlesToLookup;
     QList<WatchData> locals;
     foreach (const QVariant &property, objectData.properties) {
-        QmlV8ObjectData localData = d->extractData(property, refsVal);
+        QmlV8ObjectData localData = extractData(property, refsVal);
         WatchData data;
         data.exp = localData.name;
         //Check for v8 specific local data
@@ -1794,8 +1757,44 @@ void QmlV8DebuggerClient::updateScope(const QVariant &bodyVal, const QVariant &r
         d->engine->watchHandler()->insertData(locals);
 }
 
-void QmlV8DebuggerClient::updateEvaluationResult(int sequence, bool success, const QVariant &bodyVal,
+QmlJS::ConsoleItem *constructLogItemTree(QmlJS::ConsoleItem *parent,
+                                                 const QmlV8ObjectData &objectData,
                                                  const QVariant &refsVal)
+{
+    using namespace QmlJS;
+    bool sorted = debuggerCore()->boolSetting(SortStructMembers);
+    if (!objectData.value.isValid())
+        return 0;
+
+    QString text;
+    if (objectData.name.isEmpty())
+        text = objectData.value.toString();
+    else
+        text = QString(_("%1: %2")).arg(QString::fromLatin1(objectData.name))
+                .arg(objectData.value.toString());
+
+    ConsoleItem *item = new ConsoleItem(parent, ConsoleItem::UndefinedType, text);
+
+    QSet<QString> childrenFetched;
+    foreach (const QVariant &property, objectData.properties) {
+        const QmlV8ObjectData childObjectData = extractData(property, refsVal);
+        if (childObjectData.handle == objectData.handle)
+            continue;
+        ConsoleItem *child = constructLogItemTree(item, childObjectData, refsVal);
+        if (child) {
+            const QString text = child->text();
+            if (childrenFetched.contains(text))
+                continue;
+            childrenFetched.insert(text);
+            item->insertChild(child, sorted);
+        }
+    }
+
+    return item;
+}
+
+void QmlV8DebuggerClient::updateEvaluationResult(int sequence, bool success,
+                                                 const QVariant &bodyVal, const QVariant &refsVal)
 {
     //    { "seq"         : <number>,
     //      "type"        : "response",
@@ -1819,17 +1818,20 @@ void QmlV8DebuggerClient::updateEvaluationResult(int sequence, bool success, con
 
     } else if (d->debuggerCommands.contains(sequence)) {
         d->updateLocalsAndWatchers.removeOne(sequence);
-        QmlV8ObjectData body = d->extractData(bodyVal, refsVal);
-        QtMessageLogItem *item = d->constructLogItemTree(d->engine->qtMessageLogHandler()->root(),
-                                                         body, refsVal);
-        if (item)
-            d->engine->qtMessageLogHandler()->appendItem(item);
+        QmlV8ObjectData body = extractData(bodyVal, refsVal);
+        using namespace QmlJS;
+        ConsoleManagerInterface *consoleManager = ConsoleManagerInterface::instance();
+        if (consoleManager) {
+            ConsoleItem *item = constructLogItemTree(consoleManager->rootItem(), body, refsVal);
+            if (item)
+                consoleManager->printToConsolePane(item);
+        }
         //Update the locals
         foreach (int index, d->currentFrameScopes)
             d->scope(index);
 
     } else {
-        QmlV8ObjectData body = d->extractData(bodyVal, refsVal);
+        QmlV8ObjectData body = extractData(bodyVal, refsVal);
         if (d->evaluatingExpression.contains(sequence)) {
             QString exp =  d->evaluatingExpression.take(sequence);
             QList<WatchData> watchDataList;
@@ -1928,7 +1930,7 @@ void QmlV8DebuggerClient::expandLocalsAndWatchers(const QVariant &bodyVal, const
     QStringList handlesList = body.keys();
     WatchHandler *watchHandler = d->engine->watchHandler();
     foreach (const QString &handle, handlesList) {
-        QmlV8ObjectData bodyObjectData = d->extractData(
+        QmlV8ObjectData bodyObjectData = extractData(
                     body.value(handle), refsVal);
         QByteArray prepend = d->localsAndWatchers.take(handle.toInt());
 
@@ -1965,7 +1967,7 @@ QList<WatchData> QmlV8DebuggerClient::createWatchDataList(const WatchData *paren
     if (properties.count()) {
         QTC_ASSERT(parent, return watchDataList);
         foreach (const QVariant &property, properties) {
-            QmlV8ObjectData propertyData = d->extractData(property, refsVal);
+            QmlV8ObjectData propertyData = extractData(property, refsVal);
             WatchData data;
             data.name = QString::fromUtf8(propertyData.name);
 
@@ -2031,7 +2033,7 @@ void QmlV8DebuggerClient::highlightExceptionCode(int lineNumber,
 
             QString message = QString(_("%1: %2: %3")).arg(filePath).arg(lineNumber)
                     .arg(errorMessage);
-            d->engine->showMessage(message, QtMessageLogOutput);
+            d->engine->showMessage(message, ConsoleOutput);
         }
     }
 }
