@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -32,34 +32,30 @@
 #include "breakhandler.h"
 #include "registerhandler.h"
 #include "debuggeractions.h"
-#include "debuggerconstants.h"
 #include "debuggerinternalconstants.h"
 #include "debuggercore.h"
 #include "debuggerdialogs.h"
 #include "debuggerengine.h"
-#include "debuggerstartparameters.h"
 #include "watchdelegatewidgets.h"
 #include "watchhandler.h"
 #include "debuggertooltipmanager.h"
 #include "memoryagent.h"
 
+#include <texteditor/syntaxhighlighter.h>
+
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
+#include <utils/fancylineedit.h>
 
 #include <QDebug>
-#include <QMetaObject>
 #include <QMetaProperty>
-#include <QVariant>
 #include <QMimeData>
 
 #include <QApplication>
-#include <QPalette>
 #include <QClipboard>
-#include <QContextMenuEvent>
 #include <QHeaderView>
 #include <QItemDelegate>
 #include <QMenu>
-#include <QPainter>
 #include <QInputDialog>
 #include <QMessageBox>
 
@@ -107,8 +103,9 @@ public:
         }
 
         // Standard line edits for the rest.
-        QLineEdit *lineEdit = new QLineEdit(parent);
+        Utils::FancyLineEdit *lineEdit = new Utils::FancyLineEdit(parent);
         lineEdit->setFrame(false);
+        lineEdit->setHistoryCompleter(QLatin1String("WatchItems"));
         return lineEdit;
     }
 
@@ -146,12 +143,12 @@ private:
 // Watch model query helpers.
 static inline quint64 addressOf(const QModelIndex &m)
 {
-    return m.data(LocalsAddressRole).toULongLong();
+    return m.data(LocalsObjectAddressRole).toULongLong();
 }
 
-static inline quint64 pointerValueOf(const QModelIndex &m)
+static inline quint64 pointerAddressOf(const QModelIndex &m)
 {
-    return m.data(LocalsPointerValueRole).toULongLong();
+    return m.data(LocalsPointerAddressRole).toULongLong();
 }
 
 static inline QString nameOf(const QModelIndex &m)
@@ -237,9 +234,9 @@ static int memberVariableRecursion(const QAbstractItemModel *model,
     return childCount;
 }
 
-/*!
-    \fn variableMemoryMarkup()
+typedef QList<MemoryMarkup> MemoryMarkupList;
 
+/*!
     \brief Creates markup for a variable in the memory view.
 
     Marks the visible children with alternating colors in the parent, that is, for
@@ -277,9 +274,6 @@ static int memberVariableRecursion(const QAbstractItemModel *model,
 
     \sa Debugger::Internal::MemoryViewWidget
 */
-
-typedef QList<MemoryMarkup> MemoryMarkupList;
-
 static MemoryMarkupList
     variableMemoryMarkup(const QAbstractItemModel *model,
                          const QModelIndex &modelIndex,
@@ -301,6 +295,7 @@ static MemoryMarkupList
     MemoryMarkupList result;
     int colorNumber = 0;
     ColorNumberToolTips ranges(size, ColorNumberToolTip(colorNumber, rootToolTip));
+    colorNumber++;
     const int childCount = memberVariableRecursion(model, modelIndex,
                                                    rootName, address, address + size,
                                                    &colorNumber, &ranges);
@@ -331,30 +326,19 @@ static MemoryMarkupList
             }
     }
 
-    // Condense ranges of identical color into markup ranges. Assign member colors
-    // interchangeably.
-    const QColor baseColor = sizeIsEstimate ? defaultBackground : Qt::lightGray;
+    // Assign colors from a list, use base color for 0 (contrast to black text).
+    // Overwrite the first color (which is usually very bright) by the base color.
+    QList<QColor> colors = TextEditor::SyntaxHighlighter::generateColors(colorNumber + 2,
+                                                                         QColor(Qt::black));
+    colors[0] = sizeIsEstimate ? defaultBackground : Qt::lightGray;
     const QColor registerColor = Qt::green;
-    QColor memberColor1 = QColor(Qt::yellow).lighter();
-    QColor memberColor2 = QColor(Qt::cyan).lighter();
-
     int lastColorNumber = 0;
-    int childNumber = 0;
     for (unsigned i = 0; i < size; ++i) {
         const ColorNumberToolTip &range = ranges.at(i);
         if (result.isEmpty() || lastColorNumber != range.first) {
             lastColorNumber = range.first;
-            // Base colors: Parent/register
-            QColor color = range.first == registerColorNumber ? registerColor : baseColor;
-            if (range.first && range.first != registerColorNumber) {
-                if (childNumber++ & 1) { // Alternating member colors.
-                    color = memberColor1;
-                    memberColor1 = memberColor1.darker(120);
-                } else {
-                    color = memberColor2;
-                    memberColor2 = memberColor2.darker(120);
-                }
-            } // color switch
+            const QColor color = range.first == registerColorNumber ?
+                         registerColor : colors.at(range.first);
             result.push_back(MemoryMarkup(address + i, 1, color, range.second));
         } else {
             result.back().length++;
@@ -381,15 +365,15 @@ static MemoryMarkupList
 
 // Convenience to create a memory view of a variable.
 static void addVariableMemoryView(DebuggerEngine *engine, bool separateView,
-    const QModelIndex &m, bool deferencePointer,
+    const QModelIndex &m, bool atPointerAddress,
     const QPoint &p, QWidget *parent)
 {
     const QColor background = parent->palette().color(QPalette::Normal, QPalette::Base);
-    const quint64 address = deferencePointer ? pointerValueOf(m) : addressOf(m);
+    const quint64 address = atPointerAddress ? pointerAddressOf(m) : addressOf(m);
     // Fixme: Get the size of pointee (see variableMemoryMarkup())?
     const QString rootToolTip = variableToolTip(nameOf(m), typeOf(m), 0);
     const quint64 typeSize = sizeOf(m);
-    const bool sizeIsEstimate = deferencePointer || !typeSize;
+    const bool sizeIsEstimate = atPointerAddress || !typeSize;
     const quint64 size    = sizeIsEstimate ? 1024 : typeSize;
     if (!address)
          return;
@@ -400,10 +384,10 @@ static void addVariableMemoryView(DebuggerEngine *engine, bool separateView,
                              sizeIsEstimate, background);
     const unsigned flags = separateView
         ? DebuggerEngine::MemoryView|DebuggerEngine::MemoryReadOnly : 0;
-    const QString title = deferencePointer
-        ?  WatchTreeView::tr("Memory Referenced by Pointer \"%1\" (0x%2)")
+    const QString title = atPointerAddress
+        ?  WatchTreeView::tr("Memory at Pointer's Address \"%1\" (0x%2)")
                 .arg(nameOf(m)).arg(address, 0, 16)
-        : WatchTreeView::tr("Memory at Variable \"%1\" (0x%2)")
+        : WatchTreeView::tr("Memory at Object's Address \"%1\" (0x%2)")
                 .arg(nameOf(m)).arg(address, 0, 16);
     engine->openMemoryView(address, flags, markup, p, title, parent);
 }
@@ -413,7 +397,6 @@ static void addVariableMemoryView(DebuggerEngine *engine, bool separateView,
 static void addStackLayoutMemoryView(DebuggerEngine *engine, bool separateView,
     const QAbstractItemModel *m, const QPoint &p, QWidget *parent)
 {
-    typedef QPair<quint64, QString> RegisterValueNamePair;
     QTC_ASSERT(engine && m, return);
 
     // Determine suitable address range from locals.
@@ -426,7 +409,8 @@ static void addStackLayoutMemoryView(DebuggerEngine *engine, bool separateView,
     // pointer' items as they are outside the address range.
     for (int r = 0; r < localsItemCount; r++) {
         const QModelIndex idx = localsIndex.child(r, 0);
-        if (idx.data(LocalsReferencingAddressRole).toULongLong() == 0) {
+        const quint64 pointerAddress = pointerAddressOf(idx);
+        if (pointerAddress == 0) {
             const quint64 address = addressOf(idx);
             if (address) {
                 if (address < start)
@@ -452,11 +436,10 @@ static void addStackLayoutMemoryView(DebuggerEngine *engine, bool separateView,
     const RegisterMapConstIt regcEnd = regMap.constEnd();
     for (RegisterMapConstIt it = regMap.constBegin(); it != regcEnd; ++it) {
         const quint64 value = it.key();
-        if (value < start && start - value < 512) {
+        if (value < start && start - value < 512)
             start = value;
-        } else if (value > end && value - end < 512) {
+        else if (value > end && value - end < 512)
             end = value + 1;
-        }
     }
     // Indicate all variables.
     const QColor background = parent->palette().color(QPalette::Normal, QPalette::Base);
@@ -511,11 +494,8 @@ void WatchTreeView::collapseNode(const QModelIndex &idx)
 void WatchTreeView::keyPressEvent(QKeyEvent *ev)
 {
     if (ev->key() == Qt::Key_Delete && m_type == WatchersType) {
-        QModelIndexList indices = selectionModel()->selectedRows();
-        if (indices.isEmpty() && selectionModel()->currentIndex().isValid())
-            indices.append(selectionModel()->currentIndex());
         WatchHandler *handler = currentEngine()->watchHandler();
-        foreach (const QModelIndex &idx, indices)
+        foreach (const QModelIndex &idx, activeRows())
             handler->removeData(idx.data(LocalsINameRole).toByteArray());
     } else if (ev->key() == Qt::Key_Return
             && ev->modifiers() == Qt::ControlModifier
@@ -593,7 +573,8 @@ static QString removeWatchActionText(QString exp)
         exp.truncate(30);
         exp.append(QLatin1String("..."));
     }
-    return WatchTreeView::tr("Remove Expression Evaluator for \"%1\"").arg(exp);
+    return WatchTreeView::tr("Remove Expression Evaluator for \"%1\"")
+        .arg(exp.replace(QLatin1Char('&'), QLatin1String("&&")));
 }
 
 static void copyToClipboard(const QString &clipboardText)
@@ -608,24 +589,27 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
     DebuggerEngine *engine = currentEngine();
     WatchHandler *handler = engine->watchHandler();
 
+    const QModelIndexList active = activeRows();
     const QModelIndex idx = indexAt(ev->pos());
     const QModelIndex mi0 = idx.sibling(idx.row(), 0);
     const QModelIndex mi1 = idx.sibling(idx.row(), 1);
     const QModelIndex mi2 = idx.sibling(idx.row(), 2);
     const quint64 address = addressOf(mi0);
     const uint size = sizeOf(mi0);
-    const quint64 pointerValue = pointerValueOf(mi0);
+    const quint64 pointerAddress = pointerAddressOf(mi0);
     const QString exp = mi0.data(LocalsExpressionRole).toString();
     const QString name = mi0.data(LocalsNameRole).toString();
     const QString type = mi2.data().toString();
 
     // Offer to open address pointed to or variable address.
-    const bool createPointerActions = pointerValue && pointerValue != address;
+    const bool createPointerActions = pointerAddress && pointerAddress != address;
 
     const QStringList alternativeFormats =
         mi0.data(LocalsTypeFormatListRole).toStringList();
-    const int typeFormat =
+    int typeFormat =
         mi0.data(LocalsTypeFormatRole).toInt();
+    if (typeFormat >= alternativeFormats.size())
+        typeFormat = -1;
     const int individualFormat =
         mi0.data(LocalsIndividualFormatRole).toInt();
     const int unprintableBase = handler->unprintableBase();
@@ -713,29 +697,29 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
 
     QMenu breakpointMenu;
     breakpointMenu.setTitle(tr("Add Data Breakpoint..."));
-    QAction *actSetWatchpointAtVariableAddress = 0;
-    QAction *actSetWatchpointAtPointerValue = 0;
+    QAction *actSetWatchpointAtObjectAddress = 0;
+    QAction *actSetWatchpointAtPointerAddress = 0;
     const bool canSetWatchpoint = engine->hasCapability(WatchpointByAddressCapability);
     if (canSetWatchpoint && address) {
-        actSetWatchpointAtVariableAddress =
+        actSetWatchpointAtObjectAddress =
             new QAction(tr("Add Data Breakpoint at Object's Address (0x%1)")
                 .arg(address, 0, 16), &breakpointMenu);
-        actSetWatchpointAtVariableAddress->
-            setChecked(mi0.data(LocalsIsWatchpointAtAddressRole).toBool());
+        actSetWatchpointAtObjectAddress->
+            setChecked(mi0.data(LocalsIsWatchpointAtObjectAddressRole).toBool());
         if (createPointerActions) {
-            actSetWatchpointAtPointerValue =
-                new QAction(tr("Add Data Breakpoint at Referenced Address (0x%1)")
-                    .arg(pointerValue, 0, 16), &breakpointMenu);
-            actSetWatchpointAtPointerValue->setCheckable(true);
-            actSetWatchpointAtPointerValue->
-                setChecked(mi0.data(LocalsIsWatchpointAtPointerValueRole).toBool());
+            actSetWatchpointAtPointerAddress =
+                new QAction(tr("Add Data Breakpoint at Pointer's Address (0x%1)")
+                    .arg(pointerAddress, 0, 16), &breakpointMenu);
+            actSetWatchpointAtPointerAddress->setCheckable(true);
+            actSetWatchpointAtPointerAddress->
+                setChecked(mi0.data(LocalsIsWatchpointAtPointerAddressRole).toBool());
         }
     } else {
-        actSetWatchpointAtVariableAddress =
+        actSetWatchpointAtObjectAddress =
             new QAction(tr("Add Data Breakpoint"), &breakpointMenu);
-        actSetWatchpointAtVariableAddress->setEnabled(false);
+        actSetWatchpointAtObjectAddress->setEnabled(false);
     }
-    actSetWatchpointAtVariableAddress->setToolTip(
+    actSetWatchpointAtObjectAddress->setToolTip(
         tr("Setting a data breakpoint on an address will cause the program "
            "to stop when the data at the address is modified."));
 
@@ -756,9 +740,9 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
            "to stop when the data at the address given by the expression "
            "is modified."));
 
-    breakpointMenu.addAction(actSetWatchpointAtVariableAddress);
-    if (actSetWatchpointAtPointerValue)
-        breakpointMenu.addAction(actSetWatchpointAtPointerValue);
+    breakpointMenu.addAction(actSetWatchpointAtObjectAddress);
+    if (actSetWatchpointAtPointerAddress)
+        breakpointMenu.addAction(actSetWatchpointAtPointerAddress);
     breakpointMenu.addAction(actSetWatchpointAtExpression);
 
     QMenu menu;
@@ -800,51 +784,51 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
 
     QMenu memoryMenu;
     memoryMenu.setTitle(tr("Open Memory Editor..."));
-    QAction *actOpenMemoryEditAtVariableAddress = new QAction(&memoryMenu);
-    QAction *actOpenMemoryEditAtPointerValue = new QAction(&memoryMenu);
+    QAction *actOpenMemoryEditAtObjectAddress = new QAction(&memoryMenu);
+    QAction *actOpenMemoryEditAtPointerAddress = new QAction(&memoryMenu);
     QAction *actOpenMemoryEditor = new QAction(&memoryMenu);
     QAction *actOpenMemoryEditorStackLayout = new QAction(&memoryMenu);
-    QAction *actOpenMemoryViewAtVariableAddress = new QAction(&memoryMenu);
-    QAction *actOpenMemoryViewAtPointerValue = new QAction(&memoryMenu);
+    QAction *actOpenMemoryViewAtObjectAddress = new QAction(&memoryMenu);
+    QAction *actOpenMemoryViewAtPointerAddress = new QAction(&memoryMenu);
     if (engine->hasCapability(ShowMemoryCapability)) {
         actOpenMemoryEditor->setText(tr("Open Memory Editor..."));
         if (address) {
-            actOpenMemoryEditAtVariableAddress->setText(
+            actOpenMemoryEditAtObjectAddress->setText(
                 tr("Open Memory Editor at Object's Address (0x%1)")
                     .arg(address, 0, 16));
-            actOpenMemoryViewAtVariableAddress->setText(
+            actOpenMemoryViewAtObjectAddress->setText(
                     tr("Open Memory View at Object's Address (0x%1)")
                         .arg(address, 0, 16));
         } else {
-            actOpenMemoryEditAtVariableAddress->setText(
+            actOpenMemoryEditAtObjectAddress->setText(
                 tr("Open Memory Editor at Object's Address"));
-            actOpenMemoryEditAtVariableAddress->setEnabled(false);
-            actOpenMemoryViewAtVariableAddress->setText(
+            actOpenMemoryEditAtObjectAddress->setEnabled(false);
+            actOpenMemoryViewAtObjectAddress->setText(
                     tr("Open Memory View at Object's Address"));
-            actOpenMemoryViewAtVariableAddress->setEnabled(false);
+            actOpenMemoryViewAtObjectAddress->setEnabled(false);
         }
         if (createPointerActions) {
-            actOpenMemoryEditAtPointerValue->setText(
-                tr("Open Memory Editor at Referenced Address (0x%1)")
-                    .arg(pointerValue, 0, 16));
-            actOpenMemoryViewAtPointerValue->setText(
-                tr("Open Memory View at Referenced Address (0x%1)")
-                    .arg(pointerValue, 0, 16));
+            actOpenMemoryEditAtPointerAddress->setText(
+                tr("Open Memory Editor at Pointer's Address (0x%1)")
+                    .arg(pointerAddress, 0, 16));
+            actOpenMemoryViewAtPointerAddress->setText(
+                tr("Open Memory View at Pointer's Address (0x%1)")
+                    .arg(pointerAddress, 0, 16));
         } else {
-            actOpenMemoryEditAtPointerValue->setText(
-                tr("Open Memory Editor at Referenced Address"));
-            actOpenMemoryEditAtPointerValue->setEnabled(false);
-            actOpenMemoryViewAtPointerValue->setText(
-                tr("Open Memory View at Referenced Address"));
-            actOpenMemoryViewAtPointerValue->setEnabled(false);
+            actOpenMemoryEditAtPointerAddress->setText(
+                tr("Open Memory Editor at Pointer's Address"));
+            actOpenMemoryEditAtPointerAddress->setEnabled(false);
+            actOpenMemoryViewAtPointerAddress->setText(
+                tr("Open Memory View at Pointer's Address"));
+            actOpenMemoryViewAtPointerAddress->setEnabled(false);
         }
         actOpenMemoryEditorStackLayout->setText(
             tr("Open Memory Editor Showing Stack Layout"));
         actOpenMemoryEditorStackLayout->setEnabled(m_type == LocalsType);
-        memoryMenu.addAction(actOpenMemoryViewAtVariableAddress);
-        memoryMenu.addAction(actOpenMemoryViewAtPointerValue);
-        memoryMenu.addAction(actOpenMemoryEditAtVariableAddress);
-        memoryMenu.addAction(actOpenMemoryEditAtPointerValue);
+        memoryMenu.addAction(actOpenMemoryViewAtObjectAddress);
+        memoryMenu.addAction(actOpenMemoryViewAtPointerAddress);
+        memoryMenu.addAction(actOpenMemoryEditAtObjectAddress);
+        memoryMenu.addAction(actOpenMemoryEditAtPointerAddress);
         memoryMenu.addAction(actOpenMemoryEditorStackLayout);
         memoryMenu.addAction(actOpenMemoryEditor);
     } else {
@@ -895,9 +879,9 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
                                    QString(), &ok);
         if (ok && !newExp.isEmpty())
             watchExpression(newExp);
-    } else if (act == actOpenMemoryEditAtVariableAddress) {
+    } else if (act == actOpenMemoryEditAtObjectAddress) {
         addVariableMemoryView(currentEngine(), false, mi0, false, ev->globalPos(), this);
-    } else if (act == actOpenMemoryEditAtPointerValue) {
+    } else if (act == actOpenMemoryEditAtPointerAddress) {
         addVariableMemoryView(currentEngine(), false, mi0, true, ev->globalPos(), this);
     } else if (act == actOpenMemoryEditor) {
         AddressDialog dialog;
@@ -905,16 +889,16 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
             dialog.setAddress(address);
         if (dialog.exec() == QDialog::Accepted)
             currentEngine()->openMemoryView(dialog.address(), false, MemoryMarkupList(), QPoint());
-    } else if (act == actOpenMemoryViewAtVariableAddress) {
+    } else if (act == actOpenMemoryViewAtObjectAddress) {
         addVariableMemoryView(currentEngine(), true, mi0, false, ev->globalPos(), this);
-    } else if (act == actOpenMemoryViewAtPointerValue) {
+    } else if (act == actOpenMemoryViewAtPointerAddress) {
         addVariableMemoryView(currentEngine(), true, mi0, true, ev->globalPos(), this);
     } else if (act == actOpenMemoryEditorStackLayout) {
         addStackLayoutMemoryView(currentEngine(), false, model(), ev->globalPos(), this);
-    } else if (act == actSetWatchpointAtVariableAddress) {
+    } else if (act == actSetWatchpointAtObjectAddress) {
         breakHandler()->setWatchpointAtAddress(address, size);
-    } else if (act == actSetWatchpointAtPointerValue) {
-        breakHandler()->setWatchpointAtAddress(pointerValue, sizeof(void *)); // FIXME: an approximation..
+    } else if (act == actSetWatchpointAtPointerAddress) {
+        breakHandler()->setWatchpointAtAddress(pointerAddress, sizeof(void *)); // FIXME: an approximation..
     } else if (act == actSetWatchpointAtExpression) {
         breakHandler()->setWatchpointAtExpression(name);
     } else if (act == actSelectWidgetToWatch) {
@@ -931,9 +915,11 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
     } else if (act == actRemoveWatches) {
         handler->clearWatches();
     } else if (act == clearTypeFormatAction) {
-        setModelData(LocalsTypeFormatRole, -1, mi1);
+        foreach (const QModelIndex &idx, active)
+            setModelData(LocalsTypeFormatRole, -1, idx);
     } else if (act == clearIndividualFormatAction) {
-        setModelData(LocalsIndividualFormatRole, -1, mi1);
+        foreach (const QModelIndex &idx, active)
+            setModelData(LocalsIndividualFormatRole, -1, idx);
     } else if (act == actShowInEditor) {
         QString contents = handler->editorContents();
         debuggerCore()->openTextEditor(tr("Locals & Expressions"), contents);
@@ -950,13 +936,20 @@ void WatchTreeView::contextMenuEvent(QContextMenuEvent *ev)
     } else if (handleBaseContextAction(act)) {
         ;
     } else {
+        // Restrict multiple changes to items of the same type
+        // to avoid assigning illegal formats.
+        const QVariant currentType = mi1.data(LocalsTypeRole);
         for (int i = 0; i != typeFormatActions.size(); ++i) {
             if (act == typeFormatActions.at(i))
-                setModelData(LocalsTypeFormatRole, i, mi1);
+                foreach (const QModelIndex &idx, active)
+                    if (idx.data(LocalsTypeRole) == currentType)
+                        setModelData(LocalsTypeFormatRole, i, idx);
         }
         for (int i = 0; i != individualFormatActions.size(); ++i) {
             if (act == individualFormatActions.at(i))
-                setModelData(LocalsIndividualFormatRole, i, mi1);
+                foreach (const QModelIndex &idx, active)
+                    if (idx.data(LocalsTypeRole) == currentType)
+                        setModelData(LocalsIndividualFormatRole, i, idx);
         }
     }
 }
@@ -972,6 +965,12 @@ bool WatchTreeView::event(QEvent *ev)
     return BaseTreeView::event(ev);
 }
 
+void WatchTreeView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
+{
+    emit currentIndexChanged(current);
+    BaseTreeView::currentChanged(current, previous);
+}
+
 void WatchTreeView::editItem(const QModelIndex &idx)
 {
     Q_UNUSED(idx) // FIXME
