@@ -29,7 +29,6 @@
 
 #include "formeditorw.h"
 #include "formwindoweditor.h"
-#include "designerconstants.h"
 #include "settingsmanager.h"
 #include "settingspage.h"
 #include "editorwidget.h"
@@ -37,7 +36,6 @@
 #include "qtcreatorintegration.h"
 #include "designerxmleditor.h"
 #include "designercontext.h"
-#include "editorwidget.h"
 #include "resourcehandler.h"
 #include <widgethost.h>
 
@@ -48,14 +46,10 @@
 #include <coreplugin/helpmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
-#include <coreplugin/actionmanager/command.h>
-#include <coreplugin/id.h>
-#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/minisplitter.h>
 #include <coreplugin/mimedatabase.h>
 #include <coreplugin/outputpane.h>
 #include <texteditor/texteditorsettings.h>
-#include <extensionsystem/pluginmanager.h>
 #include <cpptools/cpptoolsconstants.h>
 #include <utils/qtcassert.h>
 
@@ -76,14 +70,12 @@
 #include <abstractobjectinspector.h>
 #include <QDesignerPropertyEditorInterface>
 #include <QDesignerActionEditorInterface>
-#include <QDesignerFormEditorInterface>
 
 #include <QAction>
 #include <QActionGroup>
 #include <QApplication>
 #include <QCursor>
 #include <QDockWidget>
-#include <QMenu>
 #include <QMessageBox>
 #include <QKeySequence>
 #include <QPrintDialog>
@@ -158,7 +150,7 @@ FormEditorW::FormEditorW() :
     qFill(m_designerSubWindows, m_designerSubWindows + Designer::Constants::DesignerSubWindowCount,
           static_cast<QWidget *>(0));
 
-    m_formeditor->setTopLevel(Core::ICore::editorManager());
+    m_formeditor->setTopLevel(Core::ICore::mainWindow());
     m_formeditor->setSettingsManager(new SettingsManager());
 
 #if QT_VERSION >= 0x050000
@@ -561,7 +553,7 @@ void FormEditorW::setupActions()
     addToolAction(actionFormSettings, m_contexts, Core::Id("FormEditor.FormSettings"), mformtools);
 
     mformtools->addSeparator(m_contexts);
-    m_actionAboutPlugins = new QAction(tr("About Qt Designer plugins..."), this);
+    m_actionAboutPlugins = new QAction(tr("About Qt Designer Plugins..."), this);
     m_actionAboutPlugins->setMenuRole(QAction::NoRole);
     addToolAction(m_actionAboutPlugins, m_contexts,
                    Core::Id("FormEditor.AboutPlugins"), mformtools);
@@ -620,7 +612,7 @@ Core::ActionContainer *FormEditorW::createPreviewStyleMenu(QActionGroup *actionG
             name += dot;
         }
         name += data.toString();
-        Core::Command *command = Core::ActionManager::registerAction(a, Core::Id(name), m_contexts);
+        Core::Command *command = Core::ActionManager::registerAction(a, Core::Id::fromString(name), m_contexts);
         bindShortcut(command, a);
         if (isDeviceProfile) {
             command->setAttribute(Core::Command::CA_UpdateText);
@@ -920,7 +912,7 @@ void FormEditorW::switchSourceForm()
 {
     const QString fileToOpen = otherFile();
     if (!fileToOpen.isEmpty())
-        Core::EditorManager::openEditor(fileToOpen, Core::Id(), Core::EditorManager::ModeSwitch);
+        Core::EditorManager::openEditor(fileToOpen);
 }
 
 } // namespace Internal
