@@ -1,67 +1,48 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
-#include "formeditorplugin.h"
+#include "qtcreatorintegration.h"
 #include "formwindoweditor.h"
 #include "formclasswizardpage.h"
-#include "qtcreatorintegration.h"
 #include "formeditorw.h"
 #include "editordata.h"
-#include "codemodelhelpers.h"
 #include <widgethost.h>
 
-#include <cpptools/cpprefactoringchanges.h>
 #include <cpptools/cpptoolsconstants.h>
 #include <cpptools/insertionpointlocator.h>
 #include <cpptools/symbolfinder.h>
-#include <cplusplus/ModelManagerInterface.h>
-#include <cplusplus/Symbols.h>
 #include <cplusplus/Overview.h>
-#include <cplusplus/CoreTypes.h>
-#include <cplusplus/Name.h>
-#include <cplusplus/Names.h>
-#include <cplusplus/Literals.h>
-#include <cplusplus/Scope.h>
-#include <cplusplus/Control.h>
-#include <cplusplus/TranslationUnit.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/mimedatabase.h>
-#include <coreplugin/editormanager/editormanager.h>
-#include <extensionsystem/pluginmanager.h>
 #include <texteditor/basetexteditor.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/session.h>
 #include <utils/qtcassert.h>
-#include <utils/fileutils.h>
 
 #include <QDesignerFormWindowInterface>
 #include <QDesignerFormEditorInterface>
@@ -145,7 +126,7 @@ QWidget *QtCreatorIntegration::containerWindow(QWidget * /*widget*/) const
     return 0;
 }
 
-static QList<Document::Ptr> findDocumentsIncluding(const CPlusPlus::Snapshot &docTable,
+static QList<Document::Ptr> findDocumentsIncluding(const Snapshot &docTable,
                                                    const QString &fileName, bool checkFileNameOnly)
 {
     QList<Document::Ptr> docList;
@@ -195,7 +176,8 @@ static bool matchMemberClassName(const QString &needle, const QString &hayStack)
 // Find class definition in namespace (that is, the outer class
 // containing a member of the desired class type) or inheriting the desired class
 // in case of forms using the Multiple Inheritance approach
-static const Class *findClass(const Namespace *parentNameSpace, const QString &className, QString *namespaceName)
+static const Class *findClass(const Namespace *parentNameSpace,
+                                         const QString &className, QString *namespaceName)
 {
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << className;
@@ -272,14 +254,14 @@ static Function *findDeclaration(const Class *cl, const QString &functionName)
 // TODO: remove me, this is taken from cppeditor.cpp. Find some common place for this method
 static Document::Ptr findDefinition(Function *functionDeclaration, int *line)
 {
-    if (CppModelManagerInterface *cppModelManager = CppModelManagerInterface::instance()) {
+    if (CppTools::CppModelManagerInterface *cppModelManager = CppTools::CppModelManagerInterface::instance()) {
         const Snapshot snapshot = cppModelManager->snapshot();
         CppTools::SymbolFinder symbolFinder;
-        if (Symbol *def = symbolFinder.findMatchingDefinition(functionDeclaration, snapshot)) {
+        if (Function *fun = symbolFinder.findMatchingDefinition(functionDeclaration, snapshot)) {
             if (line)
-                *line = def->line();
+                *line = fun->line();
 
-            return snapshot.document(QString::fromUtf8(def->fileName(), def->fileNameLength()));
+            return snapshot.document(QString::fromUtf8(fun->fileName(), fun->fileNameLength()));
         }
     }
 
@@ -288,7 +270,9 @@ static Document::Ptr findDefinition(Function *functionDeclaration, int *line)
 
 static inline ITextEditor *editableAt(const QString &fileName, int line, int column)
 {
-    return qobject_cast<ITextEditor *>(TextEditor::BaseTextEditorWidget::openEditorAt(fileName, line, column));
+    return qobject_cast<ITextEditor *>(Core::EditorManager::openEditorAt(fileName, line, column,
+                                                                         Core::Id(),
+                                                                         Core::EditorManager::DoNotMakeVisible));
 }
 
 static void addDeclaration(const Snapshot &snapshot,
@@ -323,7 +307,7 @@ static void addDeclaration(const Snapshot &snapshot,
     }
 }
 
-static Document::Ptr addDefinition(const CPlusPlus::Snapshot &docTable,
+static Document::Ptr addDefinition(const Snapshot &docTable,
                                    const QString &headerFileName,
                                    const QString &className,
                                    const QString &functionName,
@@ -358,7 +342,7 @@ static Document::Ptr addDefinition(const CPlusPlus::Snapshot &docTable,
                 //! \todo use the InsertionPointLocator to insert at the correct place.
                 // (we'll have to extend that class first to do definition insertions)
 
-                const QString contents = editable->contents();
+                const QString contents = editable->textDocument()->contents();
                 int column;
                 editable->convertPosition(contents.length(), line, &column);
                 editable->gotoLine(*line, column);
@@ -458,7 +442,7 @@ static QString addParameterNames(const QString &functionSignature, const QString
 typedef QPair<const Class *, Document::Ptr> ClassDocumentPtrPair;
 
 static ClassDocumentPtrPair
-        findClassRecursively(const CPlusPlus::Snapshot &docTable,
+        findClassRecursively(const Snapshot &docTable,
                              const Document::Ptr &doc, const QString &className,
                              unsigned maxIncludeDepth, QString *namespaceName)
 {
@@ -471,7 +455,7 @@ static ClassDocumentPtrPair
         // Check the includes
         const unsigned recursionMaxIncludeDepth = maxIncludeDepth - 1u;
         foreach (const QString &include, doc->includedFiles()) {
-            const CPlusPlus::Snapshot::const_iterator it = docTable.find(include);
+            const Snapshot::const_iterator it = docTable.find(include);
             if (it != docTable.end()) {
                 const Document::Ptr includeDoc = it.value();
                 const ClassDocumentPtrPair irc = findClassRecursively(docTable, it.value(), className, recursionMaxIncludeDepth, namespaceName);
@@ -487,9 +471,8 @@ void QtCreatorIntegration::slotNavigateToSlot(const QString &objectName, const Q
         const QStringList &parameterNames)
 {
     QString errorMessage;
-    if (!navigateToSlot(objectName, signalSignature, parameterNames, &errorMessage) && !errorMessage.isEmpty()) {
+    if (!navigateToSlot(objectName, signalSignature, parameterNames, &errorMessage) && !errorMessage.isEmpty())
         QMessageBox::warning(m_few->designerEditor()->topLevel(), tr("Error finding/adding a slot."), errorMessage);
-    }
 }
 
 // Build name of the class as generated by uic, insert Ui namespace
@@ -503,7 +486,9 @@ static inline QString uiClassName(QString formObjectName)
     return formObjectName;
 }
 
-static Document::Ptr getParsedDocument(const QString &fileName, CppModelManagerInterface::WorkingCopy &workingCopy, Snapshot &snapshot)
+static Document::Ptr getParsedDocument(const QString &fileName,
+                                       CppTools::CppModelManagerInterface::WorkingCopy &workingCopy,
+                                       Snapshot &snapshot)
 {
     QString src;
     if (workingCopy.contains(fileName)) {
@@ -514,9 +499,7 @@ static Document::Ptr getParsedDocument(const QString &fileName, CppModelManagerI
             src = QString::fromLocal8Bit(reader.data()); // ### FIXME encoding
     }
 
-    QByteArray source = snapshot.preprocessedCode(src, fileName);
-
-    Document::Ptr doc = snapshot.documentFromSource(source, fileName);
+    Document::Ptr doc = snapshot.preprocessedDocument(src, fileName);
     doc->check();
     snapshot.insert(doc);
     return doc;
@@ -533,7 +516,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     typedef QMap<int, Document::Ptr> DocumentMap;
 
     const EditorData ed = m_few->activeEditor();
-    QTC_ASSERT(ed, return false)
+    QTC_ASSERT(ed, return false);
     const QString currentUiFile = ed.formWindowEditor->document()->fileName();
 #if 0
     return Designer::Internal::navigateToSlot(currentUiFile, objectName, signalSignature, parameterNames, errorMessage);
@@ -553,10 +536,10 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
         *errorMessage = tr("Internal error: No project could be found for %1.").arg(currentUiFile);
         return false;
     }
-    CPlusPlus::Snapshot docTable = CppModelManagerInterface::instance()->snapshot();
-    CPlusPlus::Snapshot newDocTable;
+    Snapshot docTable = CppTools::CppModelManagerInterface::instance()->snapshot();
+    Snapshot newDocTable;
 
-    for  (CPlusPlus::Snapshot::iterator it = docTable.begin(); it != docTable.end(); ++it) {
+    for (Snapshot::iterator it = docTable.begin(); it != docTable.end(); ++it) {
         const ProjectExplorer::Project *project = ProjectExplorer::ProjectExplorerPlugin::instance()->session()->projectForFile(it.key());
         if (project == uiProject)
             newDocTable.insert(it.value());
@@ -629,8 +612,8 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
         }
     } else {
         // add function declaration to cl
-        CppModelManagerInterface::WorkingCopy workingCopy =
-            CppModelManagerInterface::instance()->workingCopy();
+        CppTools::CppModelManagerInterface::WorkingCopy workingCopy =
+            CppTools::CppModelManagerInterface::instance()->workingCopy();
         const QString fileName = doc->fileName();
         getParsedDocument(fileName, workingCopy, docTable);
         addDeclaration(docTable, fileName, cl, functionNameWithParameterNames);
@@ -645,7 +628,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     }
 
     // jump to function definition, position within code
-    TextEditor::BaseTextEditorWidget::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
+    Core::EditorManager::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
 
     return true;
 }
