@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef FINDTOOLBAR_H
 #define FINDTOOLBAR_H
@@ -38,7 +35,7 @@
 
 #include <utils/styledbar.h>
 
-#include <QtCore/QTimer>
+#include <QTimer>
 
 namespace Core {
 class FindToolBarPlaceHolder;
@@ -60,7 +57,7 @@ public:
     void readSettings();
     void writeSettings();
 
-    void openFindToolBar();
+    void openFindToolBar(bool focus = true);
     void setUseFakeVim(bool on);
 
 public slots:
@@ -84,7 +81,9 @@ private slots:
     void updateFromFindClipboard();
 
     void hideAndResetFocus();
-    void openFind();
+    void openFind(bool focus = true);
+    void findNextSelected();
+    void findPreviousSelected();
     void updateFindAction();
     void updateToolBar();
     void findFlagsChanged();
@@ -92,11 +91,13 @@ private slots:
     void setCaseSensitive(bool sensitive);
     void setWholeWord(bool wholeOnly);
     void setRegularExpressions(bool regexp);
+    void setPreserveCase(bool preserveCase);
 
     void adaptToCandidate();
 
 protected:
     bool focusNextPrevChild(bool next);
+    void keyPressEvent(QKeyEvent *event);
 
 private:
     void installEventFilters();
@@ -115,12 +116,16 @@ private:
     void updateIcons();
     void updateFlagMenus();
 
+    bool shouldSetFocusOnKeyEvent(QKeyEvent *event);
+
     FindPlugin *m_plugin;
     CurrentDocumentFind *m_currentDocumentFind;
     Ui::FindWidget m_ui;
     QCompleter *m_findCompleter;
     QCompleter *m_replaceCompleter;
     QAction *m_findInDocumentAction;
+    QAction *m_findNextSelectedAction;
+    QAction *m_findPreviousSelectedAction;
     QAction *m_enterFindStringAction;
     QAction *m_findNextAction;
     QAction *m_findPreviousAction;
@@ -131,12 +136,9 @@ private:
     QAction *m_caseSensitiveAction;
     QAction *m_wholeWordAction;
     QAction *m_regularExpressionAction;
+    QAction *m_preserveCaseAction;
     Find::FindFlags m_findFlags;
 
-    QPixmap m_casesensitiveIcon;
-    QPixmap m_regexpIcon;
-    QPixmap m_wholewordsIcon;
-
     QTimer m_findIncrementalTimer;
     QTimer m_findStepTimer;
     bool m_useFakeVim;
