@@ -1,58 +1,54 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "searchresultwindow.h"
 #include "searchresultwidget.h"
-#include "findtoolwindow.h"
+#include "searchresultcolor.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icontext.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QFile>
-#include <QtCore/QSettings>
-#include <QtCore/QDebug>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QFont>
-#include <QtGui/QAction>
-#include <QtGui/QComboBox>
-#include <QtGui/QScrollArea>
-#include <QtGui/QStackedWidget>
+#include <QSettings>
+#include <QDebug>
+#include <QFont>
+#include <QAction>
+#include <QToolButton>
+#include <QComboBox>
+#include <QScrollArea>
+#include <QStackedWidget>
 
 static const char SETTINGSKEYSECTIONNAME[] = "SearchResults";
 static const char SETTINGSKEYEXPANDRESULTS[] = "ExpandResults";
+static const int MAX_SEARCH_HISTORY = 12;
 
 namespace Find {
 
@@ -85,6 +81,7 @@ namespace Internal {
         SearchResultWindowPrivate(SearchResultWindow *window);
         bool isSearchVisible() const;
         int visibleSearchIndex() const;
+        void setCurrentIndex(int index, bool focus);
 
         SearchResultWindow *q;
         QList<Internal::SearchResultWidget *> m_searchResultWidgets;
@@ -97,9 +94,12 @@ namespace Internal {
         QList<SearchResult *> m_searchResults;
         int m_currentIndex;
         QFont m_font;
+        SearchResultColor m_color;
 
     public slots:
         void setCurrentIndex(int index);
+        void moveWidgetToTop();
+        void popupRequested(bool focus);
     };
 
     SearchResultWindowPrivate::SearchResultWindowPrivate(SearchResultWindow *window)
@@ -117,7 +117,7 @@ namespace Internal {
         return m_currentIndex - 1;
     }
 
-    void SearchResultWindowPrivate::setCurrentIndex(int index)
+    void SearchResultWindowPrivate::setCurrentIndex(int index, bool focus)
     {
         if (isSearchVisible())
             m_searchResultWidgets.at(visibleSearchIndex())->notifyVisibilityChanged(false);
@@ -125,15 +125,66 @@ namespace Internal {
         m_widget->setCurrentIndex(index);
         m_recentSearchesBox->setCurrentIndex(index);
         if (!isSearchVisible()) {
-            m_widget->currentWidget()->setFocus();
+            if (focus)
+                m_widget->currentWidget()->setFocus();
             m_expandCollapseButton->setEnabled(false);
         } else {
-            m_searchResultWidgets.at(visibleSearchIndex())->setFocusInternally();
+            if (focus)
+                m_searchResultWidgets.at(visibleSearchIndex())->setFocusInternally();
             m_searchResultWidgets.at(visibleSearchIndex())->notifyVisibilityChanged(true);
             m_expandCollapseButton->setEnabled(true);
         }
         q->navigateStateChanged();
     }
+
+    void SearchResultWindowPrivate::setCurrentIndex(int index)
+    {
+        setCurrentIndex(index, true/*focus*/);
+    }
+
+    void SearchResultWindowPrivate::moveWidgetToTop()
+    {
+        SearchResultWidget *widget = qobject_cast<SearchResultWidget *>(sender());
+        QTC_ASSERT(widget, return);
+        int index = m_searchResultWidgets.indexOf(widget);
+        if (index == 0)
+            return; // nothing to do
+        int internalIndex = index + 1/*account for "new search" entry*/;
+        QString searchEntry = m_recentSearchesBox->itemText(internalIndex);
+
+        m_searchResultWidgets.removeAt(index);
+        m_widget->removeWidget(widget);
+        m_recentSearchesBox->removeItem(internalIndex);
+        SearchResult *result = m_searchResults.takeAt(index);
+
+        m_searchResultWidgets.prepend(widget);
+        m_widget->insertWidget(1, widget);
+        m_recentSearchesBox->insertItem(1, searchEntry);
+        m_searchResults.prepend(result);
+
+        // adapt the current index
+        if (index == visibleSearchIndex()) {
+            // was visible, so we switch
+            // this is the default case
+            m_currentIndex = 1;
+            m_widget->setCurrentIndex(1);
+            m_recentSearchesBox->setCurrentIndex(1);
+        } else if (visibleSearchIndex() < index) {
+            // academical case where the widget moved before the current widget
+            // only our internal book keeping needed
+            ++m_currentIndex;
+        }
+    }
+
+    void SearchResultWindowPrivate::popupRequested(bool focus)
+    {
+        SearchResultWidget *widget = qobject_cast<SearchResultWidget *>(sender());
+        QTC_ASSERT(widget, return);
+        int internalIndex = m_searchResultWidgets.indexOf(widget) + 1/*account for "new search" entry*/;
+        setCurrentIndex(internalIndex, focus);
+        q->popup(focus ? Core::IOutputPane::ModeSwitch | Core::IOutputPane::WithFocus
+                       : Core::IOutputPane::NoModeSwitch);
+    }
 }
 
 using namespace Find::Internal;
@@ -150,7 +201,8 @@ using namespace Find::Internal;
 
 /*!
     \class Find::SearchResult
-    \brief Reports user interaction like activation of a search result item.
+    \brief The SearchResult class reports user interaction, such as the
+    activation of a search result item.
 
     Whenever a new search is initiated via startNewSearch, an instance of this
     class is returned to provide the initiator with the hooks for handling user
@@ -164,7 +216,7 @@ using namespace Find::Internal;
 */
 
 /*!
-    \fn void SearchResult::replaceButtonClicked(const QString &replaceText, const QList<Find::SearchResultItem> &checkedItems)
+    \fn void SearchResult::replaceButtonClicked(const QString &replaceText, const QList<Find::SearchResultItem> &checkedItems, bool preserveCase)
     \brief Sent when the user initiated a replace, e.g. by pressing the replace
     all button.
 
@@ -214,7 +266,6 @@ using namespace Find::Internal;
 SearchResultWindow *SearchResultWindow::m_instance = 0;
 
 /*!
-    \fn SearchResultWindow::SearchResultWindow()
     \internal
 */
 SearchResultWindow::SearchResultWindow(QWidget *newSearchPanel)
@@ -243,7 +294,7 @@ SearchResultWindow::SearchResultWindow(QWidget *newSearchPanel)
     d->m_expandCollapseAction = new QAction(tr("Expand All"), this);
     d->m_expandCollapseAction->setCheckable(true);
     d->m_expandCollapseAction->setIcon(QIcon(QLatin1String(":/find/images/expand.png")));
-    Core::Command *cmd = Core::ICore::instance()->actionManager()->registerAction(
+    Core::Command *cmd = Core::ActionManager::registerAction(
             d->m_expandCollapseAction, "Find.ExpandAll",
             Core::Context(Core::Constants::C_GLOBAL));
     cmd->setAttribute(Core::Command::CA_UpdateText);
@@ -254,7 +305,6 @@ SearchResultWindow::SearchResultWindow(QWidget *newSearchPanel)
 }
 
 /*!
-    \fn SearchResultWindow::~SearchResultWindow()
     \internal
 */
 SearchResultWindow::~SearchResultWindow()
@@ -267,7 +317,6 @@ SearchResultWindow::~SearchResultWindow()
 }
 
 /*!
-    \fn SearchResultWindow *SearchResultWindow::instance()
     \brief Returns the single shared instance of the Search Results window.
 */
 SearchResultWindow *SearchResultWindow::instance()
@@ -276,7 +325,6 @@ SearchResultWindow *SearchResultWindow::instance()
 }
 
 /*!
-    \fn void SearchResultWindow::visibilityChanged(bool)
     \internal
 */
 void SearchResultWindow::visibilityChanged(bool visible)
@@ -286,7 +334,6 @@ void SearchResultWindow::visibilityChanged(bool visible)
 }
 
 /*!
-    \fn QWidget *SearchResultWindow::outputWidget(QWidget *)
     \internal
 */
 QWidget *SearchResultWindow::outputWidget(QWidget *)
@@ -295,7 +342,6 @@ QWidget *SearchResultWindow::outputWidget(QWidget *)
 }
 
 /*!
-    \fn QList<QWidget*> SearchResultWindow::toolBarWidgets() const
     \internal
 */
 QList<QWidget*> SearchResultWindow::toolBarWidgets() const
@@ -329,10 +375,11 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
                                                  SearchMode searchOrSearchAndReplace,
                                                  const QString &cfgGroup)
 {
-    if (d->m_searchResults.size() >= 5) {
+    if (d->m_searchResults.size() >= MAX_SEARCH_HISTORY) {
         d->m_searchResultWidgets.last()->notifyVisibilityChanged(false);
-        delete d->m_searchResults.takeLast();
+        // widget first, because that might send interesting signals to SearchResult
         delete d->m_searchResultWidgets.takeLast();
+        delete d->m_searchResults.takeLast();
         d->m_recentSearchesBox->removeItem(d->m_recentSearchesBox->count()-1);
         if (d->m_currentIndex >= d->m_recentSearchesBox->count()) {
             // temporarily set the index to the last existing
@@ -343,7 +390,9 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
     d->m_searchResultWidgets.prepend(widget);
     d->m_widget->insertWidget(1, widget);
     connect(widget, SIGNAL(navigateStateChanged()), this, SLOT(navigateStateChanged()));
-    widget->setTextEditorFont(d->m_font);
+    connect(widget, SIGNAL(restarted()), d, SLOT(moveWidgetToTop()));
+    connect(widget, SIGNAL(requestPopup(bool)), d, SLOT(popupRequested(bool)));
+    widget->setTextEditorFont(d->m_font, d->m_color);
     widget->setShowReplaceUI(searchOrSearchAndReplace != SearchOnly);
     widget->setAutoExpandResults(d->m_expandCollapseAction->isChecked());
     widget->setInfo(label, toolTip, searchTerm);
@@ -359,7 +408,6 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
 }
 
 /*!
-    \fn void SearchResultWindow::clearContents()
     \brief Clears the current contents in the search result window.
 */
 void SearchResultWindow::clearContents()
@@ -380,7 +428,6 @@ void SearchResultWindow::clearContents()
 }
 
 /*!
-    \fn bool SearchResultWindow::hasFocus()
     \internal
 */
 bool SearchResultWindow::hasFocus() const
@@ -389,7 +436,6 @@ bool SearchResultWindow::hasFocus() const
 }
 
 /*!
-    \fn bool SearchResultWindow::canFocus()
     \internal
 */
 bool SearchResultWindow::canFocus() const
@@ -400,7 +446,6 @@ bool SearchResultWindow::canFocus() const
 }
 
 /*!
-    \fn void SearchResultWindow::setFocus()
     \internal
 */
 void SearchResultWindow::setFocus()
@@ -412,24 +457,36 @@ void SearchResultWindow::setFocus()
 }
 
 /*!
-    \fn void SearchResultWindow::setTextEditorFont(const QFont &font)
     \internal
 */
-void SearchResultWindow::setTextEditorFont(const QFont &font)
+void SearchResultWindow::setTextEditorFont(const QFont &font,
+                                           const QColor &textForegroundColor,
+                                           const QColor &textBackgroundColor,
+                                           const QColor &highlightForegroundColor,
+                                           const QColor &highlightBackgroundColor)
 {
     d->m_font = font;
+    Internal::SearchResultColor color;
+    color.textBackground = textBackgroundColor;
+    color.textForeground = textForegroundColor;
+    color.highlightBackground = highlightBackgroundColor.isValid()
+            ? highlightBackgroundColor
+            : textBackgroundColor;
+    color.highlightForeground = highlightForegroundColor.isValid()
+            ? highlightForegroundColor
+            : textForegroundColor;
+    d->m_color = color;
     foreach (Internal::SearchResultWidget *widget, d->m_searchResultWidgets)
-        widget->setTextEditorFont(font);
+        widget->setTextEditorFont(font, color);
 }
 
 void SearchResultWindow::openNewSearchPanel()
 {
     d->setCurrentIndex(0);
-    popup(true/*focus*/, true/*sizeHint*/);
+    popup(IOutputPane::ModeSwitch  | IOutputPane::WithFocus | IOutputPane::EnsureSizeHint);
 }
 
 /*!
-    \fn void SearchResultWindow::handleExpandCollapseToolButton(bool checked)
     \internal
 */
 void SearchResultWindow::handleExpandCollapseToolButton(bool checked)
@@ -447,35 +504,28 @@ void SearchResultWindow::handleExpandCollapseToolButton(bool checked)
 }
 
 /*!
-    \fn void SearchResultWindow::readSettings()
     \internal
 */
 void SearchResultWindow::readSettings()
 {
-    QSettings *s = Core::ICore::instance()->settings();
-    if (s) {
-        s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
-        d->m_expandCollapseAction->setChecked(s->value(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_initiallyExpand).toBool());
-        s->endGroup();
-    }
+    QSettings *s = Core::ICore::settings();
+    s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
+    d->m_expandCollapseAction->setChecked(s->value(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_initiallyExpand).toBool());
+    s->endGroup();
 }
 
 /*!
-    \fn void SearchResultWindow::writeSettings()
     \internal
 */
 void SearchResultWindow::writeSettings()
 {
-    QSettings *s = Core::ICore::instance()->settings();
-    if (s) {
-        s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
-        s->setValue(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_expandCollapseAction->isChecked());
-        s->endGroup();
-    }
+    QSettings *s = Core::ICore::settings();
+    s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
+    s->setValue(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_expandCollapseAction->isChecked());
+    s->endGroup();
 }
 
 /*!
-    \fn int SearchResultWindow::priorityInStatusBar() const
     \internal
 */
 int SearchResultWindow::priorityInStatusBar() const
@@ -484,7 +534,6 @@ int SearchResultWindow::priorityInStatusBar() const
 }
 
 /*!
-    \fn bool SearchResultWindow::canNext()
     \internal
 */
 bool SearchResultWindow::canNext() const
@@ -495,7 +544,6 @@ bool SearchResultWindow::canNext() const
 }
 
 /*!
-    \fn bool SearchResultWindow::canPrevious()
     \internal
 */
 bool SearchResultWindow::canPrevious() const
@@ -504,7 +552,6 @@ bool SearchResultWindow::canPrevious() const
 }
 
 /*!
-    \fn void SearchResultWindow::goToNext()
     \internal
 */
 void SearchResultWindow::goToNext()
@@ -515,7 +562,6 @@ void SearchResultWindow::goToNext()
 }
 
 /*!
-    \fn void SearchResultWindow::goToPrev()
     \internal
 */
 void SearchResultWindow::goToPrev()
@@ -526,7 +572,6 @@ void SearchResultWindow::goToPrev()
 }
 
 /*!
-    \fn bool SearchResultWindow::canNavigate()
     \internal
 */
 bool SearchResultWindow::canNavigate() const
@@ -535,7 +580,6 @@ bool SearchResultWindow::canNavigate() const
 }
 
 /*!
-    \fn SearchResult::SearchResult(SearchResultWidget *widget)
     \internal
 */
 SearchResult::SearchResult(SearchResultWidget *widget)
@@ -543,16 +587,19 @@ SearchResult::SearchResult(SearchResultWidget *widget)
 {
     connect(widget, SIGNAL(activated(Find::SearchResultItem)),
             this, SIGNAL(activated(Find::SearchResultItem)));
-    connect(widget, SIGNAL(replaceButtonClicked(QString,QList<Find::SearchResultItem>)),
-            this, SIGNAL(replaceButtonClicked(QString,QList<Find::SearchResultItem>)));
+    connect(widget, SIGNAL(replaceButtonClicked(QString,QList<Find::SearchResultItem>,bool)),
+            this, SIGNAL(replaceButtonClicked(QString,QList<Find::SearchResultItem>,bool)));
     connect(widget, SIGNAL(cancelled()),
             this, SIGNAL(cancelled()));
+    connect(widget, SIGNAL(paused(bool)),
+            this, SIGNAL(paused(bool)));
     connect(widget, SIGNAL(visibilityChanged(bool)),
             this, SIGNAL(visibilityChanged(bool)));
+    connect(widget, SIGNAL(searchAgainRequested()),
+            this, SIGNAL(searchAgainRequested()));
 }
 
 /*!
-    \fn void SearchResult::setUserData(const QVariant &data)
     \brief Attach some random \a data to this search, that you can use later.
 
     \sa userData()
@@ -563,7 +610,6 @@ void SearchResult::setUserData(const QVariant &data)
 }
 
 /*!
-    \fn void SearchResult::userData()
     \brief Return the data that was attached to this search by calling setUserData().
 
     \sa setUserData()
@@ -574,7 +620,6 @@ QVariant SearchResult::userData() const
 }
 
 /*!
-    \fn QString SearchResult::textToReplace() const
     \brief Returns the text that should replace the text in search results.
 */
 QString SearchResult::textToReplace() const
@@ -582,8 +627,17 @@ QString SearchResult::textToReplace() const
     return m_widget->textToReplace();
 }
 
+int SearchResult::count() const
+{
+    return m_widget->count();
+}
+
+void SearchResult::setSearchAgainSupported(bool supported)
+{
+    m_widget->setSearchAgainSupported(supported);
+}
+
 /*!
-    \fn void SearchResult::addResult(const QString &fileName, int lineNumber, const QString &rowText, int searchTermStart, int searchTermLength, const QVariant &userData)
     \brief Adds a single result line to the search results.
 
     The \a fileName, \a lineNumber and \a rowText are shown in the result line.
@@ -599,10 +653,10 @@ void SearchResult::addResult(const QString &fileName, int lineNumber, const QStr
 {
     m_widget->addResult(fileName, lineNumber, lineText,
                         searchTermStart, searchTermLength, userData);
+    emit countChanged(m_widget->count());
 }
 
 /*!
-    \fn void SearchResult::addResults(const QList<SearchResultItem> &items, SearchResult::AddMode mode)
     \brief Adds all of the given search result \a items to the search
     results window.
 
@@ -611,20 +665,19 @@ void SearchResult::addResult(const QString &fileName, int lineNumber, const QStr
 void SearchResult::addResults(const QList<SearchResultItem> &items, AddMode mode)
 {
     m_widget->addResults(items, mode);
+    emit countChanged(m_widget->count());
 }
 
 /*!
-    \fn void SearchResult::finishSearch()
     \brief Notifies the search result window that the current search
     has finished, and the UI should reflect that.
 */
-void SearchResult::finishSearch()
+void SearchResult::finishSearch(bool canceled)
 {
-    m_widget->finishSearch();
+    m_widget->finishSearch(canceled);
 }
 
 /*!
-    \fn void SearchResult::setTextToReplace(const QString &textToReplace)
     \brief Sets the value in the UI element that allows the user to type
     the text that should replace text in search results to \a textToReplace.
 */
@@ -633,6 +686,27 @@ void SearchResult::setTextToReplace(const QString &textToReplace)
     m_widget->setTextToReplace(textToReplace);
 }
 
+/*!
+ * \brief Removes all search results.
+ */
+void SearchResult::restart()
+{
+    m_widget->restart();
+}
+
+void SearchResult::setSearchAgainEnabled(bool enabled)
+{
+    m_widget->setSearchAgainEnabled(enabled);
+}
+
+/*!
+ * \brief Pops up the search result panel with this search.
+ */
+void SearchResult::popup()
+{
+    m_widget->sendRequestPopup();
+}
+
 } // namespace Find
 
 #include "searchresultwindow.moc"
