@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -34,8 +34,6 @@
 #include <vcsbase/vcsbaseoutputwindow.h>
 
 #include <QFont>
-#include <QRegExp>
-#include <QTimer>
 
 namespace Git {
 namespace Internal {
@@ -48,16 +46,20 @@ class BranchNode
 {
 public:
     BranchNode() :
-        parent(0), current(false)
+        parent(0),
+        name(QLatin1String("<ROOT>"))
     { }
 
-    BranchNode(const QString &n, const QString &s = QString(), bool c = false) :
-        parent(0), current(c), name(n), sha(s)
+    BranchNode(const QString &n, const QString &s = QString(), const QString &t = QString()) :
+        parent(0), name(n), sha(s), tracking(t)
     { }
 
     ~BranchNode()
     {
-        qDeleteAll(children);
+        while (!children.isEmpty())
+            delete children.first();
+        if (parent)
+            parent->children.removeAll(this);
     }
 
     BranchNode *rootNode() const
@@ -72,7 +74,19 @@ public:
 
     bool isLeaf() const
     {
-        return children.isEmpty();
+        return children.isEmpty() && parent && parent->parent;
+    }
+
+    bool isTag() const
+    {
+        if (!parent)
+            return false;
+        for (const BranchNode *p = this; p->parent; p = p->parent) {
+            // find root child with name "tags"
+            if (!p->parent->parent && p->name == QLatin1String("tags"))
+                return true;
+        }
+        return false;
     }
 
     bool childOf(BranchNode *node) const
@@ -152,12 +166,17 @@ public:
         return QStringList(fullName().join(QString(QLatin1Char('/'))));
     }
 
+    int rowOf(BranchNode *node)
+    {
+        return children.indexOf(node);
+    }
+
     BranchNode *parent;
     QList<BranchNode *> children;
 
-    bool current;
     QString name;
     QString sha;
+    QString tracking;
     mutable QString toolTip;
 };
 
@@ -168,7 +187,8 @@ public:
 BranchModel::BranchModel(GitClient *client, QObject *parent) :
     QAbstractItemModel(parent),
     m_client(client),
-    m_rootNode(new BranchNode)
+    m_rootNode(new BranchNode),
+    m_currentBranch(0)
 {
     QTC_CHECK(m_client);
     m_rootNode->append(new BranchNode(tr("Local Branches")));
@@ -179,32 +199,34 @@ BranchModel::~BranchModel()
     delete m_rootNode;
 }
 
-QModelIndex BranchModel::index(int row, int column, const QModelIndex &parent) const
+QModelIndex BranchModel::index(int row, int column, const QModelIndex &parentIdx) const
 {
-    BranchNode *node = m_rootNode;
-    if (parent.isValid())
-        node = static_cast<BranchNode *>(parent.internalPointer());
-    if (row >= node->count())
+    if (column != 0)
         return QModelIndex();
-    return createIndex(row, column, static_cast<void *>(node->children.at(row)));
+    BranchNode *parentNode = indexToNode(parentIdx);
+
+    if (row >= parentNode->count())
+        return QModelIndex();
+    return nodeToIndex(parentNode->children.at(row));
 }
 
 QModelIndex BranchModel::parent(const QModelIndex &index) const
 {
-    BranchNode *node = static_cast<BranchNode *>(index.internalPointer());
+    if (!index.isValid())
+        return QModelIndex();
+
+    BranchNode *node = indexToNode(index);
     if (node->parent == m_rootNode)
         return QModelIndex();
-    int row = node->parent->children.indexOf(node);
-    return createIndex(row, 0, static_cast<void *>(node->parent));
+    return nodeToIndex(node->parent);
 }
 
-int BranchModel::rowCount(const QModelIndex &parent) const
+int BranchModel::rowCount(const QModelIndex &parentIdx) const
 {
-    if (!parent.isValid())
-        return m_rootNode->count();
-    if (parent.column() != 0)
+    if (parentIdx.column() > 0)
         return 0;
-    return static_cast<BranchNode *>(parent.internalPointer())->count();
+
+    return indexToNode(parentIdx)->count();
 }
 
 int BranchModel::columnCount(const QModelIndex &parent) const
@@ -215,10 +237,17 @@ int BranchModel::columnCount(const QModelIndex &parent) const
 
 QVariant BranchModel::data(const QModelIndex &index, int role) const
 {
-    BranchNode *node = static_cast<BranchNode *>(index.internalPointer());
+    BranchNode *node = indexToNode(index);
+    if (!node)
+        return QVariant();
 
     switch (role) {
-    case Qt::DisplayRole:
+    case Qt::DisplayRole: {
+        QString res = node->name;
+        if (!node->tracking.isEmpty())
+            res += QLatin1String(" [") + node->tracking + QLatin1Char(']');
+        return res;
+    }
     case Qt::EditRole:
         return node->name;
     case Qt::ToolTipRole:
@@ -232,7 +261,7 @@ QVariant BranchModel::data(const QModelIndex &index, int role) const
         QFont font;
         if (!node->isLeaf()) {
             font.setBold(true);
-        } else if (node->current) {
+        } else if (node == m_currentBranch) {
             font.setBold(true);
             font.setUnderline(true);
         }
@@ -247,7 +276,9 @@ bool BranchModel::setData(const QModelIndex &index, const QVariant &value, int r
 {
     if (role != Qt::EditRole)
         return false;
-    BranchNode *node = static_cast<BranchNode *>(index.internalPointer());
+    BranchNode *node = indexToNode(index);
+    if (!node)
+        return false;
 
     const QString newName = value.toString();
     if (newName.isEmpty())
@@ -278,7 +309,9 @@ bool BranchModel::setData(const QModelIndex &index, const QVariant &value, int r
 
 Qt::ItemFlags BranchModel::flags(const QModelIndex &index) const
 {
-    BranchNode *node = static_cast<BranchNode *>(index.internalPointer());
+    BranchNode *node = indexToNode(index);
+    if (!node)
+        return Qt::NoItemFlags;
     if (node->isLeaf() && node->isLocal())
         return Qt::ItemIsSelectable | Qt::ItemIsEditable | Qt::ItemIsEnabled;
     else
@@ -287,44 +320,62 @@ Qt::ItemFlags BranchModel::flags(const QModelIndex &index) const
 
 void BranchModel::clear()
 {
-    while (m_rootNode->count() > 1) {
-        BranchNode *n = m_rootNode->children.takeLast();
-        delete n;
-    }
+    while (m_rootNode->count() > 1)
+        delete m_rootNode->children.takeLast();
     BranchNode *locals = m_rootNode->children.at(0);
-    while (locals->count()) {
-        BranchNode *n = locals->children.takeLast();
-        delete n;
-    }
+    while (locals->count())
+        delete locals->children.takeLast();
+
+    m_currentBranch = 0;
 }
 
 bool BranchModel::refresh(const QString &workingDirectory, QString *errorMessage)
 {
-    if (workingDirectory.isEmpty())
-        return false;
-
-    QStringList branchArgs;
-    branchArgs << QLatin1String(GitClient::noColorOption)
-               << QLatin1String("-v") << QLatin1String("-a");
-    QString output;
-    if (!m_client->synchronousBranchCmd(workingDirectory, branchArgs, &output, errorMessage)) {
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(*errorMessage);
+    beginResetModel();
+    clear();
+    if (workingDirectory.isEmpty()) {
+        endResetModel();
         return false;
     }
 
-    beginResetModel();
-
-    clear();
+    m_currentSha = m_client->synchronousTopRevision(workingDirectory);
+    QStringList args;
+    args << QLatin1String("--format=%(objectname)\t%(refname)\t%(upstream:short)\t%(*objectname)");
+    QString output;
+    if (!m_client->synchronousForEachRefCmd(workingDirectory, args, &output, errorMessage))
+        VcsBase::VcsBaseOutputWindow::instance()->appendError(*errorMessage);
 
     m_workingDirectory = workingDirectory;
     const QStringList lines = output.split(QLatin1Char('\n'));
     foreach (const QString &l, lines)
         parseOutputLine(l);
 
+    if (m_currentBranch) {
+        if (m_currentBranch->parent == m_rootNode->children[0])
+            m_currentBranch = 0;
+        setCurrentBranch();
+    }
+
     endResetModel();
+
     return true;
 }
 
+void BranchModel::setCurrentBranch()
+{
+    QString currentBranch = m_client->synchronousCurrentLocalBranch(m_workingDirectory);
+    if (currentBranch.isEmpty())
+        return;
+
+    BranchNode *local = m_rootNode->children.at(0);
+    int pos = 0;
+    for (pos = 0; pos < local->count(); ++pos) {
+        if (local->children.at(pos)->name == currentBranch) {
+            m_currentBranch = local->children[pos];
+        }
+    }
+}
+
 void BranchModel::renameBranch(const QString &oldName, const QString &newName)
 {
     QString errorMessage;
@@ -349,23 +400,17 @@ GitClient *BranchModel::client() const
 
 QModelIndex BranchModel::currentBranch() const
 {
-    if (!m_rootNode || !m_rootNode->count())
+    if (!m_currentBranch)
         return QModelIndex();
-    BranchNode *localBranches = m_rootNode->children.at(0);
-    QModelIndex localIdx = index(0, 0, QModelIndex());
-    for (int i = 0; i < localBranches->count(); ++i) {
-        if (localBranches->children.at(i)->current)
-            return index(i, 0, localIdx);
-    }
-    return QModelIndex();
+    return nodeToIndex(m_currentBranch);
 }
 
 QString BranchModel::branchName(const QModelIndex &idx) const
 {
     if (!idx.isValid())
         return QString();
-    BranchNode *node = static_cast<BranchNode *>(idx.internalPointer());
-    if (!node->isLeaf())
+    BranchNode *node = indexToNode(idx);
+    if (!node || !node->isLeaf())
         return QString();
     QStringList path = node->fullName();
     return path.join(QString(QLatin1Char('/')));
@@ -383,7 +428,7 @@ QString BranchModel::sha(const QModelIndex &idx) const
 {
     if (!idx.isValid())
         return QString();
-    BranchNode *node = static_cast<BranchNode *>(idx.internalPointer());
+    BranchNode *node = indexToNode(idx);
     return node->sha;
 }
 
@@ -391,7 +436,7 @@ bool BranchModel::isLocal(const QModelIndex &idx) const
 {
     if (!idx.isValid())
         return false;
-    BranchNode *node = static_cast<BranchNode *>(idx.internalPointer());
+    BranchNode *node = indexToNode(idx);
     return node->isLocal();
 }
 
@@ -399,10 +444,17 @@ bool BranchModel::isLeaf(const QModelIndex &idx) const
 {
     if (!idx.isValid())
         return false;
-    BranchNode *node = static_cast<BranchNode *>(idx.internalPointer());
+    BranchNode *node = indexToNode(idx);
     return node->isLeaf();
 }
 
+bool BranchModel::isTag(const QModelIndex &idx) const
+{
+    if (!idx.isValid())
+        return false;
+    return indexToNode(idx)->isTag();
+}
+
 void BranchModel::removeBranch(const QModelIndex &idx)
 {
     QString branch = branchName(idx);
@@ -419,11 +471,15 @@ void BranchModel::removeBranch(const QModelIndex &idx)
         return;
     }
 
-    QModelIndex parentIdx = parent(idx);
-    beginRemoveRows(parentIdx, idx.row(), idx.row());
-    static_cast<BranchNode *>(parentIdx.internalPointer())->children.removeAt(parentIdx.row());
-    delete static_cast<BranchNode *>(idx.internalPointer());
-    endRemoveRows();
+    QModelIndex tmp = idx; // tmp is a leaf, so count must be 0.
+    while (indexToNode(tmp)->count() == 0) {
+        QModelIndex tmpParent = parent(tmp);
+        beginRemoveRows(tmpParent, tmp.row(), tmp.row());
+        indexToNode(tmpParent)->children.removeAt(tmp.row());
+        delete indexToNode(tmp);
+        endRemoveRows();
+        tmp = tmpParent;
+    }
 }
 
 void BranchModel::checkoutBranch(const QModelIndex &idx)
@@ -432,33 +488,9 @@ void BranchModel::checkoutBranch(const QModelIndex &idx)
     if (branch.isEmpty())
         return;
 
-    QString errorMessage;
-    switch (m_client->ensureStash(m_workingDirectory, &errorMessage)) {
-    case GitClient::StashUnchanged:
-    case GitClient::Stashed:
-    case GitClient::NotStashed:
-        break;
-    case GitClient::StashCanceled:
-        return;
-    case GitClient::StashFailed:
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
-        return;
-    }
-    if (m_client->synchronousCheckoutBranch(m_workingDirectory, branch, &errorMessage)) {
-        if (errorMessage.isEmpty()) {
-            QModelIndex currentIdx = currentBranch();
-            if (currentIdx.isValid()) {
-                static_cast<BranchNode *>(currentIdx.internalPointer())->current = false;
-                emit dataChanged(currentBranch(), currentBranch());
-            }
-            static_cast<BranchNode *>(idx.internalPointer())->current = true;
-            emit dataChanged(idx, idx);
-        } else {
-            refresh(m_workingDirectory, &errorMessage); // not sure all went well... better refresh!
-        }
-    }
-    if (!errorMessage.isEmpty())
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+    // No StashGuard since this function for now is only used with clean working dir.
+    // If it is ever used from another place, please add StashGuard here
+    m_client->synchronousCheckout(m_workingDirectory, branch);
 }
 
 bool BranchModel::branchIsMerged(const QModelIndex &idx)
@@ -472,10 +504,8 @@ bool BranchModel::branchIsMerged(const QModelIndex &idx)
     QStringList args;
 
     args << QLatin1String("-a") << QLatin1String("--contains") << sha(idx);
-    if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage)) {
+    if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage))
         VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
-        return false;
-    }
 
     QStringList lines = output.split(QLatin1Char('\n'), QString::SkipEmptyParts);
     foreach (const QString &l, lines) {
@@ -488,19 +518,30 @@ bool BranchModel::branchIsMerged(const QModelIndex &idx)
     return false;
 }
 
-QModelIndex BranchModel::addBranch(const QString &branchName, bool track, const QString &startPoint)
+static int positionForName(BranchNode *node, const QString &name)
+{
+    int pos = 0;
+    for (pos = 0; pos < node->count(); ++pos) {
+        if (node->children.at(pos)->name >= name)
+            break;
+    }
+    return pos;
+}
+
+QModelIndex BranchModel::addBranch(const QString &name, bool track, const QModelIndex &startPoint)
 {
     if (!m_rootNode || !m_rootNode->count())
         return QModelIndex();
 
+    const QString trackedBranch = branchName(startPoint);
     QString output;
     QString errorMessage;
 
     QStringList args;
     args << (track ? QLatin1String("--track") : QLatin1String("--no-track"));
-    args << branchName;
-    if (!startPoint.isEmpty())
-        args << startPoint;
+    args << name;
+    if (!trackedBranch.isEmpty())
+        args << trackedBranch;
 
     if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage)) {
         VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
@@ -508,29 +549,30 @@ QModelIndex BranchModel::addBranch(const QString &branchName, bool track, const
     }
 
     BranchNode *local = m_rootNode->children.at(0);
-    int pos = 0;
-    for (pos = 0; pos < local->count(); ++pos) {
-        if (local->children.at(pos)->name > branchName)
-            break;
-    }
-    BranchNode *newNode = new BranchNode(branchName);
-
-    // find the sha of the new branch:
-    output = toolTip(branchName); // abuse toolTip to get the data;-)
-    QStringList lines = output.split(QLatin1Char('\n'));
-    foreach (const QString &l, lines) {
-        if (l.startsWith(QLatin1String("commit "))) {
-            newNode->sha = l.mid(7, 8);
-            break;
+    const int slash = name.indexOf(QLatin1Char('/'));
+    const QString leafName = slash == -1 ? name : name.mid(slash + 1);
+    bool added = false;
+    if (slash != -1) {
+        const QString nodeName = name.left(slash);
+        int pos = positionForName(local, nodeName);
+        BranchNode *child = (pos == local->count()) ? 0 : local->children.at(pos);
+        if (!child || child->name != nodeName) {
+            child = new BranchNode(nodeName);
+            beginInsertRows(nodeToIndex(local), pos, pos);
+            added = true;
+            child->parent = local;
+            local->children.insert(pos, child);
         }
+        local = child;
     }
-
-    beginInsertRows(index(0, 0), pos, pos);
+    int pos = positionForName(local, leafName);
+    BranchNode *newNode = new BranchNode(leafName, sha(startPoint), track ? trackedBranch : QString());
+    if (!added)
+        beginInsertRows(nodeToIndex(local), pos, pos);
     newNode->parent = local;
     local->children.insert(pos, newNode);
     endInsertRows();
-
-    return index(pos, 0, index(0, 0));
+    return nodeToIndex(newNode);
 }
 
 void BranchModel::parseOutputLine(const QString &line)
@@ -538,38 +580,57 @@ void BranchModel::parseOutputLine(const QString &line)
     if (line.size() < 3)
         return;
 
-    bool current = line.startsWith(QLatin1String("* "));
+    QStringList lineParts = line.split(QLatin1Char('\t'));
+    const QString shaDeref = lineParts.at(3);
+    const QString sha = shaDeref.isEmpty() ? lineParts.at(0) : shaDeref;
+    const QString fullName = lineParts.at(1);
 
-    const QString branchInfo = line.mid(2);
-    if (current && branchInfo.startsWith(QLatin1String("(no branch)")))
-        return;
-
-    QStringList tokens = branchInfo.split(QLatin1Char(' '), QString::SkipEmptyParts);
-    if (tokens.size() < 2)
-        return;
-
-    QString sha = tokens.at(1);
+    bool current = (sha == m_currentSha);
+    bool showTags = m_client->settings()->boolValue(GitSettings::showTagsKey);
 
     // insert node into tree:
-    QStringList nameParts = tokens.at(0).split(QLatin1Char('/'));
-    if (nameParts.count() < 1)
+    QStringList nameParts = fullName.split(QLatin1Char('/'));
+    nameParts.removeFirst(); // remove refs...
+
+    if (nameParts.first() == QLatin1String("heads"))
+        nameParts[0] = m_rootNode->children.at(0)->name; // Insert the local designator
+    else if (nameParts.first() == QLatin1String("remotes"))
+        nameParts.removeFirst(); // remove "remotes"
+    else if (nameParts.first() == QLatin1String("stash"))
+        return;
+    else if (!showTags && (nameParts.first() == QLatin1String("tags")))
         return;
 
-    QString name = nameParts.last();
+    // limit depth of list. Git basically only ever wants one / and considers the rest as part of
+    // the name.
+    while (nameParts.count() > 3) {
+        nameParts[2] = nameParts.at(2) + QLatin1Char('/') + nameParts.at(3);
+        nameParts.removeAt(3);
+    }
+
+    const QString name = nameParts.last();
     nameParts.removeLast();
 
-    if (nameParts.isEmpty() || nameParts.at(0) != QLatin1String("remotes")) {
-        // local branch:
-        while (nameParts.count() > 2)
-            nameParts[1] = nameParts.at(1) + QLatin1Char('/') + nameParts.at(2);
-        m_rootNode->children.at(0)->insert(nameParts, new BranchNode(name, sha, current));
-    } else {
-        // remote branch:
-        nameParts.removeFirst(); // remove "remotes"
-        while (nameParts.count() > 2)
-            nameParts[1] = nameParts.at(1) + QLatin1Char('/') + nameParts.at(2);
-        m_rootNode->insert(nameParts, new BranchNode(name, sha, current));
-    }
+    BranchNode *newNode = new BranchNode(name, sha, lineParts.at(2));
+    m_rootNode->insert(nameParts, newNode);
+    if (current)
+        m_currentBranch = newNode;
+}
+
+BranchNode *BranchModel::indexToNode(const QModelIndex &index) const
+{
+    if (index.column() > 0)
+        return 0;
+    if (!index.isValid())
+        return m_rootNode;
+    return static_cast<BranchNode *>(index.internalPointer());
+}
+
+QModelIndex BranchModel::nodeToIndex(BranchNode *node) const
+{
+    if (node == m_rootNode)
+        return QModelIndex();
+    return createIndex(node->parent->rowOf(node), 0, static_cast<void *>(node));
 }
 
 QString BranchModel::toolTip(const QString &sha) const
@@ -577,12 +638,10 @@ QString BranchModel::toolTip(const QString &sha) const
     // Show the sha description excluding diff as toolTip
     QString output;
     QString errorMessage;
-    if (!m_client->synchronousShow(m_workingDirectory, sha, &output, &errorMessage))
+    QStringList arguments(QLatin1String("-n1"));
+    arguments << sha;
+    if (!m_client->synchronousLog(m_workingDirectory, arguments, &output, &errorMessage))
         return errorMessage;
-    // Remove 'diff' output
-    const int diffPos = output.indexOf(QLatin1String("\ndiff --"));
-    if (diffPos != -1)
-        output.remove(diffPos, output.size() - diffPos);
     return output;
 }
 
