@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -27,44 +27,33 @@
 **
 ****************************************************************************/
 
-#include <qglobal.h>
-
 #include "locatorwidget.h"
 #include "locatorplugin.h"
 #include "locatorconstants.h"
 #include "ilocatorfilter.h"
 
-#include <extensionsystem/pluginmanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/actionmanager/command.h>
-#include <coreplugin/editormanager/editormanager.h>
-#include <coreplugin/coreconstants.h>
 #include <coreplugin/fileiconprovider.h>
+#include <coreplugin/icontext.h>
+#include <utils/appmainwindow.h>
 #include <utils/filterlineedit.h>
+#include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 
-#include <QtConcurrentRun>
 #include <QColor>
 #include <QFileInfo>
-#include <QFile>
 #include <QTimer>
-#include <QSettings>
 #include <QEvent>
 #include <QAction>
 #include <QApplication>
-#include <QContextMenuEvent>
 #include <QHBoxLayout>
 #include <QHeaderView>
 #include <QKeyEvent>
-#include <QLabel>
-#include <QLineEdit>
 #include <QMenu>
-#include <QMouseEvent>
-#include <QPushButton>
 #include <QScrollBar>
 #include <QTreeView>
 #include <QToolTip>
@@ -72,6 +61,8 @@
 Q_DECLARE_METATYPE(Locator::ILocatorFilter*)
 Q_DECLARE_METATYPE(Locator::FilterEntry)
 
+using namespace Core;
+
 namespace Locator {
 namespace Internal {
 
@@ -104,12 +95,14 @@ public:
     void updatePreferredSize();
     QSize preferredSize() const { return m_preferredSize; }
 
-#if defined(Q_OS_WIN)
     void focusOutEvent (QFocusEvent * event)  {
-        if (event->reason() == Qt::ActiveWindowFocusReason)
-            hide();
+        if (Utils::HostOsInfo::isWindowsHost()) {
+            if (event->reason() == Qt::ActiveWindowFocusReason)
+                hide();
+        } else {
+            QTreeView::focusOutEvent(event);
+        }
     }
-#endif
 
     void next() {
         int index = currentIndex().row();
@@ -173,11 +166,10 @@ QVariant LocatorModel::data(const QModelIndex &index, int role) const
         return QVariant();
 
     if (role == Qt::DisplayRole) {
-        if (index.column() == 0) {
+        if (index.column() == 0)
             return mEntries.at(index.row()).displayName;
-        } else if (index.column() == 1) {
+        else if (index.column() == 1)
             return mEntries.at(index.row()).extraInfo;
-        }
     } else if (role == Qt::ToolTipRole) {
         if (mEntries.at(index.row()).extraInfo.isEmpty())
             return QVariant(mEntries.at(index.row()).displayName);
@@ -186,11 +178,9 @@ QVariant LocatorModel::data(const QModelIndex &index, int role) const
                             + QLatin1String("\n\n") + mEntries.at(index.row()).extraInfo);
     } else if (role == Qt::DecorationRole && index.column() == 0) {
         FilterEntry &entry = mEntries[index.row()];
-        if (entry.resolveFileIcon && entry.displayIcon.isNull()) {
-            entry.resolveFileIcon = false;
-            QString path = entry.internalData.toString();
-            Core::EditorManager::splitLineNumber(&path);
-            entry.displayIcon = Core::FileIconProvider::instance()->icon(QFileInfo(path));
+        if (!entry.fileIconResolved && !entry.fileName.isEmpty() && entry.displayIcon.isNull()) {
+            entry.fileIconResolved = true;
+            entry.displayIcon = Core::FileIconProvider::instance()->icon(QFileInfo(entry.fileName));
         }
         return entry.displayIcon;
     } else if (role == Qt::ForegroundRole && index.column() == 1) {
@@ -204,8 +194,9 @@ QVariant LocatorModel::data(const QModelIndex &index, int role) const
 
 void LocatorModel::setEntries(const QList<FilterEntry> &entries)
 {
+    beginResetModel();
     mEntries = entries;
-    reset();
+    endResetModel();
 }
 
 // =========== CompletionList ===========
@@ -221,12 +212,12 @@ CompletionList::CompletionList(QWidget *parent)
     // This is too slow when done on all results
     //header()->setResizeMode(QHeaderView::ResizeToContents);
     setWindowFlags(Qt::ToolTip);
-#ifdef Q_OS_MAC
-    if (horizontalScrollBar())
-        horizontalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
-    if (verticalScrollBar())
-        verticalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
-#endif
+    if (Utils::HostOsInfo::isMacHost()) {
+        if (horizontalScrollBar())
+            horizontalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+        if (verticalScrollBar())
+            verticalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+    }
 }
 
 void CompletionList::updatePreferredSize()
@@ -248,7 +239,8 @@ LocatorWidget::LocatorWidget(LocatorPlugin *qop) :
     m_configureAction(new QAction(tr("Configure..."), this)),
     m_fileLineEdit(new Utils::FilterLineEdit),
     m_updateRequested(false),
-    m_acceptRequested(false)
+    m_acceptRequested(false),
+    m_possibleToolTipRequest(false)
 {
     // Explicitly hide the completion list popup.
     m_completionList->hide();
@@ -313,40 +305,44 @@ void LocatorWidget::setPlaceholderText(const QString &text)
 
 void LocatorWidget::updateFilterList()
 {
+    typedef QMap<Id, QAction *> IdActionMap;
+
     m_filterMenu->clear();
 
     // update actions and menu
-    QMap<QString, QAction *> actionCopy = m_filterActionMap;
+    IdActionMap actionCopy = m_filterActionMap;
     m_filterActionMap.clear();
     // register new actions, update existent
     foreach (ILocatorFilter *filter, m_locatorPlugin->filters()) {
         if (filter->shortcutString().isEmpty() || filter->isHidden())
             continue;
-        Core::Id locatorId = Core::Id(QLatin1String("Locator.") + filter->id());
+        Id filterId = filter->id();
+        Id locatorId = filterId.withPrefix("Locator.");
         QAction *action = 0;
-        Core::Command *cmd = 0;
-        if (!actionCopy.contains(filter->id())) {
+        Command *cmd = 0;
+        if (!actionCopy.contains(filterId)) {
             // register new action
             action = new QAction(filter->displayName(), this);
-            cmd = Core::ActionManager::registerAction(action, locatorId,
-                               Core::Context(Core::Constants::C_GLOBAL));
-            cmd->setAttribute(Core::Command::CA_UpdateText);
+            cmd = ActionManager::registerAction(action, locatorId,
+                               Context(Core::Constants::C_GLOBAL));
+            cmd->setAttribute(Command::CA_UpdateText);
             connect(action, SIGNAL(triggered()), this, SLOT(filterSelected()));
             action->setData(qVariantFromValue(filter));
         } else {
-            action = actionCopy.take(filter->id());
-            action->setText(filter->displayName());
-            cmd = Core::ActionManager::command(locatorId);
+            action = actionCopy.take(filterId);
+            action->setText(filterId.toString());
+            cmd = ActionManager::command(locatorId);
         }
-        m_filterActionMap.insert(filter->id(), action);
+        m_filterActionMap.insert(filterId, action);
         m_filterMenu->addAction(cmd->action());
     }
 
     // unregister actions that are deleted now
-    foreach (const QString &id, actionCopy.keys()) {
-        Core::ActionManager::unregisterAction(actionCopy.value(id), Core::Id(QLatin1String("Locator.") + id));
+    const IdActionMap::Iterator end = actionCopy.end();
+    for (IdActionMap::Iterator it = actionCopy.begin(); it != end; ++it) {
+        ActionManager::unregisterAction(it.value(), it.key().withPrefix("Locator."));
+        delete it.value();
     }
-    qDeleteAll(actionCopy);
 
     m_filterMenu->addSeparator();
     m_filterMenu->addAction(m_refreshAction);
@@ -408,11 +404,14 @@ bool LocatorWidget::eventFilter(QObject *obj, QEvent *event)
             }
         }
     } else if (obj == m_fileLineEdit && event->type() == QEvent::FocusOut) {
-#if defined(Q_OS_WIN)
-        QFocusEvent *fev = static_cast<QFocusEvent*>(event);
-        if (fev->reason() != Qt::ActiveWindowFocusReason ||
-            (fev->reason() == Qt::ActiveWindowFocusReason && !m_completionList->isActiveWindow()))
-#endif
+        bool hideList = true;
+        if (Utils::HostOsInfo::isWindowsHost()) {
+            QFocusEvent *fev = static_cast<QFocusEvent*>(event);
+            if (fev->reason() == Qt::ActiveWindowFocusReason &&
+                    !(fev->reason() == Qt::ActiveWindowFocusReason && !m_completionList->isActiveWindow()))
+                hideList = false;
+        }
+        if (hideList)
             m_completionList->hide();
     } else if (obj == m_fileLineEdit && event->type() == QEvent::FocusIn) {
         showPopupNow();
@@ -535,9 +534,8 @@ void LocatorWidget::updateEntries()
 
     const QList<FilterEntry> entries = m_entriesWatcher->future().results();
     m_locatorModel->setEntries(entries);
-    if (m_locatorModel->rowCount() > 0) {
+    if (m_locatorModel->rowCount() > 0)
         m_completionList->setCurrentIndex(m_locatorModel->index(0, 0));
-    }
 #if 0
     m_completionList->updatePreferredSize();
 #endif
@@ -577,6 +575,7 @@ void LocatorWidget::show(const QString &text, int selectionStart, int selectionL
         m_fileLineEdit->setFocus();
     else
         showPopupNow();
+    ICore::raiseWindow(ICore::mainWindow());
 
     if (selectionStart >= 0) {
         m_fileLineEdit->setSelection(selectionStart, selectionLength);
@@ -619,6 +618,5 @@ void LocatorWidget::showEvent(QShowEvent *event)
 
 void LocatorWidget::showConfigureDialog()
 {
-    Core::ICore::showOptionsDialog(QLatin1String(Core::Constants::SETTINGS_CATEGORY_CORE),
-          QLatin1String(Constants::FILTER_OPTIONS_PAGE));
+    ICore::showOptionsDialog(Core::Constants::SETTINGS_CATEGORY_CORE, Constants::FILTER_OPTIONS_PAGE);
 }
