@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -32,6 +32,7 @@
 #include "maemoqemusettings.h"
 
 #include <qtsupport/baseqtversion.h>
+#include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 
 #include <QDir>
@@ -94,7 +95,7 @@ MaemoQemuRuntime MaemoQemuRuntimeParser::parseRuntime(const QtSupport::BaseQtVer
         return runtime;
     if (!madProc.waitForStarted() || !madProc.waitForFinished())
         return runtime;
-    const QByteArray &madInfoOutput = madProc.readAllStandardOutput();
+    QString madInfoOutput = QString::fromLocal8Bit(madProc.readAllStandardOutput());
     const QString &targetName = MaemoGlobal::targetName(qtVersion->qmakeCommand().toString());
     runtime = MaemoQemuRuntimeParserV2(madInfoOutput, targetName, maddeRootPath)
         .parseRuntime();
@@ -127,7 +128,7 @@ MaemoQemuRuntime MaemoQemuRuntimeParserV1::parseRuntime()
                 && m_madInfoReader.name() == QLatin1String("target")) {
                 const QXmlStreamAttributes &attrs = m_madInfoReader.attributes();
                 if (attrs.value(QLatin1String("target_id")) == m_targetName)
-                    targetRuntime = attrs.value("runtime_id").toString();
+                    targetRuntime = attrs.value(QLatin1String("runtime_id")).toString();
             } else if (m_madInfoReader.name() == QLatin1String("runtime")) {
                 const QXmlStreamAttributes attrs = m_madInfoReader.attributes();
                 while (!m_madInfoReader.atEnd()) {
@@ -198,17 +199,17 @@ void MaemoQemuRuntimeParserV1::fillRuntimeInformation(MaemoQemuRuntime *runtime)
             // This is complex because of extreme MADDE weirdness.
             const QString root = m_maddeRoot + QLatin1Char('/');
             const bool pathIsRelative = QFileInfo(runtime->m_bin).isRelative();
-            runtime->m_bin =
-        #ifdef Q_OS_WIN
-                root + (pathIsRelative
-                    ? QLatin1String("madlib/") + runtime->m_bin // Fremantle.
-                    : runtime->m_bin)                           // Harmattan.
-                    + QLatin1String(".exe");
-        #else
-                pathIsRelative
-                    ? root + QLatin1String("madlib/") + runtime->m_bin // Fremantle.
-                    : runtime->m_bin;                                  // Harmattan.
-        #endif
+            if (Utils::HostOsInfo::isWindowsHost()) {
+                runtime->m_bin =
+                        root + (pathIsRelative
+                                ? QLatin1String("madlib/") + runtime->m_bin // Fremantle.
+                                : runtime->m_bin)                           // Harmattan.
+                        + QLatin1String(".exe");
+            } else {
+                runtime->m_bin = pathIsRelative
+                        ? root + QLatin1String("madlib/") + runtime->m_bin // Fremantle.
+                        : runtime->m_bin;                                  // Harmattan.
+            }
         }
     }
 }
@@ -335,17 +336,17 @@ void MaemoQemuRuntimeParserV2::handleEnvironmentTag(MaemoQemuRuntime &runtime)
     while (m_madInfoReader.readNextStartElement())
         handleVariableTag(runtime);
 
-#ifdef Q_OS_WIN
-    const QString root = QDir::toNativeSeparators(m_maddeRoot)
-        + QLatin1Char('/');
-    const QLatin1Char colon(';');
-    const QLatin1String key("PATH");
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    runtime.m_normalVars << MaemoQemuRuntime::Variable(key,
-        root + QLatin1String("bin") + colon + env.value(key));
-    runtime.m_normalVars << MaemoQemuRuntime::Variable(key,
-        root + QLatin1String("madlib") + colon + env.value(key));
-#endif
+    if (Utils::HostOsInfo::isWindowsHost()) {
+        const QString root = QDir::toNativeSeparators(m_maddeRoot)
+                + QLatin1Char('/');
+        const QLatin1Char colon(';');
+        const QLatin1String key("PATH");
+        QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+        runtime.m_normalVars << MaemoQemuRuntime::Variable(key,
+                root + QLatin1String("bin") + colon + env.value(key));
+        runtime.m_normalVars << MaemoQemuRuntime::Variable(key,
+                root + QLatin1String("madlib") + colon + env.value(key));
+    }
 }
 
 void MaemoQemuRuntimeParserV2::handleVariableTag(MaemoQemuRuntime &runtime)
@@ -378,11 +379,10 @@ void MaemoQemuRuntimeParserV2::handleVariableTag(MaemoQemuRuntime &runtime)
 
     if (varName.isEmpty())
         return;
-    if (isGlBackend) {
+    if (isGlBackend)
         runtime.m_openGlBackendVarName = varName;
-    } else {
+    else
         runtime.m_normalVars << MaemoQemuRuntime::Variable(varName, varValue);
-    }
 }
 
 QList<MaemoQemuRuntimeParserV2::Port> MaemoQemuRuntimeParserV2::handleTcpPortListTag()
