@@ -46,6 +46,7 @@
 #include <qtsupport/customexecutablerunconfiguration.h>
 #include <remotelinux/remotelinuxdebugsupport.h>
 #include <remotelinux/remotelinuxruncontrol.h>
+#include <utils/qtcassert.h>
 
 using namespace Debugger;
 using namespace ProjectExplorer;
@@ -110,8 +111,8 @@ bool MaemoRunConfigurationFactory::canClone(Target *parent,
         return false;
     const RemoteLinuxRunConfiguration * const rlrc
             = qobject_cast<RemoteLinuxRunConfiguration *>(source);
-    QString idStr = QString::fromLatin1(source->id().name()) + QLatin1Char('.') + rlrc->proFilePath();
-    return rlrc && canCreate(parent, Core::Id(idStr));
+    Core::Id id = source->id().withSuffix(QLatin1Char('.') + rlrc->projectFilePath());
+    return rlrc && canCreate(parent, id);
 }
 
 QList<Core::Id> MaemoRunConfigurationFactory::availableCreationIds(Target *parent) const
@@ -121,7 +122,7 @@ QList<Core::Id> MaemoRunConfigurationFactory::availableCreationIds(Target *paren
         return result;
     QStringList proFiles = static_cast<Qt4Project *>(parent->project())->applicationProFilePathes(QLatin1String(MAEMO_RC_ID_PREFIX));
     foreach (const QString &pf, proFiles)
-        result << Core::Id(pf);
+        result << Core::Id::fromString(pf);
     return result;
 }
 
@@ -131,27 +132,16 @@ QString MaemoRunConfigurationFactory::displayNameForId(const Core::Id id) const
         + QLatin1String(" (on remote Maemo device)");
 }
 
-RunConfiguration *MaemoRunConfigurationFactory::create(Target *parent, const Core::Id id)
+RunConfiguration *MaemoRunConfigurationFactory::doCreate(Target *parent, const Core::Id id)
 {
-    if (!canCreate(parent, id))
-        return 0;
     return new MaemoRunConfiguration(parent, id, pathFromId(id));
 }
 
-RunConfiguration *MaemoRunConfigurationFactory::restore(Target *parent,
+RunConfiguration *MaemoRunConfigurationFactory::doRestore(Target *parent,
     const QVariantMap &map)
 {
-    if (!canRestore(parent, map))
-        return 0;
-
     Core::Id id = ProjectExplorer::idFromMap(map);
-    MaemoRunConfiguration *rc
-        = new MaemoRunConfiguration(parent, id, pathFromId(id));
-    if (rc->fromMap(map))
-        return rc;
-
-    delete rc;
-    return 0;
+    return new MaemoRunConfiguration(parent, id, pathFromId(id));
 }
 
 RunConfiguration *MaemoRunConfigurationFactory::clone(Target *parent, RunConfiguration *source)
@@ -178,7 +168,7 @@ QList<RunConfiguration *> MaemoRunConfigurationFactory::runConfigurationsForNode
     QList<ProjectExplorer::RunConfiguration *> result;
     foreach (ProjectExplorer::RunConfiguration *rc, t->runConfigurations())
         if (MaemoRunConfiguration *mrc = qobject_cast<MaemoRunConfiguration *>(rc))
-            if (mrc->proFilePath() == n->path())
+            if (mrc->projectFilePath() == n->path())
                 result << rc;
     return result;
 }
@@ -195,40 +185,58 @@ MaemoRunControlFactory::~MaemoRunControlFactory()
 
 bool MaemoRunControlFactory::canRun(RunConfiguration *runConfiguration, RunMode mode) const
 {
+    if (mode != NormalRunMode && mode != DebugRunMode && mode != DebugRunModeWithBreakOnMain)
+        return false;
     const MaemoRunConfiguration * const maemoRunConfig
         = qobject_cast<MaemoRunConfiguration *>(runConfiguration);
-    if (!maemoRunConfig || !maemoRunConfig->isEnabled())
-        return false;
-    return maemoRunConfig->hasEnoughFreePorts(mode);
+    return maemoRunConfig && maemoRunConfig->isEnabled();
 }
 
-RunControl* MaemoRunControlFactory::create(RunConfiguration *runConfig, RunMode mode, QString *errorMessage)
+RunControl* MaemoRunControlFactory::create(RunConfiguration *runConfig, RunMode mode,
+                                           QString *errorMessage)
 {
-    Q_ASSERT(canRun(runConfig, mode));
+    QTC_ASSERT(canRun(runConfig, mode), return 0);
 
     MaemoRunConfiguration *rc = qobject_cast<MaemoRunConfiguration *>(runConfig);
-    Q_ASSERT(rc);
-
-    if (mode == NormalRunMode) {
+    QTC_ASSERT(rc, return 0);
+    switch (mode) {
+    case NormalRunMode: {
         RemoteLinuxRunControl * const runControl = new RemoteLinuxRunControl(rc);
         setHelperActions(runControl, rc, runControl);
         return runControl;
     }
+    case DebugRunMode:
+    case DebugRunModeWithBreakOnMain: {
+        IDevice::ConstPtr dev = DeviceKitInformation::device(rc->target()->kit());
+        if (!dev) {
+            *errorMessage = tr("Cannot debug: Kit has no device.");
+            return 0;
+        }
+        if (rc->portsUsedByDebuggers() > dev->freePorts().count()) {
+            *errorMessage = tr("Cannot debug: Not enough free ports available.");
+            return 0;
+        }
+        DebuggerStartParameters params = LinuxDeviceDebugSupport::startParameters(rc);
+        if (mode == ProjectExplorer::DebugRunModeWithBreakOnMain)
+            params.breakOnMain = true;
+        DebuggerRunControl * const runControl
+                = DebuggerPlugin::createDebugger(params, rc, errorMessage);
+        if (!runControl)
+            return 0;
+        LinuxDeviceDebugSupport * const debugSupport =
+                new LinuxDeviceDebugSupport(rc, runControl->engine());
+        setHelperActions(debugSupport, rc, runControl);
+        connect(runControl, SIGNAL(finished()), debugSupport, SLOT(handleDebuggingFinished()));
+        return runControl;
+    }
+    case NoRunMode:
+    case QmlProfilerRunMode:
+    case CallgrindRunMode:
+    case MemcheckRunMode:
+        QTC_ASSERT(false, return 0);
+    }
 
-    const DebuggerStartParameters params = LinuxDeviceDebugSupport::startParameters(rc);
-    DebuggerRunControl * const runControl = DebuggerPlugin::createDebugger(params, rc, errorMessage);
-    if (!runControl)
-        return 0;
-    LinuxDeviceDebugSupport * const debugSupport
-            = new LinuxDeviceDebugSupport(rc, runControl->engine());
-    setHelperActions(debugSupport, rc, runControl);
-    connect(runControl, SIGNAL(finished()), debugSupport, SLOT(handleDebuggingFinished()));
-    return runControl;
-}
-
-QString MaemoRunControlFactory::displayName() const
-{
-    return tr("Run on device");
+    QTC_ASSERT(false, return 0);
 }
 
     } // namespace Internal
