@@ -28,26 +28,21 @@
 ****************************************************************************/
 
 #include "appoutputpane.h"
-#include "projectexplorerconstants.h"
 #include "projectexplorer.h"
 #include "projectexplorersettings.h"
 #include "runconfiguration.h"
 #include "session.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/actionmanager/command.h>
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/icore.h>
-#include <coreplugin/icontext.h>
+#include <coreplugin/outputwindow.h>
 #include <find/basetextfind.h>
-#include <aggregation/aggregate.h>
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorsettings.h>
 #include <extensionsystem/pluginmanager.h>
 #include <extensionsystem/invoker.h>
 
 #include <utils/qtcassert.h>
-#include <utils/outputformatter.h>
 
 #include <QAction>
 #include <QVBoxLayout>
@@ -106,7 +101,7 @@ void TabWidget::slotContextMenuRequested(const QPoint &pos)
 }
 
 AppOutputPane::RunControlTab::RunControlTab(RunControl *rc, Core::OutputWindow *w) :
-    runControl(rc), window(w), asyncClosing(false)
+    runControl(rc), window(w), asyncClosing(false), behavivorOnOutput(Flash)
 {
 }
 
@@ -121,6 +116,8 @@ AppOutputPane::AppOutputPane() :
     m_stopButton(new QToolButton),
     m_attachButton(new QToolButton)
 {
+    setObjectName(QLatin1String("AppOutputPane")); // Used in valgrind engine
+
     // Rerun
     m_reRunButton->setIcon(QIcon(QLatin1String(ProjectExplorer::Constants::ICON_RUN_SMALL)));
     m_reRunButton->setToolTip(tr("Re-run this run-configuration"));
@@ -324,7 +321,8 @@ void AppOutputPane::createNewOutputWindow(RunControl *rc)
     }
     // Create new
     static uint counter = 0;
-    Core::Context context(Constants::C_APP_OUTPUT, counter++);
+    Core::Id contextId = Core::Id(Constants::C_APP_OUTPUT).withSuffix(counter++);
+    Core::Context context(contextId);
     Core::OutputWindow *ow = new Core::OutputWindow(context, m_tabWidget);
     ow->setWindowTitle(tr("Application Output Window"));
     ow->setWindowIcon(QIcon(QLatin1String(Constants::ICON_WINDOW)));
@@ -362,8 +360,16 @@ void AppOutputPane::updateFromSettings()
 void AppOutputPane::appendMessage(RunControl *rc, const QString &out, Utils::OutputFormat format)
 {
     const int index = indexOf(rc);
-    if (index != -1)
-        m_runControlTabs.at(index).window->appendMessage(out, format);
+    if (index != -1) {
+        Core::OutputWindow *window = m_runControlTabs.at(index).window;
+        window->appendMessage(out, format);
+        if (format != Utils::NormalMessageFormat) {
+            if (m_runControlTabs.at(index).behavivorOnOutput == Flash)
+                flash();
+            else
+                popup(NoModeSwitch);
+        }
+    }
 }
 
 void AppOutputPane::showTabFor(RunControl *rc)
@@ -371,6 +377,13 @@ void AppOutputPane::showTabFor(RunControl *rc)
     m_tabWidget->setCurrentIndex(tabWidgetIndexOf(indexOf(rc)));
 }
 
+void AppOutputPane::setBehaviorOnOutput(RunControl *rc, AppOutputPane::BehavivorOnOutput mode)
+{
+    const int index = indexOf(rc);
+    if (index != -1)
+        m_runControlTabs[index].behavivorOnOutput = mode;
+}
+
 void AppOutputPane::reRunRunControl()
 {
     const int index = currentIndex();
