@@ -30,10 +30,10 @@
 #ifndef DEVICEFACTORYSELECTIONDIALOG_H
 #define DEVICEFACTORYSELECTIONDIALOG_H
 
-#include <coreplugin/id.h>
-
 #include <QDialog>
 
+namespace Core { class Id; }
+
 namespace ProjectExplorer {
 class IDeviceFactory;
 
