@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -30,6 +30,7 @@
 #include "environmentitemswidget.h"
 
 #include <utils/environment.h>
+#include <utils/hostosinfo.h>
 #include <texteditor/snippets/snippeteditor.h>
 #include <texteditor/texteditorsettings.h>
 
@@ -53,9 +54,8 @@ QList<Utils::EnvironmentItem> EnvironmentItemsWidgetPrivate::cleanUp(
     QSet<QString> uniqueSet;
     for (int i = items.count() - 1; i >= 0; i--) {
         Utils::EnvironmentItem item = items.at(i);
-#if defined(Q_OS_WIN)
-        item.name = item.name.toUpper();
-#endif
+        if (Utils::HostOsInfo::isWindowsHost())
+            item.name = item.name.toUpper();
         const QString &itemName = item.name;
         QString emptyName = itemName;
         emptyName.remove(QLatin1Char(' '));
