@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef GCCTOOLCHAINFACTORIES_H
 #define GCCTOOLCHAINFACTORIES_H
@@ -38,7 +35,7 @@
 #include "abi.h"
 #include "abiwidget.h"
 
-#include <QtCore/QList>
+#include <QList>
 
 QT_BEGIN_NAMESPACE
 class QComboBox;
@@ -74,7 +71,6 @@ public:
 protected:
     virtual GccToolChain *createToolChain(bool autoDetect);
     QList<ToolChain *> autoDetectToolchains(const QString &compiler,
-                                            const QStringList &debuggers,
                                             const Abi &);
 };
 
@@ -88,21 +84,24 @@ class GccToolChainConfigWidget : public ToolChainConfigWidget
 
 public:
     GccToolChainConfigWidget(GccToolChain *);
-    void apply();
-    void discard() { setFromToolchain(); }
-    bool isDirty() const;
-    void makeReadOnly();
-
+    static QStringList splitString(const QString &s);
 private slots:
-    void handlePathChange();
-    void handleAbiChange();
+    void handleCompilerCommandChange();
+    void handlePlatformCodeGenFlagsChange();
+    void handlePlatformLinkerFlagsChange();
 
 private:
+    void applyImpl();
+    void discardImpl() { setFromToolchain(); }
+    bool isDirtyImpl() const;
+    void makeReadOnlyImpl();
+
     void setFromToolchain();
 
-    Utils::PathChooser *m_compilerPath;
+    Utils::PathChooser *m_compilerCommand;
+    QLineEdit *m_platformCodeGenFlagsLineEdit;
+    QLineEdit *m_platformLinkerFlagsLineEdit;
     AbiWidget *m_abiWidget;
-    QString m_autoDebuggerCommand;
 
     QList<Abi> m_abiList;
     bool m_isReadOnly;
