@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -29,8 +29,6 @@
 
 #include "projectwelcomepage.h"
 
-#include "projectexplorerconstants.h"
-
 #include <utils/stringutils.h>
 
 #include <QDeclarativeEngine>
@@ -38,10 +36,16 @@
 #include <QFileInfo>
 #include <QDir>
 
+#include <coreplugin/icore.h>
+#include <coreplugin/dialogs/iwizard.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/projectexplorer.h>
 #include <sessiondialog.h>
 
+#ifdef Q_OS_WIN
+#include <utils/winutils.h>
+#endif
+
 namespace ProjectExplorer {
 namespace Internal {
 
@@ -110,7 +114,8 @@ bool SessionModel::isDefaultVirgin() const
 
 void SessionModel::resetSessions()
 {
-    reset();
+    beginResetModel();
+    endResetModel();
 }
 
 void SessionModel::cloneSession(const QString &session)
@@ -123,19 +128,22 @@ void SessionModel::cloneSession(const QString &session)
         QString newSession = newSessionInputDialog.value();
         if (newSession.isEmpty() || m_manager->sessions().contains(newSession))
             return;
+        beginResetModel();
         m_manager->cloneSession(session, newSession);
-        reset();
+        endResetModel();
 
-        if (newSessionInputDialog.isSwitchToRequested()) {
+        if (newSessionInputDialog.isSwitchToRequested())
             m_manager->loadSession(newSession);
-        }
     }
 }
 
 void SessionModel::deleteSession(const QString &session)
 {
+    if (!m_manager->confirmSessionDelete(session))
+        return;
+    beginResetModel();
     m_manager->deleteSession(session);
-    reset();
+    endResetModel();
 }
 
 void SessionModel::renameSession(const QString &session)
@@ -148,12 +156,12 @@ void SessionModel::renameSession(const QString &session)
         QString newSession = newSessionInputDialog.value();
         if (newSession.isEmpty() || m_manager->sessions().contains(newSession))
             return;
+        beginResetModel();
         m_manager->renameSession(session, newSession);
-        reset();
+        endResetModel();
 
-        if (newSessionInputDialog.isSwitchToRequested()) {
+        if (newSessionInputDialog.isSwitchToRequested())
             m_manager->loadSession(newSession);
-        }
     }
 }
 
@@ -193,7 +201,8 @@ QVariant ProjectModel::data(const QModelIndex &index, int role) const
 
 void ProjectModel::resetProjects()
 {
-    reset();
+    beginResetModel();
+    endResetModel();
 }
 
 ///////////////////
@@ -215,6 +224,16 @@ void ProjectWelcomePage::facilitateQml(QDeclarativeEngine *engine)
     ctx->setContextProperty(QLatin1String("projectWelcomePage"), this);
 }
 
+QUrl ProjectWelcomePage::pageLocation() const
+{
+    QString resourcePath = Core::ICore::resourcePath();
+#ifdef Q_OS_WIN
+    // normalize paths so QML doesn't freak out if it's wrongly capitalized on Windows
+    resourcePath = Utils::normalizePathName(resourcePath);
+#endif
+     return QUrl::fromLocalFile(resourcePath + QLatin1String("/welcomescreen/develop.qml"));
+}
+
 ProjectWelcomePage::Id ProjectWelcomePage::id() const
 {
     return Develop;
@@ -228,5 +247,16 @@ void ProjectWelcomePage::reloadWelcomeScreenData()
         m_projectModel->resetProjects();
 }
 
+void ProjectWelcomePage::newProject()
+{
+    Core::ICore::showNewItemDialog(tr("New Project"),
+                                   Core::IWizard::wizardsOfKind(Core::IWizard::ProjectWizard));
+}
+
+void ProjectWelcomePage::openProject()
+{
+     ProjectExplorerPlugin::instance()->openOpenProjectDialog();
+}
+
 } // namespace Internal
 } // namespace ProjectExplorer
