@@ -1,158 +1,70 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "runconfiguration.h"
 
 #include "project.h"
 #include "target.h"
 #include "toolchain.h"
+#include "abi.h"
 #include "buildconfiguration.h"
-#include "projectexplorerconstants.h"
+#include "kitinformation.h"
 #include <extensionsystem/pluginmanager.h>
 
-#include <utils/qtcassert.h>
+#include <utils/outputformatter.h>
 #include <utils/checkablemessagebox.h>
 
 #include <coreplugin/icore.h>
+#include <coreplugin/icontext.h>
 
-#include <QtCore/QTimer>
-#include <QtCore/QSettings>
-#include <QtGui/QMainWindow>
-#include <QtGui/QMessageBox>
-#include <QtGui/QPushButton>
+#include <QTimer>
+#include <QPushButton>
 
 #ifdef Q_OS_MAC
-#include <Carbon/Carbon.h>
+#include <ApplicationServices/ApplicationServices.h>
 #endif
 
 using namespace ProjectExplorer;
 
 namespace {
-// Function objects:
 
-const char * const USE_CPP_DEBUGGER_KEY("RunConfiguration.UseCppDebugger");
-const char * const USE_QML_DEBUGGER_KEY("RunConfiguration.UseQmlDebugger");
-const char * const USE_QML_DEBUGGER_AUTO_KEY("RunConfiguration.UseQmlDebuggerAuto");
-const char * const QML_DEBUG_SERVER_PORT_KEY("RunConfiguration.QmlDebugServerPort");
-
-class RunConfigurationFactoryMatcher
-{
-public:
-    RunConfigurationFactoryMatcher(Target * target) : m_target(target)
-    { }
-
-    virtual ~RunConfigurationFactoryMatcher() { }
-
-    virtual bool operator()(IRunConfigurationFactory *) const = 0;
-
-    Target *target() const
-    {
-        return m_target;
-    }
-
-private:
-    Target *m_target;
-};
-
-class CreateMatcher : public RunConfigurationFactoryMatcher
-{
-public:
-    CreateMatcher(Target *target, const QString &id) :
-        RunConfigurationFactoryMatcher(target),
-        m_id(id)
-    { }
-
-    bool operator()(IRunConfigurationFactory *factory) const
-    {
-        return factory->canCreate(target(), m_id);
-    }
-
-private:
-    QString m_id;
-};
-
-class CloneMatcher : public RunConfigurationFactoryMatcher
-{
-public:
-    CloneMatcher(Target *target, RunConfiguration *source) :
-        RunConfigurationFactoryMatcher(target),
-        m_source(source)
-    { }
-
-    bool operator()(IRunConfigurationFactory *factory) const
-    {
-        return factory->canClone(target(), m_source);
-    }
-
-private:
-    RunConfiguration *m_source;
-};
-
-class RestoreMatcher : public RunConfigurationFactoryMatcher
-{
-public:
-    RestoreMatcher(Target *target, const QVariantMap &map) :
-        RunConfigurationFactoryMatcher(target),
-        m_map(map)
-    { }
-
-    bool operator()(IRunConfigurationFactory *factory) const
-    {
-        return factory->canRestore(target(), m_map);
-    }
-
-private:
-    QVariantMap m_map;
-};
-
-// Helper methods:
-
-IRunConfigurationFactory *findRunConfigurationFactory(RunConfigurationFactoryMatcher &matcher)
-{
-    QList<IRunConfigurationFactory *> factories
-        = ExtensionSystem::PluginManager::instance()->
-                getObjects<IRunConfigurationFactory>();
-    foreach (IRunConfigurationFactory *factory, factories) {
-        if (matcher(factory))
-            return factory;
-    }
-    return 0;
-}
+const char USE_CPP_DEBUGGER_KEY[] = "RunConfiguration.UseCppDebugger";
+const char USE_QML_DEBUGGER_KEY[] = "RunConfiguration.UseQmlDebugger";
+const char USE_QML_DEBUGGER_AUTO_KEY[] = "RunConfiguration.UseQmlDebuggerAuto";
+const char QML_DEBUG_SERVER_PORT_KEY[] = "RunConfiguration.QmlDebugServerPort";
+const char USE_MULTIPROCESS_KEY[] = "RunConfiguration.UseMultiProcess";
 
 } // namespace
 
 /*!
     \class ProjectExplorer::ProcessHandle
-    \brief  Helper class to describe a process.
+    \brief The ProcessHandle class is a helper class to describe a process.
 
     Encapsulates parameters of a running process, local (PID) or remote (to be done,
     address, port, etc).
@@ -191,38 +103,53 @@ bool ProcessHandle::equals(const ProcessHandle &rhs) const
     return m_pid == rhs.m_pid;
 }
 
+
+/*!
+    \brief Returns the widget used to configure this run configuration. Ownership is transferred to the caller
+*/
+RunConfigWidget *IRunConfigurationAspect::createConfigurationWidget()
+{
+    return 0;
+}
+
+
 /*!
     \class ProjectExplorer::RunConfiguration
-    \brief  Base class for a run configuration. A run configuration specifies how a
-    target should be run, while the runner (see below) does the actual running.
+    \brief The RunConfiguration class is the base class for a run configuration.
 
-    Note that all RunControls and the target hold a shared pointer to the RunConfiguration.
-    That is the lifetime of the RunConfiguration might exceed the life of the target.
+    A run configuration specifies how a target should be run, while a runner
+    does the actual running.
+
+    All RunControls and the target hold a shared pointer to the run
+    configuration. That is, the lifetime of the run configuration might exceed
+    the life of the target.
     The user might still have a RunControl running (or output tab of that RunControl open)
     and yet unloaded the target.
 
-    Also, a RunConfiguration might be already removed from the list of RunConfigurations
+    Also, a run configuration might be already removed from the list of run
+    configurations
     for a target, but still be runnable via the output tab.
 */
 
-RunConfiguration::RunConfiguration(Target *target, const QString &id) :
+RunConfiguration::RunConfiguration(Target *target, const Core::Id id) :
     ProjectConfiguration(target, id),
-    m_useCppDebugger(true),
-    m_useQmlDebugger(AutoEnableQmlDebugger),
-    m_qmlDebugServerPort(Constants::QML_DEFAULT_DEBUG_SERVER_PORT)
+    m_aspectsInitialized(false)
 {
     Q_ASSERT(target);
-    addExtraAspects();
+    ctor();
 }
 
 RunConfiguration::RunConfiguration(Target *target, RunConfiguration *source) :
     ProjectConfiguration(target, source),
-    m_useCppDebugger(source->m_useCppDebugger),
-    m_useQmlDebugger(source->m_useQmlDebugger),
-    m_qmlDebugServerPort(source->m_qmlDebugServerPort)
+    m_aspectsInitialized(true)
 {
     Q_ASSERT(target);
-    addExtraAspects();
+    ctor();
+    foreach (IRunConfigurationAspect *aspect, source->m_aspects) {
+        IRunConfigurationAspect *clone = aspect->clone(this);
+        if (clone)
+            m_aspects.append(clone);
+    }
 }
 
 RunConfiguration::~RunConfiguration()
@@ -232,10 +159,23 @@ RunConfiguration::~RunConfiguration()
 
 void RunConfiguration::addExtraAspects()
 {
-    ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
-    foreach (IRunControlFactory *factory, pm->getObjects<IRunControlFactory>())
-        if (IRunConfigurationAspect *aspect = factory->createRunConfigurationAspect())
-            m_aspects.append(aspect);
+    if (m_aspectsInitialized)
+        return;
+
+    foreach (IRunControlFactory *factory, ExtensionSystem::PluginManager::getObjects<IRunControlFactory>())
+        addExtraAspect(factory->createRunConfigurationAspect(this));
+    m_aspectsInitialized = true;
+}
+
+void RunConfiguration::addExtraAspect(IRunConfigurationAspect *aspect)
+{
+    if (aspect)
+        m_aspects += aspect;
+}
+
+void RunConfiguration::ctor()
+{
+    connect(this, SIGNAL(enabledChanged()), this, SIGNAL(requestRunActionsUpdate()));
 }
 
 /*!
@@ -252,11 +192,20 @@ QString RunConfiguration::disabledReason() const
     return QString();
 }
 
-/*!
-    \fn virtual QWidget *ProjectExplorer::RunConfiguration::createConfigurationWidget()
+bool RunConfiguration::isConfigured() const
+{
+    return true;
+}
+
+bool RunConfiguration::ensureConfigured(QString *errorMessage)
+{
+    if (isConfigured())
+        return true;
+    if (errorMessage)
+        *errorMessage = tr("Unknown error.");
+    return false;
+}
 
-    \brief Returns the widget used to configure this run configuration. Ownership is transferred to the caller
-*/
 
 BuildConfiguration *RunConfiguration::activeBuildConfiguration() const
 {
@@ -270,65 +219,10 @@ Target *RunConfiguration::target() const
     return static_cast<Target *>(parent());
 }
 
-void RunConfiguration::setUseQmlDebugger(bool value)
-{
-    if (value) {
-        m_useQmlDebugger = EnableQmlDebugger;
-    } else {
-        m_useQmlDebugger = DisableQmlDebugger;
-    }
-    emit debuggersChanged();
-}
-
-void RunConfiguration::setUseCppDebugger(bool value)
-{
-    m_useCppDebugger = value;
-    emit debuggersChanged();
-}
-
-bool RunConfiguration::useCppDebugger() const
-{
-    return m_useCppDebugger;
-}
-
-static bool isQtQuickAppProject(Project *project)
-{
-    const QString &filePath = project->projectDirectory()
-            +QLatin1String("/qmlapplicationviewer/qmlapplicationviewer.pri");
-    return project->files(Project::ExcludeGeneratedFiles).contains(filePath);
-}
-
-bool RunConfiguration::useQmlDebugger() const
-{
-    if (m_useQmlDebugger == AutoEnableQmlDebugger) {
-        if (isQtQuickAppProject(target()->project())) {
-            m_useQmlDebugger = EnableQmlDebugger;
-        } else {
-            m_useQmlDebugger = DisableQmlDebugger;
-        }
-    }
-
-    return (m_useQmlDebugger == EnableQmlDebugger);
-}
-
-uint RunConfiguration::qmlDebugServerPort() const
-{
-    return m_qmlDebugServerPort;
-}
-
-void RunConfiguration::setQmlDebugServerPort(uint port)
-{
-    m_qmlDebugServerPort = port;
-    emit qmlDebugServerPortChanged(port);
-}
-
 QVariantMap RunConfiguration::toMap() const
 {
     QVariantMap map = ProjectConfiguration::toMap();
-    map.insert(QLatin1String(USE_CPP_DEBUGGER_KEY), m_useCppDebugger);
-    map.insert(QLatin1String(USE_QML_DEBUGGER_KEY), m_useQmlDebugger == EnableQmlDebugger);
-    map.insert(QLatin1String(USE_QML_DEBUGGER_AUTO_KEY), m_useQmlDebugger == AutoEnableQmlDebugger);
-    map.insert(QLatin1String(QML_DEBUG_SERVER_PORT_KEY), m_qmlDebugServerPort);
+
     foreach (IRunConfigurationAspect *aspect, m_aspects)
         map.unite(aspect->toMap());
 
@@ -340,7 +234,7 @@ ProjectExplorer::Abi RunConfiguration::abi() const
     BuildConfiguration *bc = target()->activeBuildConfiguration();
     if (!bc)
         return Abi::hostAbi();
-    ToolChain *tc = bc->toolChain();
+    ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
     if (!tc)
         return Abi::hostAbi();
     return tc->targetAbi();
@@ -348,20 +242,10 @@ ProjectExplorer::Abi RunConfiguration::abi() const
 
 bool RunConfiguration::fromMap(const QVariantMap &map)
 {
-    m_useCppDebugger = map.value(QLatin1String(USE_CPP_DEBUGGER_KEY), true).toBool();
-    if (map.value(QLatin1String(USE_QML_DEBUGGER_AUTO_KEY), false).toBool()) {
-        m_useQmlDebugger = AutoEnableQmlDebugger;
-    } else {
-        if (map.value(QLatin1String(USE_QML_DEBUGGER_KEY), false).toBool()) {
-            m_useQmlDebugger = EnableQmlDebugger;
-        } else {
-            m_useQmlDebugger = DisableQmlDebugger;
-        }
-    }
+    addExtraAspects();
 
     foreach (IRunConfigurationAspect *aspect, m_aspects)
-        if (!aspect->fromMap(map))
-            return false;
+        aspect->fromMap(map);
 
     return ProjectConfiguration::fromMap(map);
 }
@@ -369,7 +253,8 @@ bool RunConfiguration::fromMap(const QVariantMap &map)
 /*!
     \class ProjectExplorer::IRunConfigurationAspect
 
-    \brief Extra configuration aspect.
+    \brief The IRunConfigurationAspect class provides an additional
+    configuration aspect.
 
     Aspects are a mechanism to add RunControl-specific options to a RunConfiguration without
     subclassing the RunConfiguration for every addition, preventing a combinatorical explosion
@@ -384,6 +269,7 @@ bool RunConfiguration::fromMap(const QVariantMap &map)
 
 QList<IRunConfigurationAspect *> RunConfiguration::extraAspects() const
 {
+    QTC_ASSERT(m_aspectsInitialized, return QList<IRunConfigurationAspect *>());
     return m_aspects;
 }
 
@@ -396,7 +282,8 @@ Utils::OutputFormatter *RunConfiguration::createOutputFormatter() const
 /*!
     \class ProjectExplorer::IRunConfigurationFactory
 
-    \brief Restores RunConfigurations from settings.
+    \brief The IRunConfigurationFactory class restores run configurations from
+    settings.
 
     The run configuration factory is used for restoring run configurations from
     settings. And used to create new runconfigurations in the "Run Settings" Dialog.
@@ -427,28 +314,68 @@ IRunConfigurationFactory::~IRunConfigurationFactory()
 {
 }
 
-IRunConfigurationFactory *IRunConfigurationFactory::createFactory(Target *parent, const QString &id)
+RunConfiguration *IRunConfigurationFactory::create(Target *parent, const Core::Id id)
 {
-    CreateMatcher matcher(parent, id);
-    return findRunConfigurationFactory(matcher);
+    if (!canCreate(parent, id))
+        return 0;
+    RunConfiguration *rc = doCreate(parent, id);
+    if (!rc)
+        return 0;
+    rc->addExtraAspects();
+    return rc;
 }
 
-IRunConfigurationFactory *IRunConfigurationFactory::cloneFactory(Target *parent, RunConfiguration *source)
+RunConfiguration *IRunConfigurationFactory::restore(Target *parent, const QVariantMap &map)
 {
-    CloneMatcher matcher(parent, source);
-    return findRunConfigurationFactory(matcher);
+    if (!canRestore(parent, map))
+        return 0;
+    RunConfiguration *rc = doRestore(parent, map);
+    if (!rc->fromMap(map)) {
+        delete rc;
+        rc = 0;
+    }
+    return rc;
 }
 
-IRunConfigurationFactory *IRunConfigurationFactory::restoreFactory(Target *parent, const QVariantMap &map)
+IRunConfigurationFactory *IRunConfigurationFactory::find(Target *parent, const QVariantMap &map)
 {
-    RestoreMatcher matcher(parent, map);
-    return findRunConfigurationFactory(matcher);
+    QList<IRunConfigurationFactory *> factories
+            = ExtensionSystem::PluginManager::instance()->getObjects<IRunConfigurationFactory>();
+    foreach (IRunConfigurationFactory *factory, factories) {
+        if (factory->canRestore(parent, map))
+            return factory;
+    }
+    return 0;
+}
+
+IRunConfigurationFactory *IRunConfigurationFactory::find(Target *parent, RunConfiguration *rc)
+{
+    QList<IRunConfigurationFactory *> factories
+            = ExtensionSystem::PluginManager::instance()->getObjects<IRunConfigurationFactory>();
+    foreach (IRunConfigurationFactory *factory, factories) {
+        if (factory->canClone(parent, rc))
+            return factory;
+    }
+    return 0;
+}
+
+QList<IRunConfigurationFactory *> IRunConfigurationFactory::find(Target *parent)
+{
+    QList<IRunConfigurationFactory *> factories
+            = ExtensionSystem::PluginManager::instance()->getObjects<IRunConfigurationFactory>();
+    QList<IRunConfigurationFactory *> result;
+    foreach (IRunConfigurationFactory *factory, factories) {
+        if (!factory->availableCreationIds(parent).isEmpty())
+            result << factory;
+    }
+    return result;
 }
 
 /*!
     \class ProjectExplorer::IRunControlFactory
 
-    \brief Creates RunControl objects matching a RunConfiguration
+    \brief The IRunControlFactory class creates RunControl objects matching a
+    run configuration.
 */
 
 /*!
@@ -478,14 +405,15 @@ IRunControlFactory::~IRunControlFactory()
 {
 }
 
-IRunConfigurationAspect *IRunControlFactory::createRunConfigurationAspect()
+IRunConfigurationAspect *IRunControlFactory::createRunConfigurationAspect(RunConfiguration *rc)
 {
+    Q_UNUSED(rc);
     return 0;
 }
 
 /*!
     \class ProjectExplorer::RunControl
-    \brief Each instance of this class represents one item that is run.
+    \brief The RunControl class instances represent one item that is run.
 */
 
 /*!
@@ -502,7 +430,7 @@ IRunConfigurationAspect *IRunControlFactory::createRunConfigurationAspect()
     TODO the icon differs currently only per "mode", so this is more flexible then it needs to be.
 */
 
-RunControl::RunControl(RunConfiguration *runConfiguration, QString mode)
+RunControl::RunControl(RunConfiguration *runConfiguration, RunMode mode)
     : m_runMode(mode), m_runConfiguration(runConfiguration), m_outputFormatter(0)
 {
     if (runConfiguration) {
@@ -524,7 +452,7 @@ Utils::OutputFormatter *RunControl::outputFormatter()
     return m_outputFormatter;
 }
 
-QString RunControl::runMode() const
+RunMode RunControl::runMode() const
 {
     return m_runMode;
 }
@@ -534,6 +462,18 @@ QString RunControl::displayName() const
     return m_displayName;
 }
 
+Abi RunControl::abi() const
+{
+    if (const RunConfiguration *rc = m_runConfiguration.data())
+        return rc->abi();
+    return Abi();
+}
+
+RunConfiguration *RunControl::runConfiguration() const
+{
+    return m_runConfiguration.data();
+}
+
 ProcessHandle RunControl::applicationProcessHandle() const
 {
     return m_applicationProcessHandle;
@@ -549,7 +489,7 @@ void RunControl::setApplicationProcessHandle(const ProcessHandle &handle)
 
 bool RunControl::promptToStop(bool *optionalPrompt) const
 {
-    QTC_ASSERT(isRunning(), return true;)
+    QTC_ASSERT(isRunning(), return true);
 
     if (optionalPrompt && !*optionalPrompt)
         return true;
@@ -571,10 +511,10 @@ bool RunControl::showPromptToStopDialog(const QString &title,
                                         const QString &cancelButtonText,
                                         bool *prompt) const
 {
-    QTC_ASSERT(isRunning(), return true;)
+    QTC_ASSERT(isRunning(), return true);
     // Show a question message box where user can uncheck this
     // question for this class.
-    Utils::CheckableMessageBox messageBox(Core::ICore::instance()->mainWindow());
+    Utils::CheckableMessageBox messageBox(Core::ICore::mainWindow());
     messageBox.setWindowTitle(title);
     messageBox.setText(text);
     messageBox.setStandardButtons(QDialogButtonBox::Yes|QDialogButtonBox::Cancel);
