@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -31,36 +31,27 @@
 
 #include "project.h"
 #include "projectexplorer.h"
-#include "projectexplorerconstants.h"
 #include "nodesvisitor.h"
 #include "editorconfiguration.h"
 #include "projectnodes.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/imode.h>
-#include <coreplugin/documentmanager.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/modemanager.h>
 
 #include <texteditor/itexteditor.h>
 
-#include <utils/listutils.h>
-#include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
 
 #include <QDebug>
 #include <QDir>
 #include <QFileInfo>
-#include <QFuture>
-#include <QSettings>
-#include <QTimer>
 
-#include <QApplication>
 #include <QMessageBox>
 #include <QPushButton>
-#include <QTextCodec>
 
 namespace {
     bool debug = false;
@@ -76,7 +67,7 @@ using namespace ProjectExplorer::Internal;
 /*!
      \class ProjectExplorer::SessionManager
 
-     \brief Session management.
+     \brief The SessionManager class manages sessions.
 
      TODO the interface of this class is not really great.
      The implementation suffers that all the functions from the
@@ -220,11 +211,10 @@ void SessionManager::removeDependency(Project *project, Project *depProject)
 
     QStringList proDeps = m_depMap.value(proName);
     proDeps.removeAll(depName);
-    if (proDeps.isEmpty()) {
+    if (proDeps.isEmpty())
         m_depMap.remove(proName);
-    } else {
+    else
         m_depMap[proName] = proDeps;
-    }
     emit dependencyChanged(project, depProject);
 }
 
@@ -316,6 +306,15 @@ bool SessionManager::save()
     if (m_startupProject)
         data.insert(QLatin1String("StartupProject"), m_startupProject->document()->fileName());
 
+    QColor c = Utils::StyleHelper::requestedBaseColor();
+    if (c.isValid()) {
+        QString tmp = QString::fromLatin1("#%1%2%3")
+                .arg(c.red(), 2, 16, QLatin1Char('0'))
+                .arg(c.green(), 2, 16, QLatin1Char('0'))
+                .arg(c.blue(), 2, 16, QLatin1Char('0'));
+        data.insert(QLatin1String("Color"), tmp);
+    }
+
     QStringList projectFiles;
     foreach (Project *pro, m_projects)
         projectFiles << pro->document()->fileName();
@@ -374,8 +373,6 @@ bool SessionManager::save()
 }
 
 /*!
-  \fn bool SessionManager::closeAllProjects()
-
   Closes all projects
   */
 void SessionManager::closeAllProjects()
@@ -392,13 +389,21 @@ const QList<Project *> &SessionManager::projects() const
 QStringList SessionManager::dependencies(const QString &proName) const
 {
     QStringList result;
-    foreach (const QString &dep, m_depMap.value(proName))
-        result += dependencies(dep);
-
-    result << proName;
+    dependencies(proName, result);
     return result;
 }
 
+void SessionManager::dependencies(const QString &proName, QStringList &result) const
+{
+    QStringList depends = m_depMap.value(proName);
+
+    foreach (const QString &dep, depends)
+        dependencies(dep, result);
+
+    if (!result.contains(proName))
+        result.append(proName);
+}
+
 QStringList SessionManager::dependenciesOrder() const
 {
     QList<QPair<QString, QStringList> > unordered;
@@ -438,11 +443,10 @@ QList<Project *> SessionManager::projectOrder(Project *project) const
     QList<Project *> result;
 
     QStringList pros;
-    if (project) {
+    if (project)
         pros = dependencies(project->document()->fileName());
-    } else {
+    else
         pros = dependenciesOrder();
-    }
 
     foreach (const QString &proFile, pros) {
         foreach (Project *pro, projects()) {
@@ -675,6 +679,18 @@ bool SessionManager::renameSession(const QString &original, const QString &newNa
     return deleteSession(original);
 }
 
+
+/*!
+    \brief Shows a dialog asking the user to confirm deleting the session \p session
+*/
+bool SessionManager::confirmSessionDelete(const QString &session)
+{
+    return QMessageBox::question(Core::ICore::mainWindow(),
+                                 tr("Delete Session"),
+                                 tr("Delete session %1?").arg(session),
+                                 QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes;
+}
+
 /*!
      \brief Deletes session name from session list and file from disk.
 */
@@ -861,6 +877,10 @@ bool SessionManager::loadSession(const QString &session)
         restoreValues(reader);
         emit aboutToLoadSession(session);
 
+        QColor c = QColor(reader.restoreValue(QLatin1String("Color")).toString());
+        if (c.isValid())
+            Utils::StyleHelper::setBaseColor(c);
+
         QStringList fileList =
             reader.restoreValue(QLatin1String("ProjectList")).toStringList();
         int openEditorsCount = reader.restoreValue(QLatin1String("OpenEditors")).toInt();
@@ -868,6 +888,10 @@ bool SessionManager::loadSession(const QString &session)
         m_future.setProgressRange(0, fileList.count() + openEditorsCount + 2);
         m_future.setProgressValue(1);
 
+        // if one processEvents doesn't get the job done
+        // just use two!
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
         restoreProjects(fileList);
         sessionLoadingProgress();
         restoreDependencies(reader);
@@ -878,12 +902,9 @@ bool SessionManager::loadSession(const QString &session)
         m_future = QFutureInterface<void>();
 
         // restore the active mode
-        QString modeIdentifier = value(QLatin1String("ActiveMode")).toString();
-        Id modeId;
-        if (modeIdentifier.isEmpty())
+        Id modeId = Id::fromSetting(value(QLatin1String("ActiveMode")));
+        if (!modeId.isValid())
             modeId = Id(Core::Constants::MODE_EDIT);
-        else
-            modeId = Id(modeIdentifier);
 
         ModeManager::activateMode(modeId);
         ModeManager::setFocusToCurrentMode();
@@ -901,8 +922,7 @@ bool SessionManager::loadSession(const QString &session)
 
 QString SessionManager::lastSession() const
 {
-    QString fileName = ICore::settings()->value(QLatin1String("ProjectExplorer/StartupSession")).toString();
-    return QFileInfo(fileName).completeBaseName();
+    return ICore::settings()->value(QLatin1String("ProjectExplorer/StartupSession")).toString();
 }
 
 SessionNode *SessionManager::sessionNode() const
