@@ -27,33 +27,28 @@
 **
 ****************************************************************************/
 
+#include "formeditorview.h"
 #include "selectiontool.h"
 #include "movetool.h"
 #include "resizetool.h"
 #include "dragtool.h"
-#include "formeditorview.h"
 #include "formeditorwidget.h"
-#include "formeditornodeinstanceview.h"
 #include "formeditoritem.h"
 #include "formeditorscene.h"
-#include "toolbox.h"
-#include <rewritertransaction.h>
+#include "abstractcustomtool.h"
+
+#include <designmodecontext.h>
 #include <modelnode.h>
-#include <itemlibraryinfo.h>
-#include <metainfo.h>
 #include <model.h>
-#include <QApplication>
 #include <QDebug>
 #include <QPair>
 #include <QString>
-#include <QDir>
-#include <QAction>
 #include <QTimer>
 #include <zoomaction.h>
 #include <nodeabstractproperty.h>
 #include <nodelistproperty.h>
-#include <commondefines.h>
 
+#include <coreplugin/icore.h>
 
 namespace QmlDesigner {
 
@@ -68,9 +63,12 @@ FormEditorView::FormEditorView(QObject *parent)
       m_currentTool(m_selectionTool),
       m_transactionCounter(0)
 {
-    connect(widget()->zoomAction(), SIGNAL(zoomLevelChanged(double)), SLOT(updateGraphicsIndicators()));
-    connect(widget()->showBoundingRectAction(), SIGNAL(toggled(bool)), scene(), SLOT(setShowBoundingRects(bool)));
-    connect(widget()->selectOnlyContentItemsAction(), SIGNAL(toggled(bool)), this, SLOT(setSelectOnlyContentItemsAction(bool)));
+    Internal::FormEditorContext *formEditorContext = new Internal::FormEditorContext(m_formEditorWidget.data());
+    Core::ICore::addContextObject(formEditorContext);
+
+    connect(formEditorWidget()->zoomAction(), SIGNAL(zoomLevelChanged(double)), SLOT(updateGraphicsIndicators()));
+    connect(formEditorWidget()->showBoundingRectAction(), SIGNAL(toggled(bool)), scene(), SLOT(setShowBoundingRects(bool)));
+    connect(formEditorWidget()->selectOnlyContentItemsAction(), SIGNAL(toggled(bool)), this, SLOT(setSelectOnlyContentItemsAction(bool)));
 
 }
 
@@ -90,6 +88,7 @@ FormEditorView::~FormEditorView()
     delete m_dragTool;
     m_dragTool = 0;
 
+    qDeleteAll(m_customToolList);
 
     // delete scene after tools to prevent double deletion
     // of items
@@ -170,11 +169,15 @@ void FormEditorView::modelAboutToBeDetached(Model *model)
     m_moveTool->clear();
     m_resizeTool->clear();
     m_dragTool->clear();
+    foreach (AbstractCustomTool *customTool, m_customToolList)
+        customTool->clear();
     m_scene->clearFormEditorItems();
     m_formEditorWidget->updateActions();
     m_formEditorWidget->resetView();
     scene()->resetScene();
 
+    m_currentTool = m_selectionTool;
+
     QmlModelView::modelAboutToBeDetached(model);
 }
 
@@ -215,12 +218,12 @@ void FormEditorView::nodeAboutToBeRemoved(const ModelNode &removedNode)
 
 void FormEditorView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList)
 {
-    foreach(const AbstractProperty &property, propertyList) {
+    foreach (const AbstractProperty &property, propertyList) {
         if (property.isNodeAbstractProperty()) {
             NodeAbstractProperty nodeAbstractProperty = property.toNodeAbstractProperty();
             QList<FormEditorItem*> removedItemList;
 
-            foreach(const ModelNode &modelNode, nodeAbstractProperty.allSubNodes()) {
+            foreach (const ModelNode &modelNode, nodeAbstractProperty.allSubNodes()) {
                 QmlItemNode qmlItemNode(modelNode);
 
                 if (qmlItemNode.isValid() && m_scene->hasItemForQmlItemNode(qmlItemNode)) {
@@ -267,9 +270,17 @@ void FormEditorView::bindingPropertiesChanged(const QList<BindingProperty>& prop
     QmlModelView::bindingPropertiesChanged(propertyList, propertyChange);
 }
 
-FormEditorWidget *FormEditorView::widget() const
+void FormEditorView::signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty> & /*propertyList*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
+{
+}
+
+WidgetInfo FormEditorView::widgetInfo()
+{
+    return createWidgetInfo(m_formEditorWidget.data(), 0, "FormEditor", WidgetInfo::CentralPane, 0, tr("Form Editor"));
+}
+
+FormEditorWidget *FormEditorView::formEditorWidget()
 {
-    Q_ASSERT(!m_formEditorWidget.isNull());
     return m_formEditorWidget.data();
 }
 
@@ -402,10 +413,47 @@ void FormEditorView::changeToTransformTools()
     changeToSelectionTool();
 }
 
+void FormEditorView::changeToCustomTool(const ModelNode &modelNode)
+{
+    int handlingRank = 0;
+    AbstractCustomTool *selectedCustomTool;
+
+    foreach (AbstractCustomTool *customTool, m_customToolList) {
+        if (customTool->wantHandleItem(modelNode) > handlingRank) {
+            handlingRank = customTool->wantHandleItem(modelNode);
+            selectedCustomTool = customTool;
+        }
+
+    }
+
+    if (handlingRank > 0) {
+        m_scene->updateAllFormEditorItems();
+        m_currentTool->clear();
+        m_currentTool = selectedCustomTool;
+        m_currentTool->clear();
+        m_currentTool->setItems(scene()->itemsForQmlItemNodes(selectedQmlItemNodes()));
+    }
+}
+
+void FormEditorView::changeToCustomTool(AbstractCustomTool *customTool)
+{
+    m_scene->updateAllFormEditorItems();
+    m_currentTool->clear();
+    m_currentTool = customTool;
+    m_currentTool->clear();
+    m_currentTool->setItems(scene()->itemsForQmlItemNodes(selectedQmlItemNodes()));
+}
+
+void FormEditorView::registerTool(AbstractCustomTool *tool)
+{
+    tool->setView(this);
+    m_customToolList.append(tool);
+}
+
 void FormEditorView::nodeSlidedToIndex(const NodeListProperty &listProperty, int /*newIndex*/, int /*oldIndex*/)
 {
     QList<ModelNode> newOrderModelNodeList = listProperty.toModelNodeList();
-    foreach(const ModelNode &node, newOrderModelNodeList) {
+    foreach (const ModelNode &node, newOrderModelNodeList) {
         FormEditorItem *item = m_scene->itemForQmlItemNode(QmlItemNode(node));
         if (item) {
             FormEditorItem *oldParentItem = item->parentItem();
@@ -417,7 +465,7 @@ void FormEditorView::nodeSlidedToIndex(const NodeListProperty &listProperty, int
     m_currentTool->formEditorItemsChanged(scene()->allFormEditorItems());
 }
 
-void FormEditorView::auxiliaryDataChanged(const ModelNode &node, const QString &name, const QVariant &data)
+void FormEditorView::auxiliaryDataChanged(const ModelNode &node, const PropertyName &name, const QVariant &data)
 {
     QmlModelView::auxiliaryDataChanged(node, name, data);
     if (name == "invisible" && m_scene->hasItemForQmlItemNode(QmlItemNode(node))) {
@@ -453,8 +501,8 @@ void FormEditorView::instanceInformationsChange(const QMultiHash<ModelNode, Info
         if (qmlItemNode.isValid() && scene()->hasItemForQmlItemNode(qmlItemNode)) {
             scene()->synchronizeTransformation(qmlItemNode);
             if (qmlItemNode.isRootModelNode() && informationChangeHash.values(node).contains(Size)) {
-                widget()->setRootItemRect(qmlItemNode.instanceBoundingRect());
-                widget()->centerScene();
+                formEditorWidget()->setRootItemRect(qmlItemNode.instanceBoundingRect());
+                formEditorWidget()->centerScene();
             }
 
             itemNodeList.append(scene()->itemForQmlItemNode(qmlItemNode));
@@ -468,9 +516,8 @@ void FormEditorView::instancesRenderImageChanged(const QVector<ModelNode> &nodeL
 {
     foreach (const ModelNode &node, nodeList) {
         QmlItemNode qmlItemNode(node);
-        if (qmlItemNode.isValid() && scene()->hasItemForQmlItemNode(qmlItemNode)) {
+        if (qmlItemNode.isValid() && scene()->hasItemForQmlItemNode(qmlItemNode))
            scene()->itemForQmlItemNode(qmlItemNode)->update();
-        }
     }
 }
 
@@ -509,9 +556,9 @@ void FormEditorView::rewriterEndTransaction()
     m_transactionCounter--;
 }
 
-double FormEditorView::margins() const
+double FormEditorView::containerPadding() const
 {
-    return m_formEditorWidget->margins();
+    return m_formEditorWidget->containerPadding();
 }
 
 double FormEditorView::spacing() const
@@ -545,14 +592,14 @@ QmlItemNode findRecursiveQmlItemNode(const QmlObjectNode &firstQmlObjectNode)
     return QmlItemNode();
 }
 
-void FormEditorView::instancePropertyChange(const QList<QPair<ModelNode, QString> > &propertyList)
+void FormEditorView::instancePropertyChange(const QList<QPair<ModelNode, PropertyName> > &propertyList)
 {
-    typedef QPair<ModelNode, QString> NodePropertyPair;
+    typedef QPair<ModelNode, PropertyName> NodePropertyPair;
     foreach (const NodePropertyPair &nodePropertyPair, propertyList) {
         const QmlItemNode itemNode(nodePropertyPair.first);
-        const QString propertyName = nodePropertyPair.second;
+        const PropertyName propertyName = nodePropertyPair.second;
         if (itemNode.isValid() && scene()->hasItemForQmlItemNode(itemNode)) {
-            static QStringList skipList = QStringList() << "x" << "y" << "width" << "height";
+            static PropertyNameList skipList = PropertyNameList() << "x" << "y" << "width" << "height";
             if (!skipList.contains(propertyName)) {
                 m_scene->synchronizeOtherProperty(itemNode, propertyName);
                 m_currentTool->formEditorItemsChanged(QList<FormEditorItem*>() << m_scene->itemForQmlItemNode(itemNode));
@@ -578,7 +625,7 @@ bool FormEditorView::isMoveToolAvailable() const
 {
     if (selectedQmlItemNodes().count() == 1)
         return selectedQmlItemNodes().first().instanceIsMovable() &&
-               !selectedQmlItemNodes().first().instanceIsInPositioner();
+               !selectedQmlItemNodes().first().instanceIsInLayoutable();
     return true;
 }
 
@@ -589,14 +636,6 @@ void FormEditorView::actualStateChanged(const ModelNode &node)
     QmlModelState newQmlModelState(node);
 }
 
-Utils::CrumblePath *FormEditorView::crumblePath() const
-{
-    if (widget() && widget()->toolBox()) {
-        return widget()->toolBox()->crumblePath();
-    }
-    return 0;
-}
-
 void FormEditorView::reset()
 {
    QTimer::singleShot(200, this, SLOT(delayedReset()));
