@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "navigatortreemodel.h"
 
@@ -37,16 +34,15 @@
 #include <nodeproperty.h>
 #include <variantproperty.h>
 #include <metainfo.h>
-#include <qgraphicswidget.h>
 #include <qmlmodelview.h>
 #include <rewriterview.h>
 #include <invalididexception.h>
 #include <rewritingexception.h>
+#include <modelnodecontextmenu.h>
 
 #include <QMimeData>
 #include <QMessageBox>
 #include <QApplication>
-#include <QTransform>
 #include <QPointF>
 
 static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPointF &pos)
@@ -54,8 +50,8 @@ static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPo
     QmlDesigner::QmlItemNode parentNode = modelNode.parentProperty().parentQmlObjectNode().toQmlItemNode();
     if (parentNode.isValid()) {
         QPointF localPos = parentNode.instanceSceneTransform().inverted().map(pos);
-        modelNode.variantProperty(QLatin1String("x")) = localPos.toPoint().x();
-        modelNode.variantProperty(QLatin1String("y")) = localPos.toPoint().y();
+        modelNode.variantProperty("x") = localPos.toPoint().x();
+        modelNode.variantProperty("y") = localPos.toPoint().y();
     }
 }
 
@@ -140,7 +136,7 @@ bool NavigatorTreeModel::dropMimeData(const QMimeData *data,
         return false;
 
     QModelIndex parentIndex, parentItemIndex;
-    QString parentPropertyName;
+    PropertyName parentPropertyName;
     int targetIndex;
 
     parentIndex = dropIndex.sibling(dropIndex.row(), 0);
@@ -156,7 +152,7 @@ bool NavigatorTreeModel::dropMimeData(const QMimeData *data,
     }
     else {
         parentItemIndex = parentIndex.parent();
-        parentPropertyName = parentIndex.data(Qt::DisplayRole).toString();
+        parentPropertyName = parentIndex.data(Qt::DisplayRole).toByteArray();
     }
 
     // Disallow dropping items between properties, which are listed first.
@@ -210,11 +206,10 @@ NavigatorTreeModel::ItemRow NavigatorTreeModel::createItemRow(const ModelNode &n
     idItem->setDropEnabled(dropEnabled);
     idItem->setEditable(true);
     idItem->setData(hash, NavigatorRole);
-    if (node.metaInfo().isValid()) {
+    if (node.metaInfo().isValid())
         idItem->setToolTip(node.type());
-    } else {
+    else
         idItem->setToolTip(msgUnknownItem(node.type()));
-    }
 #    ifdef _LOCK_ITEMS_
     QStandardItem *lockItem = new QStandardItem;
     lockItem->setDragEnabled(true);
@@ -229,9 +224,8 @@ NavigatorTreeModel::ItemRow NavigatorTreeModel::createItemRow(const ModelNode &n
     visibilityItem->setCheckable(true);
     visibilityItem->setEditable(false);
     visibilityItem->setData(hash, NavigatorRole);
-    if (node.isRootNode()) {
+    if (node.isRootNode())
         visibilityItem->setCheckable(false);
-    }
 
     QMap<QString, QStandardItem *> propertyItems;
     foreach (const QString &propertyName, visibleProperties(node)) {
@@ -254,15 +248,14 @@ NavigatorTreeModel::ItemRow NavigatorTreeModel::createItemRow(const ModelNode &n
 
 void NavigatorTreeModel::updateItemRow(const ModelNode &node, ItemRow items)
 {
-    bool blockSignal = blockItemChangedSignal(true);    
+    bool blockSignal = blockItemChangedSignal(true);
 
     items.idItem->setText(node.id());
     items.visibilityItem->setCheckState(node.auxiliaryData("invisible").toBool() ? Qt::Unchecked : Qt::Checked);
-    if (node.metaInfo().isValid()) {
+    if (node.metaInfo().isValid())
         items.idItem->setToolTip(node.type());
-    } else {
+    else
         items.idItem->setToolTip(msgUnknownItem(node.type()));
-    }
 
     blockItemChangedSignal(blockSignal);
 }
@@ -498,9 +491,8 @@ void NavigatorTreeModel::removeSubTree(const ModelNode &node)
 
     QList<QStandardItem*> rowList;
     ItemRow itemRow = itemRowForNode(node);
-    if (itemRow.idItem->parent()) {
+    if (itemRow.idItem->parent())
         rowList = itemRow.idItem->parent()->takeRow(itemRow.idItem->row());
-    }
 
     foreach (const ModelNode &childNode, modelNodeChildren(node)) {
         removeSubTree(childNode);
@@ -515,7 +507,7 @@ void NavigatorTreeModel::removeSubTree(const ModelNode &node)
 void NavigatorTreeModel::moveNodesInteractive(NodeAbstractProperty parentProperty, const QList<ModelNode> &modelNodes, int targetIndex)
 {
     try {
-        QString propertyQmlType = qmlTypeInQtContainer(parentProperty.parentModelNode().metaInfo().propertyTypeName(parentProperty.name()));
+        TypeName propertyQmlType = qmlTypeInQtContainer(parentProperty.parentModelNode().metaInfo().propertyTypeName(parentProperty.name()));
 
         RewriterTransaction transaction = m_view->beginRewriterTransaction();
         foreach (const ModelNode &node, modelNodes) {
@@ -533,9 +525,8 @@ void NavigatorTreeModel::moveNodesInteractive(NodeAbstractProperty parentPropert
                         if (parentProperty.isNodeProperty()) {
                             ModelNode propertyNode = parentProperty.toNodeProperty().modelNode();
                             // Destruction of ancestors is not allowed
-                            if (propertyNode.isAncestorOf(node)) {
+                            if (propertyNode.isAncestorOf(node))
                                 continue;
-                            }
                             if (propertyNode.isValid()) {
                                 QApplication::setOverrideCursor(Qt::ArrowCursor);
                                 if (QMessageBox::warning(0, tr("Warning"), tr("Reparenting the component %1 here will cause the component %2 to be deleted. Do you want to proceed?").arg(node.id(), propertyNode.id()), QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel) {
@@ -547,7 +538,7 @@ void NavigatorTreeModel::moveNodesInteractive(NodeAbstractProperty parentPropert
                             }
                         }
 
-                        if (parentProperty.isDefaultProperty() && parentProperty.parentModelNode().metaInfo().isSubclassOf("<cpp>.QDeclarativeBasePositioner", -1, -1)) {
+                        if (parentProperty.isDefaultProperty() && parentProperty.parentModelNode().metaInfo().isLayoutable()) {
                              ModelNode currentNode = node;
                              if (currentNode.hasProperty("x"))
                                  currentNode.removeProperty("x");
@@ -571,9 +562,8 @@ void NavigatorTreeModel::moveNodesInteractive(NodeAbstractProperty parentPropert
                         if (index < targetIndex) { // item is first removed from oldIndex, then inserted at new index
                             --targetIndex;
                         }
-                        if (index != targetIndex) {
+                        if (index != targetIndex)
                             parentProperty.toNodeListProperty().slide(index, targetIndex);
-                        }
                     }
             }
         }
@@ -585,14 +575,14 @@ void NavigatorTreeModel::moveNodesInteractive(NodeAbstractProperty parentPropert
 QList<ModelNode> NavigatorTreeModel::modelNodeChildren(const ModelNode &parentNode)
 {
     QList<ModelNode> children;
-    QStringList properties;
+    PropertyNameList properties;
 
     if (parentNode.metaInfo().hasDefaultProperty())
         properties << parentNode.metaInfo().defaultPropertyName();
 
     properties << visibleProperties(parentNode);
 
-    foreach (const QString &propertyName, properties) {
+    foreach (const PropertyName &propertyName, properties) {
         AbstractProperty property(parentNode.property(propertyName));
         if (property.isNodeProperty())
             children << property.toNodeProperty().modelNode();
@@ -603,9 +593,9 @@ QList<ModelNode> NavigatorTreeModel::modelNodeChildren(const ModelNode &parentNo
     return children;
 }
 
-QString NavigatorTreeModel::qmlTypeInQtContainer(const QString &qtContainerType) const
+TypeName NavigatorTreeModel::qmlTypeInQtContainer(const TypeName &qtContainerType) const
 {
-    QString typeName(qtContainerType);
+    TypeName typeName(qtContainerType);
     if (typeName.startsWith("QDeclarativeListProperty<") &&
         typeName.endsWith('>')) {
         typeName.remove(0, 25);
@@ -618,23 +608,24 @@ QString NavigatorTreeModel::qmlTypeInQtContainer(const QString &qtContainerType)
 }
 
 
-QStringList NavigatorTreeModel::visibleProperties(const ModelNode &node) const
+PropertyNameList NavigatorTreeModel::visibleProperties(const ModelNode &node) const
 {
-    QStringList propertyList;
+    PropertyNameList propertyList;
 
-    foreach (const QString &propertyName, node.metaInfo().propertyNames()) {
+    foreach (const PropertyName &propertyName, node.metaInfo().propertyNames()) {
         if (!propertyName.contains('.') && //do not show any dot properties, since they are tricky and unlikely to make sense
             node.metaInfo().propertyIsWritable(propertyName) && !m_hiddenProperties.contains(propertyName) &&
             !node.metaInfo().propertyIsEnumType(propertyName) && //Some enums have the same name as Qml types (e. g. Flow)
+            !node.metaInfo().propertyIsPrivate(propertyName) && //Do not show private properties
             propertyName != node.metaInfo().defaultPropertyName()) { // TODO: ask the node instances
 
-            QString qmlType = qmlTypeInQtContainer(node.metaInfo().propertyTypeName(propertyName));
+            TypeName qmlType = qmlTypeInQtContainer(node.metaInfo().propertyTypeName(propertyName));
             if (node.model()->metaInfo(qmlType).isValid() &&
-                node.model()->metaInfo(qmlType).isSubclassOf("<cpp>.QGraphicsObject", -1, -1)) {
+                node.model()->metaInfo(qmlType).isSubclassOf("QtQuick.Item", -1, -1)) {
                 propertyList.append(propertyName);
             }
         }
-    }    
+    }
 
     return propertyList;
 }
@@ -661,10 +652,9 @@ void NavigatorTreeModel::setVisible(const QModelIndex &index, bool visible)
     itemRow.visibilityItem->setCheckState(visible ? Qt::Checked : Qt::Unchecked);
 }
 
-void NavigatorTreeModel::openContextMenu(const QPoint &p)
+void NavigatorTreeModel::openContextMenu(const QPoint &position)
 {
-    if (m_view)
-        m_view->showContextMenu(p, QPoint(), false);
+    ModelNodeContextMenu::showContextMenu(m_view.data(), position, QPoint(), false);
 }
 
 } // QmlDesigner
