@@ -1,39 +1,36 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef QmlItemNode_H
 #define QmlItemNode_H
 
-#include <corelib_global.h>
+#include <qmldesignercorelib_global.h>
 #include <modelnode.h>
 #include "qmlobjectnode.h"
 #include "qmlstate.h"
@@ -47,15 +44,17 @@ namespace QmlDesigner {
 class QmlModelStateGroup;
 class QmlAnchors;
 
-class CORESHARED_EXPORT QmlItemNode : public QmlObjectNode
+class QMLDESIGNERCORE_EXPORT QmlItemNode : public QmlObjectNode
 {
-    friend class CORESHARED_EXPORT QmlAnchors;
+    friend class QmlAnchors;
 public:
     QmlItemNode() : QmlObjectNode() {}
     QmlItemNode(const ModelNode &modelNode)  : QmlObjectNode(modelNode) {}
     bool isValid() const;
     bool isRootNode() const;
 
+    static bool isItemOrWindow(const ModelNode &modelNode);
+
     QmlModelStateGroup states() const;
     QList<QmlItemNode> children() const;
     QList<QmlObjectNode> resources() const;
@@ -73,17 +72,22 @@ public:
     bool instanceIsAnchoredByChildren() const;
     bool instanceIsMovable() const;
     bool instanceIsResizable() const;
-    bool instanceIsInPositioner() const;
+    bool instanceIsInLayoutable() const;
     bool instanceHasRotationTransform() const;
 
     QRectF instanceBoundingRect() const;
     QRectF instancePaintedBoundingRect() const;
+    QRectF instanceContentItemBoundingRect() const;
     QTransform instanceTransform() const;
+    QTransform instanceTransformWithContentTransform() const;
+    QTransform instanceTransformWithContentItemTransform() const;
     QTransform instanceSceneTransform() const;
+    QTransform instanceSceneContentItemTransform() const;
     QPointF instanceScenePosition() const;
     QPointF instancePosition() const;
     QSizeF instanceSize() const;
     int instancePenWidth() const;
+    bool instanceIsRenderPixmapNull() const;
 
 
     void paintInstance(QPainter *painter);
@@ -92,16 +96,16 @@ public:
     void deselectNode();
     bool isSelected() const;
 
-    QString simplifiedTypeName() const;
+    TypeName simplifiedTypeName() const;
 
     const QList<QmlItemNode> allDirectSubModelNodes() const;
     const QList<QmlItemNode> allSubModelNodes() const;
     bool hasAnySubModelNodes() const;
 };
 
-CORESHARED_EXPORT uint qHash(const QmlItemNode &node);
+QMLDESIGNERCORE_EXPORT uint qHash(const QmlItemNode &node);
 
-class CORESHARED_EXPORT QmlModelStateGroup
+class QMLDESIGNERCORE_EXPORT QmlModelStateGroup
 {
     friend class QmlItemNode;
     friend class QmlModelView;
@@ -124,8 +128,8 @@ private:
     ModelNode m_modelNode;
 };
 
-CORESHARED_EXPORT QList<ModelNode> toModelNodeList(const QList<QmlItemNode> &fxItemNodeList);
-CORESHARED_EXPORT QList<QmlItemNode> toQmlItemNodeList(const QList<ModelNode> &modelNodeList);
+QMLDESIGNERCORE_EXPORT QList<ModelNode> toModelNodeList(const QList<QmlItemNode> &fxItemNodeList);
+QMLDESIGNERCORE_EXPORT QList<QmlItemNode> toQmlItemNodeList(const QList<ModelNode> &modelNodeList);
 
 } //QmlDesigner
 
