@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -32,14 +32,8 @@
 #include "internalvariantproperty.h"
 #include "internalnodelistproperty.h"
 #include "internalnodeproperty.h"
+#include "internalsignalhandlerproperty.h"
 #include "internalnode_p.h"
-#include <QVariant>
-#include <QString>
-#include <QRegExp>
-#include <QSize>
-#include <QSizeF>
-#include <QPoint>
-#include <QPointF>
 
 namespace QmlDesigner {
 
@@ -54,7 +48,7 @@ InternalProperty::~InternalProperty()
 {
 }
 
-InternalProperty::InternalProperty(const QString &name, const InternalNode::Pointer &propertyOwner)
+InternalProperty::InternalProperty(const PropertyName &name, const InternalNode::Pointer &propertyOwner)
      : m_name(name),
      m_propertyOwner(propertyOwner)
 {
@@ -79,7 +73,7 @@ bool InternalProperty::isValid() const
     return m_propertyOwner && !m_name.isEmpty();
 }
 
-QString InternalProperty::name() const
+PropertyName InternalProperty::name() const
 {
     return m_name;
 }
@@ -116,6 +110,11 @@ bool InternalProperty::isNodeAbstractProperty() const
     return false;
 }
 
+bool InternalProperty::isSignalHandlerProperty() const
+{
+    return false;
+}
+
 QSharedPointer<InternalVariantProperty> InternalProperty::toVariantProperty() const
 
 {
@@ -146,18 +145,24 @@ QSharedPointer<InternalNodeAbstractProperty> InternalProperty::toNodeAbstractPro
     return internalPointer().staticCast<InternalNodeAbstractProperty>();
 }
 
+QSharedPointer<InternalSignalHandlerProperty> InternalProperty::toSignalHandlerProperty() const
+{
+    Q_ASSERT(internalPointer().dynamicCast<InternalSignalHandlerProperty>());
+    return internalPointer().staticCast<InternalSignalHandlerProperty>();
+}
+
 void InternalProperty::remove()
 {
     propertyOwner()->removeProperty(name());
     m_propertyOwner.clear();
 }
 
-QString InternalProperty::dynamicTypeName() const
+TypeName InternalProperty::dynamicTypeName() const
 {
     return m_dynamicType;
 }
 
-void InternalProperty::setDynamicTypeName(const QString &name)
+void InternalProperty::setDynamicTypeName(const TypeName &name)
 {
     m_dynamicType = name;
 }
