@@ -27,11 +27,10 @@
 **
 ****************************************************************************/
 
-#include <modelnode.h>
+#include "modelnode.h"
 #include <abstractproperty.h>
 #include <abstractview.h>
 #include <model.h>
-#include <metainfo.h>
 #include <nodemetainfo.h>
 #include "internalnode_p.h"
 #include <QHash>
@@ -39,13 +38,10 @@
 #include "invalidargumentexception.h"
 #include "invalididexception.h"
 #include "invalidmodelnodeexception.h"
-#include "invalidpropertyexception.h"
-#include "invalidslideindexexception.h"
 #include "model_p.h"
-#include "abstractview.h"
-#include "abstractproperty.h"
 #include "variantproperty.h"
 #include "bindingproperty.h"
+#include "signalhandlerproperty.h"
 #include "nodeabstractproperty.h"
 #include "nodelistproperty.h"
 #include "nodeproperty.h"
@@ -132,11 +128,11 @@ ModelNode::~ModelNode()
 QString ModelNode::generateNewId() const
 {
     int counter = 1;
-    QString newId = QString("%1%2").arg(simplifiedTypeName().toLower()).arg(counter);
+    QString newId = QString("%1%2").arg(QString::fromUtf8(simplifiedTypeName()).toLower()).arg(counter);
 
-    while(view()->hasId(newId)) {
+    while (view()->hasId(newId)) {
         counter += 1;
-        newId = QString("%1%2").arg(simplifiedTypeName().toLower()).arg(counter);
+        newId = QString("%1%2").arg(QString::fromUtf8(simplifiedTypeName()).toLower()).arg(counter);
     }
 
     return newId;
@@ -147,9 +143,8 @@ QString ModelNode::generateNewId() const
 */
 QString ModelNode::id() const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return m_internalNode->id();
 }
@@ -204,7 +199,7 @@ void ModelNode::setId(const QString& id)
 /*! \brief the fully-qualified type name of the node is represented as string
 \return type of the node as a string
 */
-QString ModelNode::type() const
+TypeName ModelNode::type() const
 {
     if (!isValid()) {
         Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
@@ -237,6 +232,36 @@ int ModelNode::majorVersion() const
     return m_internalNode->majorVersion();
 }
 
+int getMajorVersionFromImport(Model *model)
+{
+    foreach (const Import &import, model->imports()) {
+        if (import.isLibraryImport() && import.url() == QLatin1String("QtQuick")) {
+            const QString versionString = import.version();
+            if (versionString.contains(QLatin1String("."))) {
+                const QString majorVersionString = versionString.split(QLatin1String(".")).first();
+                return majorVersionString.toInt();
+            }
+        }
+    }
+
+    return -1;
+}
+
+int getMajorVersionFromNode(const ModelNode &modelNode)
+{
+    if (modelNode.metaInfo().isValid()) {
+        if (modelNode.type() == "QtQuick.QtObject" || modelNode.type() == "QtQuick.Item")
+            return modelNode.majorVersion();
+
+        foreach (const NodeMetaInfo &superClass,  modelNode.metaInfo().superClasses()) {
+            if (modelNode.type() == "QtQuick.QtObject" || modelNode.type() == "QtQuick.Item")
+                return superClass.majorVersion();
+        }
+    }
+
+    return 1; //default
+}
+
 /*! \brief major number of the QtQuick version used
 \return major number of QtQuickVersion
 */
@@ -247,29 +272,23 @@ int ModelNode::majorQtQuickVersion() const
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    if (metaInfo().isValid()) {
-        NodeMetaInfo superClass = metaInfo().directSuperClass();
+    int majorVersionFromImport = getMajorVersionFromImport(model());
+    if (majorVersionFromImport >= 0)
+        return majorVersionFromImport;
 
-        while (superClass.isValid()) {
-            if (superClass.typeName() == "QtQuick.Item")
-                return superClass.majorVersion();
-            superClass = superClass.directSuperClass();
-        }
-        return 1; //default
-    }
-    return 1; //default
+    return getMajorVersionFromNode(*this);
 }
 
 
 /*! \return the short-hand type name of the node. */
-QString ModelNode::simplifiedTypeName() const
+TypeName ModelNode::simplifiedTypeName() const
 {
     if (!isValid()) {
         Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return type().split(QLatin1Char('.')).last();
+    return type().split('.').last();
 }
 
 /*! \brief Returns whether the node is valid
@@ -351,9 +370,8 @@ void ModelNode::setParentProperty(NodeAbstractProperty parent)
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    if (!parent.parentModelNode().isValid()) {
+    if (!parent.parentModelNode().isValid())
         throw InvalidArgumentException(__LINE__, __FUNCTION__, __FILE__, "newParentNode");
-    }
 
     if (*this == parent.parentModelNode()) {
         Q_ASSERT_X(*this != parent.parentModelNode(), Q_FUNC_INFO, "cannot set parent to itself");
@@ -366,7 +384,7 @@ void ModelNode::setParentProperty(NodeAbstractProperty parent)
     parent.reparentHere(*this);
 }
 
-void ModelNode::setParentProperty(const ModelNode &newParentNode, const QString &propertyName)
+void ModelNode::setParentProperty(const ModelNode &newParentNode, const PropertyName &propertyName)
 {
     setParentProperty(newParentNode.nodeAbstractProperty(propertyName));
 }
@@ -398,7 +416,7 @@ bool ModelNode::hasParentProperty() const
   \return BindingProperty named name
   */
 
-BindingProperty ModelNode::bindingProperty(const QString &name) const
+BindingProperty ModelNode::bindingProperty(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -406,6 +424,14 @@ BindingProperty ModelNode::bindingProperty(const QString &name) const
     return BindingProperty(name, m_internalNode, model(), view());
 }
 
+SignalHandlerProperty ModelNode::signalHandlerProperty(const PropertyName &name) const
+{
+    if (!isValid())
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+
+    return SignalHandlerProperty(name, m_internalNode, model(), view());
+}
+
 
 /*!
   \brief Returns a NodeProperty
@@ -417,7 +443,7 @@ BindingProperty ModelNode::bindingProperty(const QString &name) const
   \return NodeProperty named name
   */
 
-NodeProperty ModelNode::nodeProperty(const QString &name) const
+NodeProperty ModelNode::nodeProperty(const PropertyName &name) const
 {
       if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -436,7 +462,7 @@ NodeProperty ModelNode::nodeProperty(const QString &name) const
   \return NodeListProperty named name
   */
 
-NodeListProperty ModelNode::nodeListProperty(const QString &name) const
+NodeListProperty ModelNode::nodeListProperty(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -444,7 +470,7 @@ NodeListProperty ModelNode::nodeListProperty(const QString &name) const
     return NodeListProperty(name, m_internalNode, model(), view());
 }
 
-NodeAbstractProperty ModelNode::nodeAbstractProperty(const QString &name) const
+NodeAbstractProperty ModelNode::nodeAbstractProperty(const PropertyName &name) const
 {
      if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -463,7 +489,7 @@ NodeAbstractProperty ModelNode::nodeAbstractProperty(const QString &name) const
   \return VariantProperty named name
   */
 
-VariantProperty ModelNode::variantProperty(const QString &name) const
+VariantProperty ModelNode::variantProperty(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -471,7 +497,7 @@ VariantProperty ModelNode::variantProperty(const QString &name) const
     return VariantProperty(name, m_internalNode, model(), view());
 }
 
-AbstractProperty ModelNode::property(const QString &name) const
+AbstractProperty ModelNode::property(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -501,7 +527,7 @@ QList<AbstractProperty> ModelNode::properties() const
 
     QList<AbstractProperty> propertyList;
 
-    foreach (const QString &propertyName, internalNode()->propertyNameList()) {
+    foreach (const PropertyName &propertyName, internalNode()->propertyNameList()) {
         AbstractProperty property(propertyName, internalNode(), model(), view());
         propertyList.append(property);
     }
@@ -581,7 +607,7 @@ Does nothing if the node state does not set this property.
 
 \see addProperty property  properties hasProperties
 */
-void ModelNode::removeProperty(const QString &name)
+void ModelNode::removeProperty(const PropertyName &name)
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -629,9 +655,8 @@ void ModelNode::destroy()
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    if (isRootNode()) {
+    if (isRootNode())
         throw InvalidArgumentException(__LINE__, __FUNCTION__, __FILE__, "rootNode");
-    }
 
     removeModelNodeFromSelection(*this);
     model()->d->removeNode(internalNode());
@@ -791,7 +816,7 @@ The list of properties set in this state.
 
 \see addProperty property changePropertyValue removeProperty hasProperties
 */
-QStringList ModelNode::propertyNames() const
+PropertyNameList ModelNode::propertyNames() const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -801,7 +826,7 @@ QStringList ModelNode::propertyNames() const
 /*! \brief test a if a property is set for this node
 \return true if property a property ins this or a ancestor state exists
 */
-bool ModelNode::hasProperty(const QString &name) const
+bool ModelNode::hasProperty(const PropertyName &name) const
 {
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
@@ -809,27 +834,27 @@ bool ModelNode::hasProperty(const QString &name) const
     return internalNode()->hasProperty(name);
 }
 
-bool ModelNode::hasVariantProperty(const QString &name) const
+bool ModelNode::hasVariantProperty(const PropertyName &name) const
 {
     return hasProperty(name) && internalNode()->property(name)->isVariantProperty();
 }
 
-bool ModelNode::hasBindingProperty(const QString &name) const
+bool ModelNode::hasBindingProperty(const PropertyName &name) const
 {
     return hasProperty(name) && internalNode()->property(name)->isBindingProperty();
 }
 
-bool ModelNode::hasNodeAbstracProperty(const QString &name) const
+bool ModelNode::hasNodeAbstracProperty(const PropertyName &name) const
 {
     return hasProperty(name) && internalNode()->property(name)->isNodeAbstractProperty();
 }
 
-bool ModelNode::hasNodeProperty(const QString &name) const
+bool ModelNode::hasNodeProperty(const PropertyName &name) const
 {
     return hasProperty(name) && internalNode()->property(name)->isNodeProperty();
 }
 
-bool ModelNode::hasNodeListProperty(const QString &name) const
+bool ModelNode::hasNodeListProperty(const PropertyName &name) const
 {
     return hasProperty(name) && internalNode()->property(name)->isNodeListProperty();
 }
@@ -879,9 +904,8 @@ QTextStream& operator<<(QTextStream &stream, const ModelNode &modelNode)
 
 void ModelNode::selectNode()
 {
-    if (!isValid()) {
+    if (!isValid())
             throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     QList<ModelNode> selectedNodeList;
     selectedNodeList.append(*this);
@@ -891,9 +915,8 @@ void ModelNode::selectNode()
 
 void ModelNode::deselectNode()
 {
-    if (!isValid()) {
+    if (!isValid())
             throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     QList<ModelNode> selectedNodeList(view()->selectedModelNodes());
     selectedNodeList.removeAll(*this);
@@ -911,35 +934,32 @@ QVariant ModelNode::toVariant() const
     return QVariant::fromValue(*this);
 }
 
-QVariant ModelNode::auxiliaryData(const QString &name) const
+QVariant ModelNode::auxiliaryData(const PropertyName &name) const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return internalNode()->auxiliaryData(name);
 }
 
-void ModelNode::setAuxiliaryData(const QString &name, const QVariant &data) const
+void ModelNode::setAuxiliaryData(const PropertyName &name, const QVariant &data) const
 {
     Internal::WriteLocker locker(m_model.data());
     m_model.data()->d->setAuxiliaryData(internalNode(), name, data);
 }
 
-bool ModelNode::hasAuxiliaryData(const QString &name) const
+bool ModelNode::hasAuxiliaryData(const PropertyName &name) const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return internalNode()->hasAuxiliaryData(name);
 }
 
-QHash<QString, QVariant> ModelNode::auxiliaryData() const
+QHash<PropertyName, QVariant> ModelNode::auxiliaryData() const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return internalNode()->auxiliaryData();
 }
@@ -979,18 +999,16 @@ void ModelNode::setNodeSource(const QString &newNodeSource)
 
 QString ModelNode::nodeSource() const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return internalNode()->nodeSource();
 }
 
 QString ModelNode::convertTypeToImportAlias() const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     if (model()->rewriterView())
         return model()->rewriterView()->convertTypeToImportAlias(type());
@@ -1000,9 +1018,8 @@ QString ModelNode::convertTypeToImportAlias() const
 
 ModelNode::NodeSourceType ModelNode::nodeSourceType() const
 {
-    if (!isValid()) {
+    if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
 
     return static_cast<ModelNode::NodeSourceType>(internalNode()->nodeSourceType());
 
