@@ -30,11 +30,9 @@
 #include "plaintexteditmodifier.h"
 
 #include <utils/changeset.h>
-#include <extensionsystem/pluginmanager.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
 
 #include <QPlainTextEdit>
-#include <QUndoStack>
 
 #include <QDebug>
 
@@ -60,7 +58,7 @@ PlainTextEditModifier::~PlainTextEditModifier()
 
 void PlainTextEditModifier::save(QIODevice *device)
 {
-    device->write(m_textEdit->toPlainText().toLatin1());
+    device->write(m_textEdit->toPlainText().toUtf8());
 }
 
 void PlainTextEditModifier::replace(int offset, int length, const QString &replacement)
