@@ -27,22 +27,20 @@
 **
 ****************************************************************************/
 
+#include "rewriterview.h"
+
 #include <filemanager/astobjecttextextractor.h>
 #include <filemanager/objectlengthcalculator.h>
 #include <filemanager/firstdefinitionfinder.h>
 #include <customnotifications.h>
 
-#include <qmljs/parser/qmljsengine_p.h>
 
-#include "rewriterview.h"
 #include "rewritingexception.h"
-#include "textmodifier.h"
 #include "texttomodelmerger.h"
 #include "modelnodepositionstorage.h"
 #include "modeltotextmerger.h"
 #include "nodelistproperty.h"
-#include "nodeproperty.h"
-#include "invalidmodelnodeexception.h"
+#include "signalhandlerproperty.h"
 
 
 
@@ -107,7 +105,7 @@ QString RewriterView::Error::toString() const
         str += tr("line %1").arg(line());
     }
 
-    if(column() != -1) {
+    if (column() != -1) {
         if (!str.isEmpty())
             str += QLatin1Char(' ');
 
@@ -155,9 +153,8 @@ void RewriterView::modelAttached(Model *model)
 
     ModelAmender differenceHandler(m_textToModelMerger.data());
     const QString qmlSource = m_textModifier->text();
-    if (m_textToModelMerger->load(qmlSource, differenceHandler)) {
+    if (m_textToModelMerger->load(qmlSource, differenceHandler))
         lastCorrectQmlSource = qmlSource;
-    }
 }
 
 void RewriterView::modelAboutToBeDetached(Model * /*model*/)
@@ -194,11 +191,6 @@ void RewriterView::nodeRemoved(const ModelNode &removedNode, const NodeAbstractP
         applyChanges();
 }
 
-void RewriterView::propertiesAdded(const ModelNode &/*node*/, const QList<AbstractProperty>& /*propertyList*/)
-{
-    Q_ASSERT(0);
-}
-
 void RewriterView::propertiesAboutToBeRemoved(const QList<AbstractProperty> &propertyList)
 {
     Q_ASSERT(textModifier());
@@ -264,6 +256,22 @@ void RewriterView::bindingPropertiesChanged(const QList<BindingProperty>& proper
         applyChanges();
 }
 
+void RewriterView::signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty> &propertyList, AbstractView::PropertyChangeFlags propertyChange)
+{
+    Q_ASSERT(textModifier());
+    if (textToModelMerger()->isActive())
+        return;
+
+    QList<AbstractProperty> usefulPropertyList;
+    foreach (const SignalHandlerProperty &property, propertyList)
+        usefulPropertyList.append(property);
+
+    modelToTextMerger()->propertiesChanged(usefulPropertyList, propertyChange);
+
+    if (!isModificationGroupActive())
+        applyChanges();
+}
+
 void RewriterView::nodeReparented(const ModelNode &node, const NodeAbstractProperty &newPropertyParent, const NodeAbstractProperty &oldPropertyParent, AbstractView::PropertyChangeFlags propertyChange)
 {
     Q_ASSERT(textModifier());
@@ -376,7 +384,7 @@ void RewriterView::scriptFunctionsChanged(const ModelNode & /*node*/, const QStr
 {
 }
 
-void RewriterView::instancePropertyChange(const QList<QPair<ModelNode, QString> > & /*propertyList*/)
+void RewriterView::instancePropertyChange(const QList<QPair<ModelNode, PropertyName> > & /*propertyList*/)
 {
 }
 
@@ -473,6 +481,16 @@ QString RewriterView::textModifierContent() const
     return QString();
 }
 
+void RewriterView::reactivateTextMofifierChangeSignals()
+{
+    textModifier()->reactivateChangeSignals();
+}
+
+void RewriterView::deactivateTextMofifierChangeSignals()
+{
+    textModifier()->deactivateChangeSignals();
+}
+
 void RewriterView::applyModificationGroupChanges()
 {
     Q_ASSERT(transactionLevel == 0);
@@ -493,11 +511,12 @@ void RewriterView::applyChanges()
         throw RewritingException(__LINE__, __FUNCTION__, __FILE__, "RewriterView::applyChanges() already in error state", content);
     }
 
+    m_differenceHandling = Validate;
+
     try {
         modelToTextMerger()->applyChanges();
-        if (!errors().isEmpty()) {
+        if (!errors().isEmpty())
             enterErrorState(errors().first().description());
-        }
     } catch (Exception &e) {
         const QString content = textModifierContent();
         qDebug() << "RewriterException:" << m_rewritingErrorMessage;
@@ -505,6 +524,8 @@ void RewriterView::applyChanges()
         enterErrorState(e.description());
     }
 
+    m_differenceHandling = Amend;
+
     if (inErrorState()) {
         const QString content = textModifierContent();
         qDebug() << "RewriterException:" << m_rewritingErrorMessage;
@@ -548,7 +569,7 @@ void RewriterView::resetToLastCorrectQml()
     m_textModifier->textDocument()->undo();
     m_textModifier->textDocument()->clearUndoRedoStacks(QTextDocument::RedoStack);
     ModelAmender differenceHandler(m_textToModelMerger.data());
-    m_textToModelMerger->load(m_textModifier->text().toUtf8(), differenceHandler);
+    m_textToModelMerger->load(m_textModifier->text(), differenceHandler);
 
     leaveErrorState();
 }
@@ -613,6 +634,31 @@ bool RewriterView::modificationGroupActive()
     return m_modificationGroupActive;
 }
 
+static bool isInNodeDefinition(int nodeTextOffset, int nodeTextLength, int cursorPosition)
+{
+    return (nodeTextOffset <= cursorPosition) && (nodeTextOffset + nodeTextLength > cursorPosition);
+}
+
+ModelNode RewriterView::nodeAtTextCursorPosition(int cursorPosition) const
+{
+    const QList<ModelNode> allNodes = allModelNodes();
+
+    ModelNode nearestNode;
+    int nearestNodeTextOffset = -1;
+
+    foreach (const ModelNode &currentNode, allNodes) {
+        const int nodeTextOffset = nodeOffset(currentNode);
+        const int nodeTextLength = nodeLength(currentNode);
+        if (isInNodeDefinition(nodeTextOffset, nodeTextLength, cursorPosition)
+            && (nodeTextOffset > nearestNodeTextOffset)) {
+            nearestNode = currentNode;
+            nearestNodeTextOffset = nodeTextOffset;
+        }
+    }
+
+    return nearestNode;
+}
+
 bool RewriterView::renameId(const QString& oldId, const QString& newId)
 {
     if (textModifier())
@@ -677,6 +723,24 @@ QString RewriterView::convertTypeToImportAlias(const QString &type) const
     return result;
 }
 
+QString RewriterView::pathForImport(const Import &import)
+{
+    if (scopeChain() && scopeChain()->context() && document()) {
+        const QString importStr = import.isFileImport() ? import.file() : import.url();
+        const QmlJS::Imports *imports = scopeChain()->context()->imports(document());
+
+        QmlJS::ImportInfo importInfo;
+
+        foreach (const QmlJS::Import &qmljsImport, imports->all()) {
+            if (qmljsImport.info.name() == importStr)
+                importInfo = qmljsImport.info;
+        }
+        const QString importPath = importInfo.path();
+        return importPath;
+    }
+
+    return QString();
+}
 
 void RewriterView::qmlTextChanged()
 {
@@ -695,9 +759,8 @@ void RewriterView::qmlTextChanged()
         switch (m_differenceHandling) {
             case Validate: {
                 ModelValidator differenceHandler(m_textToModelMerger.data());
-                if (m_textToModelMerger->load(newQmlText.toUtf8(), differenceHandler)) {
+                if (m_textToModelMerger->load(newQmlText, differenceHandler))
                     lastCorrectQmlSource = newQmlText;
-                }
                 break;
             }
 
@@ -705,9 +768,8 @@ void RewriterView::qmlTextChanged()
             default: {
                 emitCustomNotification(StartRewriterAmend);
                 ModelAmender differenceHandler(m_textToModelMerger.data());
-                if (m_textToModelMerger->load(newQmlText, differenceHandler)) {
+                if (m_textToModelMerger->load(newQmlText, differenceHandler))
                     lastCorrectQmlSource = newQmlText;
-                }
                 emitCustomNotification(EndRewriterAmend);
                 break;
             }
