@@ -1,46 +1,45 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qmljscompletionassist.h"
 #include "qmljseditorconstants.h"
 #include "qmljsreuse.h"
 #include "qmlexpressionundercursor.h"
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 
 #include <texteditor/codeassist/iassistinterface.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/functionhintproposal.h>
 #include <texteditor/codeassist/ifunctionhintproposalmodel.h>
+#include <texteditor/texteditorsettings.h>
+#include <texteditor/completionsettings.h>
 
 #include <utils/qtcassert.h>
 
@@ -52,22 +51,28 @@
 #include <qmljs/qmljsscanner.h>
 #include <qmljs/qmljsbind.h>
 #include <qmljs/qmljscompletioncontextfinder.h>
+#include <qmljs/qmljsbundle.h>
 #include <qmljs/qmljsscopebuilder.h>
+#include <projectexplorer/projectexplorer.h>
 
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QDir>
-#include <QtCore/QDebug>
-#include <QtCore/QtAlgorithms>
-#include <QtCore/QDirIterator>
-#include <QtCore/QStringList>
-#include <QtGui/QIcon>
+#include <QFile>
+#include <QFileInfo>
+#include <QDir>
+#include <QDebug>
+#include <QtAlgorithms>
+#include <QDirIterator>
+#include <QStringList>
+#include <QIcon>
+#include <QTextDocumentFragment>
 
 using namespace QmlJS;
-using namespace QmlJSEditor;
-using namespace Internal;
+using namespace QmlJSTools;
 using namespace TextEditor;
 
+namespace QmlJSEditor {
+
+using namespace Internal;
+
 namespace {
 
 enum CompletionOrder {
@@ -111,6 +116,13 @@ public:
     virtual void operator()(const Value *base, const QString &name, const Value *value) = 0;
 };
 
+class CompleteFunctionCall
+{
+public:
+    CompleteFunctionCall(bool hasArguments = true) : hasArguments(hasArguments) {}
+    bool hasArguments;
+};
+
 class CompletionAdder : public PropertyProcessor
 {
 protected:
@@ -127,8 +139,14 @@ public:
     virtual void operator()(const Value *base, const QString &name, const Value *value)
     {
         Q_UNUSED(base)
-        Q_UNUSED(value)
-        addCompletion(completions, name, icon, order);
+        QVariant data;
+        if (const FunctionValue *func = value->asFunctionValue()) {
+            // constructors usually also have other interesting members,
+            // don't consider them pure functions and complete the '()'
+            if (!func->lookupMember(QLatin1String("prototype"), 0, 0, false))
+                data = QVariant::fromValue(CompleteFunctionCall(func->namedArgumentCount() || func->isVariadic()));
+        }
+        addCompletion(completions, name, icon, order, data);
     }
 
     QIcon icon;
@@ -148,7 +166,7 @@ public:
 
     virtual void operator ()(const Value *base, const QString &name, const Value *)
     {
-        const QmlObjectValue *qmlBase = dynamic_cast<const QmlObjectValue *>(base);
+        const CppComponentValue *qmlBase = value_cast<CppComponentValue>(base);
 
         QString itemText = name;
         QString postfix;
@@ -319,6 +337,12 @@ bool isLiteral(AST::Node *ast)
 
 } // Anonymous
 
+} // namesapce QmlJSEditor
+
+Q_DECLARE_METATYPE(QmlJSEditor::CompleteFunctionCall)
+
+namespace QmlJSEditor {
+
 // -----------------------
 // QmlJSAssistProposalItem
 // -----------------------
@@ -338,16 +362,25 @@ void QmlJSAssistProposalItem::applyContextualContent(TextEditor::BaseTextEditor
     editor->setCursorPosition(basePosition);
     editor->remove(currentPosition - basePosition);
 
-    QString replaceable;
-    const QString &content = text();
-    if (content.endsWith(QLatin1String(": ")))
-        replaceable = QLatin1String(": ");
-    else if (content.endsWith(QLatin1Char('.')))
-        replaceable = QLatin1String(".");
+    QString content = text();
+    int cursorOffset = 0;
+
+    const CompletionSettings &completionSettings =
+            TextEditorSettings::instance()->completionSettings();
+    const bool autoInsertBrackets = completionSettings.m_autoInsertBrackets;
+
+    if (autoInsertBrackets && data().canConvert<CompleteFunctionCall>()) {
+        CompleteFunctionCall function = data().value<CompleteFunctionCall>();
+        content += QLatin1String("()");
+        if (function.hasArguments)
+            cursorOffset = -1;
+    }
+
+    QString replaceable = content;
     int replacedLength = 0;
     for (int i = 0; i < replaceable.length(); ++i) {
         const QChar a = replaceable.at(i);
-        const QChar b = editor->characterAt(editor->position() + i);
+        const QChar b = editor->textDocument()->characterAt(editor->position() + i);
         if (a == b)
             ++replacedLength;
         else
@@ -355,6 +388,8 @@ void QmlJSAssistProposalItem::applyContextualContent(TextEditor::BaseTextEditor
     }
     const int length = editor->position() - basePosition + replacedLength;
     editor->replace(length, content);
+    if (cursorOffset)
+        editor->setCursorPosition(editor->position() + cursorOffset);
 }
 
 // -------------------------
@@ -363,10 +398,12 @@ void QmlJSAssistProposalItem::applyContextualContent(TextEditor::BaseTextEditor
 class FunctionHintProposalModel : public TextEditor::IFunctionHintProposalModel
 {
 public:
-    FunctionHintProposalModel(const QString &functionName, const QStringList &signature)
+    FunctionHintProposalModel(const QString &functionName, const QStringList &namedArguments,
+                              int optionalNamedArguments, bool isVariadic)
         : m_functionName(functionName)
-        , m_signature(signature)
-        , m_minimumArgumentCount(signature.size())
+        , m_namedArguments(namedArguments)
+        , m_optionalNamedArguments(optionalNamedArguments)
+        , m_isVariadic(isVariadic)
     {}
 
     virtual void reset() {}
@@ -376,8 +413,9 @@ public:
 
 private:
     QString m_functionName;
-    QStringList m_signature;
-    int m_minimumArgumentCount;
+    QStringList m_namedArguments;
+    int m_optionalNamedArguments;
+    bool m_isVariadic;
 };
 
 QString FunctionHintProposalModel::text(int index) const
@@ -388,11 +426,13 @@ QString FunctionHintProposalModel::text(int index) const
     prettyMethod += QString::fromLatin1("function ");
     prettyMethod += m_functionName;
     prettyMethod += QLatin1Char('(');
-    for (int i = 0; i < m_minimumArgumentCount; ++i) {
+    for (int i = 0; i < m_namedArguments.size(); ++i) {
+        if (i == m_namedArguments.size() - m_optionalNamedArguments)
+            prettyMethod += QLatin1Char('[');
         if (i != 0)
             prettyMethod += QLatin1String(", ");
 
-        QString arg = m_signature.at(i);
+        QString arg = m_namedArguments.at(i);
         if (arg.isEmpty()) {
             arg = QLatin1String("arg");
             arg += QString::number(i + 1);
@@ -400,6 +440,13 @@ QString FunctionHintProposalModel::text(int index) const
 
         prettyMethod += arg;
     }
+    if (m_isVariadic) {
+        if (m_namedArguments.size())
+            prettyMethod += QLatin1String(", ");
+        prettyMethod += QLatin1String("...");
+    }
+    if (m_optionalNamedArguments)
+        prettyMethod += QLatin1Char(']');
     prettyMethod += QLatin1Char(')');
     return prettyMethod;
 }
@@ -429,9 +476,9 @@ int FunctionHintProposalModel::activeArgument(const QString &prefix) const
 // -----------------------------
 // QmlJSCompletionAssistProvider
 // -----------------------------
-bool QmlJSCompletionAssistProvider::supportsEditor(const QString &editorId) const
+bool QmlJSCompletionAssistProvider::supportsEditor(const Core::Id &editorId) const
 {
-    return editorId == QLatin1String(Constants::C_QMLJSEDITOR_ID);
+    return editorId == Constants::C_QMLJSEDITOR_ID;
 }
 
 int QmlJSCompletionAssistProvider::activationCharSequenceLength() const
@@ -459,7 +506,7 @@ IAssistProcessor *QmlJSCompletionAssistProvider::createProcessor() const
 // ------------------------------
 QmlJSCompletionAssistProcessor::QmlJSCompletionAssistProcessor()
     : m_startPosition(0)
-    , m_snippetCollector(Constants::QML_SNIPPETS_GROUP_ID, iconForColor(Qt::red), SnippetOrder)
+    , m_snippetCollector(QLatin1String(Constants::QML_SNIPPETS_GROUP_ID), iconForColor(Qt::red), SnippetOrder)
 {}
 
 QmlJSCompletionAssistProcessor::~QmlJSCompletionAssistProcessor()
@@ -472,10 +519,12 @@ IAssistProposal *QmlJSCompletionAssistProcessor::createContentProposal() const
     return proposal;
 }
 
-IAssistProposal *QmlJSCompletionAssistProcessor::createHintProposal(const QString &functionName,
-                                                                    const QStringList &signature) const
+IAssistProposal *QmlJSCompletionAssistProcessor::createHintProposal(
+        const QString &functionName, const QStringList &namedArguments,
+        int optionalNamedArguments, bool isVariadic) const
 {
-    IFunctionHintProposalModel *model = new FunctionHintProposalModel(functionName, signature);
+    IFunctionHintProposalModel *model = new FunctionHintProposalModel(
+                functionName, namedArguments, optionalNamedArguments, isVariadic);
     IAssistProposal *proposal = new FunctionHintProposal(m_startPosition, model);
     return proposal;
 }
@@ -487,10 +536,10 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
     if (assistInterface->reason() == IdleEditor && !acceptsIdleEditor())
         return 0;
 
-    const QString &fileName = m_interface->file()->fileName();
+    const QString &fileName = m_interface->fileName();
 
     m_startPosition = assistInterface->position();
-    while (isIdentifierChar(m_interface->document()->characterAt(m_startPosition - 1), false, false))
+    while (isIdentifierChar(m_interface->textDocument()->characterAt(m_startPosition - 1), false, false))
         --m_startPosition;
     const bool onIdentifier = m_startPosition != assistInterface->position();
 
@@ -520,9 +569,9 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
     // a +b<complete> -> '+'
     QChar completionOperator;
     if (m_startPosition > 0)
-        completionOperator = m_interface->document()->characterAt(m_startPosition - 1);
+        completionOperator = m_interface->textDocument()->characterAt(m_startPosition - 1);
 
-    QTextCursor startPositionCursor(qmlInterface->document());
+    QTextCursor startPositionCursor(qmlInterface->textDocument());
     startPositionCursor.setPosition(m_startPosition);
     CompletionContextFinder contextFinder(startPositionCursor);
 
@@ -552,7 +601,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
                 }
                 const Value *v = newScopeType->lookupMember(it->name.toString(), context);
                 v = context->lookupReference(v);
-                newScopeType = value_cast<const ObjectValue *>(v);
+                newScopeType = value_cast<ObjectValue>(v);
             }
             if (!newScopeType)
                 break;
@@ -566,7 +615,7 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
     if (contextFinder.isInStringLiteral()) {
         // get the text of the literal up to the cursor position
         //QTextCursor tc = textWidget->textCursor();
-        QTextCursor tc(qmlInterface->document());
+        QTextCursor tc(qmlInterface->textDocument());
         tc.setPosition(qmlInterface->position());
         QmlExpressionUnderCursor expressionUnderCursor;
         expressionUnderCursor(tc);
@@ -603,12 +652,46 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
 
         return 0;
     }
+
+    // currently path-in-stringliteral is the only completion available in imports
+    if (contextFinder.isInImport()) {
+        QmlJS::ModelManagerInterface::ProjectInfo pInfo = QmlJS::ModelManagerInterface::instance()
+                ->projectInfo(ProjectExplorer::ProjectExplorerPlugin::currentProject());
+        QmlBundle platform = pInfo.extendedBundle.bundleForLanguage(document->language());
+        if (!platform.supportedImports().isEmpty()) {
+            QTextCursor tc(qmlInterface->textDocument());
+            tc.setPosition(qmlInterface->position());
+            QmlExpressionUnderCursor expressionUnderCursor;
+            expressionUnderCursor(tc);
+            QString libVersion = contextFinder.libVersionImport();
+            if (!libVersion.isNull()) {
+                QStringList completions=platform.supportedImports().complete(libVersion, QString(), QmlJS::PersistentTrie::LookupFlags(QmlJS::PersistentTrie::CaseInsensitive|QmlJS::PersistentTrie::SkipChars|QmlJS::PersistentTrie::SkipSpaces));
+                completions = QmlJS::PersistentTrie::matchStrengthSort(libVersion, completions);
+
+                int toSkip = qMax(libVersion.lastIndexOf(QLatin1Char(' '))
+                                  , libVersion.lastIndexOf(QLatin1Char('.')));
+                if (++toSkip > 0) {
+                    QStringList nCompletions;
+                    QString prefix(libVersion.left(toSkip));
+                    nCompletions.reserve(completions.size());
+                    foreach (const QString &completion, completions)
+                        if (completion.startsWith(prefix))
+                            nCompletions.append(completion.right(completion.size()-toSkip));
+                    completions = nCompletions;
+                }
+                addCompletions(&m_completions, completions, m_interface->fileNameIcon(), KeywordOrder);
+                return createContentProposal();
+            }
+        }
+        return 0;
+    }
+
     // member "a.bc<complete>" or function "foo(<complete>" completion
-    else if (completionOperator == QLatin1Char('.')
-             || (completionOperator == QLatin1Char('(') && !onIdentifier)) {
+    if (completionOperator == QLatin1Char('.')
+            || (completionOperator == QLatin1Char('(') && !onIdentifier)) {
         // Look at the expression under cursor.
         //QTextCursor tc = textWidget->textCursor();
-        QTextCursor tc(qmlInterface->document());
+        QTextCursor tc(qmlInterface->textDocument());
         tc.setPosition(m_startPosition - 1);
 
         QmlExpressionUnderCursor expressionUnderCursor;
@@ -642,11 +725,12 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
                     if (indexOfDot != -1)
                         functionName = functionName.mid(indexOfDot + 1);
 
-                    QStringList signature;
-                    for (int i = 0; i < f->argumentCount(); ++i)
-                        signature.append(f->argumentName(i));
+                    QStringList namedArguments;
+                    for (int i = 0; i < f->namedArgumentCount(); ++i)
+                        namedArguments.append(f->argumentName(i));
 
-                    return createHintProposal(functionName.trimmed(), signature);
+                    return createHintProposal(functionName.trimmed(), namedArguments,
+                                              f->optionalNamedArgumentCount(), f->isVariadic());
                 }
             }
         }
@@ -655,8 +739,9 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
             return createContentProposal();
         return 0;
     }
+
     // global completion
-    else if (onIdentifier || assistInterface->reason() == ExplicitlyInvoked) {
+    if (onIdentifier || assistInterface->reason() == ExplicitlyInvoked) {
 
         bool doGlobalCompletion = true;
         bool doQmlKeywordCompletion = true;
@@ -700,14 +785,14 @@ IAssistProposal *QmlJSCompletionAssistProcessor::perform(const IAssistInterface
             const Value *value =
                     getPropertyValue(qmlScopeType, contextFinder.bindingPropertyName(), context);
             if (const QmlEnumValue *enumValue =
-                    dynamic_cast<const QmlEnumValue *>(value)) {
+                    value_cast<QmlEnumValue>(value)) {
                 const QString &name = context->imports(document.data())->nameForImportedObject(enumValue->owner(), context.data());
                 foreach (const QString &key, enumValue->keys()) {
                     QString completion;
                     if (name.isEmpty())
-                        completion = QString("\"%1\"").arg(key);
+                        completion = QString::fromLatin1("\"%1\"").arg(key);
                     else
-                        completion = QString("%1.%2").arg(name, key);
+                        completion = QString::fromLatin1("%1.%2").arg(name, key);
                     addCompletion(&m_completions, key, m_interface->symbolIcon(),
                                   EnumValueOrder, completion);
                 }
@@ -772,11 +857,11 @@ bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
     const int cursorPos = m_interface->position();
 
     bool maybeAccept = false;
-    const QChar &charBeforeCursor = m_interface->document()->characterAt(cursorPos - 1);
+    const QChar &charBeforeCursor = m_interface->textDocument()->characterAt(cursorPos - 1);
     if (isActivationChar(charBeforeCursor)) {
         maybeAccept = true;
     } else {
-        const QChar &charUnderCursor = m_interface->document()->characterAt(cursorPos);
+        const QChar &charUnderCursor = m_interface->textDocument()->characterAt(cursorPos);
         if (isIdentifierChar(charBeforeCursor)
                 && ((charUnderCursor.isSpace()
                     || charUnderCursor.isNull()
@@ -785,7 +870,7 @@ bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
 
             int startPos = cursorPos - 1;
             for (; startPos != -1; --startPos) {
-                if (!isIdentifierChar(m_interface->document()->characterAt(startPos)))
+                if (!isIdentifierChar(m_interface->textDocument()->characterAt(startPos)))
                     break;
             }
             ++startPos;
@@ -802,7 +887,7 @@ bool QmlJSCompletionAssistProcessor::acceptsIdleEditor() const
     }
 
     if (maybeAccept) {
-        QTextCursor tc(m_interface->document());
+        QTextCursor tc(m_interface->textDocument());
         tc.setPosition(m_interface->position());
         const QTextBlock &block = tc.block();
         const QString &blockText = block.text();
@@ -883,12 +968,12 @@ bool QmlJSCompletionAssistProcessor::completeUrl(const QString &relativeBasePath
 // ------------------------------
 // QmlJSCompletionAssistInterface
 // ------------------------------
-QmlJSCompletionAssistInterface::QmlJSCompletionAssistInterface(QTextDocument *document,
+QmlJSCompletionAssistInterface::QmlJSCompletionAssistInterface(QTextDocument *textDocument,
                                                                int position,
-                                                               Core::IFile *file,
+                                                               const QString &fileName,
                                                                TextEditor::AssistReason reason,
                                                                const SemanticInfo &info)
-    : DefaultAssistInterface(document, position, file, reason)
+    : DefaultAssistInterface(textDocument, position, fileName, reason)
     , m_semanticInfo(info)
     , m_darkBlueIcon(iconForColor(Qt::darkBlue))
     , m_darkYellowIcon(iconForColor(Qt::darkYellow))
@@ -927,6 +1012,19 @@ struct QmlJSLessThan
 // -------------------------
 // QmlJSAssistProposalModel
 // -------------------------
+void QmlJSAssistProposalModel::filter(const QString &prefix)
+{
+    BasicProposalItemListModel::filter(prefix);
+    if (prefix.startsWith(QLatin1String("__")))
+        return;
+    QList<BasicProposalItem *> newCurrentItems;
+    newCurrentItems.reserve(m_currentItems.size());
+    foreach (BasicProposalItem *item, m_currentItems)
+        if (!item->text().startsWith(QLatin1String("__")))
+            newCurrentItems << item;
+    m_currentItems = newCurrentItems;
+}
+
 void QmlJSAssistProposalModel::sort()
 {
     qSort(currentItems().first, currentItems().second, QmlJSLessThan());
@@ -936,3 +1034,5 @@ bool QmlJSAssistProposalModel::keepPerfectMatch(TextEditor::AssistReason reason)
 {
     return reason == ExplicitlyInvoked;
 }
+
+} // namespace QmlJSEditor
