@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qmljssemantichighlighter.h"
 
@@ -43,6 +40,7 @@
 #include <qmljs/qmljsutils.h>
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsastvisitor_p.h>
+#include <qmljs/qmljsstaticanalysismessage.h>
 #include <texteditor/syntaxhighlighter.h>
 #include <texteditor/basetextdocument.h>
 #include <texteditor/texteditorconstants.h>
@@ -53,11 +51,13 @@
 #include <QFutureInterface>
 #include <QRunnable>
 
-using namespace QmlJSEditor;
-using namespace QmlJSEditor::Internal;
 using namespace QmlJS;
 using namespace QmlJS::AST;
 
+namespace QmlJSEditor {
+
+using namespace Internal;
+
 namespace {
 
 template <typename T>
@@ -192,10 +192,15 @@ class CollectionTask :
         protected Visitor
 {
 public:
-    CollectionTask(const ScopeChain &scopeChain)
-        : m_scopeChain(scopeChain)
+    CollectionTask(const QmlJSTools::SemanticInfo &semanticInfo,
+                   SemanticHighlighter &semanticHighlighter)
+        : m_semanticInfo(semanticInfo)
+        , m_semanticHighlighter(semanticHighlighter)
+        , m_scopeChain(semanticInfo.scopeChain())
         , m_scopeBuilder(&m_scopeChain)
         , m_lineOfLastUse(0)
+        , m_nextExtraFormat(SemanticHighlighter::Max)
+        , m_currentDelayedUse(0)
     {}
 
 protected:
@@ -273,11 +278,10 @@ protected:
 
     bool visit(UiObjectDefinition *ast)
     {
-        if (m_scopeChain.document()->bind()->isGroupedPropertyBinding(ast)) {
+        if (m_scopeChain.document()->bind()->isGroupedPropertyBinding(ast))
             processBindingName(ast->qualifiedTypeNameId);
-        } else {
+        else
             processTypeId(ast->qualifiedTypeNameId);
-        }
         scopedAccept(ast, ast->initializer);
         return false;
     }
@@ -345,19 +349,111 @@ protected:
             return false;
 
         const QString &value = ast->value.toString();
-        if (m_stateNames.contains(value)) {
+        if (m_stateNames.contains(value))
             addUse(ast->literalToken, SemanticHighlighter::LocalStateNameType);
-        }
 
         return false;
     }
 
+    void addMessages(QList<QmlJS::DiagnosticMessage> messages,
+            const Document::Ptr &doc)
+    {
+        foreach (const DiagnosticMessage &d, messages) {
+            int line = d.loc.startLine;
+            int column = qMax(1U, d.loc.startColumn);
+            int length = d.loc.length;
+            int begin = d.loc.begin();
+
+            if (d.loc.length == 0) {
+                QString source(doc->source());
+                int end = begin;
+                if (begin == source.size() || source.at(begin) == QLatin1Char('\n')
+                        || source.at(begin) == QLatin1Char('\r')) {
+                    while (begin > end - column && !source.at(--begin).isSpace()) { }
+                } else {
+                    while (end < source.size() && source.at(++end).isLetterOrNumber()) { }
+                }
+                column += begin - d.loc.begin();
+                length = end-begin;
+            }
+
+            QTextCharFormat format;
+            if (d.isWarning())
+                format.setUnderlineColor(Qt::darkYellow);
+            else
+                format.setUnderlineColor(Qt::red);
+
+            format.setUnderlineStyle(QTextCharFormat::WaveUnderline);
+            format.setToolTip(d.message);
+
+            collectRanges(begin, length, format);
+            addDelayedUse(SemanticHighlighter::Use(line, column, length, addFormat(format)));
+        }
+    }
+
+    void addMessages(const QList<QmlJS::StaticAnalysis::Message> &messages,
+                     const Document::Ptr &doc)
+    {
+        foreach (const QmlJS::StaticAnalysis::Message &d, messages) {
+            int line = d.location.startLine;
+            int column = qMax(1U, d.location.startColumn);
+            int length = d.location.length;
+            int begin = d.location.begin();
+
+            if (d.location.length == 0) {
+                QString source(doc->source());
+                int end = begin;
+                if (begin == source.size() || source.at(begin) == QLatin1Char('\n')
+                        || source.at(begin) == QLatin1Char('\r')) {
+                    while (begin > end - column && !source.at(--begin).isSpace()) { }
+                } else {
+                    while (end < source.size() && source.at(++end).isLetterOrNumber()) { }
+                }
+                column += begin - d.location.begin();
+                length = end-begin;
+            }
+            QTextCharFormat format;
+            if (d.severity == StaticAnalysis::Warning || d.severity == StaticAnalysis::MaybeWarning)
+                format.setUnderlineColor(Qt::darkYellow);
+            else if (d.severity == StaticAnalysis::Error || d.severity == StaticAnalysis::MaybeError)
+                format.setUnderlineColor(Qt::red);
+            else if (d.severity == StaticAnalysis::Hint)
+                format.setUnderlineColor(Qt::darkGreen);
+
+            format.setUnderlineStyle(QTextCharFormat::WaveUnderline);
+            format.setToolTip(d.message);
+
+            collectRanges(begin, length, format);
+            addDelayedUse(SemanticHighlighter::Use(line, column, length, addFormat(format)));
+        }
+    }
+
 private:
     void run()
     {
+        int nMessages = 0;
+        if (Document::isFullySupportedLanguage(m_scopeChain.document()->language())) {
+            nMessages = m_scopeChain.document()->diagnosticMessages().size()
+                    + m_semanticInfo.semanticMessages.size()
+                    + m_semanticInfo.staticAnalysisMessages.size();
+            m_delayedUses.reserve(nMessages);
+            m_diagnosticRanges.reserve(nMessages);
+            m_extraFormats.reserve(nMessages);
+            addMessages(m_scopeChain.document()->diagnosticMessages(), m_scopeChain.document());
+            addMessages(m_semanticInfo.semanticMessages, m_semanticInfo.document);
+            addMessages(m_semanticInfo.staticAnalysisMessages, m_semanticInfo.document);
+
+            qSort(m_delayedUses.begin(), m_delayedUses.end(), sortByLinePredicate);
+        }
+        m_currentDelayedUse = 0;
+
+        m_semanticHighlighter.reportMessagesInfo(m_diagnosticRanges, m_extraFormats);
+
         Node *root = m_scopeChain.document()->ast();
         m_stateNames = CollectStateNames(m_scopeChain)(root);
         accept(root);
+        while (m_currentDelayedUse < m_delayedUses.size())
+            m_uses.append(m_delayedUses.value(m_currentDelayedUse++));
         flush();
         reportFinished();
     }
@@ -371,6 +467,10 @@ private:
 
     void addUse(const SemanticHighlighter::Use &use)
     {
+        while (m_currentDelayedUse < m_delayedUses.size()
+               && m_delayedUses.value(m_currentDelayedUse).line < use.line)
+            m_uses.append(m_delayedUses.value(m_currentDelayedUse++));
+
         if (m_uses.size() >= chunkSize) {
             if (use.line > m_lineOfLastUse)
                 flush();
@@ -380,6 +480,26 @@ private:
         m_uses.append(use);
     }
 
+    void addDelayedUse(const SemanticHighlighter::Use &use)
+    {
+        m_delayedUses.append(use);
+    }
+
+    int addFormat(const QTextCharFormat &format)
+    {
+        int res = m_nextExtraFormat++;
+        m_extraFormats.insert(res, format);
+        return res;
+    }
+
+    void collectRanges(int start, int length, const QTextCharFormat &format) {
+        QTextLayout::FormatRange range;
+        range.start = start;
+        range.length = length;
+        range.format = format;
+        m_diagnosticRanges.append(range);
+    }
+
     static bool sortByLinePredicate(const SemanticHighlighter::Use &lhs, const SemanticHighlighter::Use &rhs)
     {
         return lhs.line < rhs.line;
@@ -398,11 +518,18 @@ private:
         m_uses.reserve(chunkSize);
     }
 
+    const QmlJSTools::SemanticInfo &m_semanticInfo;
+    SemanticHighlighter &m_semanticHighlighter;
     ScopeChain m_scopeChain;
     ScopeBuilder m_scopeBuilder;
     QStringList m_stateNames;
     QVector<SemanticHighlighter::Use> m_uses;
     unsigned m_lineOfLastUse;
+    QVector<SemanticHighlighter::Use> m_delayedUses;
+    int m_nextExtraFormat;
+    int m_currentDelayedUse;
+    QHash<int, QTextCharFormat> m_extraFormats;
+    QVector<QTextLayout::FormatRange> m_diagnosticRanges;
 };
 
 } // anonymous namespace
@@ -418,13 +545,13 @@ SemanticHighlighter::SemanticHighlighter(QmlJSTextEditorWidget *editor)
             this, SLOT(finished()));
 }
 
-void SemanticHighlighter::rerun(const ScopeChain &scopeChain)
+void SemanticHighlighter::rerun(const QmlJSTools::SemanticInfo &semanticInfo)
 {
     m_watcher.cancel();
 
     // this does not simply use QtConcurrentRun because we want a low-priority future
     // the thread pool deletes the task when it is done
-    CollectionTask::Future f = (new CollectionTask(scopeChain))->start(QThread::LowestPriority);
+    CollectionTask::Future f = (new CollectionTask(semanticInfo, *this))->start(QThread::LowestPriority);
     m_startRevision = m_editor->editorRevision();
     m_watcher.setFuture(f);
 }
@@ -441,13 +568,13 @@ void SemanticHighlighter::applyResults(int from, int to)
     if (m_startRevision != m_editor->editorRevision())
         return;
 
-    TextEditor::BaseTextDocument *baseTextDocument = m_editor->baseTextDocument();
+    TextEditor::BaseTextDocument *baseTextDocument = m_editor->baseTextDocument().data();
     QTC_ASSERT(baseTextDocument, return);
     TextEditor::SyntaxHighlighter *highlighter = qobject_cast<TextEditor::SyntaxHighlighter *>(baseTextDocument->syntaxHighlighter());
     QTC_ASSERT(highlighter, return);
 
     TextEditor::SemanticHighlighter::incrementalApplyExtraAdditionalFormats(
-                highlighter, m_watcher.future(), from, to, m_formats);
+                highlighter, m_watcher.future(), from, to, m_extraFormats);
 }
 
 void SemanticHighlighter::finished()
@@ -457,10 +584,11 @@ void SemanticHighlighter::finished()
     if (m_startRevision != m_editor->editorRevision())
         return;
 
-    TextEditor::BaseTextDocument *baseTextDocument = m_editor->baseTextDocument();
+    TextEditor::BaseTextDocument *baseTextDocument = m_editor->baseTextDocument().data();
     QTC_ASSERT(baseTextDocument, return);
     TextEditor::SyntaxHighlighter *highlighter = qobject_cast<TextEditor::SyntaxHighlighter *>(baseTextDocument->syntaxHighlighter());
     QTC_ASSERT(highlighter, return);
+    m_editor->m_diagnosticRanges = m_diagnosticRanges;
 
     TextEditor::SemanticHighlighter::clearExtraAdditionalFormatsUntilEnd(
                 highlighter, m_watcher.future());
@@ -468,21 +596,35 @@ void SemanticHighlighter::finished()
 
 void SemanticHighlighter::updateFontSettings(const TextEditor::FontSettings &fontSettings)
 {
-    m_formats[LocalIdType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_LOCAL_ID));
-    m_formats[ExternalIdType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_EXTERNAL_ID));
-    m_formats[QmlTypeType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_TYPE_ID));
-    m_formats[RootObjectPropertyType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_ROOT_OBJECT_PROPERTY));
-    m_formats[ScopeObjectPropertyType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_SCOPE_OBJECT_PROPERTY));
-    m_formats[ExternalObjectPropertyType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_EXTERNAL_OBJECT_PROPERTY));
-    m_formats[JsScopeType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_JS_SCOPE_VAR));
-    m_formats[JsImportType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_JS_IMPORT_VAR));
-    m_formats[JsGlobalType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_JS_GLOBAL_VAR));
-    m_formats[LocalStateNameType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_QML_STATE_NAME));
-    m_formats[BindingNameType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_BINDING));
-    m_formats[FieldType] = fontSettings.toTextCharFormat(QLatin1String(TextEditor::Constants::C_FIELD));
+    m_formats[LocalIdType] = fontSettings.toTextCharFormat(TextEditor::C_QML_LOCAL_ID);
+    m_formats[ExternalIdType] = fontSettings.toTextCharFormat(TextEditor::C_QML_EXTERNAL_ID);
+    m_formats[QmlTypeType] = fontSettings.toTextCharFormat(TextEditor::C_QML_TYPE_ID);
+    m_formats[RootObjectPropertyType] = fontSettings.toTextCharFormat(TextEditor::C_QML_ROOT_OBJECT_PROPERTY);
+    m_formats[ScopeObjectPropertyType] = fontSettings.toTextCharFormat(TextEditor::C_QML_SCOPE_OBJECT_PROPERTY);
+    m_formats[ExternalObjectPropertyType] = fontSettings.toTextCharFormat(TextEditor::C_QML_EXTERNAL_OBJECT_PROPERTY);
+    m_formats[JsScopeType] = fontSettings.toTextCharFormat(TextEditor::C_JS_SCOPE_VAR);
+    m_formats[JsImportType] = fontSettings.toTextCharFormat(TextEditor::C_JS_IMPORT_VAR);
+    m_formats[JsGlobalType] = fontSettings.toTextCharFormat(TextEditor::C_JS_GLOBAL_VAR);
+    m_formats[LocalStateNameType] = fontSettings.toTextCharFormat(TextEditor::C_QML_STATE_NAME);
+    m_formats[BindingNameType] = fontSettings.toTextCharFormat(TextEditor::C_BINDING);
+    m_formats[FieldType] = fontSettings.toTextCharFormat(TextEditor::C_FIELD);
+}
+
+void SemanticHighlighter::reportMessagesInfo(const QVector<QTextLayout::FormatRange> &diagnosticRanges,
+                                             const QHash<int,QTextCharFormat> &formats)
+
+{
+    // tricky usage of m_extraFormats and diagnosticMessages we call this in another thread...
+    // but will use them only after a signal sent by that same thread, maybe we should transfer
+    // them more explicitly
+    m_extraFormats = formats;
+    m_extraFormats.unite(m_formats);
+    m_diagnosticRanges = diagnosticRanges;
 }
 
 int SemanticHighlighter::startRevision() const
 {
     return m_startRevision;
 }
+
+} // namespace QmlJSEditor
