@@ -1,46 +1,41 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "qmljsplugindumper.h"
 #include "qmljsmodelmanager.h"
 
-#include <qmljs/qmljsdocument.h>
 #include <qmljs/qmljsinterpreter.h>
 #include <projectexplorer/projectexplorer.h>
 #include <coreplugin/messagemanager.h>
 #include <utils/filesystemwatcher.h>
 #include <utils/fileutils.h>
 
-#include <QMetaType>
 #include <QDir>
 
 using namespace LanguageUtils;
@@ -122,6 +117,13 @@ void PluginDumper::onLoadBuiltinTypes(const QmlJS::ModelManagerInterface::Projec
         loadQmltypesFile(QStringList(builtinQmltypesPath), info.qtImportsPath, builtinInfo);
         return;
     }
+    // QTDIR/imports/QtQuick1/builtins.qmltypes was used in developer builds of 5.0.0, 5.0.1
+    const QString builtinQmltypesPath2 = info.qtImportsPath
+            + QLatin1String("/QtQuick1/builtins.qmltypes");
+    if (QFile::exists(builtinQmltypesPath2)) {
+        loadQmltypesFile(QStringList(builtinQmltypesPath2), info.qtImportsPath, builtinInfo);
+        return;
+    }
 
     // run qmldump
     QProcess *process = new QProcess(this);
@@ -138,7 +140,7 @@ static QString makeAbsolute(const QString &path, const QString &base)
 {
     if (QFileInfo(path).isAbsolute())
         return path;
-    return QString("%1%2%3").arg(base, QDir::separator(), path);
+    return QString::fromLatin1("%1%2%3").arg(base, QDir::separator(), path);
 }
 
 void PluginDumper::onLoadPluginTypes(const QString &libraryPath, const QString &importPath, const QString &importUri, const QString &importVersion)
@@ -254,7 +256,48 @@ static void printParseWarnings(const QString &libraryPath, const QString &warnin
     Core::MessageManager *messageManager = Core::MessageManager::instance();
     messageManager->printToOutputPane(
                 PluginDumper::tr("Warnings while parsing qmltypes information of %1:\n"
-                                 "%2").arg(libraryPath, warning));
+                                 "%2").arg(libraryPath, warning),
+                Core::MessageManager::Flash);
+}
+
+static QString qmlPluginDumpErrorMessage(QProcess *process)
+{
+    QString errorMessage;
+#if QT_VERSION >= 0x050000
+    const QString binary = QDir::toNativeSeparators(process->program());
+#else
+    const QString binary = QLatin1String("qmlplugindump");
+#endif
+    switch (process->error()) {
+    case QProcess::FailedToStart:
+        errorMessage = PluginDumper::tr("\"%1\" failed to start: %2").arg(binary, process->errorString());
+        break;
+    case QProcess::Crashed:
+        errorMessage = PluginDumper::tr("\"%1\" crashed.").arg(binary);
+        break;
+    case QProcess::Timedout:
+        errorMessage = PluginDumper::tr("\"%1\" timed out.").arg(binary);
+        break;
+    case QProcess::ReadError:
+    case QProcess::WriteError:
+        errorMessage = PluginDumper::tr("I/O error running \"%1\".").arg(binary);
+        break;
+    case QProcess::UnknownError:
+        if (process->exitCode())
+            errorMessage = PluginDumper::tr("\"%1\" returned exit code %2.").arg(binary).arg(process->exitCode());
+        break;
+    }
+#if QT_VERSION >= 0x050000
+    errorMessage += QLatin1Char('\n') + PluginDumper::tr("Arguments: %1").arg(process->arguments().join(QLatin1Char(' ')));
+#endif
+    if (process->error() != QProcess::FailedToStart) {
+        const QString stdErr = QString::fromLocal8Bit(process->readAllStandardError());
+        if (!stdErr.isEmpty()) {
+            errorMessage += QLatin1Char('\n');
+            errorMessage += stdErr;
+        }
+    }
+    return errorMessage;
 }
 
 void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
@@ -272,8 +315,9 @@ void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
 
     if (exitCode != 0) {
         Core::MessageManager *messageManager = Core::MessageManager::instance();
-        const QString errorMessages = process->readAllStandardError();
-        messageManager->printToOutputPane(qmldumpErrorMessage(libraryPath, errorMessages));
+        const QString errorMessages = qmlPluginDumpErrorMessage(process);
+        messageManager->printToOutputPane(qmldumpErrorMessage(libraryPath, errorMessages),
+                                          Core::MessageManager::Flash);
         libraryInfo.setPluginTypeInfoStatus(LibraryInfo::DumpError, qmldumpFailedMessage(libraryPath, errorMessages));
     }
 
@@ -282,7 +326,8 @@ void PluginDumper::qmlPluginTypeDumpDone(int exitCode)
     QString warning;
     CppQmlTypesLoader::BuiltinObjects objectsList;
     QList<ModuleApiInfo> moduleApis;
-    CppQmlTypesLoader::parseQmlTypeDescriptions(output, &objectsList, &moduleApis, &error, &warning);
+    CppQmlTypesLoader::parseQmlTypeDescriptions(output, &objectsList, &moduleApis, &error, &warning,
+                                                QLatin1String("<dump of ") + libraryPath + QLatin1String(">"));
     if (exitCode == 0) {
         if (!error.isEmpty()) {
             libraryInfo.setPluginTypeInfoStatus(LibraryInfo::DumpError,
@@ -312,9 +357,9 @@ void PluginDumper::qmlPluginTypeDumpError(QProcess::ProcessError)
         return;
 
     Core::MessageManager *messageManager = Core::MessageManager::instance();
-    const QString errorMessages = process->readAllStandardError();
-    messageManager->printToOutputPane(qmldumpErrorMessage(libraryPath, errorMessages));
-
+    const QString errorMessages = qmlPluginDumpErrorMessage(process);
+    messageManager->printToOutputPane(qmldumpErrorMessage(libraryPath, errorMessages),
+                                      Core::MessageManager::Flash);
     if (!libraryPath.isEmpty()) {
         const Snapshot snapshot = m_modelManager->snapshot();
         LibraryInfo libraryInfo = snapshot.libraryInfo(libraryPath);
@@ -353,7 +398,7 @@ void PluginDumper::loadQmltypesFile(const QStringList &qmltypesFilePaths,
         QString warning;
         CppQmlTypesLoader::BuiltinObjects newObjects;
         QList<ModuleApiInfo> newModuleApis;
-        CppQmlTypesLoader::parseQmlTypeDescriptions(reader.data(), &newObjects, &newModuleApis, &error, &warning);
+        CppQmlTypesLoader::parseQmlTypeDescriptions(reader.data(), &newObjects, &newModuleApis, &error, &warning, qmltypesFilePath);
         if (!error.isEmpty()) {
             errors += tr("Failed to parse '%1'.\nError: %2").arg(qmltypesFilePath, error);
         } else {
@@ -429,6 +474,8 @@ void PluginDumper::dump(const Plugin &plugin)
         if (ComponentVersion(plugin.importVersion).isValid())
             args << plugin.importVersion;
     } else {
+        if (info.qmlDumpHasRelocatableFlag)
+            args << QLatin1String("-relocatable");
         args << plugin.importUri;
         args << plugin.importVersion;
         args << plugin.importPath;
@@ -492,13 +539,12 @@ QString PluginDumper::resolvePlugin(const QDir &qmldirPath, const QString &qmldi
   Adapted from QDeclarativeImportDatabase::resolvePlugin.
 
   \table
-  \header \i Platform \i Valid suffixes
-  \row \i Windows     \i \c .dll
-  \row \i Unix/Linux  \i \c .so
-  \row \i AIX  \i \c .a
-  \row \i HP-UX       \i \c .sl, \c .so (HP-UXi)
-  \row \i Mac OS X    \i \c .dylib, \c .bundle, \c .so
-  \row \i Symbian     \i \c .dll
+  \header \li Platform \li Valid suffixes
+  \row \li Windows     \li \c .dll
+  \row \li Unix/Linux  \li \c .so
+  \row \li AIX  \li \c .a
+  \row \li HP-UX       \li \c .sl, \c .so (HP-UXi)
+  \row \li Mac OS X    \li \c .dylib, \c .bundle, \c .so
   \endtable
 
   Version number on unix are ignored.
