@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -35,6 +35,7 @@
 #include <projectexplorer/project.h>
 
 #include <QDeclarativeEngine>
+#include <QPointer>
 
 namespace ProjectExplorer { class RunConfiguration; }
 namespace QmlJS { class ModelManagerInterface; }
@@ -67,8 +68,6 @@ public:
 
     bool supportsKit(ProjectExplorer::Kit *k, QString *errorMessage) const;
 
-    QList<ProjectExplorer::BuildConfigWidget*> subConfigWidgets();
-
     ProjectExplorer::ProjectNode *rootProjectNode() const;
     QStringList files(FilesMode fileMode) const;
 
@@ -93,9 +92,13 @@ public:
 
     void refreshProjectFile();
 
+    enum QmlImport { UnknownImport, QtQuick1Import, QtQuick2Import };
+    QmlImport defaultImport() const;
+
 private slots:
     void refreshFiles(const QSet<QString> &added, const QSet<QString> &removed);
     void addedTarget(ProjectExplorer::Target *target);
+    void onActiveTargetChanged(ProjectExplorer::Target *target);
     void addedRunConfiguration(ProjectExplorer::RunConfiguration *);
 
 protected:
@@ -110,6 +113,7 @@ private:
     QString m_fileName;
     Internal::QmlProjectFile *m_file;
     QString m_projectName;
+    QmlImport m_defaultImport;
     QmlJS::ModelManagerInterface *m_modelManager;
 
     // plain format
@@ -117,7 +121,7 @@ private:
 
     // qml based, new format
     QDeclarativeEngine m_engine;
-    QWeakPointer<QmlProjectItem> m_projectItem;
+    QPointer<QmlProjectItem> m_projectItem;
 
     Internal::QmlProjectNode *m_rootNode;
 };
