@@ -1,49 +1,45 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "externaleditors.h"
 #include "qt4project.h"
-#include "qt4target.h"
 #include "qt4projectmanagerconstants.h"
-#include "qt4buildconfiguration.h"
 
+#include <utils/hostosinfo.h>
 #include <utils/synchronousprocess.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/target.h>
 #include <projectexplorer/session.h>
-#include <qtsupport/qtversionmanager.h>
+#include <qtsupport/qtkitinformation.h>
 #include <designer/designerconstants.h>
 
 #include <QProcess>
-#include <QFileInfo>
 #include <QDebug>
 #include <QSignalMapper>
 
@@ -78,9 +74,15 @@ static inline QString msgAppNotFound(const QString &id)
 }
 
 // -- Commands and helpers
-#ifdef Q_OS_MAC
-static const char linguistBinaryC[] = "Linguist";
-static const char designerBinaryC[] = "Designer";
+static QString linguistBinary()
+{
+    return QLatin1String(Utils::HostOsInfo::isMacHost() ? "Linguist" : "linguist");
+}
+
+static QString designerBinary()
+{
+    return QLatin1String(Utils::HostOsInfo::isMacHost() ? "Designer" : "designer");
+}
 
 // Mac: Change the call 'Foo.app/Contents/MacOS/Foo <filelist>' to
 // 'open -a Foo.app <filelist>'. doesn't support generic command line arguments
@@ -94,10 +96,6 @@ static void createMacOpenCommand(QString *binary, QStringList *arguments)
         *binary = QLatin1String("open");
     }
 }
-#else
-static const char linguistBinaryC[] = "linguist";
-static const char designerBinaryC[] = "designer";
-#endif
 
 static const char designerIdC[] = "Qt.Designer";
 static const char linguistIdC[] = "Qt.Linguist";
@@ -106,7 +104,7 @@ static const char designerDisplayName[] = QT_TRANSLATE_NOOP("OpenWith::Editors",
 static const char linguistDisplayName[] = QT_TRANSLATE_NOOP("OpenWith::Editors", "Qt Linguist");
 
 // -------------- ExternalQtEditor
-ExternalQtEditor::ExternalQtEditor(const QString &id,
+ExternalQtEditor::ExternalQtEditor(Core::Id id,
                                    const QString &displayName,
                                    const QString &mimetype,
                                    QObject *parent) :
@@ -142,12 +140,10 @@ bool ExternalQtEditor::getEditorLaunchData(const QString &fileName,
 {
     // Get the binary either from the current Qt version of the project or Path
     if (const Qt4Project *project = qt4ProjectFor(fileName)) {
-        if (const Qt4BaseTarget *target = project->activeTarget()) {
-            if (const Qt4BuildConfiguration *qt4bc = target->activeQt4BuildConfiguration()) {
-                if (const QtSupport::BaseQtVersion *qtVersion = qt4bc->qtVersion()) {
-                    data->binary = (qtVersion->*commandAccessor)();
-                    data->workingDirectory = project->projectDirectory();
-                }
+        if (const ProjectExplorer::Target *target = project->activeTarget()) {
+            if (const QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(target->kit())) {
+                data->binary = (qtVersion->*commandAccessor)();
+                data->workingDirectory = project->projectDirectory();
             }
         }
     }
@@ -162,12 +158,8 @@ bool ExternalQtEditor::getEditorLaunchData(const QString &fileName,
     // Setup binary + arguments, use Mac Open if appropriate
     data->arguments = additionalArguments;
     data->arguments.push_back(fileName);
-#ifdef Q_OS_MAC
-    if (useMacOpenCommand)
+    if (Utils::HostOsInfo::isMacHost() && useMacOpenCommand)
         createMacOpenCommand(&(data->binary), &(data->arguments));
-#else
-    Q_UNUSED(useMacOpenCommand)
-#endif
     if (debug)
         qDebug() << Q_FUNC_INFO << '\n' << data->binary << data->arguments;
     return true;
@@ -187,7 +179,7 @@ bool ExternalQtEditor::startEditorProcess(const EditorLaunchData &data, QString
 
 // --------------- LinguistExternalEditor
 LinguistExternalEditor::LinguistExternalEditor(QObject *parent) :
-       ExternalQtEditor(QLatin1String(linguistIdC),
+       ExternalQtEditor(linguistIdC,
                         QLatin1String(linguistDisplayName),
                         QLatin1String(Qt4ProjectManager::Constants::LINGUIST_MIMETYPE),
                         parent)
@@ -198,14 +190,13 @@ bool LinguistExternalEditor::startEditor(const QString &fileName, QString *error
 {
     EditorLaunchData data;
     return getEditorLaunchData(fileName, &QtSupport::BaseQtVersion::linguistCommand,
-                            QLatin1String(linguistBinaryC),
-                            QStringList(), true, &data, errorMessage)
-    && startEditorProcess(data, errorMessage);
+                            linguistBinary(), QStringList(), true, &data, errorMessage)
+            && startEditorProcess(data, errorMessage);
 }
 
 // --------------- MacDesignerExternalEditor, using Mac 'open'
 MacDesignerExternalEditor::MacDesignerExternalEditor(QObject *parent) :
-       ExternalQtEditor(QLatin1String(designerIdC),
+       ExternalQtEditor(designerIdC,
                         QLatin1String(designerDisplayName),
                         QLatin1String(Qt4ProjectManager::Constants::FORM_MIMETYPE),
                         parent)
@@ -216,15 +207,14 @@ bool MacDesignerExternalEditor::startEditor(const QString &fileName, QString *er
 {
     EditorLaunchData data;
     return getEditorLaunchData(fileName, &QtSupport::BaseQtVersion::designerCommand,
-                            QLatin1String(designerBinaryC),
-                            QStringList(), true, &data, errorMessage)
+                            designerBinary(), QStringList(), true, &data, errorMessage)
         && startEditorProcess(data, errorMessage);
     return false;
 }
 
 // --------------- DesignerExternalEditor with Designer Tcp remote control.
 DesignerExternalEditor::DesignerExternalEditor(QObject *parent) :
-    ExternalQtEditor(QLatin1String(designerIdC),
+    ExternalQtEditor(designerIdC,
                      QLatin1String(designerDisplayName),
                      QLatin1String(Designer::Constants::FORM_MIMETYPE),
                      parent),
@@ -252,8 +242,7 @@ bool DesignerExternalEditor::startEditor(const QString &fileName, QString *error
     EditorLaunchData data;
     // Find the editor binary
     if (!getEditorLaunchData(fileName, &QtSupport::BaseQtVersion::designerCommand,
-                            QLatin1String(designerBinaryC),
-                            QStringList(), false, &data, errorMessage)) {
+                            designerBinary(), QStringList(), false, &data, errorMessage)) {
         return false;
     }
     // Known one?
