@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef MAKESTEP_H
 #define MAKESTEP_H
@@ -36,27 +33,22 @@
 #include "qt4projectmanager_global.h"
 
 #include <projectexplorer/abstractprocessstep.h>
-#include <projectexplorer/buildstep.h>
-#include <projectexplorer/task.h>
-
-QT_BEGIN_NAMESPACE
-namespace Ui {
-class MakeStep;
-}
-QT_END_NAMESPACE
 
 namespace ProjectExplorer {
 class BuildStep;
-class GnuMakeParser;
 class IBuildStepFactory;
-class Project;
+class Task;
 }
 
 namespace Qt4ProjectManager {
+
 class Qt4BuildConfiguration;
+class MakeStepConfigWidget;
 
 namespace Internal {
 
+namespace Ui { class MakeStep; }
+
 class MakeStepFactory : public ProjectExplorer::IBuildStepFactory
 {
     Q_OBJECT
@@ -65,15 +57,15 @@ public:
     explicit MakeStepFactory(QObject *parent = 0);
     virtual ~MakeStepFactory();
 
-    bool canCreate(ProjectExplorer::BuildStepList *parent, const QString &id) const;
-    ProjectExplorer::BuildStep *create(ProjectExplorer::BuildStepList *parent, const QString &id);
+    bool canCreate(ProjectExplorer::BuildStepList *parent, const Core::Id id) const;
+    ProjectExplorer::BuildStep *create(ProjectExplorer::BuildStepList *parent, const Core::Id id);
     bool canClone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *source) const;
     ProjectExplorer::BuildStep *clone(ProjectExplorer::BuildStepList *parent, ProjectExplorer::BuildStep *source);
     bool canRestore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map) const;
     ProjectExplorer::BuildStep *restore(ProjectExplorer::BuildStepList *parent, const QVariantMap &map);
 
-    QStringList availableCreationIds(ProjectExplorer::BuildStepList *parent) const;
-    QString displayNameForId(const QString &id) const;
+    QList<Core::Id> availableCreationIds(ProjectExplorer::BuildStepList *parent) const;
+    QString displayNameForId(const Core::Id id) const;
 };
 } //namespace Internal
 
@@ -83,8 +75,7 @@ class QT4PROJECTMANAGER_EXPORT MakeStep : public ProjectExplorer::AbstractProces
 {
     Q_OBJECT
     friend class Internal::MakeStepFactory;
-    friend class MakeStepConfigWidget; // TODO remove this
-    // used to access internal stuff
+    friend class MakeStepConfigWidget;
 
 public:
     explicit MakeStep(ProjectExplorer::BuildStepList *bsl);
@@ -102,6 +93,7 @@ public:
     void setUserArguments(const QString &arguments);
     void setClean(bool clean);
     bool isClean() const;
+    QString makeCommand() const;
 
     QVariantMap toMap() const;
 
@@ -110,12 +102,15 @@ signals:
 
 protected:
     MakeStep(ProjectExplorer::BuildStepList *bsl, MakeStep *bs);
-    MakeStep(ProjectExplorer::BuildStepList *bsl, const QString &id);
+    MakeStep(ProjectExplorer::BuildStepList *bsl, const Core::Id id);
     virtual bool fromMap(const QVariantMap &map);
 
 private:
     void ctor();
+    void setMakeCommand(const QString &make);
+    QStringList automaticallyAddedArguments() const;
     bool m_clean;
+    bool m_scriptTarget;
     QString m_makeFileToCheck;
     QString m_userArgs;
     QString m_makeCmd;
@@ -136,14 +131,16 @@ private slots:
     void makeEdited();
     void makeArgumentsLineEdited();
 
-    void updateMakeOverrideLabel();
     void updateDetails();
     void userArgumentsChanged();
-    void qtVersionChanged();
+    void activeBuildConfigurationChanged();
 private:
-    Ui::MakeStep *m_ui;
+    void setSummaryText(const QString &text);
+
+    Internal::Ui::MakeStep *m_ui;
     MakeStep *m_makeStep;
     QString m_summaryText;
+    ProjectExplorer::BuildConfiguration *m_bc;
     bool m_ignoreChange;
 };
 
