@@ -33,15 +33,11 @@
 #include "qt4projectmanagerconstants.h"
 #include "profileeditor.h"
 
+#include <qtsupport/qtsupportconstants.h>
 #include <coreplugin/fileiconprovider.h>
-#include <coreplugin/editormanager/editormanager.h>
-#include <texteditor/texteditoractionhandler.h>
 #include <texteditor/texteditorsettings.h>
 
 #include <QCoreApplication>
-#include <QFileInfo>
-#include <QAction>
-#include <QMenu>
 
 using namespace Qt4ProjectManager;
 using namespace Qt4ProjectManager::Internal;
@@ -54,11 +50,11 @@ ProFileEditorFactory::ProFileEditorFactory(Qt4Manager *manager, TextEditor::Text
     m_actionHandler(handler)
 {
     Core::FileIconProvider *iconProvider = Core::FileIconProvider::instance();
-    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(Constants::ICON_QT_PROJECT)),
+    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(QtSupport::Constants::ICON_QT_PROJECT)),
                                         QLatin1String("pro"));
-    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(Constants::ICON_QT_PROJECT)),
+    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(QtSupport::Constants::ICON_QT_PROJECT)),
                                         QLatin1String("pri"));
-    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(Constants::ICON_QT_PROJECT)),
+    iconProvider->registerIconOverlayForSuffix(QIcon(QLatin1String(QtSupport::Constants::ICON_QT_PROJECT)),
                                         QLatin1String("prf"));
 }
 
@@ -68,7 +64,7 @@ ProFileEditorFactory::~ProFileEditorFactory()
 
 Core::Id ProFileEditorFactory::id() const
 {
-    return Qt4ProjectManager::Constants::PROFILE_EDITOR_ID;
+    return Core::Id(Qt4ProjectManager::Constants::PROFILE_EDITOR_ID);
 }
 
 QString ProFileEditorFactory::displayName() const
