@@ -35,21 +35,23 @@
 #include <utils/fileutils.h>
 
 #include <projectexplorer/abi.h>
-#include <projectexplorer/headerpath.h>
-#include <projectexplorer/task.h>
-#include <coreplugin/featureprovider.h>
 
 #include <QVariantMap>
-#include <QWidget>
 
 namespace Utils {
 class Environment;
 } // namespace Utils
 
+namespace Core {
+class FeatureSet;
+} // namespace Core
+
 namespace ProjectExplorer {
 class IOutputParser;
 class Kit;
 class ToolChain;
+class HeaderPath;
+class Task;
 } // namespace ProjectExplorer
 
 QT_BEGIN_NAMESPACE
@@ -59,6 +61,8 @@ QT_END_NAMESPACE
 
 namespace QtSupport
 {
+class QtConfigWidget;
+
 class QTSUPPORT_EXPORT QtVersionNumber
 {
 public:
@@ -75,21 +79,10 @@ public:
     bool operator >=(const QtVersionNumber &b) const;
     bool operator !=(const QtVersionNumber &b) const;
     bool operator ==(const QtVersionNumber &b) const;
-private:
-    bool checkVersionString(const QString &version) const;
 };
 
 namespace Internal { class QtOptionsPageWidget; }
 
-class QTSUPPORT_EXPORT QtConfigWidget : public QWidget
-{
-    Q_OBJECT
-public:
-    QtConfigWidget();
-signals:
-    void changed();
-};
-
 class QTSUPPORT_EXPORT BaseQtVersion
 {
     friend class QtVersionFactory;
@@ -128,9 +121,12 @@ public:
 
     // Returns the PREFIX, BINPREFIX, DOCPREFIX and similar information
     QHash<QString,QString> versionInfo() const;
-    static QString qmakeProperty(const QHash<QString,QString> &versionInfo, const QByteArray &name);
+    enum PropertyVariant { PropertyVariantGet, PropertyVariantSrc };
+    static QString qmakeProperty(const QHash<QString,QString> &versionInfo, const QByteArray &name,
+                                 PropertyVariant variant = PropertyVariantGet);
     QString qmakeProperty(const QByteArray &name) const;
     virtual void addToEnvironment(const ProjectExplorer::Kit *k, Utils::Environment &env) const;
+    virtual Utils::Environment qmakeRunEnvironment() const;
 
     virtual Utils::FileName sourcePath() const;
     // used by QtUiCodeModelSupport
@@ -185,13 +181,15 @@ public:
     ///         warnings and finally info items.
     QList<ProjectExplorer::Task> reportIssues(const QString &proFile, const QString &buildDir) const;
 
-    virtual ProjectExplorer::IOutputParser *createOutputParser() const;
-
-    static bool queryQMakeVariables(const Utils::FileName &binary, QHash<QString, QString> *versionInfo);
-    static bool queryQMakeVariables(const Utils::FileName &binary, QHash<QString, QString> *versionInfo, bool *qmakeIsExecutable);
+    static bool queryQMakeVariables(const Utils::FileName &binary, const Utils::Environment &env,
+                                    QHash<QString, QString> *versionInfo, QString *error = 0);
     static Utils::FileName mkspecDirectoryFromVersionInfo(const QHash<QString, QString> &versionInfo);
     static Utils::FileName mkspecFromVersionInfo(const QHash<QString, QString> &versionInfo);
 
+    static bool isQmlDebuggingSupported(ProjectExplorer::Kit *k, QString *reason = 0);
+    bool isQmlDebuggingSupported(QString *reason = 0) const;
+    static void buildDebuggingHelper(ProjectExplorer::Kit *k, int tools);
+    void buildDebuggingHelper(ProjectExplorer::ToolChain *tc, int tools);
 
     virtual bool supportsBinaryDebuggingHelper() const;
     virtual QString gdbDebuggingHelperLibrary() const;
@@ -202,6 +200,7 @@ public:
 
     virtual bool hasGdbDebuggingHelper() const;
     virtual bool hasQmlDump() const;
+    virtual bool hasQmlDumpWithRelocatableFlag() const;
     virtual bool needsQmlDump() const;
     virtual bool hasQmlDebuggingLibrary() const;
     virtual bool needsQmlDebuggingLibrary() const;
@@ -221,6 +220,19 @@ public:
 
     virtual QList<ProjectExplorer::Task> validateKit(const ProjectExplorer::Kit *k);
 
+    Utils::FileName headerPath() const;
+    Utils::FileName docsPath() const;
+    Utils::FileName libraryPath() const;
+    Utils::FileName binPath() const;
+    Utils::FileName mkspecsPath() const;
+
+    QString qtNamespace() const;
+    QString qtLibInfix() const;
+    bool isFrameworkBuild() const;
+    // Note: A Qt version can have both a debug and a release built at the same time!
+    bool hasDebugBuild() const;
+    bool hasReleaseBuild() const;
+
 protected:
     BaseQtVersion();
     BaseQtVersion(const Utils::FileName &path, bool isAutodetected = false, const QString &autodetectionSource = QString());
@@ -244,32 +256,37 @@ private:
     void updateMkspec() const;
     void setId(int id); // used by the qtversionmanager for legacy restore
                         // and by the qtoptionspage to replace Qt versions
-    QString m_displayName;
-    int m_id;
-    bool m_isAutodetected;
-    QString m_autodetectionSource;
 
-    mutable Utils::FileName m_sourcePath;
+    int m_id;
+
+    bool m_isAutodetected;
     mutable bool m_hasDebuggingHelper; // controlled by m_versionInfoUpToDate
     mutable bool m_hasQmlDump;         // controlled by m_versionInfoUpToDate
     mutable bool m_hasQmlDebuggingLibrary; // controlled by m_versionInfoUpdate
     mutable bool m_hasQmlObserver;     // controlled by m_versionInfoUpToDate
-
     mutable bool m_mkspecUpToDate;
-    mutable Utils::FileName m_mkspec;
-    mutable Utils::FileName m_mkspecFullPath;
-
     mutable bool m_mkspecReadUpToDate;
     mutable bool m_defaultConfigIsDebug;
     mutable bool m_defaultConfigIsDebugAndRelease;
-    mutable QHash<QString, QString> m_mkspecValues;
-
+    mutable bool m_frameworkBuild;
     mutable bool m_versionInfoUpToDate;
-    mutable QHash<QString,QString> m_versionInfo;
     mutable bool m_installed;
     mutable bool m_hasExamples;
     mutable bool m_hasDemos;
     mutable bool m_hasDocumentation;
+    mutable bool m_qmakeIsExecutable;
+    mutable bool m_hasQtAbis;
+
+    QString m_displayName;
+    QString m_autodetectionSource;
+    mutable Utils::FileName m_sourcePath;
+
+    mutable Utils::FileName m_mkspec;
+    mutable Utils::FileName m_mkspecFullPath;
+
+    mutable QHash<QString, QString> m_mkspecValues;
+
+    mutable QHash<QString,QString> m_versionInfo;
 
     mutable Utils::FileName m_qmakeCommand;
     mutable QString m_qtVersionString;
@@ -279,7 +296,6 @@ private:
     mutable QString m_qmlsceneCommand;
     mutable QString m_qmlviewerCommand;
 
-    mutable bool m_qmakeIsExecutable;
     mutable QList<ProjectExplorer::Abi> m_qtAbis;
 };
 }
