@@ -1,62 +1,60 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "genericproposalwidget.h"
-#include "iassistprovider.h"
 #include "igenericproposalmodel.h"
 #include "iassistproposalitem.h"
-#include "genericproposal.h"
 #include "codeassistant.h"
 
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/completionsettings.h>
+#include <texteditor/texteditorconstants.h>
 
 #include <utils/faketooltip.h>
+#include <utils/hostosinfo.h>
 
-#include <QtCore/QRect>
-#include <QtCore/QLatin1String>
-#include <QtCore/QAbstractListModel>
-#include <QtCore/QPointer>
-#include <QtCore/QDebug>
-#include <QtCore/QTimer>
-#include <QtGui/QApplication>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QListView>
-#include <QtGui/QAbstractItemView>
-#include <QtGui/QScrollBar>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QDesktopWidget>
-#include <QtGui/QLabel>
+#include <QRect>
+#include <QLatin1String>
+#include <QAbstractListModel>
+#include <QPointer>
+#include <QDebug>
+#include <QTimer>
+#include <QApplication>
+#include <QVBoxLayout>
+#include <QListView>
+#include <QAbstractItemView>
+#include <QScrollBar>
+#include <QKeyEvent>
+#include <QDesktopWidget>
+#include <QLabel>
 
+using namespace Utils;
 
 namespace TextEditor {
 
@@ -136,13 +134,12 @@ QVariant ModelAdapter::data(const QModelIndex &index, int role) const
     if (index.row() >= m_completionModel->size())
         return QVariant();
 
-    if (role == Qt::DisplayRole) {
+    if (role == Qt::DisplayRole)
         return m_completionModel->text(index.row());
-    } else if (role == Qt::DecorationRole) {
+    else if (role == Qt::DecorationRole)
         return m_completionModel->icon(index.row());
-    } else if (role == Qt::WhatsThisRole) {
+    else if (role == Qt::WhatsThisRole)
         return m_completionModel->detail(index.row());
-    }
 
     return QVariant();
 }
@@ -180,7 +177,7 @@ private:
 class GenericProposalListView : public QListView
 {
 public:
-    GenericProposalListView(QWidget *parent) : QListView(parent) {}
+    GenericProposalListView(QWidget *parent);
 
     QSize calculateSize() const;
     QPoint infoFramePos() const;
@@ -193,19 +190,24 @@ public:
     void selectLastRow() { selectRow(model()->rowCount() - 1); }
 };
 
+GenericProposalListView::GenericProposalListView(QWidget *parent)
+    : QListView(parent)
+{
+    setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
+}
+
 QSize GenericProposalListView::calculateSize() const
 {
     static const int maxVisibleItems = 10;
 
     // Determine size by calculating the space of the visible items
-    int visibleItems = model()->rowCount();
-    if (visibleItems > maxVisibleItems)
-        visibleItems = maxVisibleItems;
+    const int visibleItems = qMin(model()->rowCount(), maxVisibleItems);
+    const int firstVisibleRow = verticalScrollBar()->value();
 
     const QStyleOptionViewItem &option = viewOptions();
     QSize shint;
     for (int i = 0; i < visibleItems; ++i) {
-        QSize tmp = itemDelegate()->sizeHint(option, model()->index(i, 0));
+        QSize tmp = itemDelegate()->sizeHint(option, model()->index(i + firstVisibleRow, 0));
         if (shint.width() < tmp.width())
             shint = tmp;
     }
@@ -246,6 +248,7 @@ public:
     QPointer<GenericProposalInfoFrame> m_infoFrame;
     QTimer m_infoTimer;
     CodeAssistant *m_assistant;
+    bool m_autoWidth;
 
 public slots:
     void handleActivation(const QModelIndex &modelIndex);
@@ -260,11 +263,12 @@ GenericProposalWidgetPrivate::GenericProposalWidgetPrivate(QWidget *completionWi
     , m_explicitlySelected(false)
     , m_justInvoked(false)
     , m_assistant(0)
+    , m_autoWidth(true)
 {
     connect(m_completionListView, SIGNAL(activated(QModelIndex)),
             this, SLOT(handleActivation(QModelIndex)));
 
-    m_infoTimer.setInterval(1000);
+    m_infoTimer.setInterval(Constants::COMPLETION_ASSIST_TOOLTIP_DELAY);
     m_infoTimer.setSingleShot(true);
     connect(&m_infoTimer, SIGNAL(timeout()), SLOT(maybeShowInfoTip()));
 }
@@ -304,30 +308,36 @@ void GenericProposalWidgetPrivate::maybeShowInfoTip()
 // GenericProposalWidget
 // ------------------------
 GenericProposalWidget::GenericProposalWidget()
-    : m_d(new GenericProposalWidgetPrivate(this))
+    : d(new GenericProposalWidgetPrivate(this))
 {
-#ifdef Q_WS_MAC
-    if (m_d->m_completionListView->horizontalScrollBar())
-        m_d->m_completionListView->horizontalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
-    if (m_d->m_completionListView->verticalScrollBar())
-        m_d->m_completionListView->verticalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
-#else
-    // This improves the look with QGTKStyle.
-    setFrameStyle(m_d->m_completionListView->frameStyle());
-#endif
-    m_d->m_completionListView->setFrameStyle(QFrame::NoFrame);
-    m_d->m_completionListView->setAttribute(Qt::WA_MacShowFocusRect, false);
-    m_d->m_completionListView->setUniformItemSizes(true);
-    m_d->m_completionListView->setSelectionBehavior(QAbstractItemView::SelectItems);
-    m_d->m_completionListView->setSelectionMode(QAbstractItemView::SingleSelection);
-    m_d->m_completionListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    m_d->m_completionListView->setMinimumSize(1, 1);
+    if (HostOsInfo::isMacHost()) {
+        if (d->m_completionListView->horizontalScrollBar())
+            d->m_completionListView->horizontalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+        if (d->m_completionListView->verticalScrollBar())
+            d->m_completionListView->verticalScrollBar()->setAttribute(Qt::WA_MacMiniSize);
+    } else {
+        // This improves the look with QGTKStyle.
+        setFrameStyle(d->m_completionListView->frameStyle());
+    }
+    d->m_completionListView->setFrameStyle(QFrame::NoFrame);
+    d->m_completionListView->setAttribute(Qt::WA_MacShowFocusRect, false);
+    d->m_completionListView->setUniformItemSizes(true);
+    d->m_completionListView->setSelectionBehavior(QAbstractItemView::SelectItems);
+    d->m_completionListView->setSelectionMode(QAbstractItemView::SingleSelection);
+    d->m_completionListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    d->m_completionListView->setMinimumSize(1, 1);
+    connect(d->m_completionListView->verticalScrollBar(), SIGNAL(valueChanged(int)),
+            this, SLOT(updatePositionAndSize()));
+    connect(d->m_completionListView->verticalScrollBar(), SIGNAL(sliderPressed()),
+            this, SLOT(turnOffAutoWidth()));
+    connect(d->m_completionListView->verticalScrollBar(), SIGNAL(sliderReleased()),
+            this, SLOT(turnOnAutoWidth()));
 
     QVBoxLayout *layout = new QVBoxLayout(this);
     layout->setMargin(0);
-    layout->addWidget(m_d->m_completionListView);
+    layout->addWidget(d->m_completionListView);
 
-    m_d->m_completionListView->installEventFilter(this);
+    d->m_completionListView->installEventFilter(this);
 
     setObjectName(QLatin1String("m_popupFrame"));
     setMinimumSize(1, 1);
@@ -335,62 +345,63 @@ GenericProposalWidget::GenericProposalWidget()
 
 GenericProposalWidget::~GenericProposalWidget()
 {
-    delete m_d->m_model;
+    delete d->m_model;
+    delete d;
 }
 
 void GenericProposalWidget::setAssistant(CodeAssistant *assistant)
 {
-    m_d->m_assistant = assistant;
+    d->m_assistant = assistant;
 }
 
 void GenericProposalWidget::setReason(AssistReason reason)
 {
-    m_d->m_reason = reason;
-    if (m_d->m_reason == ExplicitlyInvoked)
-        m_d->m_justInvoked = true;
+    d->m_reason = reason;
+    if (d->m_reason == ExplicitlyInvoked)
+        d->m_justInvoked = true;
 }
 
 void GenericProposalWidget::setKind(AssistKind kind)
 {
-    m_d->m_kind = kind;
+    d->m_kind = kind;
 }
 
 void GenericProposalWidget::setUnderlyingWidget(const QWidget *underlyingWidget)
 {
     setFont(underlyingWidget->font());
-    m_d->m_underlyingWidget = underlyingWidget;
+    d->m_underlyingWidget = underlyingWidget;
 }
 
 void GenericProposalWidget::setModel(IAssistProposalModel *model)
 {
-    delete m_d->m_model;
-    m_d->m_model = static_cast<IGenericProposalModel *>(model);
-    m_d->m_completionListView->setModel(new ModelAdapter(m_d->m_model, m_d->m_completionListView));
+    delete d->m_model;
+    d->m_model = static_cast<IGenericProposalModel *>(model);
+    d->m_completionListView->setModel(new ModelAdapter(d->m_model, d->m_completionListView));
 
-    connect(m_d->m_completionListView->selectionModel(),
+    connect(d->m_completionListView->selectionModel(),
             SIGNAL(currentChanged(QModelIndex,QModelIndex)),
-            &m_d->m_infoTimer,
+            &d->m_infoTimer,
             SLOT(start()));
 }
 
 void GenericProposalWidget::setDisplayRect(const QRect &rect)
 {
-    m_d->m_displayRect = rect;
+    d->m_displayRect = rect;
 }
 
 void GenericProposalWidget::setIsSynchronized(bool isSync)
 {
-    m_d->m_isSynchronized = isSync;
+    d->m_isSynchronized = isSync;
 }
 
 void GenericProposalWidget::showProposal(const QString &prefix)
 {
     ensurePolished();
-    m_d->m_model->removeDuplicates();
+    d->m_model->removeDuplicates();
     if (!updateAndCheck(prefix))
         return;
     show();
-    m_d->m_completionListView->setFocus();
+    d->m_completionListView->setFocus();
 }
 
 void GenericProposalWidget::updateProposal(const QString &prefix)
@@ -408,7 +419,7 @@ void GenericProposalWidget::closeProposal()
 void GenericProposalWidget::notifyActivation(int index)
 {
     abort();
-    emit proposalItemActivated(m_d->m_model->proposalItem(index));
+    emit proposalItemActivated(d->m_model->proposalItem(index));
 }
 
 void GenericProposalWidget::abort()
@@ -422,62 +433,62 @@ bool GenericProposalWidget::updateAndCheck(const QString &prefix)
 {
     // Keep track in the case there has been an explicit selection.
     int preferredItemId = -1;
-    if (m_d->m_explicitlySelected)
+    if (d->m_explicitlySelected)
         preferredItemId =
-                m_d->m_model->persistentId(m_d->m_completionListView->currentIndex().row());
+                d->m_model->persistentId(d->m_completionListView->currentIndex().row());
 
     // Filter, sort, etc.
-    m_d->m_model->reset();
+    d->m_model->reset();
     if (!prefix.isEmpty())
-        m_d->m_model->filter(prefix);
-    if (m_d->m_model->size() == 0
-            || (!m_d->m_model->keepPerfectMatch(m_d->m_reason)
-                && isPerfectMatch(prefix, m_d->m_model))) {
+        d->m_model->filter(prefix);
+    if (d->m_model->size() == 0
+            || (!d->m_model->keepPerfectMatch(d->m_reason)
+                && isPerfectMatch(prefix, d->m_model))) {
         abort();
         return false;
     }
-    if (m_d->m_model->isSortable())
-        m_d->m_model->sort();
-    m_d->m_completionListView->reset();
+    if (d->m_model->isSortable(prefix))
+        d->m_model->sort();
+    d->m_completionListView->reset();
 
     // Try to find the previosly explicit selection (if any). If we can find the item set it
     // as the current. Otherwise (it might have been filtered out) select the first row.
-    if (m_d->m_explicitlySelected) {
+    if (d->m_explicitlySelected) {
         Q_ASSERT(preferredItemId != -1);
-        for (int i = 0; i < m_d->m_model->size(); ++i) {
-            if (m_d->m_model->persistentId(i) == preferredItemId) {
-                m_d->m_completionListView->selectRow(i);
+        for (int i = 0; i < d->m_model->size(); ++i) {
+            if (d->m_model->persistentId(i) == preferredItemId) {
+                d->m_completionListView->selectRow(i);
                 break;
             }
         }
     }
-    if (!m_d->m_completionListView->currentIndex().isValid()) {
-        m_d->m_completionListView->selectFirstRow();
-        if (m_d->m_explicitlySelected)
-            m_d->m_explicitlySelected = false;
+    if (!d->m_completionListView->currentIndex().isValid()) {
+        d->m_completionListView->selectFirstRow();
+        if (d->m_explicitlySelected)
+            d->m_explicitlySelected = false;
     }
 
     if (TextEditorSettings::instance()->completionSettings().m_partiallyComplete
-            && m_d->m_kind == Completion
-            && m_d->m_justInvoked
-            && m_d->m_isSynchronized) {
-        if (m_d->m_model->size() == 1) {
-            IAssistProposalItem *item = m_d->m_model->proposalItem(0);
+            && d->m_kind == Completion
+            && d->m_justInvoked
+            && d->m_isSynchronized) {
+        if (d->m_model->size() == 1) {
+            IAssistProposalItem *item = d->m_model->proposalItem(0);
             if (item->implicitlyApplies()) {
                 abort();
                 emit proposalItemActivated(item);
                 return false;
             }
         }
-        if (m_d->m_model->supportsPrefixExpansion()) {
-            const QString &proposalPrefix = m_d->m_model->proposalPrefix();
+        if (d->m_model->supportsPrefixExpansion()) {
+            const QString &proposalPrefix = d->m_model->proposalPrefix();
             if (proposalPrefix.length() > prefix.length())
                 emit prefixExpanded(proposalPrefix);
         }
     }
 
-    if (m_d->m_justInvoked)
-        m_d->m_justInvoked = false;
+    if (d->m_justInvoked)
+        d->m_justInvoked = false;
 
     updatePositionAndSize();
     return true;
@@ -485,51 +496,72 @@ bool GenericProposalWidget::updateAndCheck(const QString &prefix)
 
 void GenericProposalWidget::updatePositionAndSize()
 {
-    const QSize &shint = m_d->m_completionListView->calculateSize();
+    if (!d->m_autoWidth)
+        return;
+
+    const QSize &shint = d->m_completionListView->calculateSize();
     const int fw = frameWidth();
     const int width = shint.width() + fw * 2 + 30;
     const int height = shint.height() + fw * 2;
 
     // Determine the position, keeping the popup on the screen
     const QDesktopWidget *desktop = QApplication::desktop();
-#ifdef Q_WS_MAC
-    const QRect screen = desktop->availableGeometry(desktop->screenNumber(m_d->m_underlyingWidget));
-#else
-    const QRect screen = desktop->screenGeometry(desktop->screenNumber(m_d->m_underlyingWidget));
-#endif
+    const QRect screen = HostOsInfo::isMacHost()
+            ? desktop->availableGeometry(desktop->screenNumber(d->m_underlyingWidget))
+            : desktop->screenGeometry(desktop->screenNumber(d->m_underlyingWidget));
 
-    QPoint pos = m_d->m_displayRect.bottomLeft();
+    QPoint pos = d->m_displayRect.bottomLeft();
     pos.rx() -= 16 + fw;    // Space for the icons
     if (pos.y() + height > screen.bottom())
-        pos.setY(m_d->m_displayRect.top() - height);
+        pos.setY(d->m_displayRect.top() - height);
     if (pos.x() + width > screen.right())
         pos.setX(screen.right() - width);
     setGeometry(pos.x(), pos.y(), width, height);
 }
 
+void GenericProposalWidget::turnOffAutoWidth()
+{
+    d->m_autoWidth = false;
+}
+
+void GenericProposalWidget::turnOnAutoWidth()
+{
+    d->m_autoWidth = true;
+    updatePositionAndSize();
+}
+
+static bool useCarbonWorkaround()
+{
+#if (QT_VERSION < 0x050000) && defined(Q_OS_DARWIN) && ! defined(QT_MAC_USE_COCOA)
+    return true;
+#else
+    return false;
+#endif
+}
+
 bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
 {
     if (e->type() == QEvent::FocusOut) {
         abort();
-#if defined(Q_OS_DARWIN) && ! defined(QT_MAC_USE_COCOA)
-        QFocusEvent *fe = static_cast<QFocusEvent *>(e);
-        if (fe->reason() == Qt::OtherFocusReason) {
-            // Qt/carbon workaround
-            // focus out is received before the key press event.
-            if (m_d->m_completionListView->currentIndex().isValid())
-                emit proposalItemActivated(m_d->m_model->proposalItem(
-                                              m_d->m_completionListView->currentIndex().row()));
+        if (useCarbonWorkaround()) {
+            QFocusEvent *fe = static_cast<QFocusEvent *>(e);
+            if (fe->reason() == Qt::OtherFocusReason) {
+                // Qt/carbon workaround
+                // focus out is received before the key press event.
+                if (d->m_completionListView->currentIndex().isValid())
+                    emit proposalItemActivated(d->m_model->proposalItem(
+                                                   d->m_completionListView->currentIndex().row()));
+            }
         }
-#endif
-        if (m_d->m_infoFrame)
-            m_d->m_infoFrame->close();
+        if (d->m_infoFrame)
+            d->m_infoFrame->close();
         return true;
     } else if (e->type() == QEvent::ShortcutOverride) {
         QKeyEvent *ke = static_cast<QKeyEvent *>(e);
         switch (ke->key()) {
         case Qt::Key_N:
         case Qt::Key_P:
-            if (ke->modifiers() == Qt::ControlModifier) {
+            if (ke->modifiers() == Qt::KeyboardModifiers(HostOsInfo::controlModifier())) {
                 e->accept();
                 return true;
             }
@@ -544,40 +576,40 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
         case Qt::Key_N:
         case Qt::Key_P:
             // select next/previous completion
-            m_d->m_explicitlySelected = true;
-            if (ke->modifiers() == Qt::ControlModifier) {
+            d->m_explicitlySelected = true;
+            if (ke->modifiers() == Qt::KeyboardModifiers(HostOsInfo::controlModifier())) {
                 int change = (ke->key() == Qt::Key_N) ? 1 : -1;
-                int nrows = m_d->m_model->size();
-                int row = m_d->m_completionListView->currentIndex().row();
+                int nrows = d->m_model->size();
+                int row = d->m_completionListView->currentIndex().row();
                 int newRow = (row + change + nrows) % nrows;
                 if (newRow == row + change || !ke->isAutoRepeat())
-                    m_d->m_completionListView->selectRow(newRow);
+                    d->m_completionListView->selectRow(newRow);
                 return true;
             }
             break;
 
         case Qt::Key_Tab:
         case Qt::Key_Return:
-#if defined(QT_MAC_USE_COCOA) || !defined(Q_OS_DARWIN)
-            abort();
-            if (m_d->m_completionListView->currentIndex().isValid())
-                emit proposalItemActivated(m_d->m_model->proposalItem(
-                                              m_d->m_completionListView->currentIndex().row()));
-#endif
+            if (!useCarbonWorkaround()) {
+                abort();
+                if (d->m_completionListView->currentIndex().isValid())
+                    emit proposalItemActivated(d->m_model->proposalItem(
+                                                   d->m_completionListView->currentIndex().row()));
+            }
             return true;
 
         case Qt::Key_Up:
-            m_d->m_explicitlySelected = true;
-            if (!ke->isAutoRepeat() && m_d->m_completionListView->isFirstRowSelected()) {
-                m_d->m_completionListView->selectLastRow();
+            d->m_explicitlySelected = true;
+            if (!ke->isAutoRepeat() && d->m_completionListView->isFirstRowSelected()) {
+                d->m_completionListView->selectLastRow();
                 return true;
             }
             return false;
 
         case Qt::Key_Down:
-            m_d->m_explicitlySelected = true;
-            if (!ke->isAutoRepeat() && m_d->m_completionListView->isLastRowSelected()) {
-                m_d->m_completionListView->selectFirstRow();
+            d->m_explicitlySelected = true;
+            if (!ke->isAutoRepeat() && d->m_completionListView->isLastRowSelected()) {
+                d->m_completionListView->selectFirstRow();
                 return true;
             }
             return false;
@@ -603,22 +635,22 @@ bool GenericProposalWidget::eventFilter(QObject *o, QEvent *e)
         }
 
         if (ke->text().length() == 1
-                && m_d->m_completionListView->currentIndex().isValid()
+                && d->m_completionListView->currentIndex().isValid()
                 && qApp->focusWidget() == o) {
             const QChar &typedChar = ke->text().at(0);
             IAssistProposalItem *item =
-                m_d->m_model->proposalItem(m_d->m_completionListView->currentIndex().row());
+                d->m_model->proposalItem(d->m_completionListView->currentIndex().row());
             if (item->prematurelyApplies(typedChar)
-                    && (m_d->m_reason == ExplicitlyInvoked || item->text().endsWith(typedChar))) {
+                    && (d->m_reason == ExplicitlyInvoked || item->text().endsWith(typedChar))) {
                 abort();
                 emit proposalItemActivated(item);
                 return true;
             }
         }
 
-        QApplication::sendEvent(const_cast<QWidget *>(m_d->m_underlyingWidget), e);
+        QApplication::sendEvent(const_cast<QWidget *>(d->m_underlyingWidget), e);
         if (isVisible())
-            m_d->m_assistant->notifyChange();
+            d->m_assistant->notifyChange();
 
         return true;
     }
