@@ -28,13 +28,10 @@
 ****************************************************************************/
 
 #include "rule.h"
-#include "highlighterexception.h"
 #include "progressdata.h"
 #include "highlightdefinition.h"
 #include "reuse.h"
 
-#include <QStringList>
-
 #include <functional>
 
 using namespace TextEditor;
