@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -31,15 +31,10 @@
 #include "basetextdocument.h"
 #include "basetextdocumentlayout.h"
 
-#include <qtextdocument.h>
-#include <qtextlayout.h>
-#include <qpointer.h>
-#include <qtextobject.h>
-#include <qtextcursor.h>
-#include <qdebug.h>
-#include <qtextedit.h>
 #include <qtimer.h>
 
+#include <math.h>
+
 using namespace TextEditor;
 
 class TextEditor::SyntaxHighlighterPrivate
@@ -558,8 +553,6 @@ void SyntaxHighlighter::applyFormatToSpaces(const QString &text, const QTextChar
 }
 
 /*!
-    \fn QTextCharFormat SyntaxHighlighter::format(int position) const
-
     Returns the format at \a position inside the syntax highlighter's
     current text block.
 */
@@ -692,8 +685,10 @@ static bool byStartOfRange(const QTextLayout::FormatRange &range, const QTextLay
     return range.start < other.start;
 }
 
+// The formats is passed in by reference in order to prevent unnecessary copying of its items.
+// After this method returns, the list is modified, and should be considered invalidated!
 void SyntaxHighlighter::setExtraAdditionalFormats(const QTextBlock& block,
-                                                  const QList<QTextLayout::FormatRange> &fmts)
+                                                  QList<QTextLayout::FormatRange> &formats)
 {
 //    qDebug() << "setAdditionalFormats() on block" << block.blockNumber();
 //    qDebug() << "   is valid:" << (block.isValid() ? "Yes" : "No");
@@ -706,21 +701,21 @@ void SyntaxHighlighter::setExtraAdditionalFormats(const QTextBlock& block,
 //                 << "color:" << overrides.at(i).format.foreground().color();
     Q_D(SyntaxHighlighter);
 
-    if (block.layout() == 0 || block.text().isEmpty())
+    const int blockLength = block.length();
+    if (block.layout() == 0 || blockLength == 0)
         return;
 
-    QList<QTextLayout::FormatRange> formats;
-    formats.reserve(fmts.size());
-    foreach (QTextLayout::FormatRange r, fmts) {
-        r.format.setProperty(QTextFormat::UserProperty, true);
-        formats.append(r);
-    }
     qSort(formats.begin(), formats.end(), byStartOfRange);
 
+    const QList<QTextLayout::FormatRange> all = block.layout()->additionalFormats();
     QList<QTextLayout::FormatRange> previousSemanticFormats;
     QList<QTextLayout::FormatRange> formatsToApply;
+    previousSemanticFormats.reserve(all.size());
+    formatsToApply.reserve(all.size() + formats.size());
+
+    for (int i = 0, ei = formats.size(); i < ei; ++i)
+        formats[i].format.setProperty(QTextFormat::UserProperty, true);
 
-    const QList<QTextLayout::FormatRange> all = block.layout()->additionalFormats();
     foreach (const QTextLayout::FormatRange &r, all) {
         if (r.format.hasProperty(QTextFormat::UserProperty))
             previousSemanticFormats.append(r);
@@ -751,8 +746,42 @@ void SyntaxHighlighter::setExtraAdditionalFormats(const QTextBlock& block,
     bool wasInReformatBlocks = d->inReformatBlocks;
     d->inReformatBlocks = true;
     block.layout()->setAdditionalFormats(formatsToApply);
-    document()->markContentsDirty(block.position(), block.length()-1);
+    document()->markContentsDirty(block.position(), blockLength - 1);
     d->inReformatBlocks = wasInReformatBlocks;
 }
 
+/* Generate at least n different colors for highlighting, excluding background
+ * color. */
+
+QList<QColor> SyntaxHighlighter::generateColors(int n, const QColor &background)
+{
+    QList<QColor> result;
+    // Assign a color gradient. Generate a sufficient number of colors
+    // by using ceil and looping from 0..step.
+    const double oneThird = 1.0 / 3.0;
+    const int step = qRound(ceil(pow(double(n), oneThird)));
+    result.reserve(step * step * step);
+    const int factor = 255 / step;
+    const int half = factor / 2;
+    const int bgRed = background.red();
+    const int bgGreen = background.green();
+    const int bgBlue = background.blue();
+    for (int r = step; r >= 0; --r) {
+        const int red = r * factor;
+        if (bgRed - half > red || bgRed + half <= red) {
+            for (int g = step; g >= 0; --g) {
+                const int green = g * factor;
+                if (bgGreen - half > green || bgGreen + half <= green) {
+                    for (int b = step; b >= 0 ; --b) {
+                        const int blue = b * factor;
+                        if (bgBlue - half > blue || bgBlue + half <= blue)
+                            result.append(QColor(red, green, blue));
+                    }
+                }
+            }
+        }
+    }
+    return result;
+}
+
 #include "moc_syntaxhighlighter.cpp"
