@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -29,16 +29,13 @@
 
 #include "commonsettingspage.h"
 #include "vcsbaseconstants.h"
-#include "nicknamedialog.h"
 
 #include "ui_commonsettingspage.h"
 
 #include <coreplugin/icore.h>
-#include <extensionsystem/pluginmanager.h>
 
 #include <QDebug>
 #include <QCoreApplication>
-#include <QMessageBox>
 
 namespace VcsBase {
 namespace Internal {
@@ -109,7 +106,7 @@ CommonOptionsPage::CommonOptionsPage(QObject *parent) :
 {
     m_settings.fromSettings(Core::ICore::settings());
 
-    setId(QLatin1String(Constants::VCS_COMMON_SETTINGS_ID));
+    setId(Constants::VCS_COMMON_SETTINGS_ID);
     setDisplayName(QCoreApplication::translate("VcsBase", Constants::VCS_COMMON_SETTINGS_NAME));
 }
 
