@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -30,55 +30,42 @@
 #include "vcsbasesubmiteditor.h"
 
 #include "commonvcssettings.h"
-#include "vcsbaseoutputwindow.h"
-#include "vcsplugin.h"
 #include "nicknamedialog.h"
 #include "submiteditorfile.h"
+#include "submiteditorwidget.h"
+#include "submitfieldwidget.h"
+#include "submitfilemodel.h"
+#include "vcsbaseoutputwindow.h"
+#include "vcsplugin.h"
 
 #include <aggregation/aggregate.h>
-#include <cplusplus/Control.h>
-#include <cplusplus/CoreTypes.h>
-#include <cplusplus/FullySpecifiedType.h>
-#include <cplusplus/Literals.h>
-#include <cpptools/ModelManagerInterface.h>
-#include <cplusplus/Symbol.h>
-#include <cplusplus/Symbols.h>
-#include <cplusplus/TranslationUnit.h>
-#include <coreplugin/idocument.h>
+#include <cpptools/cppmodelmanagerinterface.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <coreplugin/id.h>
-#include <coreplugin/actionmanager/actionmanager.h>
-#include <utils/completingtextedit.h>
-#include <utils/submiteditorwidget.h>
 #include <utils/checkablemessagebox.h>
 #include <utils/synchronousprocess.h>
-#include <utils/submitfieldwidget.h>
 #include <utils/fileutils.h>
+#include <utils/qtcassert.h>
 #include <find/basetextfind.h>
 #include <texteditor/fontsettings.h>
 #include <texteditor/texteditorsettings.h>
 
 #include <projectexplorer/projectexplorer.h>
-#include <projectexplorer/session.h>
 #include <projectexplorer/project.h>
 
 #include <QDebug>
 #include <QDir>
 #include <QProcess>
-#include <QFile>
 #include <QFileInfo>
 #include <QPointer>
 #include <QStringListModel>
-#include <QTextStream>
 #include <QStyle>
 #include <QToolBar>
 #include <QAction>
 #include <QApplication>
 #include <QMessageBox>
 #include <QCompleter>
-#include <QLineEdit>
-#include <QTextEdit>
+
 #include <cstring>
 
 enum { debug = 0 };
@@ -110,15 +97,17 @@ static const char *belongingClassName(const CPlusPlus::Function *function)
 }
 
 /*!
-    \struct VcsBase::VcsBaseSubmitEditorParameters
+    \class VcsBase::VcsBaseSubmitEditorParameters
 
-    \brief Utility struct to parametrize a VcsBaseSubmitEditor.
+    \brief The VcsBaseSubmitEditorParameters class is a utility class
+    to parametrize a VcsBaseSubmitEditor.
 */
 
 /*!
     \class  VcsBase::VcsBaseSubmitEditor
 
-    \brief Base class for a submit editor based on the Utils::SubmitEditorWidget.
+    \brief The VcsBaseSubmitEditor class is the base class for a submit editor
+    based on the SubmitEditorWidget.
 
     Presents the commit message in a text editor and an
     checkable list of modified files in a list window. The user can delete
@@ -207,7 +196,7 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     connect(d->m_file, SIGNAL(saveMe(QString*,QString,bool)),
             this, SLOT(save(QString*,QString,bool)));
 
-    connect(d->m_widget, SIGNAL(diffSelected(QStringList)), this, SLOT(slotDiffSelectedVcsFiles(QStringList)));
+    connect(d->m_widget, SIGNAL(diffSelected(QList<int>)), this, SLOT(slotDiffSelectedVcsFiles(QList<int>)));
     connect(d->m_widget->descriptionEdit(), SIGNAL(textChanged()), this, SLOT(slotDescriptionChanged()));
 
     const CommonVcsSettings settings = VcsPlugin::instance()->settings();
@@ -238,6 +227,9 @@ VcsBaseSubmitEditor::VcsBaseSubmitEditor(const VcsBaseSubmitEditorParameters *pa
     connect(VcsPlugin::instance(),
             SIGNAL(settingsChanged(VcsBase::Internal::CommonVcsSettings)),
             this, SLOT(slotUpdateEditorSettings(VcsBase::Internal::CommonVcsSettings)));
+    connect(Core::EditorManager::instance(), SIGNAL(currentEditorChanged(Core::IEditor*)),
+            this, SLOT(slotRefreshCommitData()));
+    connect(Core::ICore::mainWindow(), SIGNAL(windowActivated()), this, SLOT(slotRefreshCommitData()));
 
     Aggregation::Aggregate *aggregate = new Aggregation::Aggregate;
     aggregate->add(new Find::BaseTextFind(d->m_widget->descriptionEdit()));
@@ -257,6 +249,12 @@ void VcsBaseSubmitEditor::slotUpdateEditorSettings(const CommonVcsSettings &s)
     setLineWrap(s.lineWrap);
 }
 
+void VcsBaseSubmitEditor::slotRefreshCommitData()
+{
+    if (Core::EditorManager::currentEditor() == this)
+        updateFileModel();
+}
+
 // Return a trimmed list of non-empty field texts
 static inline QStringList fieldTexts(const QString &fileContents)
 {
@@ -308,16 +306,6 @@ void VcsBaseSubmitEditor::unregisterActions(QAction *editorUndoAction,  QAction
     d->m_diffAction = d->m_submitAction = 0;
 }
 
-int VcsBaseSubmitEditor::fileNameColumn() const
-{
-    return d->m_widget->fileNameColumn();
-}
-
-void VcsBaseSubmitEditor::setFileNameColumn(int c)
-{
-    d->m_widget->setFileNameColumn(c);
-}
-
 QAbstractItemView::SelectionMode VcsBaseSubmitEditor::fileListSelectionMode() const
 {
     return d->m_widget->fileListSelectionMode();
@@ -414,16 +402,6 @@ void VcsBaseSubmitEditor::setCheckScriptWorkingDirectory(const QString &s)
     d->m_checkScriptWorkingDirectory = s;
 }
 
-bool VcsBaseSubmitEditor::duplicateSupported() const
-{
-    return false;
-}
-
-Core::IEditor *VcsBaseSubmitEditor::duplicate(QWidget * /*parent*/)
-{
-    return 0;
-}
-
 Core::Id VcsBaseSubmitEditor::id() const
 {
     return d->m_parameters->id;
@@ -461,32 +439,26 @@ QWidget *VcsBaseSubmitEditor::toolBar()
     return d->m_toolWidget;
 }
 
-QByteArray VcsBaseSubmitEditor::saveState() const
-{
-    return QByteArray();
-}
-
-bool VcsBaseSubmitEditor::restoreState(const QByteArray &/*state*/)
-{
-    return true;
-}
-
 QStringList VcsBaseSubmitEditor::checkedFiles() const
 {
     return d->m_widget->checkedFiles();
 }
 
-void VcsBaseSubmitEditor::setFileModel(QAbstractItemModel *m, const QString &repositoryDirectory)
+void VcsBaseSubmitEditor::setFileModel(SubmitFileModel *model, const QString &repositoryDirectory)
 {
-    d->m_widget->setFileModel(m);
+    QTC_ASSERT(model, return);
+    if (SubmitFileModel *oldModel = d->m_widget->fileModel()) {
+        model->updateSelections(oldModel);
+        delete oldModel;
+    }
+    d->m_widget->setFileModel(model);
 
     QSet<QString> uniqueSymbols;
-    const CPlusPlus::Snapshot cppSnapShot = CPlusPlus::CppModelManagerInterface::instance()->snapshot();
+    const CPlusPlus::Snapshot cppSnapShot = CppTools::CppModelManagerInterface::instance()->snapshot();
 
     // Iterate over the files and get interesting symbols
-    for (int row = 0; row < m->rowCount(); ++row) {
-        const QString fileName = m->data(m->index(row, d->m_widget->fileNameColumn())).toString();
-        const QFileInfo fileInfo(repositoryDirectory, fileName);
+    for (int row = 0; row < model->rowCount(); ++row) {
+        const QFileInfo fileInfo(repositoryDirectory, model->file(row));
 
         // Add file name
         uniqueSymbols.insert(fileInfo.fileName());
@@ -529,14 +501,30 @@ void VcsBaseSubmitEditor::setFileModel(QAbstractItemModel *m, const QString &rep
     }
 }
 
-QAbstractItemModel *VcsBaseSubmitEditor::fileModel() const
+SubmitFileModel *VcsBaseSubmitEditor::fileModel() const
 {
     return d->m_widget->fileModel();
 }
 
-void VcsBaseSubmitEditor::slotDiffSelectedVcsFiles(const QStringList &rawList)
+QStringList VcsBaseSubmitEditor::rowsToFiles(const QList<int> &rows) const
 {
-     emit diffSelectedFiles(rawList);
+    if (rows.empty())
+        return QStringList();
+
+    QStringList rc;
+    const SubmitFileModel *model = fileModel();
+    const int count = rows.size();
+    for (int i = 0; i < count; i++)
+        rc.push_back(model->file(rows.at(i)));
+    return rc;
+}
+
+void VcsBaseSubmitEditor::slotDiffSelectedVcsFiles(const QList<int> &rawList)
+{
+    if (d->m_parameters->diffType == VcsBaseSubmitEditorParameters::DiffRows)
+        emit diffSelectedFiles(rawList);
+    else
+        emit diffSelectedFiles(rowsToFiles(rawList));
 }
 
 bool VcsBaseSubmitEditor::save(QString *errorString, const QString &fileName, bool autoSave)
@@ -583,12 +571,11 @@ VcsBaseSubmitEditor::PromptSubmitResult
                                           const QString &checkFailureQuestion,
                                           bool *promptSetting,
                                           bool forcePrompt,
-                                          bool canCommitOnFailure) const
+                                          bool canCommitOnFailure)
 {
-    SubmitEditorWidget *submitWidget =
-            static_cast<SubmitEditorWidget *>(const_cast<VcsBaseSubmitEditor *>(this)->widget());
+    SubmitEditorWidget *submitWidget = static_cast<SubmitEditorWidget *>(this->widget());
 
-    raiseSubmitEditor();
+    Core::EditorManager::activateEditor(this, Core::EditorManager::IgnoreNavigationHistory);
 
     QString errorMessage;
     QMessageBox::StandardButton answer = QMessageBox::Yes;
@@ -801,31 +788,11 @@ void VcsBaseSubmitEditor::filterUntrackedFilesOfProject(const QString &repositor
     const QDir repoDir(repositoryDirectory);
     for (QStringList::iterator it = untrackedFiles->begin(); it != untrackedFiles->end(); ) {
         const QString path = QDir::toNativeSeparators(repoDir.absoluteFilePath(*it));
-        if (nativeProjectFiles.contains(path)) {
+        if (nativeProjectFiles.contains(path))
             ++it;
-        } else {
+        else
             it = untrackedFiles->erase(it);
-        }
     }
 }
 
-// Helper to raise an already open submit editor to prevent opening twice.
-bool VcsBaseSubmitEditor::raiseSubmitEditor()
-{
-    // Nothing to do?
-    if (Core::IEditor *ce = Core::EditorManager::currentEditor())
-        if (qobject_cast<VcsBaseSubmitEditor*>(ce))
-            return true;
-    // Try to activate a hidden one
-    Core::EditorManager *em = Core::EditorManager::instance();
-    foreach (Core::IEditor *e, em->openedEditors()) {
-        if (qobject_cast<VcsBaseSubmitEditor*>(e)) {
-            Core::EditorManager::activateEditor(e,
-                Core::EditorManager::IgnoreNavigationHistory | Core::EditorManager::ModeSwitch);
-            return true;
-        }
-    }
-    return false;
-}
-
 } // namespace VcsBase
