@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include <QtTest>
 #include <QObject>
@@ -119,6 +116,7 @@ private Q_SLOTS:
     void functionBodyAndBraces4();
     void constructor1();
     void constructor2();
+    void constructor3();
     void caseBody1();
     void caseBody2();
     void caseBody3();
@@ -128,6 +126,7 @@ private Q_SLOTS:
     void blockBraces1();
     void functionDefaultArgument();
     void attributeInAccessSpecifier();
+    void braceReturn();
 };
 
 struct Line {
@@ -1878,6 +1877,42 @@ void tst_CodeFormatter::constructor2()
     checkIndent(data);
 }
 
+void tst_CodeFormatter::constructor3()
+{
+    QList<Line> data;
+    data << Line("class Foo {")
+         << Line("    Foo() : _a{0}, _b{1, {2, {3, \"foo\"}, 3}}")
+         << Line("    {")
+         << Line("        _b = 0")
+         << Line("    }")
+         << Line("    int _a;")
+         << Line("    Foo()")
+         << Line("    ~   : _foo{1},")
+         << Line("    ~     _bar{2},")
+         << Line("    ~     _carooooo(")
+         << Line("    ~         foo() + 12),")
+         << Line("    ~     _carooooo{foo(),")
+         << Line("    ~               12}")
+         << Line("    {")
+         << Line("        _b = 0")
+         << Line("    }")
+         << Line("    int _b;")
+         << Line("    Foo()")
+         << Line("    ~   : _foo{1}")
+         << Line("    ~   , _bar{2}")
+         << Line("    ~   , _carooooo{")
+         << Line("    ~         foo() + 12}")
+         << Line("    ~   , _carooooo{foo(),")
+         << Line("    ~               12}")
+         << Line("    {")
+         << Line("        _b = 0")
+         << Line("    }")
+         << Line("};")
+         ;
+    CppCodeStyleSettings codeStyle;
+    checkIndent(data);
+}
+
 void tst_CodeFormatter::caseBody1()
 {
     QList<Line> data;
@@ -2063,7 +2098,20 @@ void tst_CodeFormatter::attributeInAccessSpecifier()
     checkIndent(data);
 }
 
-QTEST_APPLESS_MAIN(tst_CodeFormatter)
+void tst_CodeFormatter::braceReturn()
+{
+    QList<Line> data;
+    data << Line("class X {")
+         << Line("    void the_answer() {")
+         << Line("        return {42};")
+         << Line("    }")
+         << Line("    int x;")
+         << Line("};")
+         ;
+    checkIndent(data);
+}
+
+QTEST_MAIN(tst_CodeFormatter)
 
 #include "tst_codeformatter.moc"
 
