@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -43,6 +43,7 @@
 #include <cstdlib>
 #include <fstream>
 #include <iostream>
+#include <string>
 #ifdef __GNUC__
 #  include <cxxabi.h>
 #endif
@@ -57,7 +58,7 @@ public:
     void operator()(const QString &fileName, const QByteArray &src, Node *ast) {
         _src = src;
         QString basename = fileName;
-        int dotIdx = basename.lastIndexOf('.');
+        int dotIdx = basename.lastIndexOf(QLatin1Char('.'));
         if (dotIdx != -1)
             basename.truncate(dotIdx);
         basename.append(QLatin1String(".ast.dot"));
@@ -83,7 +84,7 @@ protected:
     void alignTerminals() {
         out<<"{ rank=same;" << endl;
         foreach (const QByteArray &terminalShape, _terminalShapes) {
-            out << "  " << string(terminalShape) << ";" << endl;
+            out << "  " << string(terminalShape.constData(), terminalShape.size()).c_str() << ";" << endl;
         }
         out<<"}"<<endl;
     }
@@ -98,7 +99,7 @@ protected:
     }
 
     QString spell(const SourceLocation &token) {
-        return _src.mid(token.offset, token.length).replace('\'', "\\\\").replace('"', "\\\"");
+        return QString::fromLatin1(_src.mid(token.offset, token.length).replace('\'', "\\\\").replace('"', "\\\""));
     }
 
     void terminal(const SourceLocation &token) {
@@ -331,7 +332,7 @@ int main(int argc, char *argv[])
         file.close();
 
         Document::MutablePtr doc = Document::create(fileName, Document::guessLanguageFromSuffix(fileName));
-        doc->setSource(source);
+        doc->setSource(QString::fromUtf8(source));
         doc->parse();
 
         foreach (const DiagnosticMessage &m, doc->diagnosticMessages()) {
